@@ -8,10 +8,10 @@ import (
 	"net/url"
 	"os"
 	"path/filepath"
+	"slices"
 	"strings"
 	"time"
 
-	"github.com/gorilla/websocket"
 	"github.com/zitadel/oidc/v3/pkg/oidc"
 
 	"github.com/canonical/lxd/shared"
@@ -116,14 +116,13 @@ func ConnectLXDHTTPWithContext(ctx context.Context, args *ConnectionArgs, client
 
 	// Initialize the client struct
 	server := ProtocolLXD{
-		ctx:                ctx,
-		httpBaseURL:        *httpBaseURL,
-		httpProtocol:       "custom",
-		httpUserAgent:      args.UserAgent,
-		ctxConnected:       ctxConnected,
-		ctxConnectedCancel: ctxConnectedCancel,
-		eventConns:         make(map[string]*websocket.Conn),
-		eventListeners:     make(map[string][]*EventListener),
+		ctx:                  ctx,
+		httpBaseURL:          *httpBaseURL,
+		httpProtocol:         "custom",
+		httpUserAgent:        args.UserAgent,
+		ctxConnected:         ctxConnected,
+		ctxConnectedCancel:   ctxConnectedCancel,
+		eventListenerManager: newEventListenerManager(ctx),
 	}
 
 	// Setup the HTTP client
@@ -175,15 +174,14 @@ func ConnectLXDUnixWithContext(ctx context.Context, path string, args *Connectio
 
 	// Initialize the client struct
 	server := ProtocolLXD{
-		ctx:                ctx,
-		httpBaseURL:        *httpBaseURL,
-		httpUnixPath:       path,
-		httpProtocol:       "unix",
-		httpUserAgent:      args.UserAgent,
-		ctxConnected:       ctxConnected,
-		ctxConnectedCancel: ctxConnectedCancel,
-		eventConns:         make(map[string]*websocket.Conn),
-		eventListeners:     make(map[string][]*EventListener),
+		ctx:                  ctx,
+		httpBaseURL:          *httpBaseURL,
+		httpUnixPath:         path,
+		httpProtocol:         "unix",
+		httpUserAgent:        args.UserAgent,
+		ctxConnected:         ctxConnected,
+		ctxConnectedCancel:   ctxConnectedCancel,
+		eventListenerManager: newEventListenerManager(ctx),
 	}
 
 	// Determine the socket path.
@@ -267,7 +265,10 @@ func ConnectSimpleStreams(url string, args *ConnectionArgs) (ImageServer, error)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy, args.TransportWrapper)
+	// Do not include modern post-quantum curves in the ClientHello to avoid
+	// compatibility issues (connection resets) with broken middleboxes that
+	// can't handle large ClientHello messages split over multiple TCP packets.
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, true, args.Proxy, args.TransportWrapper)
 	if err != nil {
 		return nil, err
 	}
@@ -305,6 +306,111 @@ func ConnectSimpleStreams(url string, args *ConnectionArgs) (ImageServer, error)
 	return &server, nil
 }
 
+// ConnectDevLXD lets you connect to a LXD agent over a local unix socket.
+func ConnectDevLXD(socketPath string, args *ConnectionArgs) (DevLXDServer, error) {
+	return ConnectDevLXDWithContext(context.Background(), socketPath, args)
+}
+
+// ConnectDevLXDWithContext lets you connect to a LXD agent over a local unix socket.
+func ConnectDevLXDWithContext(ctx context.Context, socketPath string, args *ConnectionArgs) (DevLXDServer, error) {
+	logger.Debug("Connecting to a devLXD over a Unix socket")
+
+	if args == nil {
+		args = &ConnectionArgs{}
+	}
+
+	socketPath = shared.HostPath(socketPath)
+
+	// Verify provided socket path.
+	socketInfo, err := os.Stat(socketPath)
+	if err != nil {
+		return nil, err
+	}
+
+	if socketInfo.Mode()&os.ModeSocket == 0 {
+		return nil, fmt.Errorf("Invalid unix socket path %q: Not a socket", socketPath)
+	}
+
+	// Base LXD agent url.
+	baseURL, err := url.Parse("http://lxd")
+	if err != nil {
+		return nil, err
+	}
+
+	// Create a new HTTP client.
+	client, err := unixHTTPClient(args, socketPath, args.TransportWrapper)
+	if err != nil {
+		return nil, err
+	}
+
+	useragent := "devlxd"
+	if args.UserAgent != "" {
+		useragent = args.UserAgent
+	}
+
+	ctxConnected, ctxConnectedCancel := context.WithCancel(context.Background())
+
+	return &ProtocolDevLXD{
+		ctx:                  ctx,
+		ctxConnected:         ctxConnected,
+		ctxConnectedCancel:   ctxConnectedCancel,
+		http:                 client,
+		httpBaseURL:          *baseURL,
+		httpUnixPath:         socketPath,
+		httpUserAgent:        useragent,
+		eventListenerManager: newEventListenerManager(ctx),
+	}, nil
+}
+
+// ConnectDevLXDHTTPWithContext lets you connect to devLXD over a VM socket.
+func ConnectDevLXDHTTPWithContext(ctx context.Context, args *ConnectionArgs, client *http.Client) (DevLXDServer, error) {
+	logger.Debug("Connecting to a devLXD over a VM socket")
+
+	// Use empty args if not specified.
+	if args == nil {
+		args = &ConnectionArgs{}
+	}
+
+	httpBaseURL, err := url.Parse("https://custom.socket")
+	if err != nil {
+		return nil, err
+	}
+
+	ctxConnected, ctxConnectedCancel := context.WithCancel(context.Background())
+
+	// Initialize the client.
+	server := ProtocolDevLXD{
+		ctx:                  ctx,
+		httpBaseURL:          *httpBaseURL,
+		httpUserAgent:        args.UserAgent,
+		ctxConnected:         ctxConnected,
+		ctxConnectedCancel:   ctxConnectedCancel,
+		eventListenerManager: newEventListenerManager(ctx),
+	}
+
+	// Setup the HTTP client.
+	if client != nil {
+		// If the http.Transport has a TLSClientConfig, it indicates that the
+		// connection is using vsock.
+		transport, ok := client.Transport.(*http.Transport)
+		if ok && transport != nil && transport.TLSClientConfig != nil {
+			server.isDevLXDOverVsock = true
+		}
+
+		server.http = client
+	}
+
+	// Test the connection.
+	if !args.SkipGetServer {
+		_, err := server.GetState()
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	return &server, nil
+}
+
 // Internal function called by ConnectLXD and ConnectPublicLXD.
 func httpsLXD(ctx context.Context, requestURL string, args *ConnectionArgs) (InstanceServer, error) {
 	// Use empty args if not specified
@@ -321,23 +427,22 @@ func httpsLXD(ctx context.Context, requestURL string, args *ConnectionArgs) (Ins
 
 	// Initialize the client struct
 	server := ProtocolLXD{
-		ctx:                ctx,
-		httpCertificate:    args.TLSServerCert,
-		httpBaseURL:        *httpBaseURL,
-		httpProtocol:       "https",
-		httpUserAgent:      args.UserAgent,
-		ctxConnected:       ctxConnected,
-		ctxConnectedCancel: ctxConnectedCancel,
-		eventConns:         make(map[string]*websocket.Conn),
-		eventListeners:     make(map[string][]*EventListener),
+		ctx:                  ctx,
+		httpCertificate:      args.TLSServerCert,
+		httpBaseURL:          *httpBaseURL,
+		httpProtocol:         "https",
+		httpUserAgent:        args.UserAgent,
+		ctxConnected:         ctxConnected,
+		ctxConnectedCancel:   ctxConnectedCancel,
+		eventListenerManager: newEventListenerManager(ctx),
 	}
 
-	if shared.ValueInSlice(args.AuthType, []string{api.AuthenticationMethodOIDC}) {
+	if slices.Contains([]string{api.AuthenticationMethodOIDC}, args.AuthType) {
 		server.RequireAuthenticated(true)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy, args.TransportWrapper)
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, false, args.Proxy, args.TransportWrapper)
 	if err != nil {
 		return nil, err
 	}
