@@ -3,6 +3,7 @@ package lxd
 import (
 	"fmt"
 	"net"
+	"net/http"
 
 	"github.com/canonical/lxd/shared/api"
 )
@@ -17,7 +18,7 @@ func (r *ProtocolLXD) GetNetworkLoadBalancerAddresses(networkName string) ([]str
 	// Fetch the raw URL values.
 	urls := []string{}
 	u := api.NewURL().Path("networks", networkName, "load-balancers")
-	_, err = r.queryStruct("GET", u.String(), nil, "", &urls)
+	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -37,7 +38,7 @@ func (r *ProtocolLXD) GetNetworkLoadBalancers(networkName string) ([]api.Network
 
 	// Fetch the raw value.
 	u := api.NewURL().Path("networks", networkName, "load-balancers").WithQuery("recursion", "1")
-	_, err = r.queryStruct("GET", u.String(), nil, "", &loadBalancers)
+	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &loadBalancers)
 	if err != nil {
 		return nil, err
 	}
@@ -56,7 +57,7 @@ func (r *ProtocolLXD) GetNetworkLoadBalancer(networkName string, listenAddress s
 
 	// Fetch the raw value.
 	u := api.NewURL().Path("networks", networkName, "load-balancers", listenAddress)
-	etag, err := r.queryStruct("GET", u.String(), nil, "", &loadBalancer)
+	etag, err := r.queryStruct(http.MethodGet, u.String(), nil, "", &loadBalancer)
 	if err != nil {
 		return nil, "", err
 	}
@@ -85,7 +86,7 @@ func (r *ProtocolLXD) CreateNetworkLoadBalancer(networkName string, loadBalancer
 
 	// Send the request.
 	u := api.NewURL().Path("networks", networkName, "load-balancers")
-	_, _, err = r.query("POST", u.String(), loadBalancer, "")
+	_, _, err = r.query(http.MethodPost, u.String(), loadBalancer, "")
 	if err != nil {
 		return err
 	}
@@ -102,7 +103,7 @@ func (r *ProtocolLXD) UpdateNetworkLoadBalancer(networkName string, listenAddres
 
 	// Send the request.
 	u := api.NewURL().Path("networks", networkName, "load-balancers", listenAddress)
-	_, _, err = r.query("PUT", u.String(), loadBalancer, ETag)
+	_, _, err = r.query(http.MethodPut, u.String(), loadBalancer, ETag)
 	if err != nil {
 		return err
 	}
@@ -119,7 +120,7 @@ func (r *ProtocolLXD) DeleteNetworkLoadBalancer(networkName string, listenAddres
 
 	// Send the request.
 	u := api.NewURL().Path("networks", networkName, "load-balancers", listenAddress)
-	_, _, err = r.query("DELETE", u.String(), nil, "")
+	_, _, err = r.query(http.MethodDelete, u.String(), nil, "")
 	if err != nil {
 		return err
 	}
