@@ -5,7 +5,9 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"slices"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -95,11 +97,16 @@ lxc cluster group assign foo default
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterMembers(toComplete)
+			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+		}
+
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpClusterGroupNames(args[0])
+			return c.global.cmpTopLevelResourceInRemote(remote, "cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -179,7 +186,7 @@ lxc cluster group create g1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, false)
+			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -260,7 +267,7 @@ func (c *cmdClusterGroupDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterGroups(toComplete)
+			return c.global.cmpTopLevelResource("cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -320,7 +327,7 @@ func (c *cmdClusterGroupEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterGroups(toComplete)
+			return c.global.cmpTopLevelResource("cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -445,7 +452,7 @@ func (c *cmdClusterGroupList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, false)
+			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -493,7 +500,7 @@ func (c *cmdClusterGroupList) run(cmd *cobra.Command, args []string) error {
 	// Render the table
 	data := [][]string{}
 	for _, group := range groups {
-		line := []string{group.Name, group.Description, fmt.Sprint(len(group.Members))}
+		line := []string{group.Name, group.Description, strconv.Itoa(len(group.Members))}
 		data = append(data, line)
 	}
 
@@ -526,11 +533,16 @@ func (c *cmdClusterGroupRemove) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterMembers(toComplete)
+			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+		}
+
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpClusterGroupNames(args[0])
+			return c.global.cmpTopLevelResourceInRemote(remote, "cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -565,7 +577,7 @@ func (c *cmdClusterGroupRemove) run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if !shared.ValueInSlice(args[1], member.Groups) {
+	if !slices.Contains(member.Groups, args[1]) {
 		return fmt.Errorf(i18n.G("Cluster group %s isn't currently applied to %s"), args[1], resource.name)
 	}
 
@@ -611,7 +623,7 @@ func (c *cmdClusterGroupRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterGroups(toComplete)
+			return c.global.cmpTopLevelResource("cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -667,7 +679,7 @@ func (c *cmdClusterGroupShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterGroups(toComplete)
+			return c.global.cmpTopLevelResource("cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -729,11 +741,16 @@ func (c *cmdClusterGroupAdd) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpClusterMembers(toComplete)
+			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+		}
+
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpClusterGroupNames(args[0])
+			return c.global.cmpTopLevelResourceInRemote(remote, "cluster_group", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -767,7 +784,7 @@ func (c *cmdClusterGroupAdd) run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if shared.ValueInSlice(args[1], member.Groups) {
+	if slices.Contains(member.Groups, args[1]) {
 		return fmt.Errorf(i18n.G("Cluster member %s is already in group %s"), resource.name, args[1])
 	}
 
