@@ -82,10 +82,51 @@ func (c *cmdConfig) command() *cobra.Command {
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
-	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
+	cmd.Run = func(cmd *cobra.Command, _ []string) { _ = cmd.Usage() }
 	return cmd
 }
 
+// configSetValidArgsFunc returns completions for server and instance configuration keys (and remotes if no remote is
+// specified). If the command is targeting an instance, only configuration keys valid for that instance type are offered.
+// This function can also be used for the `config get` command, since it is valid to get any key (even if unset). If the
+// command name is "set", configuration key completions are suffixed with an `=`.
+func (c *cmdConfig) configSetValidArgsFunc(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	var suffix string
+	if cmd.Name() == "set" {
+		suffix = "="
+	}
+
+	if len(args) == 0 {
+		remote, partial, err := c.global.conf.ParseRemote(toComplete)
+		if err != nil {
+			return handleCompletionError(err)
+		}
+
+		// Default remote: Return all config keys, remotes, and instances.
+		if partial == toComplete {
+			serverConfigKeys, _ := c.global.cmpServerAllKeys(remote, suffix, toComplete)
+			instancesAndRemotes, _ := c.global.cmpTopLevelResource("instance", toComplete)
+			return append(serverConfigKeys, instancesAndRemotes...), cobra.ShellCompDirectiveNoFileComp | cobra.ShellCompDirectiveNoSpace
+		}
+
+		// Non-default remote: Return instances and remotes
+		return c.global.cmpTopLevelResource("instance", toComplete)
+	}
+
+	remote, instanceName, err := c.global.conf.ParseRemote(args[0])
+	if err != nil {
+		return handleCompletionError(err)
+	}
+
+	if instanceName == "" {
+		return c.global.cmpServerAllKeys(remote, suffix, toComplete)
+	}
+
+	instanceType := c.global.getInstanceType(remote, instanceName)
+
+	return c.global.cmpInstanceKeysByType(instanceType, suffix, toComplete)
+}
+
 // Edit.
 type cmdConfigEdit struct {
 	global *cmdGlobal
@@ -106,9 +147,9 @@ func (c *cmdConfigEdit) command() *cobra.Command {
 	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpInstances(toComplete)
+			return c.global.cmpTopLevelResource("instance", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -399,19 +440,13 @@ func (c *cmdConfigGet) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) == 0 {
-			if strings.Contains(toComplete, ".") {
-				return c.global.cmpServerAllKeys(toComplete)
-			}
-
-			return c.global.cmpInstances(toComplete)
+		// Only one key is allowed.
+		if len(args) > 1 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		if len(args) == 1 {
-			return c.global.cmpInstanceKeys(args[0])
-		}
-
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		// The caller can get any key, even if unset. So call completions for 'config set'.
+		return c.config.configSetValidArgsFunc(cmd, args, toComplete)
 	}
 
 	return cmd
@@ -454,7 +489,7 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 			}
 
 			if c.flagIsProperty {
-				res, err := getFieldByJsonTag(inst, args[len(args)-1])
+				res, err := getFieldByJSONTag(inst, args[len(args)-1])
 				if err != nil {
 					return fmt.Errorf(i18n.G("The property %q does not exist on the instance snapshot %s/%s: %v"), args[len(args)-1], fields[0], fields[1], err)
 				}
@@ -478,7 +513,7 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 
 		if c.flagIsProperty {
 			w := resp.Writable()
-			res, err := getFieldByJsonTag(&w, args[len(args)-1])
+			res, err := getFieldByJSONTag(&w, args[len(args)-1])
 			if err != nil {
 				return fmt.Errorf(i18n.G("The property %q does not exist on the instance %q: %v"), args[len(args)-1], resource.name, err)
 			}
@@ -512,11 +547,12 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 		}
 
 		value := resp.Config[args[len(args)-1]]
-		if value == nil {
+		switch value {
+		case nil:
 			value = ""
-		} else if value == true { //nolint:revive
+		case true:
 			value = "true"
-		} else if value == false { //nolint:revive
+		case false:
 			value = "false"
 		}
 
@@ -555,22 +591,7 @@ lxc config set core.https_address=[::]:8443
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as an instance property"))
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) == 0 {
-			if strings.Contains(toComplete, ".") {
-				return c.global.cmpServerAllKeys(toComplete)
-			}
-
-			return c.global.cmpInstances(toComplete)
-		}
-
-		if len(args) == 1 {
-			return c.global.cmpInstanceKeys(args[0])
-		}
-
-		return nil, cobra.ShellCompDirectiveNoFileComp
-	}
-
+	cmd.ValidArgsFunction = c.config.configSetValidArgsFunc
 	return cmd
 }
 
@@ -651,7 +672,7 @@ func (c *cmdConfigSet) run(cmd *cobra.Command, args []string) error {
 			if c.flagIsProperty {
 				if cmd.Name() == "unset" {
 					for k := range keys {
-						err := unsetFieldByJsonTag(&writable, k)
+						err := unsetFieldByJSONTag(&writable, k)
 						if err != nil {
 							return fmt.Errorf(i18n.G("Error unsetting properties: %v"), err)
 						}
@@ -683,7 +704,7 @@ func (c *cmdConfigSet) run(cmd *cobra.Command, args []string) error {
 		if c.flagIsProperty {
 			if cmd.Name() == "unset" {
 				for k := range keys {
-					err := unsetFieldByJsonTag(&writable, k)
+					err := unsetFieldByJSONTag(&writable, k)
 					if err != nil {
 						return fmt.Errorf(i18n.G("Error unsetting properties: %v"), err)
 					}
@@ -776,12 +797,12 @@ func (c *cmdConfigShow) command() *cobra.Command {
 	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) != 0 {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpInstances(toComplete)
+		return c.global.cmpTopLevelResource("instance", toComplete)
 	}
 
 	return cmd
@@ -908,22 +929,40 @@ func (c *cmdConfigUnset) command() *cobra.Command {
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as an instance property"))
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 1 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
-			if strings.Contains(toComplete, ".") {
-				// Only complete config keys which are currently set.
-				return c.global.cmpServerSetKeys(toComplete)
+			remote, partial, err := c.global.conf.ParseRemote(toComplete)
+			if err != nil {
+				return handleCompletionError(err)
 			}
 
-			return c.global.cmpInstances(toComplete)
+			// Default remote: Return set config keys in the default remote, instance remotes, and instances in the default remote.
+			if partial == toComplete {
+				serverConfigKeys, _ := c.global.cmpServerSetKeys(remote, toComplete)
+				instancesAndRemotes, _ := c.global.cmpTopLevelResource("instance", toComplete)
+				return append(serverConfigKeys, instancesAndRemotes...), cobra.ShellCompDirectiveNoFileComp | cobra.ShellCompDirectiveNoSpace
+			}
+
+			// Non-default remote: Return instances and remotes
+			return c.global.cmpTopLevelResource("instance", toComplete)
 		}
 
-		if len(args) == 1 {
-			// Only complete config keys which are currently set.
-			return c.global.cmpInstanceSetKeys(args[0])
+		remote, instanceName, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		// If we have a remote but no instance, return set config keys in the specified remote.
+		if instanceName == "" {
+			return c.global.cmpServerSetKeys(remote, toComplete)
+		}
+
+		// Return set config keys for the instance.
+		return c.global.cmpInstanceSetKeys(remote, instanceName, toComplete)
 	}
 
 	return cmd
@@ -979,7 +1018,7 @@ func (c *cmdConfigUefi) command() *cobra.Command {
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
-	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
+	cmd.Run = func(cmd *cobra.Command, _ []string) { _ = cmd.Usage() }
 	return cmd
 }
 
