@@ -2,11 +2,10 @@ package config
 
 import (
 	"encoding/json"
-	"fmt"
 	"os"
 	"path/filepath"
 
-	"github.com/zitadel/oidc/v2/pkg/oidc"
+	"github.com/zitadel/oidc/v3/pkg/oidc"
 
 	"github.com/canonical/lxd/shared"
 )
@@ -64,23 +63,27 @@ func (c *Config) ConfigPath(paths ...string) string {
 // ServerCertPath returns the path for the remote's server certificate.
 func (c *Config) ServerCertPath(remote string) string {
 	if c.Remotes[remote].Global {
-		return c.GlobalConfigPath("servercerts", fmt.Sprintf("%s.crt", remote))
+		return c.GlobalConfigPath("servercerts", remote+".crt")
 	}
 
-	return c.ConfigPath("servercerts", fmt.Sprintf("%s.crt", remote))
+	return c.ConfigPath("servercerts", remote+".crt")
 }
 
 // OIDCTokenPath returns the path for the remote's OIDC tokens.
 func (c *Config) OIDCTokenPath(remote string) string {
-	return c.ConfigPath("oidctokens", fmt.Sprintf("%s.json", remote))
+	return c.ConfigPath("oidctokens", remote+".json")
 }
 
 // SaveOIDCTokens saves OIDC tokens to disk.
 func (c *Config) SaveOIDCTokens() {
+	if len(c.oidcTokens) == 0 {
+		return
+	}
+
 	tokenParentPath := c.ConfigPath("oidctokens")
 
 	if !shared.PathExists(tokenParentPath) {
-		_ = os.MkdirAll(tokenParentPath, 0755)
+		_ = os.MkdirAll(tokenParentPath, 0750)
 	}
 
 	for remote, tokens := range c.oidcTokens {
