@@ -3,6 +3,7 @@ package main
 import (
 	"errors"
 	"fmt"
+	"maps"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -67,11 +68,11 @@ The pull transfer mode is the default as it is compatible with all LXD versions.
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpInstances(toComplete)
+			return c.global.cmpTopLevelResource("instance", toComplete)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpRemotes(toComplete, false)
+			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -224,14 +225,13 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		// Allow setting additional config keys.
-		for key, value := range configMap {
-			entry.Config[key] = value
-		}
+		maps.Copy(entry.Config, configMap)
 
 		// Allow overriding the ephemeral status
-		if ephemeral == 1 {
+		switch ephemeral {
+		case 1:
 			entry.Ephemeral = true
-		} else if ephemeral == 0 {
+		case 0:
 			entry.Ephemeral = false
 		}
 
@@ -299,6 +299,15 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 			entry.Profiles = []string{}
 		}
 
+		// Traditionally, if instance with snapshots is transferred across projects,
+		// the snapshots keep their own profiles.
+		// This doesn't work if the snapshot profiles don't exist in the target project.
+		// If different profiles are specified for the instance,
+		// instruct the server to apply the profiles of the source instance to the snapshots as well.
+		if c.flagNoProfiles || c.flagProfile != nil {
+			args.OverrideSnapshotProfiles = true
+		}
+
 		// Check to see if any of the devices overrides are for devices that are not yet defined in the
 		// local devices and thus are expected to be coming from profiles.
 		needProfileExpansion := false
@@ -328,14 +337,13 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		// Allow setting additional config keys.
-		for key, value := range configMap {
-			entry.Config[key] = value
-		}
+		maps.Copy(entry.Config, configMap)
 
 		// Allow overriding the ephemeral status
-		if ephemeral == 1 {
+		switch ephemeral {
+		case 1:
 			entry.Ephemeral = true
-		} else if ephemeral == 0 {
+		case 0:
 			entry.Ephemeral = false
 		}
 
