@@ -46,7 +46,7 @@ lxc info [<remote>:] [--resources]
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpInstances(toComplete)
+			return c.global.cmpTopLevelResource("instance", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -168,7 +168,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 
 			fmt.Println(prefix + "  - " + fmt.Sprintf(i18n.G("%s (%s) (%d available)"), k, v.Name, v.Available))
 			if v.Description != "" {
-				for _, line := range strings.Split(v.Description, "\n") {
+				for line := range strings.SplitSeq(v.Description, "\n") {
 					fmt.Printf(prefix+"      %s\n", line)
 				}
 			}
@@ -710,17 +710,20 @@ func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, name string, showLog bool)
 
 	if showLog {
 		var log io.Reader
-		if inst.Type == "container" {
+		switch inst.Type {
+		case "container":
 			log, err = d.GetInstanceLogfile(name, "lxc.log")
 			if err != nil {
 				return err
 			}
-		} else if inst.Type == "virtual-machine" {
+
+		case "virtual-machine":
 			log, err = d.GetInstanceLogfile(name, "qemu.log")
 			if err != nil {
 				return err
 			}
-		} else {
+
+		default:
 			return fmt.Errorf(i18n.G("Unsupported instance type: %s"), inst.Type)
 		}
 
