@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"net"
 	"regexp"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -136,7 +137,7 @@ lxc list -c ns,user.comment:comment
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, false)
+			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -192,7 +193,7 @@ func (c *cmdList) shouldShow(filters []string, inst *api.Instance, state *api.In
 				if c.dotPrefixMatch(key, configKey) {
 					// Try to test filter value as a regexp.
 					regexpValue := value
-					if !(strings.Contains(value, "^") || strings.Contains(value, "$")) {
+					if !strings.Contains(value, "^") && !strings.Contains(value, "$") {
 						regexpValue = "^" + regexpValue + "$"
 					}
 
@@ -222,7 +223,7 @@ func (c *cmdList) shouldShow(filters []string, inst *api.Instance, state *api.In
 			}
 		} else {
 			regexpValue := filter
-			if !(strings.Contains(filter, "^") || strings.Contains(filter, "$")) {
+			if !strings.Contains(filter, "^") && !strings.Contains(filter, "$") {
 				regexpValue = "^" + regexpValue + "$"
 			}
 
@@ -247,7 +248,7 @@ func (c *cmdList) evaluateShorthandFilter(key string, value string, inst *api.In
 	if isShorthandFilter {
 		if strings.Contains(value, shorthandValueDelimiter) {
 			matched := false
-			for _, curValue := range strings.Split(value, shorthandValueDelimiter) {
+			for curValue := range strings.SplitSeq(value, shorthandValueDelimiter) {
 				if shorthandFilterFunction(inst, state, curValue) {
 					matched = true
 				}
@@ -263,10 +264,7 @@ func (c *cmdList) evaluateShorthandFilter(key string, value string, inst *api.In
 }
 
 func (c *cmdList) listInstances(d lxd.InstanceServer, instances []api.Instance, filters []string, columns []column) error {
-	threads := 10
-	if len(instances) < threads {
-		threads = len(instances)
-	}
+	threads := min(len(instances), 10)
 
 	// Shortcut when needing state and snapshot info.
 	hasSnapshots := false
@@ -287,7 +285,7 @@ func (c *cmdList) listInstances(d lxd.InstanceServer, instances []api.Instance,
 		cInfoQueue := make(chan string, threads)
 		cInfoWg := sync.WaitGroup{}
 
-		for i := 0; i < threads; i++ {
+		for range threads {
 			cInfoWg.Add(1)
 			go func() {
 				for {
@@ -330,7 +328,7 @@ func (c *cmdList) listInstances(d lxd.InstanceServer, instances []api.Instance,
 	cSnapshotsQueue := make(chan string, threads)
 	cSnapshotsWg := sync.WaitGroup{}
 
-	for i := 0; i < threads; i++ {
+	for range threads {
 		cStatesWg.Add(1)
 		go func() {
 			for {
@@ -610,7 +608,7 @@ func (c *cmdList) parseColumns(clustered bool) ([]column, bool, error) {
 				return nil, false, errors.New(i18n.G("Can't specify column L when not clustered"))
 			}
 		}
-		c.flagColumns = strings.Replace(c.flagColumns, "L", "", -1)
+		c.flagColumns = strings.ReplaceAll(c.flagColumns, "L", "")
 	}
 
 	columnList := strings.Split(c.flagColumns, ",")
@@ -642,7 +640,7 @@ func (c *cmdList) parseColumns(clustered bool) ([]column, bool, error) {
 			colType := configColumnType
 			if (cc[0] == configColumnType || cc[0] == deviceColumnType) && len(cc) > 1 {
 				colType = cc[0]
-				cc = append(cc[:0], cc[1:]...)
+				cc = slices.Delete(cc, 0, 1)
 			}
 
 			if len(cc) > 3 {
@@ -773,7 +771,7 @@ func (c *cmdList) ipv4ColumnData(cInfo api.InstanceFull) string {
 			}
 
 			for _, addr := range net.Addresses {
-				if shared.ValueInSlice(addr.Scope, []string{"link", "local"}) {
+				if slices.Contains([]string{"link", "local"}, addr.Scope) {
 					continue
 				}
 
@@ -799,7 +797,7 @@ func (c *cmdList) ipv6ColumnData(cInfo api.InstanceFull) string {
 			}
 
 			for _, addr := range net.Addresses {
-				if shared.ValueInSlice(addr.Scope, []string{"link", "local"}) {
+				if slices.Contains([]string{"link", "local"}, addr.Scope) {
 					continue
 				}
 
@@ -880,7 +878,7 @@ func (c *cmdList) typeColumnData(cInfo api.InstanceFull) string {
 
 func (c *cmdList) numberSnapshotsColumnData(cInfo api.InstanceFull) string {
 	if cInfo.Snapshots != nil {
-		return fmt.Sprint(len(cInfo.Snapshots))
+		return strconv.Itoa(len(cInfo.Snapshots))
 	}
 
 	return "0"
@@ -888,7 +886,7 @@ func (c *cmdList) numberSnapshotsColumnData(cInfo api.InstanceFull) string {
 
 func (c *cmdList) pidColumnData(cInfo api.InstanceFull) string {
 	if cInfo.IsActive() && cInfo.State != nil {
-		return fmt.Sprint(cInfo.State.Pid)
+		return strconv.FormatInt(cInfo.State.Pid, 10)
 	}
 
 	return ""
@@ -934,7 +932,7 @@ func (c *cmdList) lastUsedColumnData(cInfo api.InstanceFull) string {
 
 func (c *cmdList) numberOfProcessesColumnData(cInfo api.InstanceFull) string {
 	if cInfo.IsActive() && cInfo.State != nil {
-		return fmt.Sprint(cInfo.State.Processes)
+		return strconv.FormatInt(cInfo.State.Processes, 10)
 	}
 
 	return ""
