@@ -3,6 +3,7 @@ package main
 import (
 	"errors"
 	"fmt"
+	"maps"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -70,12 +71,12 @@ lxc move <instance>/<old snapshot name> <instance>/<new snapshot name>
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			instances, directives := c.global.cmpInstances(toComplete)
+			instances, directives := c.global.cmpTopLevelResource("instance", toComplete)
 			return instances, directives | cobra.ShellCompDirectiveNoSpace
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpRemotes(toComplete, false)
+			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -233,19 +234,21 @@ func (c *cmdMove) run(cmd *cobra.Command, args []string) error {
 				profiles = &[]string{}
 			}
 
-			return moveInstance(conf, sourceResource, destResource, c.flagStorage, c.flagTargetProject, c.flagConfig, c.flagDevice, profiles, c.flagInstanceOnly, stateful)
+			return c.moveInstance(conf, sourceResource, destResource, profiles, stateful)
 		}
 
 		if source.HasExtension("instance_pool_move") && source.HasExtension("instance_project_move") {
 			if len(c.flagConfig) != 0 || len(c.flagDevice) != 0 || len(c.flagProfile) != 0 || c.flagNoProfiles {
-				return fmt.Errorf("The move command does not support flags --config, --device, --profile, and --no-profiles. Please use copy instead")
+				return errors.New("The move command does not support flags --config, --device, --profile, and --no-profiles. Please use copy instead")
 			}
 
 			if c.flagMode != moveDefaultMode {
 				return errors.New(i18n.G("The --mode flag can't be used with --storage or --target-project"))
 			}
 
-			return moveInstance(conf, sourceResource, destResource, c.flagStorage, c.flagTargetProject, []string{}, []string{}, nil, c.flagInstanceOnly, stateful)
+			c.flagConfig = []string{}
+			c.flagDevice = []string{}
+			return c.moveInstance(conf, sourceResource, destResource, nil, stateful)
 		}
 	}
 
@@ -356,7 +359,7 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 }
 
 // Move an instance between pools and projects using special POST /instances/<name> API.
-func moveInstance(conf *config.Config, sourceResource string, destResource string, storage string, targetProject string, config []string, devices []string, profiles *[]string, instanceOnly bool, stateful bool) error {
+func (c *cmdMove) moveInstance(conf *config.Config, sourceResource string, destResource string, profiles *[]string, stateful bool) error {
 	// Parse the source.
 	sourceRemote, sourceName, err := conf.ParseRemote(sourceResource)
 	if err != nil {
@@ -392,7 +395,7 @@ func moveInstance(conf *config.Config, sourceResource string, destResource strin
 	}
 
 	// Overwrite the config values.
-	for _, entry := range config {
+	for _, entry := range c.flagConfig {
 		key, value, found := strings.Cut(entry, "=")
 		if !found {
 			return fmt.Errorf(i18n.G("Bad key=value pair: %q"), entry)
@@ -402,7 +405,7 @@ func moveInstance(conf *config.Config, sourceResource string, destResource strin
 	}
 
 	// Parse device map and overwrite device settings.
-	deviceMap, err := parseDeviceOverrides(devices)
+	deviceMap, err := parseDeviceOverrides(c.flagDevice)
 	if err != nil {
 		return err
 	}
@@ -413,18 +416,16 @@ func moveInstance(conf *config.Config, sourceResource string, destResource strin
 			continue
 		}
 
-		for key, value := range m {
-			inst.Devices[k][key] = value
-		}
+		maps.Copy(inst.Devices[k], m)
 	}
 
 	// Pass the new pool to the migration API.
 	req := api.InstancePost{
 		Name:         destName,
 		Migration:    true,
-		InstanceOnly: instanceOnly,
-		Pool:         storage,
-		Project:      targetProject,
+		InstanceOnly: c.flagInstanceOnly,
+		Pool:         c.flagStorage,
+		Project:      c.flagTargetProject,
 		Live:         stateful,
 		Config:       inst.Config,
 		Devices:      inst.Devices,
@@ -435,6 +436,15 @@ func moveInstance(conf *config.Config, sourceResource string, destResource strin
 		req.Profiles = *profiles
 	}
 
+	// Traditionally, if instance with snapshots is transferred across projects,
+	// the snapshots keep their own profiles.
+	// This doesn't work if the snapshot profiles don't exist in the target project.
+	// If different profiles are specified for the instance,
+	// instruct the server to apply the profiles of the source instance to the snapshots as well.
+	if c.flagNoProfiles || profiles != nil {
+		req.OverrideSnapshotProfiles = true
+	}
+
 	op, err := source.MigrateInstance(sourceName, req)
 	if err != nil {
 		return fmt.Errorf(i18n.G("Migration API failure: %w"), err)
