@@ -4,8 +4,11 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"os"
+	"slices"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -140,11 +143,16 @@ func (c *cmdNetworkAttach) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
+		}
+
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpInstances(args[0])
+			return c.global.cmpTopLevelResourceInRemote(remote, "instance", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -236,15 +244,20 @@ func (c *cmdNetworkAttachProfile) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 1 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
 		}
 
-		if len(args) == 1 {
-			return c.global.cmpProfiles(args[0], false)
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		return c.global.cmpTopLevelResourceInRemote(remote, "profile", toComplete)
 	}
 
 	return cmd
@@ -332,7 +345,7 @@ lxc network create bar network=baz --type ovn
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpRemotes(toComplete, false)
+		return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 	}
 
 	return cmd
@@ -412,7 +425,7 @@ func (c *cmdNetworkDelete) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -467,7 +480,7 @@ func (c *cmdNetworkDetach) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -564,7 +577,7 @@ func (c *cmdNetworkDetachProfile) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -664,7 +677,7 @@ func (c *cmdNetworkEdit) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -799,7 +812,7 @@ func (c *cmdNetworkGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -844,7 +857,7 @@ func (c *cmdNetworkGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := resp.Writable()
-		res, err := getFieldByJsonTag(&w, args[1])
+		res, err := getFieldByJSONTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network %q: %v"), args[1], resource.name, err)
 		}
@@ -882,7 +895,7 @@ func (c *cmdNetworkInfo) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -994,8 +1007,9 @@ type cmdNetworkList struct {
 	global  *cmdGlobal
 	network *cmdNetwork
 
-	flagFormat string
-	flagTarget string
+	flagFormat      string
+	flagTarget      string
+	flagAllProjects bool
 }
 
 func (c *cmdNetworkList) command() *cobra.Command {
@@ -1009,13 +1023,14 @@ func (c *cmdNetworkList) command() *cobra.Command {
 	cmd.RunE = c.run
 	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 	cmd.Flags().StringVar(&c.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("Display networks from all projects"))
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) != 0 {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpRemotes(toComplete, false)
+		return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
 	}
 
 	return cmd
@@ -1053,14 +1068,22 @@ func (c *cmdNetworkList) run(cmd *cobra.Command, args []string) error {
 		client = client.UseTarget(c.flagTarget)
 	}
 
-	networks, err := client.GetNetworks()
-	if err != nil {
-		return err
+	var networks []api.Network
+	if c.flagAllProjects {
+		networks, err = client.GetNetworksAllProjects()
+		if err != nil {
+			return err
+		}
+	} else {
+		networks, err = client.GetNetworks()
+		if err != nil {
+			return err
+		}
 	}
 
 	data := [][]string{}
 	for _, network := range networks {
-		if shared.ValueInSlice(network.Type, []string{"loopback", "unknown"}) {
+		if slices.Contains([]string{"loopback", "unknown"}, network.Type) {
 			continue
 		}
 
@@ -1069,7 +1092,7 @@ func (c *cmdNetworkList) run(cmd *cobra.Command, args []string) error {
 			strManaged = i18n.G("YES")
 		}
 
-		strUsedBy := fmt.Sprint(len(network.UsedBy))
+		strUsedBy := strconv.Itoa(len(network.UsedBy))
 		details := []string{
 			network.Name,
 			network.Type,
@@ -1081,6 +1104,10 @@ func (c *cmdNetworkList) run(cmd *cobra.Command, args []string) error {
 			strings.ToUpper(network.Status),
 		}
 
+		if c.flagAllProjects {
+			details = append([]string{network.Project}, details...)
+		}
+
 		data = append(data, details)
 	}
 
@@ -1097,6 +1124,10 @@ func (c *cmdNetworkList) run(cmd *cobra.Command, args []string) error {
 		i18n.G("STATE"),
 	}
 
+	if c.flagAllProjects {
+		header = append([]string{i18n.G("PROJECT")}, header...)
+	}
+
 	return cli.RenderTable(c.flagFormat, header, data, networks)
 }
 
@@ -1123,7 +1154,7 @@ func (c *cmdNetworkListLeases) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -1201,7 +1232,7 @@ func (c *cmdNetworkRename) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -1266,7 +1297,7 @@ For backward compatibility, a single configuration key may still be set with:
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -1317,7 +1348,7 @@ func (c *cmdNetworkSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJsonTag(&writable, k)
+				err := unsetFieldByJSONTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -1329,9 +1360,7 @@ func (c *cmdNetworkSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		for k, v := range keys {
-			writable.Config[k] = v
-		}
+		maps.Copy(writable.Config, keys)
 	}
 
 	return client.UpdateNetwork(resource.name, writable, etag)
@@ -1358,7 +1387,7 @@ func (c *cmdNetworkShow) command() *cobra.Command {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpNetworks(toComplete)
+		return c.global.cmpTopLevelResource("network", toComplete)
 	}
 
 	return cmd
@@ -1428,7 +1457,7 @@ func (c *cmdNetworkUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpNetworks(toComplete)
+			return c.global.cmpTopLevelResource("network", toComplete)
 		}
 
 		if len(args) == 1 {
