@@ -8,11 +8,12 @@ import (
 	"io"
 	"net/http"
 	"net/url"
+	"slices"
 	"strings"
 
 	"github.com/spf13/cobra"
 
-	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/shared/api"
 	cli "github.com/canonical/lxd/shared/cmd"
 	"github.com/canonical/lxd/shared/i18n"
@@ -72,7 +73,7 @@ func (c *cmdQuery) run(cmd *cobra.Command, args []string) error {
 		return errors.New(i18n.G("--project cannot be used with the query command"))
 	}
 
-	if !shared.ValueInSlice(c.flagAction, []string{"GET", "PUT", "POST", "PATCH", "DELETE"}) {
+	if !slices.Contains([]string{"GET", "PUT", "POST", "PATCH", "DELETE"}, c.flagAction) {
 		return fmt.Errorf(i18n.G("Action %q isn't supported by this tool"), c.flagAction)
 	}
 
@@ -87,17 +88,21 @@ func (c *cmdQuery) run(cmd *cobra.Command, args []string) error {
 		return errors.New(i18n.G("Query path must start with /"))
 	}
 
-	// Attempt to connect
-	d, err := conf.GetInstanceServer(remote)
+	// Setup client
+	d, err := conf.GetInstanceServerWithConnectionArgs(remote, &lxd.ConnectionArgs{
+		SkipGetServer: true,
+	})
 	if err != nil {
 		return err
 	}
 
 	// Guess the encoding of the input
 	var data any
-	err = json.Unmarshal([]byte(c.flagData), &data)
-	if err != nil {
-		data = c.flagData
+	if c.flagData != "" {
+		err = json.Unmarshal([]byte(c.flagData), &data)
+		if err != nil {
+			data = c.flagData
+		}
 	}
 
 	// Perform the query
@@ -133,7 +138,9 @@ func (c *cmdQuery) run(cmd *cobra.Command, args []string) error {
 		}
 
 		// Set the encoding accordingly
-		req.Header.Set("Content-Type", "plain/text")
+		if c.flagData != "" {
+			req.Header.Set("Content-Type", "plain/text")
+		}
 
 		resp, err := d.DoHTTP(req)
 		if err != nil {
@@ -160,7 +167,7 @@ func (c *cmdQuery) run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		resp, _, err = d.RawQuery("GET", uri.Path+"/wait?"+uri.RawQuery, "", "")
+		resp, _, err = d.RawQuery(http.MethodGet, uri.Path+"/wait?"+uri.RawQuery, "", "")
 		if err != nil {
 			return err
 		}
