@@ -1,6 +1,7 @@
 package main
 
 import (
+	"context"
 	"crypto/x509"
 	"encoding/pem"
 	"errors"
@@ -9,7 +10,9 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"slices"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -128,7 +131,7 @@ func (c *cmdRemoteAdd) findProject(d lxd.InstanceServer, project string) (string
 			}
 
 			// Deal with multiple projects.
-			if shared.ValueInSlice("default", names) {
+			if slices.Contains(names, "default") {
 				// If we have access to the default project, use it.
 				return "", nil
 			}
@@ -221,7 +224,7 @@ func (c *cmdRemoteAdd) addRemoteFromToken(addr string, server string, token stri
 
 	_, err = conf.GetInstanceServer(server)
 	if err != nil {
-		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
+		certificate, err = shared.GetRemoteCertificate(context.Background(), addr, c.global.conf.UserAgent)
 		if err != nil {
 			return api.StatusErrorf(http.StatusServiceUnavailable, "%s: %w", i18n.G("Unavailable remote server"), err)
 		}
@@ -431,7 +434,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 		rHost = host
 		rPort = port
 	} else {
-		rPort = fmt.Sprint(shared.HTTPSDefaultPort)
+		rPort = strconv.Itoa(shared.HTTPSDefaultPort)
 	}
 
 	if rScheme == "unix" {
@@ -507,7 +510,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 	var certificate *x509.Certificate
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
+		certificate, err = shared.GetRemoteCertificate(context.Background(), addr, c.global.conf.UserAgent)
 		if err != nil {
 			return err
 		}
@@ -528,9 +531,10 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 					return err
 				}
 
+				// Check length before accessing line to prevent runtime panic.
 				// Continue with adding the remote if digest matches, or the user
 				// confirmed a fingerprint.
-				if string(line) == digest || strings.ToLower(string(line[0])) == i18n.G("y") {
+				if string(line) == digest || (len(line) > 0 && strings.ToLower(string(line[0])) == i18n.G("y")) {
 					break
 				}
 
@@ -542,7 +546,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 				}
 
 				// Error out if the user didn't confirm the fingerprint.
-				if len(line) < 1 || strings.ToLower(string(line[0])) == i18n.G("n") {
+				if len(line) == 0 || strings.ToLower(string(line[0])) == i18n.G("n") {
 					return errors.New(i18n.G("Server certificate NACKed by user"))
 				}
 
@@ -599,13 +603,13 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 
 	// If not specified, the preferred order of authentication is 1) OIDC 2) TLS.
 	if c.flagAuthType == "" {
-		if !srv.Public && shared.ValueInSlice(api.AuthenticationMethodOIDC, srv.AuthMethods) {
+		if !srv.Public && slices.Contains(srv.AuthMethods, api.AuthenticationMethodOIDC) {
 			c.flagAuthType = api.AuthenticationMethodOIDC
 		} else {
 			c.flagAuthType = api.AuthenticationMethodTLS
 		}
 
-		if shared.ValueInSlice(c.flagAuthType, []string{api.AuthenticationMethodOIDC}) {
+		if slices.Contains([]string{api.AuthenticationMethodOIDC}, c.flagAuthType) {
 			// Update the remote configuration
 			remote := conf.Remotes[server]
 			remote.AuthType = c.flagAuthType
@@ -631,7 +635,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	if !srv.Public && !shared.ValueInSlice(c.flagAuthType, srv.AuthMethods) {
+	if !srv.Public && !slices.Contains(srv.AuthMethods, c.flagAuthType) {
 		return fmt.Errorf(i18n.G("Authentication type '%s' not supported by server"), c.flagAuthType)
 	}
 
@@ -877,7 +881,7 @@ func (c *cmdRemoteRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemoteNames(true)
+			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -958,7 +962,7 @@ func (c *cmdRemoteRemove) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemoteNames(false)
+			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes, filterGlobalRemotes, filterDefaultRemote(*c.global.conf))
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1022,7 +1026,8 @@ func (c *cmdRemoteSwitch) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemoteNames(false)
+			// It's valid to switch to a public remote, but filter them from completions to prevent leading new users down a bad path.
+			return c.global.cmpRemotes(toComplete, "", false, filterDefaultRemote(*c.global.conf), filterPublicRemotes)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1070,7 +1075,7 @@ func (c *cmdRemoteSetURL) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemoteNames(true)
+			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
