@@ -4,6 +4,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net/url"
 	"os"
 	"path"
@@ -244,19 +245,24 @@ func (c *cmdStorageVolumeAttach) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 2 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
 			return c.global.cmpStoragePoolVolumes(args[0], "custom")
 		}
 
-		if len(args) == 2 {
-			return c.global.cmpInstanceNamesFromRemote(args[0])
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		return c.global.cmpTopLevelResourceInRemote(remote, "instance", toComplete)
 	}
 
 	return cmd
@@ -314,19 +320,24 @@ func (c *cmdStorageVolumeAttachProfile) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 2 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
 			return c.global.cmpStoragePoolVolumes(args[0], "custom")
 		}
 
-		if len(args) == 2 {
-			return c.global.cmpProfileNamesFromRemote(args[0])
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		return c.global.cmpTopLevelResourceInRemote(remote, "profile", toComplete)
 	}
 
 	return cmd
@@ -394,21 +405,25 @@ func (c *cmdStorageVolumeCopy) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 1 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
 			return c.global.cmpStoragePoolWithVolume(toComplete)
 		}
 
-		if len(args) == 1 {
-			completions, directive := c.global.cmpStoragePools(toComplete, true)
-			for i, completion := range completions {
-				if !strings.Contains(completion, ":") {
-					completions[i] = completion + "/"
-				}
-			}
-			return completions, directive
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		completions, directive := c.global.cmpTopLevelResourceInRemote(remote, "storage_pool", toComplete)
+		for i, completion := range completions {
+			completions[i] = completion + "/"
+		}
+
+		return completions, directive | cobra.ShellCompDirectiveNoSpace
 	}
 
 	return cmd
@@ -459,7 +474,7 @@ func (c *cmdStorageVolumeCopy) run(cmd *cobra.Command, args []string) error {
 	}
 
 	if srcIsSnapshot && c.flagVolumeOnly {
-		return fmt.Errorf("Cannot set --volume-only when copying a snapshot")
+		return errors.New("Cannot set --volume-only when copying a snapshot")
 	}
 
 	// If the volume is in local storage, set the target to its location (or provide a helpful error
@@ -616,7 +631,7 @@ lxc storage volume create p1 v1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -720,7 +735,7 @@ func (c *cmdStorageVolumeDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -806,7 +821,7 @@ func (c *cmdStorageVolumeDetach) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -911,7 +926,7 @@ func (c *cmdStorageVolumeDetachProfile) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1022,7 +1037,7 @@ lxc storage volume edit [<remote>:]<pool> [<type>/]<volume> < volume.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1249,7 +1264,7 @@ lxc storage volume get default virtual-machine/data snapshots.expiry
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1310,7 +1325,7 @@ func (c *cmdStorageVolumeGet) run(cmd *cobra.Command, args []string) error {
 		}
 
 		if c.flagIsProperty {
-			res, err := getFieldByJsonTag(resp, args[2])
+			res, err := getFieldByJSONTag(resp, args[2])
 			if err != nil {
 				return fmt.Errorf(i18n.G("The property %q does not exist on the storage pool volume snapshot %s/%s: %v"), args[2], fields[0], fields[1], err)
 			}
@@ -1333,7 +1348,7 @@ func (c *cmdStorageVolumeGet) run(cmd *cobra.Command, args []string) error {
 	}
 
 	if c.flagIsProperty {
-		res, err := getFieldByJsonTag(resp, args[2])
+		res, err := getFieldByJSONTag(resp, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the storage pool volume %q: %v"), args[2], resource.name, err)
 		}
@@ -1377,7 +1392,7 @@ lxc storage volume info default virtual-machine/data
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1506,8 +1521,7 @@ func (c *cmdStorageVolumeInfo) run(cmd *cobra.Command, args []string) error {
 			var row []string
 
 			fields := strings.Split(snap.Name, shared.SnapshotDelimiter)
-			row = append(row, fields[len(fields)-1])
-			row = append(row, snap.Description)
+			row = append(row, fields[len(fields)-1], snap.Description)
 
 			if snap.ExpiresAt != nil {
 				row = append(row, snap.ExpiresAt.Local().Format(layout))
@@ -1629,7 +1643,7 @@ Column shorthand chars:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1761,19 +1775,19 @@ func (c *cmdStorageVolumeList) parseColumns(clustered bool, allVolumes bool) ([]
 				return nil, errors.New(i18n.G("Can't specify column L when not clustered"))
 			}
 		}
-		c.flagColumns = strings.Replace(c.flagColumns, "L", "", -1)
+		c.flagColumns = strings.ReplaceAll(c.flagColumns, "L", "")
 	}
 
 	if allVolumes {
 		columnsShorthandMap['p'] = volumeColumn{Name: i18n.G("POOL"), Data: c.poolColumnData}
 	} else {
-		c.flagColumns = strings.Replace(c.flagColumns, "p", "", -1)
+		c.flagColumns = strings.ReplaceAll(c.flagColumns, "p", "")
 	}
 
 	if c.flagAllProjects {
 		columnsShorthandMap['e'] = volumeColumn{Name: i18n.G("PROJECT"), Data: c.projectColumnData}
 	} else {
-		c.flagColumns = strings.Replace(c.flagColumns, "e", "", -1)
+		c.flagColumns = strings.ReplaceAll(c.flagColumns, "e", "")
 	}
 
 	columnList := strings.Split(c.flagColumns, ",")
@@ -1869,21 +1883,25 @@ func (c *cmdStorageVolumeMove) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) > 1 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
 		if len(args) == 0 {
 			return c.global.cmpStoragePoolWithVolume(toComplete)
 		}
 
-		if len(args) == 1 {
-			completions, directive := c.global.cmpStoragePools(toComplete, true)
-			for i, completion := range completions {
-				if !strings.Contains(completion, ":") {
-					completions[i] = completion + "/"
-				}
-			}
-			return completions, directive
+		remote, _, err := c.global.conf.ParseRemote(args[0])
+		if err != nil {
+			return handleCompletionError(err)
 		}
 
-		return nil, cobra.ShellCompDirectiveNoFileComp
+		completions, directive := c.global.cmpTopLevelResourceInRemote(remote, "storage_pool", toComplete)
+		for i, completion := range completions {
+			completions[i] = completion + "/"
+		}
+
+		return completions, directive | cobra.ShellCompDirectiveNoSpace
 	}
 
 	return cmd
@@ -1966,7 +1984,7 @@ func (c *cmdStorageVolumeRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2106,7 +2124,7 @@ lxc storage volume set default virtual-machine/data snapshots.expiry=7d
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2168,7 +2186,7 @@ func (c *cmdStorageVolumeSet) run(cmd *cobra.Command, args []string) error {
 			writable := snapVol.Writable()
 			if cmd.Name() == "unset" {
 				for k := range keys {
-					err := unsetFieldByJsonTag(&writable, k)
+					err := unsetFieldByJSONTag(&writable, k)
 					if err != nil {
 						return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 					}
@@ -2206,7 +2224,7 @@ func (c *cmdStorageVolumeSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJsonTag(&writable, k)
+				err := unsetFieldByJSONTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -2219,9 +2237,7 @@ func (c *cmdStorageVolumeSet) run(cmd *cobra.Command, args []string) error {
 		}
 	} else {
 		// Update the volume config keys.
-		for k, v := range keys {
-			writable.Config[k] = v
-		}
+		maps.Copy(writable.Config, keys)
 	}
 
 	err = client.UpdateStoragePoolVolume(resource.name, vol.Type, vol.Name, writable, etag)
@@ -2265,7 +2281,7 @@ lxc storage volume show default virtual-machine/data/snap0
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2382,7 +2398,7 @@ lxc storage volume unset default virtual-machine/data snapshots.expiry
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2442,7 +2458,7 @@ lxc storage volume snapshot default v1 snap0 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2570,7 +2586,7 @@ func (c *cmdStorageVolumeRestore) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2639,6 +2655,7 @@ type cmdStorageVolumeExport struct {
 	flagVolumeOnly           bool
 	flagOptimizedStorage     bool
 	flagCompressionAlgorithm string
+	flagExportVersion        string
 }
 
 func (c *cmdStorageVolumeExport) command() *cobra.Command {
@@ -2652,12 +2669,14 @@ func (c *cmdStorageVolumeExport) command() *cobra.Command {
 	cmd.Flags().BoolVar(&c.flagOptimizedStorage, "optimized-storage", false,
 		i18n.G("Use storage driver optimized format (can only be restored on a similar pool)"))
 	cmd.Flags().StringVar(&c.flagCompressionAlgorithm, "compression", "", i18n.G("Define a compression algorithm: for backup or none")+"``")
+	cmd.Flags().StringVar(&c.flagExportVersion, "export-version", "",
+		i18n.G("Use a different metadata format version than the latest one supported by the server (to support imports on older LXD versions)")+"``")
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		if len(args) == 1 {
@@ -2699,7 +2718,7 @@ func (c *cmdStorageVolumeExport) run(cmd *cobra.Command, args []string) error {
 
 	volName, volType := parseVolume("custom", args[1])
 	if volType != "custom" {
-		return errors.New(i18n.G("Only \"custom\" volumes can be exported"))
+		return fmt.Errorf("Failed to create storage volume backup for volume %q: %w", volName, errors.New(i18n.G("Only \"custom\" volumes can be exported")))
 	}
 
 	req := api.StoragePoolVolumeBackupsPost{
@@ -2710,9 +2729,14 @@ func (c *cmdStorageVolumeExport) run(cmd *cobra.Command, args []string) error {
 		CompressionAlgorithm: c.flagCompressionAlgorithm,
 	}
 
+	req.Version, err = getExportVersion(d, c.flagExportVersion)
+	if err != nil {
+		return err
+	}
+
 	op, err := d.CreateStoragePoolVolumeBackup(name, volName, req)
 	if err != nil {
-		return fmt.Errorf("Failed to create storage volume backup: %w", err)
+		return fmt.Errorf("Failed to create storage volume backup for volume %q: %w", volName, err)
 	}
 
 	// Watch the background operation
@@ -2775,26 +2799,27 @@ func (c *cmdStorageVolumeExport) run(cmd *cobra.Command, args []string) error {
 
 	defer func() { _ = target.Close() }()
 
-	// Prepare the download request
-	progress = cli.ProgressRenderer{
+	// Prepare the download request.
+	// Assign the renderer to a new variable to not interfer with the old one.
+	exportProgress := cli.ProgressRenderer{
 		Format: i18n.G("Exporting the backup: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
 
 	backupFileRequest := lxd.BackupFileRequest{
 		BackupFile:      io.WriteSeeker(target),
-		ProgressHandler: progress.UpdateProgress,
+		ProgressHandler: exportProgress.UpdateProgress,
 	}
 
 	// Export tarball
 	_, err = d.GetStoragePoolVolumeBackupFile(name, volName, backupName, &backupFileRequest)
 	if err != nil {
 		_ = os.Remove(targetName)
-		progress.Done("")
+		exportProgress.Done("")
 		return fmt.Errorf("Failed to fetch storage volume backup file: %w", err)
 	}
 
-	progress.Done(i18n.G("Backup exported successfully!"))
+	exportProgress.Done(i18n.G("Backup exported successfully!"))
 	return nil
 }
 
@@ -2822,7 +2847,7 @@ func (c *cmdStorageVolumeImport) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpStoragePools(toComplete, false)
+			return c.global.cmpTopLevelResource("storage_pool", toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveDefault
@@ -2882,13 +2907,13 @@ func (c *cmdStorageVolumeImport) run(cmd *cobra.Command, args []string) error {
 		}
 	} else {
 		// Validate type flag
-		if !shared.ValueInSlice(c.flagType, []string{"backup", "iso"}) {
-			return fmt.Errorf("Import type needs to be \"backup\" or \"iso\"")
+		if !slices.Contains([]string{"backup", "iso"}, c.flagType) {
+			return errors.New("Import type needs to be \"backup\" or \"iso\"")
 		}
 	}
 
 	if c.flagType == "iso" && volName == "" {
-		return fmt.Errorf("Importing ISO images requires a volume name to be set")
+		return errors.New("Importing ISO images requires a volume name to be set")
 	}
 
 	progress := cli.ProgressRenderer{
