@@ -1,7 +1,6 @@
 package main
 
 import (
-	"fmt"
 	"os"
 
 	"github.com/spf13/cobra"
@@ -39,12 +38,13 @@ func main() {
 	app.SetVersionTemplate("{{.Version}}\n")
 	app.Version = version.Version
 	if version.IsLTSVersion {
-		app.Version = fmt.Sprintf("%s LTS", version.Version)
+		app.Version = version.Version + " LTS"
 	}
 
 	// Run the main command and handle errors
 	err := app.Execute()
 	if err != nil {
+		// Ensure we exit with a non-zero exit code.
 		os.Exit(1)
 	}
 }
