@@ -6,7 +6,7 @@ import (
 
 	"github.com/pkg/sftp"
 
-	"github.com/lxc/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/response"
 )
 
 var sftpCmd = APIEndpoint{
@@ -17,21 +17,21 @@ var sftpCmd = APIEndpoint{
 }
 
 func sftpHandler(d *Daemon, r *http.Request) response.Response {
-	return &sftpServe{d, r}
+	return &sftpServe{d}
 }
 
 type sftpServe struct {
 	d *Daemon
-	r *http.Request
 }
 
 func (r *sftpServe) String() string {
 	return "sftp handler"
 }
 
-func (r *sftpServe) Render(w http.ResponseWriter) error {
+// Render hijacks the connection and starts a sftp server.
+func (r *sftpServe) Render(w http.ResponseWriter, request *http.Request) error {
 	// Upgrade to sftp.
-	if r.r.Header.Get("Upgrade") != "sftp" {
+	if request.Header.Get("Upgrade") != "sftp" {
 		http.Error(w, "Missing or invalid upgrade header", http.StatusBadRequest)
 		return nil
 	}
