@@ -3,16 +3,20 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
+	"maps"
 	"math"
 	"net"
 	"net/http"
 	"os"
 	"slices"
+	"strconv"
 
 	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/auth/oidc"
+	backupConfig "github.com/canonical/lxd/lxd/backup/config"
 	"github.com/canonical/lxd/lxd/cluster"
 	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
 	"github.com/canonical/lxd/lxd/config"
@@ -230,7 +234,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	// Get the authentication methods.
 	authMethods := []string{api.AuthenticationMethodTLS}
 
-	oidcIssuer, oidcClientID, _, _, _ := s.GlobalConfig.OIDCServer()
+	oidcIssuer, oidcClientID, _, _, _, _ := s.GlobalConfig.OIDCServer()
 	if oidcIssuer != "" && oidcClientID != "" {
 		authMethods = append(authMethods, api.AuthenticationMethodOIDC)
 	}
@@ -256,7 +260,8 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(s, r)
+	target := request.QueryParam(r, "target")
+	resp := forwardedResponseToNode(r.Context(), s, target)
 	if resp != nil {
 		return resp
 	}
@@ -309,34 +314,36 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	}
 
 	env := api.ServerEnvironment{
-		Addresses:              addresses,
-		Architectures:          architectures,
-		Certificate:            certificate,
-		CertificateFingerprint: certificateFingerprint,
-		Kernel:                 s.OS.Uname.Sysname,
-		KernelArchitecture:     s.OS.Uname.Machine,
-		KernelVersion:          s.OS.Uname.Release,
-		OSName:                 s.OS.ReleaseInfo["NAME"],
-		OSVersion:              s.OS.ReleaseInfo["VERSION_ID"],
-		Project:                projectName,
-		Server:                 "lxd",
-		ServerPid:              os.Getpid(),
-		ServerVersion:          version.Version,
-		ServerLTS:              version.IsLTSVersion,
-		ServerClustered:        s.ServerClustered,
-		ServerEventMode:        string(cluster.ServerEventMode()),
-		ServerName:             serverName,
-		Firewall:               s.Firewall.String(),
+		Addresses:                  addresses,
+		Architectures:              architectures,
+		BackupMetadataVersionRange: []uint32{api.BackupMetadataVersion1, backupConfig.MaxMetadataVersion},
+		Certificate:                certificate,
+		CertificateFingerprint:     certificateFingerprint,
+		Kernel:                     s.OS.Uname.Sysname,
+		KernelArchitecture:         s.OS.Uname.Machine,
+		KernelVersion:              s.OS.Uname.Release,
+		OSName:                     s.OS.ReleaseInfo["NAME"],
+		OSVersion:                  s.OS.ReleaseInfo["VERSION_ID"],
+		Project:                    projectName,
+		Server:                     "lxd",
+		ServerPid:                  os.Getpid(),
+		ServerVersion:              version.Version,
+		ServerLTS:                  version.IsLTSVersion,
+		ServerClustered:            s.ServerClustered,
+		ServerEventMode:            string(cluster.ServerEventMode()),
+		ServerName:                 serverName,
+		Firewall:                   s.Firewall.String(),
 	}
 
 	env.KernelFeatures = map[string]string{
-		"netnsid_getifaddrs":        fmt.Sprint(s.OS.NetnsGetifaddrs),
-		"uevent_injection":          fmt.Sprint(s.OS.UeventInjection),
-		"unpriv_binfmt":             fmt.Sprint(s.OS.UnprivBinfmt),
-		"unpriv_fscaps":             fmt.Sprint(s.OS.VFS3Fscaps),
-		"seccomp_listener":          fmt.Sprint(s.OS.SeccompListener),
-		"seccomp_listener_continue": fmt.Sprint(s.OS.SeccompListenerContinue),
-		"idmapped_mounts":           fmt.Sprint(s.OS.IdmappedMounts),
+		"bpf_token":                 strconv.FormatBool(s.OS.BPFToken),
+		"netnsid_getifaddrs":        strconv.FormatBool(s.OS.NetnsGetifaddrs),
+		"uevent_injection":          strconv.FormatBool(s.OS.UeventInjection),
+		"unpriv_binfmt":             strconv.FormatBool(s.OS.UnprivBinfmt),
+		"unpriv_fscaps":             strconv.FormatBool(s.OS.VFS3Fscaps),
+		"seccomp_listener":          strconv.FormatBool(s.OS.SeccompListener),
+		"seccomp_listener_continue": strconv.FormatBool(s.OS.SeccompListenerContinue),
+		"idmapped_mounts":           strconv.FormatBool(s.OS.IdmappedMounts),
 	}
 
 	drivers := instanceDrivers.DriverStatuses()
@@ -372,7 +379,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 		// Add supported instance types.
 		instType := driver.Info.Type.String()
-		if !shared.ValueInSlice(instType, env.InstanceTypes) {
+		if !slices.Contains(env.InstanceTypes, instType) {
 			env.InstanceTypes = append(env.InstanceTypes, instType)
 		}
 	}
@@ -380,7 +387,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	if s.OS.LXCFeatures != nil {
 		env.LXCFeatures = map[string]string{}
 		for k, v := range s.OS.LXCFeatures {
-			env.LXCFeatures[k] = fmt.Sprint(v)
+			env.LXCFeatures[k] = strconv.FormatBool(v)
 		}
 	}
 
@@ -404,7 +411,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	fullSrv := &api.Server{ServerUntrusted: srv}
 	fullSrv.Environment = env
-	requestor := request.CreateRequestor(r)
+	requestor := request.CreateRequestor(r.Context())
 	fullSrv.AuthUserName = requestor.Username
 	fullSrv.AuthUserMethod = requestor.Protocol
 
@@ -467,7 +474,8 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(s, r)
+	target := request.QueryParam(r, "target")
+	resp := forwardedResponseToNode(r.Context(), s, target)
 	if resp != nil {
 		return resp
 	}
@@ -506,8 +514,13 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		d.globalConfig = config
 		d.globalConfigMu.Unlock()
 
+		// Copy the old config so that the update triggers have access to it.
+		// In this case it will not be used as we are not changing any node values.
+		oldNodeConfig := make(map[string]any)
+		maps.Copy(oldNodeConfig, s.LocalConfig.Dump())
+
 		// Run any update triggers.
-		err = doAPI10UpdateTriggers(d, nil, changed, s.LocalConfig, config)
+		err = doAPI10UpdateTriggers(d, nil, changed, oldNodeConfig, s.LocalConfig, config)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -566,7 +579,8 @@ func api10Patch(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(s, r)
+	target := request.QueryParam(r, "target")
+	resp := forwardedResponseToNode(r.Context(), s, target)
 	if resp != nil {
 		return resp
 	}
@@ -623,9 +637,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		}
 
 		// Keep old config around in case something goes wrong. In that case the config will be reverted.
-		for k, v := range newNodeConfig.Dump() {
-			oldNodeConfig[k] = v
-		}
+		maps.Copy(oldNodeConfig, newNodeConfig.Dump())
 
 		// We currently don't allow changing the cluster.https_address once it's set.
 		if s.ServerClustered {
@@ -643,7 +655,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 			}
 
 			if curConfig["cluster.https_address"] != newClusterHTTPSAddress {
-				return fmt.Errorf("Changing cluster.https_address is currently not supported")
+				return errors.New("Changing cluster.https_address is currently not supported")
 			}
 		}
 
@@ -728,7 +740,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 	var newClusterConfig *clusterConfig.Config
 	var oldClusterConfig map[string]any
 
-	err = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 		newClusterConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
@@ -739,9 +751,9 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		oldClusterConfig = newClusterConfig.Dump()
 
 		if patch {
-			clusterChanged, err = newClusterConfig.Patch(req.Config)
+			clusterChanged, err = newClusterConfig.Patch(tx, req.Config)
 		} else {
-			clusterChanged, err = newClusterConfig.Replace(req.Config)
+			clusterChanged, err = newClusterConfig.Replace(tx, req.Config)
 		}
 
 		return err
@@ -771,7 +783,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 				return fmt.Errorf("Failed to load cluster config: %w", err)
 			}
 
-			_, err = newClusterConfig.Replace(req.Config)
+			_, err = newClusterConfig.Replace(tx, req.Config)
 			if err != nil {
 				return fmt.Errorf("Failed updating cluster config: %w", err)
 			}
@@ -817,19 +829,19 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 	d.globalConfigMu.Unlock()
 
 	// Run any update triggers.
-	err = doAPI10UpdateTriggers(d, nodeChanged, clusterChanged, newNodeConfig, newClusterConfig)
+	err = doAPI10UpdateTriggers(d, nodeChanged, clusterChanged, oldNodeConfig, newNodeConfig, newClusterConfig)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	revert.Success()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r.Context()), nil))
 
 	return response.EmptySyncResponse
 }
 
-func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, nodeConfig *node.Config, clusterConfig *clusterConfig.Config) error {
+func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, oldNodeConfig map[string]any, newNodeConfig *node.Config, newClusterConfig *clusterConfig.Config) error {
 	s := d.State()
 
 	maasChanged := false
@@ -850,7 +862,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 		case "core.proxy_https":
 			fallthrough
 		case "core.proxy_ignore_hosts":
-			daemonConfigSetProxy(d, clusterConfig)
+			daemonConfigSetProxy(d, newClusterConfig)
 		case "maas.api.url":
 			fallthrough
 		case "maas.api.key":
@@ -862,7 +874,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			}
 
 		case "cluster.offline_threshold":
-			d.gateway.HeartbeatOfflineThreshold = clusterConfig.OfflineThreshold()
+			d.gateway.HeartbeatOfflineThreshold = newClusterConfig.OfflineThreshold()
 			d.taskClusterHeartbeat.Reset()
 		case "images.auto_update_interval":
 			fallthrough
@@ -893,7 +905,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			acmeCAURLChanged = true
 		case "acme.domain":
 			acmeDomainChanged = true
-		case "oidc.issuer", "oidc.client.id", "oidc.scopes", "oidc.audience", "oidc.groups.claim":
+		case "oidc.issuer", "oidc.client.id", "oidc.client.secret", "oidc.scopes", "oidc.audience", "oidc.groups.claim":
 			oidcChanged = true
 		}
 	}
@@ -925,7 +937,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			return err
 		}
 
-		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(newClusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["cluster.https_address"]
@@ -935,7 +947,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			return err
 		}
 
-		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(newClusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["core.debug_address"]
@@ -964,7 +976,8 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 	value, ok = nodeChanged["storage.backups_volume"]
 	if ok {
-		err := daemonStorageMove(s, "backups", value)
+		oldValue, _ := oldNodeConfig["storage.backups_volume"].(string)
+		err := daemonStorageMove(s, "backups", oldValue, value)
 		if err != nil {
 			return err
 		}
@@ -972,15 +985,16 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 	value, ok = nodeChanged["storage.images_volume"]
 	if ok {
-		err := daemonStorageMove(s, "images", value)
+		oldValue, _ := oldNodeConfig["storage.images_volume"].(string)
+		err := daemonStorageMove(s, "images", oldValue, value)
 		if err != nil {
 			return err
 		}
 	}
 
 	if maasChanged {
-		url, key := clusterConfig.MAASController()
-		machine := nodeConfig.MAASMachine()
+		url, key := newClusterConfig.MAASController()
+		machine := newNodeConfig.MAASMachine()
 		err := d.setupMAASController(url, key, machine)
 		if err != nil {
 			return err
@@ -988,22 +1002,22 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if bgpChanged {
-		address := nodeConfig.BGPAddress()
-		asn := clusterConfig.BGPASN()
-		routerid := nodeConfig.BGPRouterID()
+		address := newNodeConfig.BGPAddress()
+		asn := newClusterConfig.BGPASN()
+		routerid := newNodeConfig.BGPRouterID()
 
 		if asn > math.MaxUint32 {
-			return fmt.Errorf("Cannot convert BGP ASN to uint32: Upper bound exceeded")
+			return errors.New("Cannot convert BGP ASN to uint32: Upper bound exceeded")
 		}
 
-		err := s.BGP.Reconfigure(address, uint32(asn), net.ParseIP(routerid))
+		err := s.BGP.Configure(address, uint32(asn), net.ParseIP(routerid))
 		if err != nil {
 			return fmt.Errorf("Failed reconfiguring BGP: %w", err)
 		}
 	}
 
 	if dnsChanged {
-		address := nodeConfig.DNSAddress()
+		address := newNodeConfig.DNSAddress()
 
 		err := s.DNS.Reconfigure(address)
 		if err != nil {
@@ -1012,7 +1026,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if lokiChanged {
-		lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := clusterConfig.LokiServer()
+		lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := newClusterConfig.LokiServer()
 
 		if lokiURL == "" || lokiLoglevel == "" || len(lokiTypes) == 0 {
 			d.internalListener.RemoveHandler("loki")
@@ -1041,7 +1055,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if oidcChanged {
-		oidcIssuer, oidcClientID, oidcScopes, oidcAudience, oidcGroupsClaim := clusterConfig.OIDCServer()
+		oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, oidcGroupsClaim := newClusterConfig.OIDCServer()
 
 		if oidcIssuer == "" || oidcClientID == "" {
 			d.oidcVerifier = nil
@@ -1052,7 +1066,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 				return util.HTTPClient("", d.proxy)
 			}
 
-			d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcScopes, oidcAudience, s.ServerCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
+			d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, s.ServerCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
 			if err != nil {
 				return fmt.Errorf("Failed creating verifier: %w", err)
 			}
@@ -1060,7 +1074,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if syslogSocketChanged {
-		err := d.setupSyslogSocket(nodeConfig.SyslogSocket())
+		err := d.setupSyslogSocket(newNodeConfig.SyslogSocket())
 		if err != nil {
 			return err
 		}
