@@ -13,8 +13,8 @@ func TestHandleWriter(t *testing.T) {
 		status = append(status, percent)
 	})}
 
-	for i := 0; i < 101; i++ {
-		for j := 0; j < 100; j++ {
+	for i := range 101 {
+		for j := range 100 {
 			n, err := fmt.Fprintf(out, "\t    (%02d.%02d/100%s)\r", i, j, "%")
 			if err != nil {
 				t.Fatal(err, n)
