@@ -3,17 +3,17 @@ package backup
 import (
 	"archive/tar"
 	"context"
-	"fmt"
+	"errors"
 	"io"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/archive"
-	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/lxd/state"
 	"github.com/canonical/lxd/shared"
 )
 
 // TarReader rewinds backup file handle r and returns new tar reader and process cleanup function.
-func TarReader(r io.ReadSeeker, sysOS *sys.OS, outputPath string) (*tar.Reader, context.CancelFunc, error) {
+func TarReader(s *state.State, r io.ReadSeeker, outputPath string) (*tar.Reader, context.CancelFunc, error) {
 	_, err := r.Seek(0, io.SeekStart)
 	if err != nil {
 		return nil, nil, err
@@ -25,10 +25,10 @@ func TarReader(r io.ReadSeeker, sysOS *sys.OS, outputPath string) (*tar.Reader,
 	}
 
 	if unpacker == nil {
-		return nil, nil, fmt.Errorf("Unsupported backup compression")
+		return nil, nil, errors.New("Unsupported backup compression")
 	}
 
-	tr, cancelFunc, err := archive.CompressedTarReader(context.Background(), r, unpacker, sysOS, outputPath)
+	tr, cancelFunc, err := archive.CompressedTarReader(s, context.Background(), r, unpacker, outputPath)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -41,15 +41,15 @@ func TarReader(r io.ReadSeeker, sysOS *sys.OS, outputPath string) (*tar.Reader,
 // If the name is illegal, then an empty string and an error are returned.
 func ValidateBackupName(backupName string) (string, error) {
 	if strings.Contains(backupName, "/") {
-		return "", fmt.Errorf("Backup name must not contain forward slashes")
+		return "", errors.New("Backup name must not contain forward slashes")
 	}
 
 	if strings.Contains(backupName, "\\") {
-		return "", fmt.Errorf("Backup name must not contain back slashes")
+		return "", errors.New("Backup name must not contain back slashes")
 	}
 
 	if strings.Contains(backupName, "..") {
-		return "", fmt.Errorf("Backup name must not contain '..'")
+		return "", errors.New("Backup name must not contain '..'")
 	}
 
 	return backupName, nil
