@@ -9,6 +9,7 @@ import (
 	"encoding/base64"
 	"encoding/json"
 	"encoding/pem"
+	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -154,7 +155,7 @@ func certificatesGet(d *Daemon, r *http.Request) response.Response {
 		var baseCerts []dbCluster.Certificate
 		urlToCertificate := make(map[*api.URL]auth.EntitlementReporter)
 		var err error
-		err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			baseCerts, err = dbCluster.GetCertificates(ctx, tx.Tx())
 			if err != nil {
 				return err
@@ -207,7 +208,7 @@ func certificatesGet(d *Daemon, r *http.Request) response.Response {
 // clusterMemberJoinTokenValid searches for cluster join token that matches the join token provided.
 // Returns matching operation if found and cancels the operation, otherwise returns nil.
 func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName string, joinToken *api.ClusterMemberJoinToken) (*api.Operation, error) {
-	ops, err := operationsGetByType(s, r, projectName, operationtype.ClusterJoinToken)
+	ops, err := operationsGetByType(r.Context(), s, projectName, operationtype.ClusterJoinToken)
 	if err != nil {
 		return nil, fmt.Errorf("Failed getting cluster join token operations: %w", err)
 	}
@@ -250,7 +251,7 @@ func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName st
 
 	if foundOp != nil {
 		// Token is single-use, so cancel it now.
-		err = operationCancel(s, r, projectName, foundOp)
+		err = operationCancel(r.Context(), s, projectName, foundOp)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to cancel operation %q: %w", foundOp.ID, err)
 		}
@@ -294,7 +295,7 @@ func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName st
 // If an operation is found it is cancelled and the request metadata is returned. If no operation is found then a nil value
 // is returned. An error is only returned if an internal error occurs, or if a token is found but is not valid.
 func certificateTokenValid(s *state.State, r *http.Request, addToken *api.CertificateAddToken) (*api.CertificatesPost, error) {
-	ops, err := operationsGetByType(s, r, api.ProjectDefaultName, operationtype.CertificateAddToken)
+	ops, err := operationsGetByType(r.Context(), s, api.ProjectDefaultName, operationtype.CertificateAddToken)
 	if err != nil {
 		return nil, fmt.Errorf("Failed getting certificate token operations: %w", err)
 	}
@@ -337,7 +338,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 	}
 
 	// Token is single-use, so cancel it now.
-	err = operationCancel(s, r, api.ProjectDefaultName, foundOp)
+	err = operationCancel(r.Context(), s, api.ProjectDefaultName, foundOp)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to cancel operation %q: %w", foundOp.ID, err)
 	}
@@ -373,7 +374,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 	// Certificate add tokens must have a request field in the metadata.
 	tokenReqAny, ok := foundOp.Metadata["request"]
 	if !ok {
-		return nil, fmt.Errorf(`Missing "request" key in certificate add operation data`)
+		return nil, errors.New(`Missing "request" key in certificate add operation data`)
 	}
 
 	var tokenReq api.CertificatesPost
@@ -479,7 +480,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if req.Password != "" { //nolint:staticcheck
-		return response.NotImplemented(fmt.Errorf("Password authentication is no longer supported, please update your client"))
+		return response.NotImplemented(errors.New("Password authentication is no longer supported, please update your client"))
 	}
 
 	// Validate name.
@@ -491,16 +492,16 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick check.
 	if req.Token && req.Certificate != "" {
-		return response.BadRequest(fmt.Errorf("Can't use certificate if token is requested"))
+		return response.BadRequest(errors.New("Can't use certificate if token is requested"))
 	}
 
 	if req.Token {
 		if req.Type != "client" {
-			return response.BadRequest(fmt.Errorf("Tokens can only be issued for client certificates"))
+			return response.BadRequest(errors.New("Tokens can only be issued for client certificates"))
 		}
 
 		if localHTTPSAddress == "" {
-			return response.BadRequest(fmt.Errorf("Can't issue token when server isn't listening on network"))
+			return response.BadRequest(errors.New("Can't issue token when server isn't listening on network"))
 		}
 	}
 
@@ -513,14 +514,14 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		userCanCreateCertificates = true
 	}
 
-	trusted, err := request.GetCtxValue[bool](r.Context(), request.CtxTrusted)
-	if err != nil {
-		return response.SmartError(fmt.Errorf("Failed to get authentication status: %w", err))
+	reqInfo := request.GetContextInfo(r.Context())
+	if reqInfo == nil {
+		return response.SmartError(errors.New("Failed to get authentication status: Missing request context info"))
 	}
 
 	// If caller is already trusted and the trust token is provided, we validate the token and cancel
 	// the corresponding token operation.
-	if trusted && req.TrustToken != "" {
+	if reqInfo.Trusted && req.TrustToken != "" {
 		// Decode the trust token.
 		joinToken, err := shared.CertificateTokenDecode(req.TrustToken)
 		if err != nil {
@@ -534,15 +535,15 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if tokenReq == nil {
-			return response.Forbidden(fmt.Errorf("No matching certificate add operation found"))
+			return response.Forbidden(errors.New("No matching certificate add operation found"))
 		}
 
-		return response.Conflict(fmt.Errorf("Client is already trusted"))
+		return response.Conflict(errors.New("Client is already trusted"))
 	}
 
 	// If caller is already trusted and does not have permission to create certificates, they cannot create more certificates.
-	if trusted && !userCanCreateCertificates && req.Certificate == "" && !req.Token {
-		return response.BadRequest(fmt.Errorf("Client is already trusted"))
+	if reqInfo.Trusted && !userCanCreateCertificates && req.Certificate == "" && !req.Token {
+		return response.BadRequest(errors.New("Client is already trusted"))
 	}
 
 	if !userCanCreateCertificates {
@@ -566,7 +567,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			if joinOp == nil {
-				return response.Forbidden(fmt.Errorf("No matching cluster join operation found"))
+				return response.Forbidden(errors.New("No matching cluster join operation found"))
 			}
 		} else {
 			// Check if certificate add token supplied as token.
@@ -582,7 +583,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			if tokenReq == nil {
-				return response.Forbidden(fmt.Errorf("No matching certificate add operation found"))
+				return response.Forbidden(errors.New("No matching certificate add operation found"))
 			}
 
 			// Create a new request from the token data as the user isn't allowed to override anything.
@@ -614,27 +615,9 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(fmt.Errorf("Invalid certificate material: %w", err))
 		}
 	} else if req.Token {
-		// Get all addresses the server is listening on. This is encoded in the certificate token,
-		// so that the client will not have to specify a server address. The client will iterate
-		// through all these addresses until it can connect to one of them.
-		addresses, err := util.ListenAddresses(localHTTPSAddress)
+		token, err := createCertificateAddToken(s, req.Name, "")
 		if err != nil {
-			return response.InternalError(err)
-		}
-
-		// Generate join secret for new client. This will be stored inside the join token operation and will be
-		// supplied by the joining client (encoded inside the join token) which will allow us to lookup the correct
-		// operation in order to validate the requested joining client name is correct and authorised.
-		joinSecret, err := shared.RandomCryptoString()
-		if err != nil {
-			return response.InternalError(err)
-		}
-
-		// Generate fingerprint of network certificate so joining member can automatically trust the correct
-		// certificate when it is presented during the join process.
-		fingerprint, err := shared.CertFingerprintStr(string(s.Endpoints.NetworkPublicKey()))
-		if err != nil {
-			return response.InternalError(err)
+			return response.SmartError(fmt.Errorf("Failed to create certificate add token: %w", err))
 		}
 
 		if req.Projects == nil {
@@ -642,9 +625,9 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		meta := map[string]any{
-			"secret":      joinSecret,
-			"fingerprint": fingerprint,
-			"addresses":   addresses,
+			"secret":      token.Secret,
+			"fingerprint": token.Fingerprint,
+			"addresses":   token.Addresses,
 			"request":     req,
 		}
 
@@ -660,7 +643,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			meta["expiresAt"] = expiresAt
 		}
 
-		op, err := operations.OperationCreate(s, api.ProjectDefaultName, operations.OperationClassToken, operationtype.CertificateAddToken, nil, meta, nil, nil, nil, r)
+		op, err := operations.OperationCreate(r.Context(), s, api.ProjectDefaultName, operations.OperationClassToken, operationtype.CertificateAddToken, nil, meta, nil, nil, nil)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -672,12 +655,12 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			// This can happen if the client doesn't send a client certificate or if the server is in
 			// CA mode. We rely on this check to prevent non-CA trusted client certificates from being
 			// added when in CA mode.
-			return response.BadRequest(fmt.Errorf("No client certificate provided"))
+			return response.BadRequest(errors.New("No client certificate provided"))
 		}
 
 		cert = r.TLS.PeerCertificates[len(r.TLS.PeerCertificates)-1]
 	} else {
-		return response.BadRequest(fmt.Errorf("Can't use TLS data on non-TLS link"))
+		return response.BadRequest(errors.New("Can't use TLS data on non-TLS link"))
 	}
 
 	// Check validity.
@@ -706,7 +689,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check if we already have the certificate.
 		existingCert, _ := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if existingCert != nil {
@@ -752,7 +735,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 	// Reload the identity cache to add the new certificate.
 	s.UpdateIdentityCache()
 
-	lc := lifecycle.CertificateCreated.Event(fingerprint, request.CreateRequestor(r), nil)
+	lc := lifecycle.CertificateCreated.Event(fingerprint, request.CreateRequestor(r.Context()), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -805,7 +788,7 @@ func certificateGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var cert *api.Certificate
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		dbCertInfo, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -870,7 +853,7 @@ func certificatePut(d *Daemon, r *http.Request) response.Response {
 
 	// Get current database record.
 	var apiEntry *api.Certificate
-	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		oldEntry, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -937,7 +920,7 @@ func certificatePatch(d *Daemon, r *http.Request) response.Response {
 
 	// Get current database record.
 	var apiEntry *api.Certificate
-	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		oldEntry, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -997,17 +980,17 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 	certProjects := req.Projects
 	if !userCanEditCertificate {
 		if r.TLS == nil {
-			response.Forbidden(fmt.Errorf("Cannot update certificate information"))
+			response.Forbidden(errors.New("Cannot update certificate information"))
 		}
 
 		// Ensure the user in not trying to change fields other than the certificate.
 		if dbInfo.Restricted != req.Restricted || dbInfo.Name != req.Name || len(dbInfo.Projects) != len(req.Projects) {
-			return response.Forbidden(fmt.Errorf("Only the certificate can be changed"))
+			return response.Forbidden(errors.New("Only the certificate can be changed"))
 		}
 
-		for i := 0; i < len(dbInfo.Projects); i++ {
+		for i := range dbInfo.Projects {
 			if dbInfo.Projects[i] != req.Projects[i] {
-				return response.Forbidden(fmt.Errorf("Only the certificate can be changed"))
+				return response.Forbidden(errors.New("Only the certificate can be changed"))
 			}
 		}
 
@@ -1045,7 +1028,7 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 			}
 
 			if !trusted {
-				return response.Forbidden(fmt.Errorf("Certificate cannot be changed"))
+				return response.Forbidden(errors.New("Certificate cannot be changed"))
 			}
 		}
 	}
@@ -1105,7 +1088,7 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 	// Reload the identity cache.
 	s.UpdateIdentityCache()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateUpdated.Event(dbInfo.Fingerprint, request.CreateRequestor(r), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateUpdated.Event(dbInfo.Fingerprint, request.CreateRequestor(r.Context()), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1137,7 +1120,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var certInfo *dbCluster.Certificate
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get current database record.
 		var err error
 		certInfo, err = dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
@@ -1151,6 +1134,11 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	// Disallow deletion of server certificates.
+	if certInfo.Type == certificate.TypeServer {
+		return response.BadRequest(errors.New("Cannot delete a server certificate. Remove the cluster member instead"))
+	}
+
 	var userCanEditCertificate bool
 	err = s.Authorizer.CheckPermission(r.Context(), entity.CertificateURL(certInfo.Fingerprint), auth.EntitlementCanDelete)
 	if err != nil && !auth.IsDeniedError(err) {
@@ -1162,7 +1150,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	// Non-admins are able to delete only their own certificate.
 	if !userCanEditCertificate {
 		if r.TLS == nil {
-			response.Forbidden(fmt.Errorf("Cannot delete certificate"))
+			response.Forbidden(errors.New("Cannot delete certificate"))
 		}
 
 		certBlock, _ := pem.Decode([]byte(certInfo.Certificate))
@@ -1187,7 +1175,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if !trusted {
-			return response.Forbidden(fmt.Errorf("Certificate cannot be deleted"))
+			return response.Forbidden(errors.New("Certificate cannot be deleted"))
 		}
 	}
 
@@ -1216,7 +1204,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	// Reload the cache.
 	s.UpdateIdentityCache()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateDeleted.Event(fingerprint, request.CreateRequestor(r), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateDeleted.Event(fingerprint, request.CreateRequestor(r.Context()), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1241,7 +1229,7 @@ func certificateValidate(networkCert *shared.CertInfo, cert *x509.Certificate) e
 	if cert.PublicKeyAlgorithm == x509.RSA {
 		pubKey, ok := cert.PublicKey.(*rsa.PublicKey)
 		if !ok {
-			return fmt.Errorf("Unable to validate the RSA certificate")
+			return errors.New("Unable to validate the RSA certificate")
 		}
 
 		// Check that we're dealing with at least 2048bit (Size returns a value in bytes).
