@@ -2,7 +2,10 @@ package config
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"maps"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -19,8 +22,7 @@ import (
 
 // Config holds cluster-wide configuration values.
 type Config struct {
-	tx *db.ClusterTx // DB transaction the values in this config are bound to.
-	m  config.Map    // Low-level map holding the config values.
+	m config.Map // Low-level map holding the config values.
 }
 
 // Load loads a new Config object with the current cluster configuration
@@ -37,7 +39,7 @@ func Load(ctx context.Context, tx *db.ClusterTx) (*Config, error) {
 		return nil, fmt.Errorf("failed to load node config: %w", err)
 	}
 
-	return &Config{tx: tx, m: m}, nil
+	return &Config{m: m}, nil
 }
 
 // BackupsCompressionAlgorithm returns the compression algorithm to use for backups.
@@ -224,8 +226,8 @@ func (c *Config) RemoteTokenExpiry() string {
 }
 
 // OIDCServer returns all the OpenID Connect settings needed to connect to a server.
-func (c *Config) OIDCServer() (issuer string, clientID string, scopes []string, audience string, groupsClaim string) {
-	return c.m.GetString("oidc.issuer"), c.m.GetString("oidc.client.id"), strings.Fields(c.m.GetString("oidc.scopes")), c.m.GetString("oidc.audience"), c.m.GetString("oidc.groups.claim")
+func (c *Config) OIDCServer() (issuer string, clientID string, clientSecret string, scopes []string, audience string, groupsClaim string) {
+	return c.m.GetString("oidc.issuer"), c.m.GetString("oidc.client.id"), c.m.GetString("oidc.client.secret"), strings.Fields(c.m.GetString("oidc.scopes")), c.m.GetString("oidc.audience"), c.m.GetString("oidc.groups.claim")
 }
 
 // ClusterHealingThreshold returns the configured healing threshold, i.e. the
@@ -257,29 +259,27 @@ func (c *Config) Dump() map[string]any {
 // Replace the current configuration with the given values.
 //
 // Return what has actually changed.
-func (c *Config) Replace(values map[string]any) (map[string]string, error) {
-	return c.update(values)
+func (c *Config) Replace(tx *db.ClusterTx, values map[string]any) (map[string]string, error) {
+	return c.update(tx, values)
 }
 
 // Patch changes only the configuration keys in the given map.
 //
 // Return what has actually changed.
-func (c *Config) Patch(patch map[string]any) (map[string]string, error) {
+func (c *Config) Patch(tx *db.ClusterTx, patch map[string]any) (map[string]string, error) {
 	values := c.Dump() // Use current values as defaults
-	for name, value := range patch {
-		values[name] = value
-	}
+	maps.Copy(values, patch)
 
-	return c.update(values)
+	return c.update(tx, values)
 }
 
-func (c *Config) update(values map[string]any) (map[string]string, error) {
+func (c *Config) update(tx *db.ClusterTx, values map[string]any) (map[string]string, error) {
 	changed, err := c.m.Change(values)
 	if err != nil {
 		return nil, err
 	}
 
-	err = c.tx.UpdateClusterConfig(changed)
+	err = tx.UpdateClusterConfig(changed)
 	if err != nil {
 		return nil, fmt.Errorf("cannot persist configuration changes: %w", err)
 	}
@@ -664,6 +664,14 @@ var ConfigSchema = config.Schema{
 	//  shortdesc: OpenID Connect client ID
 	"oidc.client.id": {},
 
+	// lxdmeta:generate(entities=server; group=oidc; key=oidc.client.secret)
+	//
+	// ---
+	//  type: string
+	//  scope: global
+	//  shortdesc: OpenID Connect client secret
+	"oidc.client.secret": {},
+
 	// lxdmeta:generate(entities=server; group=oidc; key=oidc.issuer)
 	//
 	// ---
@@ -695,7 +703,7 @@ var ConfigSchema = config.Schema{
 		Default: strings.Join([]string{oidc.ScopeOpenID, oidc.ScopeEmail, oidc.ScopeOfflineAccess, oidc.ScopeProfile}, " "),
 		Validator: validate.Optional(func(value string) error {
 			scopes := strings.Fields(value)
-			if !shared.ValueInSlice(oidc.ScopeOpenID, scopes) || !shared.ValueInSlice(oidc.ScopeEmail, scopes) {
+			if !slices.Contains(scopes, oidc.ScopeOpenID) || !slices.Contains(scopes, oidc.ScopeEmail) {
 				return fmt.Errorf("oidc.scopes requires the %q and %q OpenID Connect scopes", oidc.ScopeOpenID, oidc.ScopeEmail)
 			}
 
@@ -792,7 +800,7 @@ func offlineThresholdValidator(value string) error {
 	// which is the lower bound granularity of the offline check.
 	threshold, err := strconv.Atoi(value)
 	if err != nil {
-		return fmt.Errorf("Offline threshold is not a number")
+		return errors.New("Offline threshold is not a number")
 	}
 
 	if threshold <= minThreshold {
@@ -805,11 +813,11 @@ func offlineThresholdValidator(value string) error {
 func imageMinimalReplicaValidator(value string) error {
 	count, err := strconv.Atoi(value)
 	if err != nil {
-		return fmt.Errorf("Minimal image replica count is not a number")
+		return errors.New("Minimal image replica count is not a number")
 	}
 
 	if count < 1 && count != -1 {
-		return fmt.Errorf("Invalid value for image replica count")
+		return errors.New("Invalid value for image replica count")
 	}
 
 	return nil
@@ -818,11 +826,11 @@ func imageMinimalReplicaValidator(value string) error {
 func maxVotersValidator(value string) error {
 	n, err := strconv.Atoi(value)
 	if err != nil {
-		return fmt.Errorf("Value is not a number")
+		return errors.New("Value is not a number")
 	}
 
 	if n < 3 || n%2 != 1 {
-		return fmt.Errorf("Value must be an odd number equal to or higher than 3")
+		return errors.New("Value must be an odd number equal to or higher than 3")
 	}
 
 	return nil
@@ -831,11 +839,11 @@ func maxVotersValidator(value string) error {
 func maxStandByValidator(value string) error {
 	n, err := strconv.Atoi(value)
 	if err != nil {
-		return fmt.Errorf("Value is not a number")
+		return errors.New("Value is not a number")
 	}
 
 	if n < 0 || n > 5 {
-		return fmt.Errorf("Value must be between 0 and 5")
+		return errors.New("Value must be between 0 and 5")
 	}
 
 	return nil
