@@ -5,6 +5,7 @@ import (
 	"context"
 	"crypto/tls"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net"
@@ -339,10 +340,10 @@ func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, identityCache
 // WaitUpgradeNotification waits for a notification from another node that all
 // nodes in the cluster should now have been upgraded and have matching schema
 // and API versions.
-func (g *Gateway) WaitUpgradeNotification() {
+func (g *Gateway) WaitUpgradeNotification(ctx context.Context) {
 	select {
 	case <-g.upgradeCh:
-	case <-time.After(time.Minute):
+	case <-ctx.Done():
 	}
 }
 
@@ -488,7 +489,7 @@ func (g *Gateway) TransferLeadership() error {
 	}
 
 	if id == 0 {
-		return fmt.Errorf("No online voter found")
+		return errors.New("No online voter found")
 	}
 
 	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
@@ -520,33 +521,39 @@ func (g *Gateway) Shutdown() error {
 	logger.Info("Stop database gateway")
 
 	var err error
-	if g.server != nil {
-		if g.info.Role == db.RaftVoter {
-			g.Sync()
-		}
-
-		err = g.server.Close()
-		close(g.stopCh)
-
-		// Unset the memory dial, since Shutdown() is also called for
-		// switching between in-memory and network mode.
-		g.lock.Lock()
-		g.memoryDial = nil
-		g.lock.Unlock()
-
-		// Record the raft term and index in the logs on every shutdown. This
-		// allows an administrator to determine the furthest-ahead cluster member
-		// in case recovery is needed.
-		lastEntryInfo, err := dqlite.ReadLastEntryInfo(g.db.Dir())
-		if err != nil {
-			return err
-		}
-
-		// This isn't really a warning, but it's important that this break through
-		// the snap's default log level of 'Warn'.
-		logger.Warn("Dqlite last entry", logger.Ctx{"term": lastEntryInfo.Term, "index": lastEntryInfo.Index})
+	if g.server == nil {
+		return nil
 	}
 
+	if g.info.Role == db.RaftVoter {
+		g.Sync()
+	}
+
+	err = g.server.Close()
+	if err != nil {
+		logger.Error("Failed stopping dqlite", logger.Ctx{"err": err})
+	}
+
+	close(g.stopCh)
+
+	// Unset the memory dial, since Shutdown() is also called for
+	// switching between in-memory and network mode.
+	g.lock.Lock()
+	g.memoryDial = nil
+	g.lock.Unlock()
+
+	// Record the raft term and index in the logs on every shutdown. This
+	// allows an administrator to determine the furthest-ahead cluster member
+	// in case recovery is needed.
+	lastEntryInfo, err := dqlite.ReadLastEntryInfo(g.db.Dir())
+	if err != nil {
+		return err
+	}
+
+	// This isn't really a warning, but it's important that this break through
+	// the snap's default log level of 'Warn'.
+	logger.Warn("Dqlite last entry", logger.Ctx{"term": lastEntryInfo.Term, "index": lastEntryInfo.Index})
+
 	return err
 }
 
@@ -618,7 +625,7 @@ func (g *Gateway) Reset(networkCert *shared.CertInfo) error {
 }
 
 // ErrNodeIsNotClustered indicates the node is not clustered.
-var ErrNodeIsNotClustered = fmt.Errorf("Server is not clustered")
+var ErrNodeIsNotClustered = errors.New("Server is not clustered")
 
 // LeaderAddress returns the address of the current raft leader.
 func (g *Gateway) LeaderAddress() (string, error) {
@@ -695,7 +702,7 @@ func (g *Gateway) LeaderAddress() (string, error) {
 		// This should never happen because the raft_nodes table should
 		// be never empty for a clustered node, but check it for good
 		// measure.
-		return "", fmt.Errorf("No raft node known")
+		return "", errors.New("No raft node known")
 	}
 
 	transport, cleanup := tlsTransport(config)
@@ -712,7 +719,7 @@ func (g *Gateway) LeaderAddress() (string, error) {
 		return leader, nil
 	}
 
-	return "", fmt.Errorf("RAFT cluster is unavailable")
+	return "", errors.New("RAFT cluster is unavailable")
 }
 
 // NetworkUpdateCert sets a new network certificate for the gateway
@@ -729,7 +736,7 @@ func (g *Gateway) NetworkUpdateCert(cert *shared.CertInfo) {
 // @bootstrap should only be true when turning a non-clustered LXD instance into
 // the first (and leader) node of a new LXD cluster.
 func (g *Gateway) init(bootstrap bool) error {
-	logger.Debugf("Initializing database gateway")
+	logger.Debug("Initializing database gateway")
 	g.stopCh = make(chan struct{})
 
 	info, err := loadInfo(g.db)
@@ -738,9 +745,6 @@ func (g *Gateway) init(bootstrap bool) error {
 	}
 
 	dir := g.db.DqliteDir()
-	if shared.PathExists(filepath.Join(dir, "logs.db")) {
-		return fmt.Errorf("Unsupported upgrade path, please first upgrade to LXD 4.0")
-	}
 
 	// If the resulting raft instance is not nil, it means that this node
 	// should serve as database node, so create a dqlite driver possibly
@@ -832,7 +836,7 @@ func (g *Gateway) init(bootstrap bool) error {
 func (g *Gateway) WaitLeadership() error {
 	n := 80
 	sleep := 250 * time.Millisecond
-	for i := 0; i < n; i++ {
+	for range n {
 		g.lock.RLock()
 		isLeader, err := g.isLeader()
 		if err != nil {
@@ -875,7 +879,7 @@ func (g *Gateway) isLeader() (bool, error) {
 }
 
 // ErrNotLeader signals that a node not the leader.
-var ErrNotLeader = fmt.Errorf("Not leader")
+var ErrNotLeader = errors.New("Not leader")
 
 // Return information about the LXD nodes that a currently part of the raft
 // cluster, as configured in the raft log. It returns an error if this node is
@@ -995,7 +999,7 @@ func attemptGetLeaderAddressFromNodeAddress(ctx context.Context, transport http.
 	}
 
 	url := "https://" + nodeAddress + databaseEndpoint
-	request, err := http.NewRequest("GET", url, nil)
+	request, err := http.NewRequest(http.MethodGet, url, nil)
 	if err != nil {
 		return "", err
 	}
@@ -1108,7 +1112,7 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 				g.upgradeTriggered = true
 			}
 		}
-		return nil, fmt.Errorf("Upgrade needed")
+		return nil, errors.New("Upgrade needed")
 	}
 
 	if response.StatusCode != http.StatusSwitchingProtocols {
@@ -1116,7 +1120,7 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 	}
 
 	if response.Header.Get("Upgrade") != "dqlite" {
-		return nil, fmt.Errorf("Missing or unexpected Upgrade header in response")
+		return nil, errors.New("Missing or unexpected Upgrade header in response")
 	}
 
 	revert.Success()
