@@ -36,7 +36,7 @@ func TestNotifyUpgradeCompleted(t *testing.T) {
 	wg.Add(1)
 
 	go func() {
-		gateway1.WaitUpgradeNotification()
+		gateway1.WaitUpgradeNotification(context.Background())
 		wg.Done()
 	}()
 
@@ -71,7 +71,7 @@ func TestMaybeUpdate_Upgrade(t *testing.T) {
 	// Create a stub upgrade script that just touches a stamp file.
 	stamp := filepath.Join(dir, "stamp")
 	script := filepath.Join(dir, "cluster-upgrade")
-	data := []byte(fmt.Sprintf("#!/bin/sh\ntouch %s\n", stamp))
+	data := fmt.Appendf(nil, "#!/bin/sh\ntouch %s\n", stamp)
 	err = os.WriteFile(script, data, 0755)
 	require.NoError(t, err)
 
@@ -124,7 +124,7 @@ func TestMaybeUpdate_NothingToDo(t *testing.T) {
 	// Create a stub upgrade script that just touches a stamp file.
 	stamp := filepath.Join(dir, "stamp")
 	script := filepath.Join(dir, "cluster-upgrade")
-	data := []byte(fmt.Sprintf("#!/bin/sh\ntouch %s\n", stamp))
+	data := fmt.Appendf(nil, "#!/bin/sh\ntouch %s\n", stamp)
 	err = os.WriteFile(script, data, 0755)
 	require.NoError(t, err)
 
