@@ -1,12 +1,12 @@
 package config
 
 import (
+	"errors"
 	"fmt"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
-
-	"github.com/canonical/lxd/shared"
 )
 
 // Schema defines the available keys of a config Map, along with the types
@@ -101,14 +101,14 @@ func (v *Key) validate(value string) error {
 	switch v.Type {
 	case String:
 	case Bool:
-		if !shared.ValueInSlice(strings.ToLower(value), booleans) {
-			return fmt.Errorf("Invalid boolean")
+		if !slices.Contains(booleans, strings.ToLower(value)) {
+			return errors.New("Invalid boolean")
 		}
 
 	case Int64:
 		_, err := strconv.ParseInt(value, 10, 64)
 		if err != nil {
-			return fmt.Errorf("Invalid integer")
+			return errors.New("Invalid integer")
 		}
 
 	default:
