@@ -16,6 +16,8 @@ import (
 	"os/exec"
 	"os/user"
 	"path/filepath"
+	"slices"
+	"strconv"
 	"strings"
 	"sync"
 	"time"
@@ -58,13 +60,13 @@ import (
 	"github.com/canonical/lxd/lxd/metrics"
 	networkZone "github.com/canonical/lxd/lxd/network/zone"
 	"github.com/canonical/lxd/lxd/node"
+	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/rsync"
 	scriptletLoad "github.com/canonical/lxd/lxd/scriptlet/load"
 	"github.com/canonical/lxd/lxd/seccomp"
 	"github.com/canonical/lxd/lxd/state"
-	storagePools "github.com/canonical/lxd/lxd/storage"
 	storageDrivers "github.com/canonical/lxd/lxd/storage/drivers"
 	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/lxd/storage/s3/miniod"
@@ -82,6 +84,9 @@ import (
 	"github.com/canonical/lxd/shared/version"
 )
 
+// secFetchSiteForbidden defines client Sec-Fetch-Site header values that will be forbidden access.
+var secFetchSiteForbidden = []string{"cross-site", "same-site"}
+
 // A Daemon can respond to requests from a shared client.
 type Daemon struct {
 	identityCache *identity.Cache
@@ -93,7 +98,7 @@ type Daemon struct {
 	dns           *dns.Server
 
 	// Event servers
-	devlxdEvents     *events.DevLXDServer
+	devLXDEvents     *events.DevLXDServer
 	events           *events.Server
 	internalListener *events.InternalListener
 
@@ -132,11 +137,13 @@ type Daemon struct {
 	serverCertInt *shared.CertInfo // Do not use this directly, use servertCert func.
 
 	// Status control.
-	setupChan      chan struct{}      // Closed when basic Daemon setup is completed
-	waitReady      *cancel.Canceller  // Cancelled when LXD is fully ready
-	shutdownCtx    context.Context    // Cancelled when shutdown starts.
-	shutdownCancel context.CancelFunc // Cancels the shutdownCtx to indicate shutdown starting.
-	shutdownDoneCh chan error         // Receives the result of the d.Stop() function and tells LXD to end.
+	startStopLock    sync.Mutex       // Prevent concurrent starts and stops.
+	setupChan        chan struct{}    // Closed when basic Daemon setup is completed
+	waitReady        cancel.Canceller // Cancelled when LXD is fully ready
+	waitNetworkReady cancel.Canceller // Closed when all networks are ready.
+	waitStorageReady cancel.Canceller // Closed when all storage pools are ready.
+	shutdownCtx      cancel.Canceller // Cancelled when shutdown starts.
+	shutdownDoneCh   chan error       // Receives the result of the d.Stop() function and tells LXD to end.
 
 	// Device monitor for watching filesystem events
 	devmonitor fsmonitor.FSMonitor
@@ -182,24 +189,25 @@ type DaemonConfig struct {
 // newDaemon returns a new Daemon object with the given configuration.
 func newDaemon(config *DaemonConfig, os *sys.OS) *Daemon {
 	lxdEvents := events.NewServer(daemon.Debug, daemon.Verbose, cluster.EventHubPush)
-	devlxdEvents := events.NewDevLXDServer(daemon.Debug, daemon.Verbose)
-	shutdownCtx, shutdownCancel := context.WithCancel(context.Background())
+	devLXDEvents := events.NewDevLXDServer(daemon.Debug, daemon.Verbose)
+	shutdownCtx := cancel.New()
 
 	d := &Daemon{
-		identityCache:  &identity.Cache{},
-		config:         config,
-		devlxdEvents:   devlxdEvents,
-		events:         lxdEvents,
-		tasks:          task.NewGroup(),
-		clusterTasks:   task.NewGroup(),
-		db:             &db.DB{},
-		http01Provider: acme.NewHTTP01Provider(),
-		os:             os,
-		setupChan:      make(chan struct{}),
-		waitReady:      cancel.New(context.Background()),
-		shutdownCtx:    shutdownCtx,
-		shutdownCancel: shutdownCancel,
-		shutdownDoneCh: make(chan error),
+		identityCache:    &identity.Cache{},
+		config:           config,
+		devLXDEvents:     devLXDEvents,
+		events:           lxdEvents,
+		tasks:            task.NewGroup(),
+		clusterTasks:     task.NewGroup(),
+		db:               &db.DB{},
+		http01Provider:   acme.NewHTTP01Provider(),
+		os:               os,
+		setupChan:        make(chan struct{}),
+		waitReady:        cancel.New(),
+		waitNetworkReady: cancel.New(),
+		waitStorageReady: cancel.New(),
+		shutdownCtx:      shutdownCtx,
+		shutdownDoneCh:   make(chan error),
 	}
 
 	d.serverCert = func() *shared.CertInfo { return d.serverCertInt }
@@ -247,6 +255,7 @@ type APIEndpointAction struct {
 	Handler        func(d *Daemon, r *http.Request) response.Response
 	AccessHandler  func(d *Daemon, r *http.Request) response.Response
 	AllowUntrusted bool
+	ContentTypes   []string // Client content types to allow.
 }
 
 // allowAuthenticated is an AccessHandler which allows only authenticated requests. This should be used in conjunction
@@ -342,14 +351,19 @@ func allowProjectResourceList(d *Daemon, r *http.Request) response.Response {
 		return response.Forbidden(nil)
 	}
 
+	requestProjectName, allProjects, err := request.ProjectParams(r)
+	if err != nil {
+		return response.SmartError(err)
+	}
+
 	// all-projects requests are not allowed
-	if shared.IsTrue(request.QueryParam(r, "all-projects")) {
-		return response.Forbidden(fmt.Errorf("Certificate is restricted"))
+	if allProjects {
+		return response.Forbidden(errors.New("Certificate is restricted"))
 	}
 
 	// Disallow listing resources in projects the caller does not have access to.
-	if !shared.ValueInSlice(request.ProjectParam(r), id.Projects) {
-		return response.Forbidden(fmt.Errorf("Certificate is restricted"))
+	if !slices.Contains(id.Projects, requestProjectName) {
+		return response.Forbidden(errors.New("Certificate is restricted"))
 	}
 
 	return response.EmptySyncResponse
@@ -369,7 +383,7 @@ func reportEntitlements(ctx context.Context, authorizer auth.Authorizer, identit
 	}
 
 	if !identity.IsFineGrainedIdentityType(id.IdentityType) {
-		return fmt.Errorf("Not fine grained")
+		return errors.New("Not fine grained")
 	}
 
 	// In the case where we have only one entity URL, we'll use the authorizer's CheckPermission method
@@ -432,7 +446,7 @@ func extractEntitlementsFromQuery(r *http.Request, entityType entity.Type, allow
 	entitlements := strings.Split(rawEntitlements, ",")
 	validEntitlements := make([]auth.Entitlement, 0, len(entitlements))
 	for _, e := range entitlements {
-		if !shared.ValueInSlice(auth.Entitlement(e), allowedEntitlements) {
+		if !slices.Contains(allowedEntitlements, auth.Entitlement(e)) {
 			return nil, api.StatusErrorf(http.StatusBadRequest, "Requested entitlement %q is not valid for entity type %q", e, entityType)
 		}
 
@@ -442,7 +456,7 @@ func extractEntitlementsFromQuery(r *http.Request, entityType entity.Type, allow
 	// Entitlements can only be requested when recursion is enabled for a request returning multiple entities (this function call uses `allowRecursion=true`).
 	// If the request is meant to return a single entity, the entitlements can be requested regardless of the recursion setting (in this case, the function is called with `allowRecursion=false`).
 	if len(validEntitlements) > 0 && (!util.IsRecursionRequest(r) && allowRecursion) {
-		return nil, fmt.Errorf("Entitlements can only be requested when recursion is enabled")
+		return nil, errors.New("Entitlements can only be requested when recursion is enabled")
 	}
 
 	return validEntitlements, nil
@@ -475,7 +489,7 @@ func (d *Daemon) Authenticate(w http.ResponseWriter, r *http.Request) (trusted b
 				return false, "", "", nil, err
 			}
 
-			u, err := user.LookupId(fmt.Sprint(cred.Uid))
+			u, err := user.LookupId(strconv.FormatUint(uint64(cred.Uid), 10))
 			if err != nil {
 				return true, fmt.Sprint("uid=", cred.Uid), auth.AuthenticationMethodUnix, nil, nil
 			}
@@ -488,16 +502,16 @@ func (d *Daemon) Authenticate(w http.ResponseWriter, r *http.Request) (trusted b
 
 	// Cluster notification with wrong certificate.
 	if isClusterNotification(r) {
-		return false, "", "", nil, fmt.Errorf("Cluster notification isn't using trusted server certificate")
+		return false, "", "", nil, errors.New("Cluster notification isn't using trusted server certificate")
 	}
 
 	// Bad query, no TLS found.
 	if r.TLS == nil {
-		return false, "", "", nil, fmt.Errorf("Bad/missing TLS on network query")
+		return false, "", "", nil, errors.New("Bad/missing TLS on network query")
 	}
 
 	if d.oidcVerifier != nil && d.oidcVerifier.IsRequest(r) {
-		result, err := d.oidcVerifier.Auth(d.shutdownCtx, w, r)
+		result, err := d.oidcVerifier.Auth(w, r)
 		if err != nil {
 			return false, "", "", nil, fmt.Errorf("Failed OIDC Authentication: %w", err)
 		}
@@ -549,7 +563,7 @@ func (d *Daemon) Authenticate(w http.ResponseWriter, r *http.Request) (trusted b
 
 			// The identity type must be in our list of candidate types (e.g. if this certificate is a metrics certificate
 			// and we're on a non-metrics related route).
-			if !shared.ValueInSlice(id.IdentityType, candidateIdentityTypes) {
+			if !slices.Contains(candidateIdentityTypes, id.IdentityType) {
 				return false, "", "", nil, nil
 			}
 
@@ -646,7 +660,7 @@ func (d *Daemon) handleOIDCAuthenticationResult(r *http.Request, result *oidc.Au
 			return fmt.Errorf("Failed to notify cluster members of new or updated OIDC identity: %w", err)
 		}
 
-		lc := action.Event(api.AuthenticationMethodOIDC, result.Email, request.CreateRequestor(r), nil)
+		lc := action.Event(api.AuthenticationMethodOIDC, result.Email, request.CreateRequestor(r.Context()), nil)
 		s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 		s.UpdateIdentityCache()
@@ -682,7 +696,7 @@ func (d *Daemon) State() *state.State {
 		OS:                  d.os,
 		Endpoints:           d.endpoints,
 		Events:              d.events,
-		DevlxdEvents:        d.devlxdEvents,
+		DevlxdEvents:        d.devLXDEvents,
 		Firewall:            d.firewall,
 		Proxy:               d.proxy,
 		ServerCert:          d.serverCert,
@@ -698,6 +712,8 @@ func (d *Daemon) State() *state.State {
 		StartTime:           d.startTime,
 		Authorizer:          d.authorizer,
 		UbuntuPro:           d.ubuntuPro,
+		NetworkReady:        d.waitNetworkReady,
+		StorageReady:        d.waitStorageReady,
 	}
 
 	s.LeaderInfo = func() (*state.LeaderInfo, error) {
@@ -722,18 +738,15 @@ func (d *Daemon) State() *state.State {
 		}, nil
 	}
 
-	return s
-}
-
-// UnixSocket returns the full path to the unix.socket file that this daemon is
-// listening on. Used by tests.
-func (d *Daemon) UnixSocket() string {
-	path := os.Getenv("LXD_SOCKET")
-	if path != "" {
-		return path
+	s.ImagesStoragePath = func() string {
+		return daemonStoragePath(s.LocalConfig.StorageImagesVolume(), "images")
 	}
 
-	return filepath.Join(d.os.VarDir, "unix.socket")
+	s.BackupsStoragePath = func() string {
+		return daemonStoragePath(s.LocalConfig.StorageBackupsVolume(), "backups")
+	}
+
+	return s
 }
 
 // createCmd creates API handlers for the provided endpoint including some useful behavior,
@@ -760,13 +773,13 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 
 		w.Header().Set("Content-Type", "application/json")
 
-		if !(r.RemoteAddr == "@" && version == "internal") {
+		if r.RemoteAddr != "@" || version != "internal" {
 			// Block public API requests until we're done with basic
 			// initialization tasks, such setting up the cluster database.
 			select {
 			case <-d.setupChan:
 			default:
-				response := response.Unavailable(fmt.Errorf("LXD daemon setup in progress"))
+				response := response.Unavailable(errors.New("LXD daemon setup in progress"))
 				_ = response.Render(w, r)
 				return
 			}
@@ -792,11 +805,17 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 			return
 		}
 
+		// Initialise the request info.
+		reqInfo := request.InitContextInfo(r)
+
 		// Set the "trusted" value in the request context.
-		request.SetCtxValue(r, request.CtxTrusted, trusted)
+		reqInfo.Trusted = trusted
+
+		// Set request source address value in the request context.
+		reqInfo.SourceAddress = r.RemoteAddr
 
 		// Reject internal queries to remote, non-cluster, clients
-		if version == "internal" && !shared.ValueInSlice(protocol, []string{auth.AuthenticationMethodUnix, auth.AuthenticationMethodCluster}) {
+		if version == "internal" && !slices.Contains([]string{auth.AuthenticationMethodUnix, auth.AuthenticationMethodCluster}, protocol) {
 			// Except for the initial cluster accept request (done over trusted TLS)
 			if !trusted || c.Path != "cluster/accept" || protocol != api.AuthenticationMethodTLS {
 				logger.Warn("Rejecting remote internal API request", logger.Ctx{"ip": r.RemoteAddr})
@@ -817,18 +836,19 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 			logger.Debug("Handling API request", logCtx)
 
 			// Add authentication/authorization context data.
-			ctx := context.WithValue(r.Context(), request.CtxUsername, username)
-			ctx = context.WithValue(ctx, request.CtxProtocol, protocol)
+			reqInfo.Username = username
+			reqInfo.Protocol = protocol
 			if len(identityProviderGroups) > 0 {
-				ctx = context.WithValue(ctx, request.CtxIdentityProviderGroups, identityProviderGroups)
+				reqInfo.IdentityProviderGroups = identityProviderGroups
 			}
 
 			// Add forwarded requestor data.
 			if protocol == auth.AuthenticationMethodCluster {
 				// Add authentication/authorization context data.
-				ctx = context.WithValue(ctx, request.CtxForwardedAddress, r.Header.Get(request.HeaderForwardedAddress))
-				ctx = context.WithValue(ctx, request.CtxForwardedUsername, r.Header.Get(request.HeaderForwardedUsername))
-				ctx = context.WithValue(ctx, request.CtxForwardedProtocol, r.Header.Get(request.HeaderForwardedProtocol))
+				reqInfo.ForwardedAddress = r.Header.Get(request.HeaderForwardedAddress)
+				reqInfo.ForwardedUsername = r.Header.Get(request.HeaderForwardedUsername)
+				reqInfo.ForwardedProtocol = r.Header.Get(request.HeaderForwardedProtocol)
+
 				forwardedIdentityProviderGroupsJSON := r.Header.Get(request.HeaderForwardedIdentityProviderGroups)
 				if forwardedIdentityProviderGroupsJSON != "" {
 					var forwardedIdentityProviderGroups []string
@@ -836,12 +856,10 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 					if err != nil {
 						logger.Error("Failed unmarshalling identity provider groups from forwarded request header", logger.Ctx{"err": err})
 					} else {
-						ctx = context.WithValue(ctx, request.CtxForwardedIdentityProviderGroups, forwardedIdentityProviderGroups)
+						reqInfo.ForwardedIdentityProviderGroups = forwardedIdentityProviderGroups
 					}
 				}
 			}
-
-			r = r.WithContext(ctx)
 		} else if untrustedOk && r.Header.Get("X-LXD-authenticated") == "" {
 			logger.Debug("Allowing untrusted "+r.Method, logger.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else {
@@ -855,7 +873,7 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 		}
 
 		// Set OpenFGA cache in request context.
-		request.SetCtxValue(r, request.CtxOpenFGARequestCache, &openfga.RequestCache{})
+		request.SetContextValue(r, request.CtxOpenFGARequestCache, &openfga.RequestCache{})
 
 		// Dump full request JSON when in debug mode
 		if daemon.Debug && r.Method != "GET" && util.IsJSONRequest(r) {
@@ -899,7 +917,7 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 		}
 
 		if d.shutdownCtx.Err() == context.Canceled && !allowedDuringShutdown() {
-			_ = response.Unavailable(fmt.Errorf("LXD is shutting down")).Render(w, r)
+			_ = response.Unavailable(errors.New("LXD is shutting down")).Render(w, r)
 			return
 		}
 
@@ -908,6 +926,27 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 				return response.NotImplemented(nil)
 			}
 
+			// Protect against CSRF when using LXD-UI with browser that supports Fetch metadata.
+			// Deny Sec-Fetch-Site when set to cross-site or same-site.
+			if slices.Contains(secFetchSiteForbidden, r.Header.Get("Sec-Fetch-Site")) {
+				return response.ErrorResponse(http.StatusForbidden, "Forbidden Sec-Fetch-Site header value")
+			}
+
+			if len(action.ContentTypes) == 0 {
+				// Require application/json if not specified by handler.
+				action.ContentTypes = []string{"application/json"}
+			}
+
+			// Validate browser Content-Type if supplied, or if non-zero Content-Length supplied.
+			if isBrowserClient(r) {
+				contentTypeParts := shared.SplitNTrimSpace(r.Header.Get("Content-Type"), ";", 2, false) // Ignore multi-part boundary part.
+				contentLength := r.Header.Get("Content-Length")
+				hasContentLength := contentLength != "" && contentLength != "0"
+				if (hasContentLength || contentTypeParts[0] != "") && !slices.Contains(action.ContentTypes, contentTypeParts[0]) {
+					return response.ErrorResponse(http.StatusUnsupportedMediaType, "Unsupported Content-Type for this request")
+				}
+			}
+
 			// All APIEndpointActions should have an access handler or should allow untrusted requests.
 			if action.AccessHandler == nil && !action.AllowUntrusted {
 				return response.InternalError(fmt.Errorf("Access handler not defined for %s %s", r.Method, r.URL.RequestURI()))
@@ -1005,18 +1044,7 @@ func setupSharedMounts() error {
 func (d *Daemon) Init() error {
 	d.startTime = time.Now()
 
-	err := d.init()
-
-	// If an error occurred synchronously while starting up, let's try to
-	// cleanup any state we produced so far. Errors happening here will be
-	// ignored.
-	if err != nil {
-		logger.Error("Failed to start the daemon", logger.Ctx{"err": err})
-		_ = d.Stop(context.Background(), unix.SIGINT)
-		return err
-	}
-
-	return nil
+	return d.init()
 }
 
 func (d *Daemon) setupLoki(URL string, cert string, key string, caCert string, instanceName string, logLevel string, labels []string, types []string) error {
@@ -1065,6 +1093,9 @@ func (d *Daemon) setupLoki(URL string, cert string, key string, caCert string, i
 }
 
 func (d *Daemon) init() error {
+	d.startStopLock.Lock()
+	defer d.startStopLock.Unlock()
+
 	var err error
 
 	var dbWarnings []dbCluster.Warning
@@ -1082,7 +1113,7 @@ func (d *Daemon) init() error {
 	d.internalListener = events.NewInternalListener(d.shutdownCtx, d.events)
 
 	// Lets check if there's an existing LXD running
-	err = endpoints.CheckAlreadyRunning(d.UnixSocket())
+	err = endpoints.CheckAlreadyRunning(d.os.GetUnixSocket())
 	if err != nil {
 		return err
 	}
@@ -1155,7 +1186,7 @@ func (d *Daemon) init() error {
 	}
 
 	// Look for kernel features
-	logger.Infof("Kernel features:")
+	logger.Info("Kernel features:")
 
 	d.os.CloseRange = canUseCloseRange()
 	if d.os.CloseRange {
@@ -1241,6 +1272,13 @@ func (d *Daemon) init() error {
 		logger.Info(" - unprivileged binfmt_misc: no")
 	}
 
+	d.os.BPFToken = canUseBPFToken()
+	if d.os.BPFToken {
+		logger.Info(" - BPF Token: yes")
+	} else {
+		logger.Info(" - BPF Token: no")
+	}
+
 	/*
 	 * During daemon startup we're the only thread that touches VFS3Fscaps
 	 * so we don't need to bother with atomic.StoreInt32() when touching
@@ -1249,10 +1287,10 @@ func (d *Daemon) init() error {
 	d.os.VFS3Fscaps = idmap.SupportsVFS3Fscaps("")
 	if d.os.VFS3Fscaps {
 		idmap.VFS3Fscaps = idmap.VFS3FscapsSupported
-		logger.Infof(" - unprivileged file capabilities: yes")
+		logger.Info(" - unprivileged file capabilities: yes")
 	} else {
 		idmap.VFS3Fscaps = idmap.VFS3FscapsUnsupported
-		logger.Infof(" - unprivileged file capabilities: no")
+		logger.Info(" - unprivileged file capabilities: no")
 	}
 
 	dbWarnings = append(dbWarnings, d.os.CGInfo.Warnings()...)
@@ -1354,9 +1392,28 @@ func (d *Daemon) init() error {
 		}
 	}
 
+	// Load local config (must come after processing incoming recovery tarball as it can update local config).
+	logger.Info("Loading daemon configuration")
+	err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		d.localConfig, err = node.ConfigLoad(ctx, tx)
+		return err
+	})
+	if err != nil {
+		return err
+	}
+
+	localHTTPAddress := d.localConfig.HTTPSAddress()
+	localClusterAddress := d.localConfig.ClusterAddress()
+	debugAddress := d.localConfig.DebugAddress()
+
+	// Sense check for clustering mode.
+	if localClusterAddress == "" && d.serverClustered {
+		return errors.New("Server is clustered (has local raft addresses) but cluster.https_address is not set")
+	}
+
 	/* Setup dqlite */
 	clusterLogLevel := "ERROR"
-	if shared.ValueInSlice("dqlite", trace) {
+	if slices.Contains(trace, "dqlite") {
 		clusterLogLevel = "TRACE"
 	}
 
@@ -1381,40 +1438,27 @@ func (d *Daemon) init() error {
 			logger.Warn("Failed setting up shared mounts", logger.Ctx{"err": err})
 		}
 
-		// Attempt to Mount the devlxd tmpfs
-		devlxd := filepath.Join(d.os.VarDir, "devlxd")
-		if !filesystem.IsMountPoint(devlxd) {
-			err = unix.Mount("tmpfs", devlxd, "tmpfs", 0, "size=100k,mode=0755")
+		// Attempt to Mount the devLXD tmpfs
+		devLXD := filepath.Join(d.os.VarDir, "devlxd")
+		if !filesystem.IsMountPoint(devLXD) {
+			err = unix.Mount("tmpfs", devLXD, "tmpfs", 0, "size=100k,mode=0755")
 			if err != nil {
-				logger.Warn("Failed to mount devlxd", logger.Ctx{"err": err})
+				logger.Warn("Failed to mount devLXD", logger.Ctx{"err": err})
 			}
 		}
 	}
 
-	logger.Info("Loading daemon configuration")
-	err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
-		d.localConfig, err = node.ConfigLoad(ctx, tx)
-		return err
-	})
-	if err != nil {
-		return err
-	}
-
-	localHTTPAddress := d.localConfig.HTTPSAddress()
-	localClusterAddress := d.localConfig.ClusterAddress()
-	debugAddress := d.localConfig.DebugAddress()
-
 	if os.Getenv("LISTEN_PID") != "" {
 		d.systemdSocketActivated = true
 	}
 
 	/* Setup the web server */
-	config := &endpoints.Config{
+	endpointsConfig := &endpoints.Config{
 		Dir:                  d.os.VarDir,
-		UnixSocket:           d.UnixSocket(),
+		UnixSocket:           d.os.GetUnixSocket(),
 		Cert:                 networkCert,
 		RestServer:           restServer(d),
-		DevLxdServer:         devLxdServer(d),
+		DevLxdServer:         devLXDServer(d),
 		LocalUnixSocketGroup: d.config.Group,
 		NetworkAddress:       localHTTPAddress,
 		ClusterAddress:       localClusterAddress,
@@ -1428,10 +1472,10 @@ func (d *Daemon) init() error {
 	// Enable vsock server support if VM instances supported.
 	err, found := d.State().InstanceTypes[instancetype.VM]
 	if found && err == nil {
-		config.VsockSupport = true
+		endpointsConfig.VsockSupport = true
 	}
 
-	d.endpoints, err = endpoints.Up(config)
+	d.endpoints, err = endpoints.Up(endpointsConfig)
 	if err != nil {
 		return err
 	}
@@ -1462,34 +1506,48 @@ func (d *Daemon) init() error {
 			driver.WithLogFunc(cluster.DqliteLog),
 		}
 
-		if shared.ValueInSlice("database", trace) {
+		if slices.Contains(trace, "database") {
 			options = append(options, driver.WithTracing(dqliteClient.LogDebug))
 		}
 
-		d.db.Cluster, err = db.OpenCluster(context.Background(), "db.bin", store, localClusterAddress, dir, d.config.DqliteSetupTimeout, nil, options...)
+		// Assign cluster DB handle to d.gateway.Cluster so its immediately usable by gateway even if DB
+		// returns StatusPreconditionFailed. This way its usable for heartbeats whilst it waits for the
+		// other members to become aligned.
+		d.gateway.Cluster, err = db.OpenCluster(d.shutdownCtx, "db.bin", store, localClusterAddress, dir, d.config.DqliteSetupTimeout, nil, options...)
 		if err == nil {
 			logger.Info("Initialized global database")
+
+			// If cluster DB handle is established without issue, make available to the rest of LXD.
+			d.db.Cluster = d.gateway.Cluster
 			break
-		} else if errors.Is(err, db.ErrSomeNodesAreBehind) {
+		} else if api.StatusErrorCheck(err, http.StatusPreconditionFailed) {
 			// If some other nodes have schema or API versions less recent
 			// than this node, we block until we receive a notification
 			// from the last node being upgraded that everything should be
 			// now fine, and then retry
-			logger.Warn("Wait for other cluster nodes to upgrade their versions, cluster not started yet")
+			logger.Warn("Wait for other cluster members to align their versions, cluster not started yet")
 
 			// The only thing we want to still do on this node is
 			// to run the heartbeat task, in case we are the raft
 			// leader.
-			d.gateway.Cluster = d.db.Cluster
 			taskFunc, taskSchedule := cluster.HeartbeatTask(d.gateway)
 			hbGroup := task.NewGroup()
 			d.taskClusterHeartbeat = hbGroup.Add(taskFunc, taskSchedule)
 			hbGroup.Start(d.shutdownCtx)
-			d.gateway.WaitUpgradeNotification()
-			_ = hbGroup.Stop(time.Second)
-			d.gateway.Cluster = nil
 
-			_ = d.db.Cluster.Close()
+			{
+				// Wait for refresh notification from other members.
+				waitNotificationCtx, cancel := context.WithTimeout(d.shutdownCtx, time.Minute)
+				d.gateway.WaitUpgradeNotification(waitNotificationCtx)
+				cancel()
+			}
+
+			_ = hbGroup.Stop(time.Second)
+			_ = d.gateway.Cluster.Close()
+
+			d.gateway.HeartbeatLock.Lock()
+			d.gateway.Cluster = nil
+			d.gateway.HeartbeatLock.Unlock()
 
 			continue
 		}
@@ -1515,28 +1573,6 @@ func (d *Daemon) init() error {
 		logger.Warn("Could not notify all nodes of database upgrade", logger.Ctx{"err": err})
 	}
 
-	d.gateway.Cluster = d.db.Cluster
-
-	// This logic used to belong to patchUpdateFromV10, but has been moved
-	// here because it needs database access.
-	if shared.PathExists(shared.VarPath("lxc")) {
-		err := os.Rename(shared.VarPath("lxc"), shared.VarPath("containers"))
-		if err != nil {
-			return err
-		}
-
-		logger.Debug("Restarting all the containers following directory rename")
-
-		s := d.State()
-		instances, err := instance.LoadNodeAll(s, instancetype.Container)
-		if err != nil {
-			return fmt.Errorf("Failed loading containers to restart: %w", err)
-		}
-
-		instancesShutdown(instances)
-		instancesStart(s, instances)
-	}
-
 	// Setup the user-agent.
 	if d.serverClustered {
 		version.UserAgentFeatures([]string{"cluster"})
@@ -1550,7 +1586,7 @@ func (d *Daemon) init() error {
 
 	// Load server name and config before patches run (so they can access them from d.State()).
 	err = d.db.Cluster.Transaction(d.shutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
-		config, err := clusterConfig.Load(ctx, tx)
+		globalConfig, err := clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
@@ -1563,7 +1599,7 @@ func (d *Daemon) init() error {
 
 		d.globalConfigMu.Lock()
 		d.serverName = serverName
-		d.globalConfig = config
+		d.globalConfig = globalConfig
 		d.globalConfigMu.Unlock()
 
 		return nil
@@ -1598,7 +1634,7 @@ func (d *Daemon) init() error {
 	d.serverUUID = serverUUID
 
 	// Mount the storage pools.
-	logger.Infof("Initializing storage pools")
+	logger.Info("Initializing storage pools")
 	err = storageStartup(d.State())
 	if err != nil {
 		return err
@@ -1611,14 +1647,14 @@ func (d *Daemon) init() error {
 	}
 
 	// Mount any daemon storage volumes.
-	logger.Infof("Initializing daemon storage mounts")
+	logger.Info("Initializing daemon storage mounts")
 	err = daemonStorageMount(d.State())
 	if err != nil {
 		return err
 	}
 
 	// Create directories on daemon storage mounts.
-	err = d.os.InitStorage()
+	err = d.os.InitStorage(d.localConfig)
 	if err != nil {
 		return err
 	}
@@ -1672,7 +1708,7 @@ func (d *Daemon) init() error {
 	maasAPIURL, maasAPIKey = d.globalConfig.MAASController()
 	d.gateway.HeartbeatOfflineThreshold = d.globalConfig.OfflineThreshold()
 	lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := d.globalConfig.LokiServer()
-	oidcIssuer, oidcClientID, oidcScopes, oidcAudience, oidcGroupsClaim := d.globalConfig.OIDCServer()
+	oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, oidcGroupsClaim := d.globalConfig.OIDCServer()
 	syslogSocketEnabled := d.localConfig.SyslogSocket()
 	instancePlacementScriptlet := d.globalConfig.InstancesPlacementScriptlet()
 
@@ -1683,7 +1719,7 @@ func (d *Daemon) init() error {
 	if lokiURL != "" {
 		err = d.setupLoki(lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes)
 		if err != nil {
-			return err
+			logger.Warn("Failed to setup Loki", logger.Ctx{"err": err})
 		}
 	}
 
@@ -1700,7 +1736,7 @@ func (d *Daemon) init() error {
 			return util.HTTPClient("", d.proxy)
 		}
 
-		d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcScopes, oidcAudience, d.serverCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
+		d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, d.serverCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
 		if err != nil {
 			return err
 		}
@@ -1708,18 +1744,6 @@ func (d *Daemon) init() error {
 
 	// Setup BGP listener.
 	d.bgp = bgp.NewServer()
-	if bgpAddress != "" && bgpASN != 0 && bgpRouterID != "" {
-		if bgpASN > math.MaxUint32 {
-			return fmt.Errorf("Cannot convert BGP ASN to uint32: Upper bound exceeded")
-		}
-
-		err := d.bgp.Start(bgpAddress, uint32(bgpASN), net.ParseIP(bgpRouterID))
-		if err != nil {
-			return err
-		}
-
-		logger.Info("Started BGP server")
-	}
 
 	// Setup DNS listener.
 	d.dns = dns.NewServer(d.db.Cluster, func(name string, full bool) (*dns.Zone, error) {
@@ -1759,16 +1783,27 @@ func (d *Daemon) init() error {
 	})
 
 	// Setup the networks.
-	if !d.db.Cluster.LocalNodeIsEvacuated() {
-		logger.Infof("Initializing networks")
+	logger.Info("Initializing networks")
 
-		err = networkStartup(d.State)
-		if err != nil {
-			return err
-		}
+	err = networkStartup(d.State, false)
+	if err != nil {
+		return err
 	}
 
 	// Setup tertiary listeners that may use managed network addresses and must be started after networks.
+	if bgpAddress != "" && bgpASN != 0 && bgpRouterID != "" {
+		if bgpASN > math.MaxUint32 {
+			return errors.New("Cannot convert BGP ASN to uint32: Upper bound exceeded")
+		}
+
+		err := d.bgp.Configure(bgpAddress, uint32(bgpASN), net.ParseIP(bgpRouterID))
+		if err != nil {
+			return err
+		}
+
+		logger.Info("Started BGP server")
+	}
+
 	dnsAddress := d.localConfig.DNSAddress()
 	if dnsAddress != "" {
 		err = d.dns.Start(dnsAddress)
@@ -1843,7 +1878,13 @@ func (d *Daemon) init() error {
 		// Setup seccomp handler
 		if d.os.SeccompListener {
 			seccompServer, err := seccomp.NewSeccompServer(d.State(), shared.VarPath("seccomp.socket"), func(pid int32, state *state.State) (seccomp.Instance, error) {
-				return findContainerForPid(pid, state)
+				c, _, err := getLXCMonitorContainer(state, pid)
+				if err != nil || c == nil {
+					logger.Warn("Could not match PID to container for seccomp", logger.Ctx{"pid": pid, "err": err})
+					return nil, errPIDNotInContainer // Don't return error to avoid leaking details about the process.
+				}
+
+				return c, nil
 			})
 			if err != nil {
 				return err
@@ -1937,34 +1978,34 @@ func (d *Daemon) init() error {
 	//        but has not been fully completed.
 	if !d.os.MockMode {
 		// Log expiry (daily)
-		d.tasks.Add(expireLogsTask(d.State()))
+		d.tasks.Add(expireLogsTask(d.State))
 
 		// Remove expired images (daily)
-		d.taskPruneImages = d.tasks.Add(pruneExpiredImagesTask(d))
+		d.taskPruneImages = d.tasks.Add(pruneExpiredImagesTask(d.State))
 
 		// Auto-update images (every 6 hours, configurable)
-		d.tasks.Add(autoUpdateImagesTask(d))
+		d.tasks.Add(autoUpdateImagesTask(d.State))
 
 		// Auto-update instance types (daily)
-		d.tasks.Add(instanceRefreshTypesTask(d))
+		d.tasks.Add(instanceRefreshTypesTask(d.State))
 
 		// Remove expired backups (hourly)
-		d.tasks.Add(pruneExpiredBackupsTask(d))
+		d.tasks.Add(pruneExpiredBackupsTask(d.State))
 
 		// Prune expired instance snapshots and take snapshot of instances (minutely check of configurable cron expression)
-		d.tasks.Add(pruneExpiredAndAutoCreateInstanceSnapshotsTask(d))
+		d.tasks.Add(pruneExpiredAndAutoCreateInstanceSnapshotsTask(d.State))
 
 		// Prune expired custom volume snapshots and take snapshots of custom volumes (minutely check of configurable cron expression)
-		d.tasks.Add(pruneExpiredAndAutoCreateCustomVolumeSnapshotsTask(d))
+		d.tasks.Add(pruneExpiredAndAutoCreateCustomVolumeSnapshotsTask(d.State))
 
 		// Remove resolved warnings (daily)
-		d.tasks.Add(pruneResolvedWarningsTask(d))
+		d.tasks.Add(pruneResolvedWarningsTask(d.State))
 
 		// Auto-renew server certificate (daily)
 		d.tasks.Add(autoRenewCertificateTask(d))
 
 		// Remove expired tokens (hourly)
-		d.tasks.Add(autoRemoveExpiredTokensTask(d))
+		d.tasks.Add(autoRemoveExpiredTokensTask(d.State))
 	}
 
 	// Start all background tasks
@@ -1998,13 +2039,13 @@ func (d *Daemon) startClusterTasks() {
 	d.taskClusterHeartbeat = d.clusterTasks.Add(cluster.HeartbeatTask(d.gateway))
 
 	// Auto-sync images across the cluster (hourly)
-	d.clusterTasks.Add(autoSyncImagesTask(d))
+	d.clusterTasks.Add(autoSyncImagesTask(d.State))
 
 	// Remove orphaned operations
-	d.clusterTasks.Add(autoRemoveOrphanedOperationsTask(d))
+	d.clusterTasks.Add(autoRemoveOrphanedOperationsTask(d.State))
 
 	// Perform automatic evacuation for offline cluster members
-	d.clusterTasks.Add(autoHealClusterTask(d))
+	d.clusterTasks.Add(autoHealClusterTask(d.State))
 
 	// Start all background tasks
 	d.clusterTasks.Start(d.shutdownCtx)
@@ -2027,12 +2068,38 @@ func (d *Daemon) numRunningInstances(instances []instance.Instance) int {
 	return count
 }
 
+// cancelCancelableOps cancels all running cancelable operations.
+func cancelCancelableOps() error {
+	ops := operations.Clone()
+	for _, op := range ops {
+		if op.Status() != api.Running || op.Class() == operations.OperationClassToken {
+			continue
+		}
+
+		_, opAPI, err := op.Render()
+		if err != nil {
+			return fmt.Errorf("Failed to render an operation while attempting to stop cancelable operations: %w", err)
+		}
+
+		if opAPI.MayCancel {
+			_, _ = op.Cancel()
+		}
+	}
+
+	return nil
+}
+
 // Stop stops the shared daemon.
 func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
+	// Cancelling the context will make everyone aware that we're shutting down.
+	d.shutdownCtx.Cancel()
+
+	d.startStopLock.Lock()
+	defer d.startStopLock.Unlock()
+
 	logger.Info("Starting shutdown sequence", logger.Ctx{"signal": sig})
 
-	// Cancelling the context will make everyone aware that we're shutting down.
-	d.shutdownCancel()
+	s := d.State()
 
 	if d.gateway != nil {
 		d.stopClusterTasks()
@@ -2044,8 +2111,6 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 		}
 	}
 
-	s := d.State()
-
 	// Stop any running minio processes cleanly before unmount storage pools.
 	miniod.StopAll()
 
@@ -2075,67 +2140,64 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 
 	// Handle shutdown (unix.SIGPWR) and reload (unix.SIGTERM) signals.
 	if sig == unix.SIGPWR || sig == unix.SIGTERM {
-		if d.db.Cluster != nil {
-			// waitForOperations will block until all operations are done, or it's forced to shut down.
-			// For the latter case, we re-use the shutdown channel which is filled when a shutdown is
-			// initiated using `lxd shutdown`.
-			waitForOperations(ctx, d.db.Cluster, s.GlobalConfig.ShutdownTimeout())
-		}
+		// Full shutdown requested.
+		if sig == unix.SIGPWR {
+			{
+				logger.Debug("Shutting down instances")
+				var instOperationWaitCtx context.Context
+				var cancel context.CancelFunc
+				if s.GlobalConfig != nil {
+					instOperationWaitCtx, cancel = context.WithTimeout(ctx, s.GlobalConfig.ShutdownTimeout())
+					defer cancel()
+				} else {
+					instOperationWaitCtx, cancel = context.WithCancel(ctx)
+					cancel() // Don't wait for operations to finish.
+				}
 
-		// Unmount daemon image and backup volumes if set.
-		logger.Info("Stopping daemon storage volumes")
-		done := make(chan struct{})
-		go func() {
-			err := daemonStorageVolumesUnmount(s)
+				instancesShutdown(instOperationWaitCtx, instances)
+			}
+
+			if d.db.Cluster != nil {
+				// Try to cancel any cancelable operations.
+				err = cancelCancelableOps()
+				if err != nil {
+					logger.Error("Failed to cancel cancelable operations", logger.Ctx{"err": err})
+				}
+			}
+
+			// Stop networks.
+			networkStop(s, false)
+
+			// Unmount daemon image and backup volumes if set.
+			logger.Info("Stopping daemon storage volumes")
+			logger.Debug("Unmounting daemon storage volumes")
+			volUnmountCtx, cancel := context.WithTimeout(context.Background(), time.Minute)
+			defer cancel()
+			err := daemonStorageVolumesUnmount(s, volUnmountCtx)
 			if err != nil {
 				logger.Error("Failed to unmount image and backup volumes", logger.Ctx{"err": err})
 			}
 
-			done <- struct{}{}
-		}()
+			logger.Debug("Daemon storage volumes unmounted")
 
-		// Only wait 60 seconds in case the storage backend is unreachable.
-		select {
-		case <-time.After(time.Minute):
-			logger.Error("Timed out waiting for image and backup volume")
-		case <-done:
+			// Unmount storage pools after instances stopped and images/backup volumes unmounted.
+			storageStop(s)
 		}
 
-		// Full shutdown requested.
-		if sig == unix.SIGPWR {
-			instancesShutdown(instances)
-
-			logger.Info("Stopping networks")
-			networkShutdown(s)
-
-			// Unmount storage pools after instances stopped.
-			logger.Info("Stopping storage pools")
-
-			var pools []string
-
+		if d.db.Cluster != nil {
+			// Remove remaining operations before closing the database.
 			err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-				var err error
+				err := dbCluster.DeleteOperations(ctx, tx.Tx(), s.DB.Cluster.GetNodeID())
+				if err != nil {
+					logger.Error("Failed cleaning up operations")
+				}
 
-				pools, err = tx.GetStoragePoolNames(ctx)
-
-				return err
+				return nil
 			})
-			if err != nil && !response.IsNotFoundError(err) {
-				logger.Error("Failed to get storage pools", logger.Ctx{"err": err})
-			}
-
-			for _, poolName := range pools {
-				pool, err := storagePools.LoadByName(s, poolName)
-				if err != nil {
-					logger.Error("Failed to get storage pool", logger.Ctx{"pool": poolName, "err": err})
-					continue
-				}
-
-				_, err = pool.Unmount()
-				if err != nil {
-					logger.Error("Unable to unmount storage pool", logger.Ctx{"pool": poolName, "err": err})
-					continue
-				}
+			if err != nil {
+				logger.Error("Failed cleaning up operations", logger.Ctx{"err": err})
+			} else {
+				logger.Debug("Operations deleted from the database")
 			}
 		}
 	}
@@ -2161,7 +2223,7 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 		logger.Info("Closing the database")
 		err := d.db.Cluster.Close()
 		if err != nil {
-			logger.Debug("Could not close global database cleanly", logger.Ctx{"err": err})
+			logger.Warn("Could not close global database cleanly", logger.Ctx{"err": err})
 		}
 	}
 
@@ -2192,6 +2254,8 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 		trackError(d.seccomp.Stop(), "Stop seccomp")
 	}
 
+	trackError(filesystem.SyncFS(filepath.Join(d.os.VarDir, "database")), "Sync database directory")
+
 	n = len(errs)
 	if n > 0 {
 		format := "%v"
@@ -2517,7 +2581,7 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 		if isDegraded || onlineVoters < maxVoters || onlineStandbys < maxStandBy {
 			d.clusterMembershipMutex.Lock()
 			logger.Debug("Rebalancing member roles in heartbeat", logger.Ctx{"local": localClusterAddress})
-			err := rebalanceMemberRoles(d.State(), d.gateway, nil, unavailableMembers)
+			err := rebalanceMemberRoles(context.Background(), d.State(), d.gateway, unavailableMembers)
 			if err != nil && !errors.Is(err, cluster.ErrNotLeader) {
 				logger.Warn("Could not rebalance cluster member roles", logger.Ctx{"err": err, "local": localClusterAddress})
 			}
