@@ -2,6 +2,7 @@ package main
 
 import (
 	"context"
+	"maps"
 
 	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
 	"github.com/canonical/lxd/lxd/db"
@@ -14,20 +15,16 @@ func daemonConfigRender(state *state.State) (map[string]any, error) {
 	config := map[string]any{}
 
 	// Turn the config into a JSON-compatible map.
-	for key, value := range state.GlobalConfig.Dump() {
-		config[key] = value
-	}
+	maps.Copy(config, state.GlobalConfig.Dump())
 
 	// Apply the local config.
-	err := state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+	err := state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
 		}
 
-		for key, value := range nodeConfig.Dump() {
-			config[key] = value
-		}
+		maps.Copy(config, nodeConfig.Dump())
 
 		return nil
 	})
