@@ -3,11 +3,14 @@ package main
 import (
 	"context"
 	"crypto/sha256"
+	"encoding/hex"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	"os"
 	"path/filepath"
+	"slices"
 	"time"
 
 	"github.com/canonical/lxd/client"
@@ -54,14 +57,14 @@ func imageOperationLock(fingerprint string) (locking.UnlockFunc, error) {
 	l.Debug("Acquiring lock for image")
 	defer l.Debug("Lock acquired for image")
 
-	return locking.Lock(context.TODO(), fmt.Sprintf("ImageOperation_%s", fingerprint))
+	return locking.Lock(context.TODO(), "ImageOperation_"+fingerprint)
 }
 
 // ImageDownload resolves the image fingerprint and if not in the database, downloads it.
-func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, args *ImageDownloadArgs) (*api.Image, error) {
-	var err error
-	var ctxMap logger.Ctx
+func ImageDownload(ctx context.Context, s *state.State, op *operations.Operation, args *ImageDownloadArgs) (*api.Image, error) {
+	l := logger.AddContext(logger.Ctx{"image": args.Alias, "member": s.ServerName, "project": args.ProjectName, "pool": args.StoragePool, "source": args.Server})
 
+	var err error
 	var remote lxd.ImageServer
 	var info *api.Image
 
@@ -78,7 +81,7 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 	fp := alias
 
 	// Attempt to resolve the alias
-	if shared.ValueInSlice(protocol, []string{"lxd", "simplestreams"}) {
+	if slices.Contains([]string{"lxd", "simplestreams"}, protocol) {
 		clientArgs := &lxd.ConnectionArgs{
 			TLSServerCert: args.Certificate,
 			UserAgent:     version.UserAgent,
@@ -179,7 +182,7 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 
 		if nodeAddress != "" {
 			// The image is available from another node, let's try to import it.
-			err = instanceImageTransfer(s, r, args.ProjectName, imgInfo.Fingerprint, nodeAddress)
+			err = instanceImageTransfer(ctx, s, args.ProjectName, imgInfo.Fingerprint, nodeAddress)
 			if err != nil {
 				return nil, fmt.Errorf("Failed transferring image %q from %q: %w", imgInfo.Fingerprint, nodeAddress, err)
 			}
@@ -240,7 +243,7 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 			// Transfer image if needed (after database record has been created above).
 			if nodeAddress != "" {
 				// The image is available from another node, let's try to import it.
-				err = instanceImageTransfer(s, r, args.ProjectName, info.Fingerprint, nodeAddress)
+				err = instanceImageTransfer(ctx, s, args.ProjectName, info.Fingerprint, nodeAddress)
 				if err != nil {
 					return nil, fmt.Errorf("Failed transferring image: %w", err)
 				}
@@ -250,8 +253,8 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 
 	if imgInfo != nil {
 		info = imgInfo
-		ctxMap = logger.Ctx{"fingerprint": info.Fingerprint}
-		logger.Debug("Image already exists in the DB", ctxMap)
+		l = l.AddContext(logger.Ctx{"fingerprint": info.Fingerprint, "autoUpdate": info.AutoUpdate, "imgProject": info.Project})
+		l.Debug("Image already exists in the DB")
 
 		var poolID int64
 		var poolIDs []int64
@@ -267,8 +270,6 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 			}
 
 			if args.StoragePool != "" {
-				ctxMap["pool"] = args.StoragePool
-
 				// Get the ID of the target storage pool.
 				poolID, err = tx.GetStoragePoolID(ctx, args.StoragePool)
 				if err != nil {
@@ -292,36 +293,33 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 			return info, nil
 		}
 
-		if shared.ValueInSlice(poolID, poolIDs) {
-			logger.Debug("Image already exists on storage pool", ctxMap)
+		if slices.Contains(poolIDs, poolID) {
+			l.Debug("Image already exists on storage pool")
 			return info, nil
 		}
 
 		// Import the image in the pool.
-		logger.Debug("Image does not exist on storage pool", ctxMap)
+		l.Debug("Image does not exist on storage pool")
 
 		err = imageCreateInPool(s, info, args.StoragePool)
 		if err != nil {
-			ctxMap["err"] = err
-			logger.Debug("Failed to create image on storage pool", ctxMap)
+			l.Debug("Failed to create image on storage pool", logger.Ctx{"err": err})
 			return nil, fmt.Errorf("Failed to create image %q on storage pool %q: %w", info.Fingerprint, args.StoragePool, err)
 		}
 
-		logger.Debug("Created image on storage pool", ctxMap)
+		l.Debug("Created image on storage pool")
 		return info, nil
 	}
 
 	// Begin downloading
-	if op == nil {
-		ctxMap = logger.Ctx{"alias": alias, "server": args.Server}
-	} else {
-		ctxMap = logger.Ctx{"trigger": op.URL(), "fingerprint": fp, "operation": op.ID(), "alias": alias, "server": args.Server}
+	if op != nil {
+		l = l.AddContext(logger.Ctx{"trigger": op.URL(), "operation": op.ID()})
 	}
 
-	logger.Info("Downloading image", ctxMap)
+	l.Info("Downloading image")
 
 	// Cleanup any leftover from a past attempt
-	destDir := shared.VarPath("images")
+	destDir := s.ImagesStoragePath()
 	destName := filepath.Join(destDir, fp)
 
 	failure := true
@@ -356,7 +354,8 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		op.SetCanceler(canceler)
 	}
 
-	if protocol == "lxd" || protocol == "simplestreams" {
+	switch protocol {
+	case "lxd", "simplestreams":
 		// Create the target files
 		dest, err := os.Create(destName)
 		if err != nil {
@@ -408,7 +407,7 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 			ProgressHandler: progress,
 			Canceler:        canceler,
 			DeltaSourceRetriever: func(fingerprint string, file string) string {
-				path := shared.VarPath("images", fmt.Sprintf("%s.%s", fingerprint, file))
+				path := filepath.Join(destDir, fingerprint+"."+file)
 				if shared.PathExists(path) {
 					return path
 				}
@@ -457,7 +456,8 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		if err != nil {
 			return nil, err
 		}
-	} else if protocol == "direct" {
+
+	case "direct":
 		// Setup HTTP client
 		httpClient, err := util.HTTPClient(args.Certificate, s.Proxy)
 		if err != nil {
@@ -467,12 +467,12 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		// Use relatively short response header timeout so as not to hold the image lock open too long.
 		httpTransport, ok := httpClient.Transport.(*http.Transport)
 		if !ok {
-			return nil, fmt.Errorf("Invalid http client type")
+			return nil, errors.New("Invalid http client type")
 		}
 
 		httpTransport.ResponseHeaderTimeout = 30 * time.Second
 
-		req, err := http.NewRequest("GET", args.Server, nil)
+		req, err := http.NewRequest(http.MethodGet, args.Server, nil)
 		if err != nil {
 			return nil, err
 		}
@@ -521,7 +521,7 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		}
 
 		// Validate hash
-		result := fmt.Sprintf("%x", sha256.Sum(nil))
+		result := hex.EncodeToString(sha256.Sum(nil))
 		if result != fp {
 			return nil, fmt.Errorf("Hash mismatch for %q: %s != %s", args.Server, result, fp)
 		}
@@ -545,7 +545,8 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		if err != nil {
 			return nil, err
 		}
-	} else {
+
+	default:
 		return nil, fmt.Errorf("Unsupported protocol: %v", protocol)
 	}
 
@@ -619,13 +620,13 @@ func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, ar
 		}
 	}
 
-	logger.Info("Image downloaded", ctxMap)
+	l.Info("Image downloaded")
 
 	var requestor *api.EventLifecycleRequestor
 	if op != nil {
 		requestor = op.Requestor()
-	} else if r != nil {
-		requestor = request.CreateRequestor(r)
+	} else if request.IsRequestContext(ctx) {
+		requestor = request.CreateRequestor(ctx)
 	}
 
 	s.Events.SendLifecycle(args.ProjectName, lifecycle.ImageCreated.Event(info.Fingerprint, args.ProjectName, requestor, logger.Ctx{"type": info.Type}))
