@@ -9,15 +9,15 @@ import (
 	"github.com/stretchr/testify/require"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/sys"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/sys"
 )
 
 // The daemon is started and a client can connect to it via unix socket.
 func TestIntegration_UnixSocket(t *testing.T) {
 	daemon, cleanup := newTestDaemon(t)
 	defer cleanup()
-	client, err := lxd.ConnectLXDUnix(daemon.UnixSocket(), nil)
+	client, err := lxd.ConnectLXDUnix(daemon.os.GetUnixSocket(), nil)
 	require.NoError(t, err)
 
 	server, _, err := client.GetServer()
