@@ -8,9 +8,10 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
+	"slices"
 	"strings"
 
-	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 )
@@ -138,33 +139,24 @@ func init() {
 	}
 }
 
-// Scan implements sql.Scanner for EntityType. This converts the integer value back into the correct entity.Type
-// constant or returns an error.
-func (e *EntityType) Scan(value any) error {
-	// Always expect null values to be coalesced into entityTypeNone (-1).
-	if value == nil {
-		return fmt.Errorf("Entity type cannot be null")
-	}
-
-	intValue, err := driver.Int32.ConvertValue(value)
-	if err != nil {
-		return fmt.Errorf("Invalid entity type `%v`: %w", value, err)
-	}
-
-	entityTypeInt, ok := intValue.(int64)
+// ScanInteger implements [query.IntegerScanner] for EntityType. This simplifies the Scan implementation.
+func (e *EntityType) ScanInteger(entityTypeCode int64) error {
+	entityType, ok := entityTypeByCode[entityTypeCode]
 	if !ok {
-		return fmt.Errorf("Entity should be an integer, got `%v` (%T)", intValue, intValue)
-	}
-
-	entityType, ok := entityTypeByCode[entityTypeInt]
-	if !ok {
-		return fmt.Errorf("Unknown entity type %d", entityTypeInt)
+		return fmt.Errorf("Unknown entity type %d", entityTypeCode)
 	}
 
 	*e = entityType
 	return nil
 }
 
+// Scan implements sql.Scanner for EntityType. This converts the integer value back into the correct entity.Type
+// constant or returns an error.
+func (e *EntityType) Scan(value any) error {
+	// Always expect null values to be coalesced into entityTypeNone (-1).
+	return query.ScanValue(value, e, false)
+}
+
 // Value implements driver.Valuer for EntityType. This converts the EntityType into an integer or throws an error.
 func (e EntityType) Value() (driver.Value, error) {
 	if e == "" {
@@ -254,7 +246,7 @@ func GetEntityURLs(ctx context.Context, tx *sql.Tx, projectName string, filterin
 	// If the server entity type is in the list of entity types, or if we are getting all entity types and
 	// not filtering by project, we need to add a server URL to the result. The entity ID of the server entity type is
 	// always zero.
-	if shared.ValueInSlice(entity.TypeServer, filteringEntityTypes) || (len(filteringEntityTypes) == 0 && projectName == "") {
+	if (len(filteringEntityTypes) == 0 && projectName == "") || slices.Contains(filteringEntityTypes, entity.TypeServer) {
 		result[entity.TypeServer] = map[int]*api.URL{0: entity.ServerURL()}
 
 		// Return early if there are no other entity types in the list (no queries to execute).
@@ -432,13 +424,13 @@ func PopulateEntityReferencesFromURLs(ctx context.Context, tx *sql.Tx, entityURL
 		}
 
 		if rowID >= len(entityURLs) {
-			return fmt.Errorf("Failed to get entityIDs from URLS: Internal error, returned row ID greater than number of URLs")
+			return errors.New("Failed to get entityIDs from URLS: Internal error, returned row ID greater than number of URLs")
 		}
 
 		// Using the row ID, get the *api.URL from the argument slice, then use it as a key in our result map to get the *EntityRef.
 		entityRef, ok := entityURLMap[entityURLs[rowID]]
 		if !ok {
-			return fmt.Errorf("Failed to get entityIDs from URLS: Internal error, entity URL missing from result object")
+			return errors.New("Failed to get entityIDs from URLS: Internal error, entity URL missing from result object")
 		}
 
 		// Set the value of the EntityID in the *EntityRef.
