@@ -68,35 +68,26 @@ const (
 	authMethodOIDC int64 = 2
 )
 
-// Scan implements sql.Scanner for AuthMethod. This converts the integer value back into the correct API constant or
-// returns an error.
-func (a *AuthMethod) Scan(value any) error {
-	if value == nil {
-		return fmt.Errorf("Authentication method cannot be null")
-	}
-
-	intValue, err := driver.Int32.ConvertValue(value)
-	if err != nil {
-		return fmt.Errorf("Invalid authentication method type: %w", err)
-	}
-
-	authMethodInt, ok := intValue.(int64)
-	if !ok {
-		return fmt.Errorf("Authentication method should be an integer, got `%v` (%T)", intValue, intValue)
-	}
-
-	switch authMethodInt {
+// ScanInteger implements [query.IntegerScanner] for AuthMethod. This simplifies the Scan implementation.
+func (a *AuthMethod) ScanInteger(authMethodCode int64) error {
+	switch authMethodCode {
 	case authMethodTLS:
 		*a = api.AuthenticationMethodTLS
 	case authMethodOIDC:
 		*a = api.AuthenticationMethodOIDC
 	default:
-		return fmt.Errorf("Unknown authentication method `%d`", authMethodInt)
+		return fmt.Errorf("Unknown authentication method `%d`", authMethodCode)
 	}
 
 	return nil
 }
 
+// Scan implements sql.Scanner for AuthMethod. This converts the integer value back into the correct API constant or
+// returns an error.
+func (a *AuthMethod) Scan(value any) error {
+	return query.ScanValue(value, a, false)
+}
+
 // Value implements driver.Valuer for AuthMethod. This converts the API constant into an integer or throws an error.
 func (a AuthMethod) Value() (driver.Value, error) {
 	switch a {
@@ -128,24 +119,9 @@ const (
 	identityTypeCertificateClientPending       int64 = 8
 )
 
-// Scan implements sql.Scanner for IdentityType. This converts the integer value back into the correct API constant or
-// returns an error.
-func (i *IdentityType) Scan(value any) error {
-	if value == nil {
-		return fmt.Errorf("Identity type cannot be null")
-	}
-
-	intValue, err := driver.Int32.ConvertValue(value)
-	if err != nil {
-		return fmt.Errorf("Invalid identity type: %w", err)
-	}
-
-	identityTypeInt, ok := intValue.(int64)
-	if !ok {
-		return fmt.Errorf("Identity type should be an integer, got `%v` (%T)", intValue, intValue)
-	}
-
-	switch identityTypeInt {
+// ScanInteger implements [query.IntegerScanner] for IdentityType. This simplifies the Scan implementation.
+func (i *IdentityType) ScanInteger(identityTypeCode int64) error {
+	switch identityTypeCode {
 	case identityTypeCertificateClientRestricted:
 		*i = api.IdentityTypeCertificateClientRestricted
 	case identityTypeCertificateClientUnrestricted:
@@ -163,12 +139,18 @@ func (i *IdentityType) Scan(value any) error {
 	case identityTypeCertificateClientPending:
 		*i = api.IdentityTypeCertificateClientPending
 	default:
-		return fmt.Errorf("Unknown identity type `%d`", identityTypeInt)
+		return fmt.Errorf("Unknown identity type `%d`", identityTypeCode)
 	}
 
 	return nil
 }
 
+// Scan implements sql.Scanner for IdentityType. This converts the integer value back into the correct API constant or
+// returns an error.
+func (i *IdentityType) Scan(value any) error {
+	return query.ScanValue(value, i, false)
+}
+
 // Value implements driver.Valuer for IdentityType. This converts the API constant into an integer or throws an error.
 func (i IdentityType) Value() (driver.Value, error) {
 	switch i {
@@ -193,6 +175,16 @@ func (i IdentityType) Value() (driver.Value, error) {
 	return nil, fmt.Errorf("Invalid identity type %q", i)
 }
 
+// ActiveType returns the active version of the identity type.
+func (i IdentityType) ActiveType() (IdentityType, error) {
+	switch i {
+	case api.IdentityTypeCertificateClientPending:
+		return api.IdentityTypeCertificateClient, nil
+	default:
+		return "", fmt.Errorf("Identities of type %q cannot be activated", i)
+	}
+}
+
 // toCertificateAPIType returns the API equivalent type.
 func (i IdentityType) toCertificateType() (certificate.Type, error) {
 	switch i {
@@ -296,7 +288,7 @@ func (i Identity) CertificateMetadata() (*CertificateMetadata, error) {
 	}
 
 	if i.Type == api.IdentityTypeCertificateClientPending {
-		return nil, fmt.Errorf("Cannot get certificate metadata: Identity is pending")
+		return nil, errors.New("Cannot get certificate metadata: Identity is pending")
 	}
 
 	var metadata CertificateMetadata
@@ -347,7 +339,7 @@ type PendingTLSMetadata struct {
 // PendingTLSMetadata returns the pending TLS identity metadata.
 func (i Identity) PendingTLSMetadata() (*PendingTLSMetadata, error) {
 	if i.Type != api.IdentityTypeCertificateClientPending {
-		return nil, api.NewStatusError(http.StatusBadRequest, "Cannot extract pending TLS identity secret: Identity is not pending")
+		return nil, api.StatusErrorf(http.StatusBadRequest, "Cannot extract pending %q TLS identity secret: Identity is not pending", i.Type)
 	}
 
 	var metadata PendingTLSMetadata
@@ -394,7 +386,7 @@ func (i *Identity) ToAPI(ctx context.Context, tx *sql.Tx, canViewGroup auth.Perm
 }
 
 // ActivateTLSIdentity updates a TLS identity to make it valid by adding the fingerprint, PEM encoded certificate, and setting
-// the type to api.IdentityTypeCertificateClient.
+// the type.
 func ActivateTLSIdentity(ctx context.Context, tx *sql.Tx, identifier uuid.UUID, cert *x509.Certificate) error {
 	fingerprint := shared.CertFingerprint(cert)
 	_, err := GetIdentityID(ctx, tx, api.AuthenticationMethodTLS, fingerprint)
@@ -402,27 +394,37 @@ func ActivateTLSIdentity(ctx context.Context, tx *sql.Tx, identifier uuid.UUID,
 		return api.StatusErrorf(http.StatusConflict, "Identity already exists")
 	}
 
+	identity, err := GetIdentity(ctx, tx, api.AuthenticationMethodTLS, identifier.String())
+	if err != nil {
+		return fmt.Errorf("Failed to get pending %q TLS identity: %w", identity.Type, err)
+	}
+
 	metadata := CertificateMetadata{Certificate: string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))}
 	b, err := json.Marshal(metadata)
 	if err != nil {
 		return fmt.Errorf("Failed to encode certificate metadata: %w", err)
 	}
 
-	stmt := `UPDATE identities SET type = ?, identifier = ?, metadata = ? WHERE identifier = ? AND auth_method = ?`
-	res, err := tx.ExecContext(ctx, stmt, identityTypeCertificateClient, fingerprint, string(b), identifier.String(), authMethodTLS)
+	identityTypeActive, err := identity.Type.ActiveType()
 	if err != nil {
-		return fmt.Errorf("Failed to activate TLS identity: %w", err)
+		return err
+	}
+
+	stmt := `UPDATE identities SET type = ?, identifier = ?, metadata = ? WHERE identifier = ? AND auth_method = ?`
+	res, err := tx.ExecContext(ctx, stmt, identityTypeActive, fingerprint, string(b), identifier.String(), authMethodTLS)
+	if err != nil {
+		return fmt.Errorf("Failed to activate %q TLS identity: %w", identity.Type, err)
 	}
 
 	n, err := res.RowsAffected()
 	if err != nil {
-		return fmt.Errorf("Failed to check for activated TLS identity: %w", err)
+		return fmt.Errorf("Failed to check for activated %q TLS identity: %w", identity.Type, err)
 	}
 
 	if n == 0 {
-		return api.StatusErrorf(http.StatusNotFound, "No pending TLS identity found with identifier %q", identifier)
+		return api.StatusErrorf(http.StatusNotFound, "No pending %q TLS identity found with identifier %q", identity.Type, identifier)
 	} else if n > 1 {
-		return fmt.Errorf("Unknown error occurred when activating a TLS identity: %w", err)
+		return fmt.Errorf("Unknown error occurred when activating %q TLS identity: %w", identity.Type, err)
 	}
 
 	return nil
