@@ -4,11 +4,12 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
+	"maps"
 	"net/http"
+	"slices"
 
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/db/query"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/logger"
@@ -72,9 +73,7 @@ func GetPermissionEntityURLs(ctx context.Context, tx *sql.Tx, permissions []Perm
 			return nil, nil, err
 		}
 
-		for k, v := range entityURLsAll {
-			entityURLs[k] = v
-		}
+		maps.Copy(entityURLs, entityURLsAll)
 	}
 
 	// Iterate over the input permissions and check which ones are present in the entityURLs map.
@@ -103,7 +102,7 @@ func GetPermissionEntityURLs(ctx context.Context, tx *sql.Tx, permissions []Perm
 		entityTypes := make([]EntityType, 0, len(danglingPermissions))
 		for _, perm := range danglingPermissions {
 			permissionIDs = append(permissionIDs, perm.ID)
-			if !shared.ValueInSlice(perm.EntityType, entityTypes) {
+			if !slices.Contains(entityTypes, perm.EntityType) {
 				entityTypes = append(entityTypes, perm.EntityType)
 			}
 		}
