@@ -1,7 +1,7 @@
 package db
 
 import (
-	"fmt"
+	"errors"
 	"net/http"
 
 	"github.com/canonical/go-dqlite/v3/driver"
@@ -10,7 +10,7 @@ import (
 
 var (
 	// ErrNoClusterMember is used to indicate no cluster member has been found for a resource.
-	ErrNoClusterMember = fmt.Errorf("No cluster member found")
+	ErrNoClusterMember = errors.New("No cluster member found")
 )
 
 // SmartErrors are used to return more appropriate errors to the caller.
