@@ -3,6 +3,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 
 	"github.com/spf13/cobra"
@@ -18,7 +19,7 @@ func newDb() *cobra.Command {
 		Use:   "db [sub-command]",
 		Short: "Database-related code generation.",
 		RunE: func(cmd *cobra.Command, args []string) error {
-			return fmt.Errorf("Not implemented")
+			return errors.New("Not implemented")
 		},
 	}
 
@@ -35,7 +36,7 @@ func newDbMapper() *cobra.Command {
 		Use:   "mapper [sub-command]",
 		Short: "Generate code mapping database rows to Go structs.",
 		RunE: func(cmd *cobra.Command, args []string) error {
-			return fmt.Errorf("Not implemented")
+			return errors.New("Not implemented")
 		},
 	}
 
@@ -81,7 +82,7 @@ func newDbMapperStmt() *cobra.Command {
 			kind := args[0]
 
 			if entity == "" {
-				return fmt.Errorf("No database entity given")
+				return errors.New("No database entity given")
 			}
 
 			config, err := parseParams(args[1:])
@@ -122,7 +123,7 @@ func newDbMapperMethod() *cobra.Command {
 			kind := args[0]
 
 			if entity == "" {
-				return fmt.Errorf("No database entity given")
+				return errors.New("No database entity given")
 			}
 
 			config, err := parseParams(args[1:])
