@@ -5,7 +5,8 @@ package db
 // Imports is a list of the package imports every generated source file has.
 var Imports = []string{
 	"database/sql",
+	"errors",
 	"fmt",
-	"github.com/lxc/lxd/lxd/db/query",
-	"github.com/lxc/lxd/shared/api",
+	"github.com/canonical/lxd/lxd/db/query",
+	"github.com/canonical/lxd/shared/api",
 }
