@@ -8,6 +8,7 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
+	"slices"
 	"strings"
 	"time"
 
@@ -18,13 +19,6 @@ import (
 	"github.com/canonical/lxd/shared/osarch"
 )
 
-// ImageSourceProtocol maps image source protocol codes to human-readable names.
-var ImageSourceProtocol = map[int]string{
-	0: "lxd",
-	1: "direct",
-	2: "simplestreams",
-}
-
 // GetLocalImagesFingerprints returns the fingerprints of all local images.
 func (c *ClusterTx) GetLocalImagesFingerprints(ctx context.Context) ([]string, error) {
 	q := `
@@ -36,149 +30,6 @@ SELECT images.fingerprint
 	return query.SelectStrings(ctx, c.tx, q, c.nodeID)
 }
 
-// GetImageSource returns the image source with the given ID.
-func (c *ClusterTx) GetImageSource(ctx context.Context, imageID int) (int, api.ImageSource, error) {
-	q := `SELECT id, server, protocol, certificate, alias FROM images_source WHERE image_id=?`
-	type imagesSource struct {
-		ID          int
-		Server      string
-		Protocol    int
-		Certificate string
-		Alias       string
-	}
-
-	sources := []imagesSource{}
-	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		s := imagesSource{}
-
-		err := scan(&s.ID, &s.Server, &s.Protocol, &s.Certificate, &s.Alias)
-		if err != nil {
-			return err
-		}
-
-		sources = append(sources, s)
-
-		return nil
-	}, imageID)
-	if err != nil {
-		return -1, api.ImageSource{}, err
-	}
-
-	if len(sources) == 0 {
-		return -1, api.ImageSource{}, api.StatusErrorf(http.StatusNotFound, "Image source not found")
-	}
-
-	source := sources[0]
-
-	protocol, found := ImageSourceProtocol[source.Protocol]
-	if !found {
-		return -1, api.ImageSource{}, fmt.Errorf("Invalid protocol: %d", source.Protocol)
-	}
-
-	result := api.ImageSource{
-		Server:      source.Server,
-		Protocol:    protocol,
-		Certificate: source.Certificate,
-		Alias:       source.Alias,
-	}
-
-	return source.ID, result, nil
-}
-
-// Fill extra image fields such as properties and alias. This is called after
-// fetching a single row from the images table.
-func (c *ClusterTx) imageFill(ctx context.Context, id int, image *api.Image, create, expire, used, upload *time.Time, arch int, imageType int) error {
-	// Some of the dates can be nil in the DB, let's process them.
-	if create != nil {
-		image.CreatedAt = *create
-	} else {
-		image.CreatedAt = time.Time{}
-	}
-
-	if expire != nil {
-		image.ExpiresAt = *expire
-	} else {
-		image.ExpiresAt = time.Time{}
-	}
-
-	if used != nil {
-		image.LastUsedAt = *used
-	} else {
-		image.LastUsedAt = time.Time{}
-	}
-
-	image.Architecture, _ = osarch.ArchitectureName(arch)
-	image.Type = instancetype.Type(imageType).String()
-
-	// The upload date is enforced by NOT NULL in the schema, so it can never be nil.
-	image.UploadedAt = *upload
-
-	// Get the properties
-	properties, err := query.SelectConfig(ctx, c.tx, "images_properties", "image_id=?", id)
-	if err != nil {
-		return err
-	}
-
-	image.Properties = properties
-
-	q := "SELECT name, description FROM images_aliases WHERE image_id=?"
-
-	// Get the aliases
-	aliases := []api.ImageAlias{}
-	err = query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		alias := api.ImageAlias{}
-
-		err := scan(&alias.Name, &alias.Description)
-		if err != nil {
-			return err
-		}
-
-		aliases = append(aliases, alias)
-		return nil
-	}, id)
-	if err != nil {
-		return err
-	}
-
-	image.Aliases = aliases
-
-	_, source, err := c.GetImageSource(ctx, id)
-	if err == nil {
-		image.UpdateSource = &source
-		image.UpdateSource.ImageType = image.Type
-	}
-
-	return nil
-}
-
-func (c *ClusterTx) imageFillProfiles(ctx context.Context, id int, image *api.Image, project string) error {
-	// Check which project name to use
-	enabled, err := cluster.ProjectHasProfiles(context.Background(), c.tx, project)
-	if err != nil {
-		return fmt.Errorf("Check if project has profiles: %w", err)
-	}
-
-	if !enabled {
-		project = "default"
-	}
-
-	// Get the profiles
-	q := `
-SELECT profiles.name FROM profiles
-	JOIN images_profiles ON images_profiles.profile_id = profiles.id
-	JOIN projects ON profiles.project_id = projects.id
-WHERE images_profiles.image_id = ? AND projects.name = ?
-`
-	profiles, err := query.SelectStrings(ctx, c.tx, q, id, project)
-	if err != nil {
-		return err
-	}
-
-	image.Profiles = profiles
-
-	return nil
-}
-
 // GetImagesFingerprints returns the names of all images (optionally only the public ones).
 func (c *ClusterTx) GetImagesFingerprints(ctx context.Context, projectName string, publicOnly bool) ([]string, error) {
 	q := `
@@ -213,7 +64,7 @@ SELECT fingerprint
 // CreateImageSource inserts a new image source.
 func (c *ClusterTx) CreateImageSource(ctx context.Context, id int, server string, protocol string, certificate string, alias string) error {
 	protocolInt := -1
-	for protoInt, protoString := range ImageSourceProtocol {
+	for protoInt, protoString := range cluster.ImageSourceProtocol {
 		if protoString == protocol {
 			protocolInt = protoInt
 		}
@@ -254,7 +105,7 @@ func (c *ClusterTx) GetCachedImageSourceFingerprint(ctx context.Context, server
 	}
 
 	protocolInt := -1
-	for protoInt, protoString := range ImageSourceProtocol {
+	for protoInt, protoString := range cluster.ImageSourceProtocol {
 		if protoString == protocol {
 			protocolInt = protoInt
 		}
@@ -360,8 +211,6 @@ func (c *ClusterTx) GetImage(ctx context.Context, fingerprintPrefix string, filt
 // publicOnly, when true, will return the image only if it is public;
 // a false value will return any image matching the fingerprint prefix.
 func (c *ClusterTx) GetImageByFingerprintPrefix(ctx context.Context, fingerprintPrefix string, filter cluster.ImageFilter) (int, *api.Image, error) {
-	var image api.Image
-	var object cluster.Image
 	if fingerprintPrefix == "" {
 		return -1, nil, errors.New("No fingerprint prefix specified for the image")
 	}
@@ -386,46 +235,27 @@ func (c *ClusterTx) GetImageByFingerprintPrefix(ctx context.Context, fingerprint
 		return -1, nil, fmt.Errorf("Failed to fetch images: %w", err)
 	}
 
+	var object cluster.Image
 	switch len(images) {
 	case 0:
 		return -1, nil, api.StatusErrorf(http.StatusNotFound, "Image not found")
 	case 1:
 		object = images[0]
 	default:
-		return -1, nil, fmt.Errorf("More than one image matches")
+		return -1, nil, errors.New("More than one image matches")
 	}
 
-	image.Fingerprint = object.Fingerprint
-	image.Filename = object.Filename
-	image.Size = object.Size
-	image.Cached = object.Cached
-	image.Public = object.Public
-	image.AutoUpdate = object.AutoUpdate
-	image.Project = object.Project
-
-	err = c.imageFill(
-		ctx, object.ID, &image,
-		&object.CreationDate.Time, &object.ExpiryDate.Time, &object.LastUseDate.Time,
-		&object.UploadDate, object.Architecture, object.Type)
+	img, err := object.ToAPI(ctx, c.Tx(), profileProject)
 	if err != nil {
-		return -1, nil, fmt.Errorf("Fill image details: %w", err)
+		return -1, nil, err
 	}
 
-	err = c.imageFillProfiles(ctx, object.ID, &image, profileProject)
-	if err != nil {
-		return -1, nil, fmt.Errorf("Fill image profiles: %w", err)
-	}
-
-	return object.ID, &image, nil
+	return object.ID, img, nil
 }
 
 // GetImageFromAnyProject returns an image matching the given fingerprint, if
 // it exists in any project.
 func (c *ClusterTx) GetImageFromAnyProject(ctx context.Context, fingerprint string) (int, *api.Image, error) {
-	// The object we'll actually return
-	var image api.Image
-	var object cluster.Image
-
 	images, err := c.getImagesByFingerprintPrefix(ctx, fingerprint, cluster.ImageFilter{})
 	if err != nil {
 		return -1, nil, fmt.Errorf("Get image %q: Failed to fetch images: %w", fingerprint, err)
@@ -435,24 +265,13 @@ func (c *ClusterTx) GetImageFromAnyProject(ctx context.Context, fingerprint stri
 		return -1, nil, fmt.Errorf("Get image %q: %w", fingerprint, api.StatusErrorf(http.StatusNotFound, "Image not found"))
 	}
 
-	object = images[0]
-
-	image.Fingerprint = object.Fingerprint
-	image.Filename = object.Filename
-	image.Size = object.Size
-	image.Cached = object.Cached
-	image.Public = object.Public
-	image.AutoUpdate = object.AutoUpdate
-
-	err = c.imageFill(
-		ctx, object.ID, &image,
-		&object.CreationDate.Time, &object.ExpiryDate.Time, &object.LastUseDate.Time,
-		&object.UploadDate, object.Architecture, object.Type)
+	object := images[0]
+	img, err := object.ToAPI(ctx, c.Tx(), "")
 	if err != nil {
-		return -1, nil, fmt.Errorf("Get image %q: Fill image details: %w", fingerprint, err)
+		return -1, nil, err
 	}
 
-	return object.ID, &image, nil
+	return object.ID, img, nil
 }
 
 // getImagesByFingerprintPrefix returns the images with fingerprints matching the prefix.
@@ -563,13 +382,11 @@ WHERE images.fingerprint = ?
 	}
 
 	if len(addresses) == 0 {
-		return "", fmt.Errorf("Image not available on any online member")
+		return "", errors.New("Image not available on any online member")
 	}
 
-	for _, address := range addresses {
-		if address == localAddress {
-			return "", nil
-		}
+	if slices.Contains(addresses, localAddress) {
+		return "", nil
 	}
 
 	return addresses[0], nil
