@@ -5,6 +5,7 @@ package db
 import (
 	"context"
 	"database/sql"
+	"errors"
 	"fmt"
 	"net/http"
 	"sort"
@@ -156,7 +157,7 @@ SELECT nodes.id, nodes.address
 	}
 
 	if rows.Next() {
-		return "", fmt.Errorf("More than one cluster member associated with instance")
+		return "", errors.New("More than one cluster member associated with instance")
 	}
 
 	err = rows.Err()
@@ -247,7 +248,7 @@ func (c *ClusterTx) GetInstancesByMemberAddress(ctx context.Context, offlineThre
 }
 
 // ErrListStop used as return value from InstanceList's instanceFunc when prematurely stopping the search.
-var ErrListStop = fmt.Errorf("search stopped")
+var ErrListStop = errors.New("search stopped")
 
 // InstanceList loads all instances across all projects and for each instance runs the instanceFunc passing in the
 // instance and it's project and profiles. Accepts optional filter arguments to specify a subset of instances.
@@ -638,7 +639,7 @@ func (c *ClusterTx) InstancesToInstanceArgs(ctx context.Context, fillProfiles bo
 	}
 
 	if instanceCount > 0 && snapshotCount > 0 {
-		return nil, fmt.Errorf("Cannot use InstancesToInstanceArgs with mixed instance and instance snapshots")
+		return nil, errors.New("Cannot use InstancesToInstanceArgs with mixed instance and instance snapshots")
 	}
 
 	// Populate instance config.
@@ -666,7 +667,7 @@ func (c *ClusterTx) InstancesToInstanceArgs(ctx context.Context, fillProfiles bo
 
 // UpdateInstanceNode changes the name of an instance and the cluster member hosting it.
 // It's meant to be used when moving a non-running instance backed by ceph from one cluster node to another.
-func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project string, oldName string, newName string, newMemberName string, poolID int64, volumeType int) error {
+func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project string, oldName string, newName string, newMemberName string, poolID int64, volumeType cluster.StoragePoolVolumeType) error {
 	// Update the name of the instance and its snapshots, and the member ID they are associated with.
 	instanceID, err := cluster.GetInstanceID(ctx, c.tx, project, oldName)
 	if err != nil {
@@ -997,7 +998,7 @@ func (c *ClusterTx) UpdateInstanceSnapshotCreationDate(ctx context.Context, inst
 // in the given project with the given name.
 // Returns snapshots slice ordered by when they were created, oldest first.
 func (c *ClusterTx) GetInstanceSnapshotsNames(ctx context.Context, project, name string) ([]string, error) {
-	result := []string{}
+	instanceSnapshotNames := []string{}
 
 	q := `
 SELECT instances_snapshots.name
@@ -1007,19 +1008,23 @@ SELECT instances_snapshots.name
 WHERE projects.name=? AND instances.name=?
 ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 `
-	inargs := []any{project, name}
-	outfmt := []any{name}
+	err := query.Scan(ctx, c.Tx(), q, func(scan func(dest ...any) error) error {
+		var instanceSnapshotName string
 
-	dbResults, err := queryScan(ctx, c, q, inargs, outfmt)
+		err := scan(&instanceSnapshotName)
+		if err != nil {
+			return err
+		}
+
+		instanceSnapshotNames = append(instanceSnapshotNames, name+shared.SnapshotDelimiter+instanceSnapshotName)
+
+		return nil
+	}, project, name)
 	if err != nil {
-		return result, err
+		return nil, err
 	}
 
-	for _, r := range dbResults {
-		result = append(result, name+shared.SnapshotDelimiter+r[0].(string))
-	}
-
-	return result, nil
+	return instanceSnapshotNames, nil
 }
 
 // GetNextInstanceSnapshotIndex returns the index that the next snapshot of the
@@ -1033,35 +1038,34 @@ SELECT instances_snapshots.name
 WHERE projects.name=? AND instances.name=?
 ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 `
-	var numstr string
-	inargs := []any{project, name}
-	outfmt := []any{numstr}
-
 	// Check if pattern is valid.
 	if !strings.Contains(pattern, "%d") {
 		return 0
 	}
 
-	results, err := queryScan(ctx, c, q, inargs, outfmt)
-	if err != nil {
-		return 0
-	}
+	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
+		var snapOnlyName string
 
-	for _, r := range results {
-		snapOnlyName, ok := r[0].(string)
-		if !ok {
-			continue
+		err := scan(&snapOnlyName)
+		if err != nil {
+			return err
 		}
 
 		var num int
 		count, err := fmt.Sscanf(snapOnlyName, pattern, &num)
 		if err != nil || count != 1 {
-			continue
+			return nil
 		}
 
 		if num >= nextIndex {
 			nextIndex = num + 1
 		}
+
+		return nil
+	}, project, name)
+
+	if err != nil {
+		return 0
 	}
 
 	return nextIndex
