@@ -5,12 +5,13 @@ package db
 import (
 	"context"
 	"database/sql"
+	"errors"
 	"fmt"
+	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/query"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/logger"
 )
 
@@ -54,7 +55,7 @@ DELETE FROM storage_volumes_config WHERE storage_volume_id NOT IN (SELECT id FRO
 	for _, table := range preClusteringTables {
 		logger.Debugf("Loading data from table %s", table)
 		data := [][]any{}
-		stmt := fmt.Sprintf("SELECT * FROM %s", table)
+		stmt := "SELECT * FROM " + table
 
 		rows, err := tx.QueryContext(ctx, stmt)
 		if err != nil {
@@ -172,7 +173,7 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 						continue
 					}
 
-					if column == "key" && shared.ValueInSlice(value, keys) {
+					if column == "key" && slices.Contains(keys, value) {
 						skip = true
 					}
 				}
@@ -194,8 +195,13 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 						break
 					}
 				}
-				key := row[index].(string)
-				if !shared.ValueInSlice(key, NodeSpecificNetworkConfig) {
+
+				key, ok := row[index].(string)
+				if !ok {
+					return fmt.Errorf("Failed to convert key to string for row %d in networks_config", i)
+				}
+
+				if !slices.Contains(NodeSpecificNetworkConfig, key) {
 					nullNodeID = true
 					break
 				}
@@ -213,8 +219,13 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 						break
 					}
 				}
-				key := row[index].(string)
-				if !shared.ValueInSlice(key, NodeSpecificStorageConfig) {
+
+				key, ok := row[index].(string)
+				if !ok {
+					return fmt.Errorf("Failed to convert key to string for row %d in storage_pools_config", i)
+				}
+
+				if !slices.Contains(NodeSpecificStorageConfig, key) {
 					nullNodeID = true
 					break
 				}
@@ -229,14 +240,14 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 				appendNodeID()
 			}
 
-			if shared.ValueInSlice(table, preClusteringTablesRequiringProjectID) {
+			if slices.Contains(preClusteringTablesRequiringProjectID, table) {
 				// These tables have a project_id reference in the new schema.
 				columns = append(columns, "project_id")
 				row = append(row, 1) // Reference the default project.
 			}
 
 			stmt := fmt.Sprintf("INSERT INTO %s(%s)", table, strings.Join(columns, ", "))
-			stmt += fmt.Sprintf(" VALUES %s", query.Params(len(columns)))
+			stmt += " VALUES " + query.Params(len(columns))
 			result, err := tx.Exec(stmt, row...)
 			if err != nil {
 				return fmt.Errorf("failed to insert row %d into %s: %w", i, table, err)
@@ -252,11 +263,11 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 			}
 
 			// Also insert the image ID to node ID association.
-			if shared.ValueInSlice(table, []string{"images", "networks", "storage_pools"}) {
+			if slices.Contains([]string{"images", "networks", "storage_pools"}, table) {
 				entity := table[:len(table)-1]
 				err := importNodeAssociation(entity, columns, row, tx)
 				if err != nil {
-					return fmt.Errorf("Failed to import node associations")
+					return errors.New("Failed to import node associations")
 				}
 			}
 		}
@@ -271,19 +282,24 @@ func importNodeAssociation(entity string, columns []string, row []any, tx *sql.T
 	stmt := fmt.Sprintf(
 		"INSERT INTO %ss_nodes(%s_id, node_id) VALUES(?, 1)", entity, entity)
 	var id int64
+	var ok bool
 	for i, column := range columns {
 		if column == "id" {
-			id = row[i].(int64)
+			id, ok = row[i].(int64)
+			if !ok {
+				return fmt.Errorf("Failed to convert %q ID to int64", entity)
+			}
+
 			break
 		}
 	}
 	if id == 0 {
-		return fmt.Errorf("entity %s has invalid ID", entity)
+		return fmt.Errorf("entity %q has invalid ID", entity)
 	}
 
 	_, err := tx.Exec(stmt, id)
 	if err != nil {
-		return fmt.Errorf("failed to associate %s to node: %w", entity, err)
+		return fmt.Errorf("failed to associate %q to node: %w", entity, err)
 	}
 
 	return nil
