@@ -5,8 +5,10 @@ package db
 import (
 	"context"
 	"database/sql"
+	"errors"
 	"fmt"
 	"net/http"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -15,7 +17,6 @@ import (
 	"github.com/canonical/lxd/lxd/db/operationtype"
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/lxd/util"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/osarch"
 	"github.com/canonical/lxd/shared/version"
@@ -84,14 +85,13 @@ type NodeInfoArgs struct {
 	FailureDomains       map[uint64]string
 	MemberFailureDomains map[string]uint64
 	OfflineThreshold     time.Duration
-	MaxMemberVersion     [2]int
+	Members              []NodeInfo
 	RaftNodes            []RaftNode
 }
 
 // ToAPI returns a LXD API entry.
 func (n NodeInfo) ToAPI(ctx context.Context, tx *ClusterTx, args NodeInfoArgs) (*api.ClusterMember, error) {
 	var err error
-	var maxVersion [2]int
 	var failureDomain string
 
 	domainID := args.MemberFailureDomains[n.Address]
@@ -155,15 +155,26 @@ func (n NodeInfo) ToAPI(ctx context.Context, tx *ClusterTx, args NodeInfoArgs) (
 		result.Status = "Offline"
 		result.Message = fmt.Sprintf("No heartbeat for %s (%s)", time.Since(n.Heartbeat), n.Heartbeat)
 	} else {
-		// Check if up to date.
-		cmp, err := util.CompareVersions(maxVersion, n.Version())
-		if err != nil {
-			return nil, err
-		}
+		for _, member := range args.Members {
+			if member.ID == n.ID {
+				continue // Skip ourselves.
+			}
 
-		if cmp == 1 {
-			result.Status = "Blocked"
-			result.Message = "Needs updating to newer version"
+			// Check if up to date.
+			cmp, err := util.CompareVersions(member.Version(), n.Version())
+			if err != nil {
+				return nil, fmt.Errorf("Failed comparing with version of member %q: %w", member.Name, err)
+			}
+
+			if cmp == 1 {
+				result.Status = "Blocked"
+				result.Message = "LXD version is older than other members"
+				break
+			} else if cmp == 2 {
+				result.Status = "Blocked"
+				result.Message = "LXD version is newer than other members"
+				break
+			}
 		}
 	}
 
@@ -190,38 +201,12 @@ func (c *ClusterTx) GetNodeByAddress(ctx context.Context, address string) (NodeI
 	case 1:
 		return nodes[0], nil
 	default:
-		return null, fmt.Errorf("more than one node matches")
+		return null, errors.New("more than one node matches")
 	}
 }
 
-// GetNodeMaxVersion returns the highest schema and API versions possible on the cluster.
-func (c *ClusterTx) GetNodeMaxVersion(ctx context.Context) ([2]int, error) {
-	version := [2]int{}
-
-	// Get the maximum DB schema.
-	var maxSchema int
-	row := c.tx.QueryRowContext(ctx, "SELECT MAX(schema) FROM nodes")
-	err := row.Scan(&maxSchema)
-	if err != nil {
-		return version, err
-	}
-
-	// Get the maximum API extension.
-	var maxAPI int
-	row = c.tx.QueryRowContext(ctx, "SELECT MAX(api_extensions) FROM nodes")
-	err = row.Scan(&maxAPI)
-	if err != nil {
-		return version, err
-	}
-
-	// Compute the combined version.
-	version = [2]int{maxSchema, maxAPI}
-
-	return version, nil
-}
-
-// GetNodeWithID returns the node with the given ID.
-func (c *ClusterTx) GetNodeWithID(ctx context.Context, nodeID int) (NodeInfo, error) {
+// getNodeByID returns the node with the given ID.
+func (c *ClusterTx) getNodeByID(ctx context.Context, nodeID int64) (NodeInfo, error) {
 	null := NodeInfo{}
 	nodes, err := c.nodes(ctx, false /* not pending */, "id=?", nodeID)
 	if err != nil {
@@ -234,7 +219,7 @@ func (c *ClusterTx) GetNodeWithID(ctx context.Context, nodeID int) (NodeInfo, er
 	case 1:
 		return nodes[0], nil
 	default:
-		return null, fmt.Errorf("More than one cluster member matches")
+		return null, errors.New("More than one cluster member matches")
 	}
 }
 
@@ -252,7 +237,7 @@ func (c *ClusterTx) GetPendingNodeByAddress(ctx context.Context, address string)
 	case 1:
 		return nodes[0], nil
 	default:
-		return null, fmt.Errorf("More than one cluster member matches")
+		return null, errors.New("More than one cluster member matches")
 	}
 }
 
@@ -270,7 +255,7 @@ func (c *ClusterTx) GetNodeByName(ctx context.Context, name string) (NodeInfo, e
 	case 1:
 		return nodes[0], nil
 	default:
-		return null, fmt.Errorf("More than one cluster member matches")
+		return null, errors.New("More than one cluster member matches")
 	}
 }
 
@@ -289,7 +274,7 @@ func (c *ClusterTx) GetLocalNodeName(ctx context.Context) (string, error) {
 	case 1:
 		return names[0], nil
 	default:
-		return "", fmt.Errorf("inconsistency: non-unique node ID")
+		return "", errors.New("inconsistency: non-unique node ID")
 	}
 }
 
@@ -307,7 +292,7 @@ func (c *ClusterTx) GetLocalNodeAddress(ctx context.Context) (string, error) {
 	case 1:
 		return addresses[0], nil
 	default:
-		return "", fmt.Errorf("inconsistency: non-unique node ID")
+		return "", errors.New("inconsistency: non-unique node ID")
 	}
 }
 
@@ -327,7 +312,7 @@ func (c *ClusterTx) NodeIsOutdated(ctx context.Context) (bool, error) {
 		}
 	}
 	if version[0] == 0 || version[1] == 0 {
-		return false, fmt.Errorf("Inconsistency: local member not found")
+		return false, errors.New("Inconsistency: local member not found")
 	}
 
 	// Check if any of the other nodes is greater than us.
@@ -338,7 +323,7 @@ func (c *ClusterTx) NodeIsOutdated(ctx context.Context) (bool, error) {
 
 		n, err := util.CompareVersions(node.Version(), version)
 		if err != nil {
-			return false, fmt.Errorf("Failed to compare with version of member %s: %w", node.Name, err)
+			return false, fmt.Errorf("Failed comparing with version of member %q: %w", node.Name, err)
 		}
 
 		if n == 1 {
@@ -422,7 +407,7 @@ func (c *ClusterTx) SetDescription(id int64, description string) error {
 	return nil
 }
 
-// Nodes returns all LXD nodes part of the cluster.
+// nodes returns all LXD nodes part of the cluster.
 func (c *ClusterTx) nodes(ctx context.Context, pending bool, where string, args ...any) ([]NodeInfo, error) {
 	// Get node roles
 	sql := "SELECT node_id, role FROM nodes_roles"
@@ -530,7 +515,7 @@ JOIN cluster_groups ON cluster_groups.id = nodes_cluster_groups.group_id`
 		}
 	}
 
-	config, err := cluster.GetConfig(context.TODO(), c.Tx(), "node")
+	config, err := cluster.GetConfig(ctx, c.Tx(), "node")
 	if err != nil {
 		return nil, fmt.Errorf("Failed to fetch nodes config: %w", err)
 	}
@@ -667,7 +652,7 @@ func (c *ClusterTx) UpdateNodeRoles(id int64, roles []ClusterRole) error {
 
 // UpdateNodeClusterGroups changes the list of cluster groups the member belongs to.
 func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, groups []string) error {
-	nodeInfo, err := c.GetNodeWithID(ctx, int(id))
+	nodeInfo, err := c.getNodeByID(ctx, id)
 	if err != nil {
 		return err
 	}
@@ -681,7 +666,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, group
 
 	// Check if node already belongs to the given groups.
 	for _, newGroup := range groups {
-		if shared.ValueInSlice(newGroup, oldGroups) {
+		if slices.Contains(oldGroups, newGroup) {
 			// Node already belongs to this group.
 			skipGroups = append(skipGroups, newGroup)
 			continue
@@ -695,7 +680,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, group
 	}
 
 	for _, oldGroup := range oldGroups {
-		if shared.ValueInSlice(oldGroup, skipGroups) {
+		if slices.Contains(skipGroups, oldGroup) {
 			continue
 		}
 
@@ -714,7 +699,7 @@ func (c *ClusterTx) UpdateNodeFailureDomain(ctx context.Context, id int64, domai
 	var domainID any
 
 	if domain == "" {
-		return fmt.Errorf("Failure domain name can't be empty")
+		return errors.New("Failure domain name can't be empty")
 	}
 
 	if domain == "default" {
@@ -1062,12 +1047,12 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 		}
 
 		// Skip group-only members if targeted cluster group doesn't match.
-		if member.Config["scheduler.instance"] == "group" && !shared.ValueInSlice(targetClusterGroup, member.Groups) {
+		if member.Config["scheduler.instance"] == "group" && !slices.Contains(member.Groups, targetClusterGroup) {
 			continue
 		}
 
 		// Skip if a group is requested and member isn't part of it.
-		if targetClusterGroup != "" && !shared.ValueInSlice(targetClusterGroup, member.Groups) {
+		if targetClusterGroup != "" && !slices.Contains(member.Groups, targetClusterGroup) {
 			continue
 		}
 
@@ -1075,7 +1060,7 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 		if allowedClusterGroups != nil {
 			found := false
 			for _, allowedClusterGroup := range allowedClusterGroups {
-				if shared.ValueInSlice(allowedClusterGroup, member.Groups) {
+				if slices.Contains(member.Groups, allowedClusterGroup) {
 					found = true
 					break
 				}
@@ -1096,7 +1081,7 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 
 			supportedArchitectures := append([]int{member.Architecture}, personalities...)
 			for _, supportedArchitecture := range supportedArchitectures {
-				if shared.ValueInSlice(supportedArchitecture, targetArchitectures) {
+				if slices.Contains(targetArchitectures, supportedArchitecture) {
 					candidateMembers = append(candidateMembers, member)
 					break
 				}
@@ -1159,7 +1144,7 @@ func (c *ClusterTx) SetNodeVersion(id int64, version [2]int) error {
 	}
 
 	if n != 1 {
-		return fmt.Errorf("Expected exactly one row to be updated")
+		return errors.New("Expected exactly one row to be updated")
 	}
 
 	return nil
