@@ -2,9 +2,8 @@ package node
 
 import (
 	"database/sql"
-	"fmt"
 
-	"github.com/CanonicalLtd/go-sqlite3"
+	"github.com/mattn/go-sqlite3"
 )
 
 func init() {
@@ -13,11 +12,11 @@ func init() {
 
 // Opens the node-level database with the correct parameters for LXD.
 func sqliteOpen(path string) (*sql.DB, error) {
-	timeout := 5 // TODO - make this command-line configurable?
+	// TODO - make the timeout command-line configurable?
 
 	// These are used to tune the transaction BEGIN behavior instead of using the
 	// similar "locking_mode" pragma (locking for the whole database connection).
-	openPath := fmt.Sprintf("%s?_busy_timeout=%d&_txlock=exclusive", path, timeout*1000)
+	openPath := path + "?_busy_timeout=5000&_txlock=exclusive"
 
 	// Open the database. If the file doesn't exist it is created.
 	return sql.Open("sqlite3_with_fk", openPath)
