@@ -232,7 +232,7 @@ func TestNodeIsEmpty_Instances(t *testing.T) {
 
 	message, err := tx.NodeIsEmpty(context.Background(), id)
 	require.NoError(t, err)
-	assert.Equal(t, "", message)
+	assert.Empty(t, message)
 
 	_, err = tx.Tx().Exec(`
 INSERT INTO instances (id, node_id, name, architecture, type, project_id, description) VALUES (1, ?, 'foo', 1, 1, 1, '')
@@ -248,7 +248,7 @@ INSERT INTO instances (id, node_id, name, architecture, type, project_id, descri
 
 	message, err = tx.NodeIsEmpty(context.Background(), id)
 	require.NoError(t, err)
-	assert.Equal(t, "", message)
+	assert.Empty(t, message)
 }
 
 // A node is considered empty only if it has no images that are available only
@@ -280,7 +280,7 @@ INSERT INTO images_nodes(image_id, node_id) VALUES(1, 1)`)
 
 	message, err = tx.NodeIsEmpty(context.Background(), id)
 	require.NoError(t, err)
-	assert.Equal(t, "", message)
+	assert.Empty(t, message)
 }
 
 // A node is considered empty only if it has no custom volumes on it.
