@@ -4,9 +4,11 @@ package openfga
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
+	"slices"
 	"strings"
 	"sync"
 	"sync/atomic"
@@ -19,7 +21,6 @@ import (
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/lxd/request"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 )
@@ -91,23 +92,23 @@ func (o *openfgaStore) Read(ctx context.Context, s string, key *openfgav1.TupleK
 
 	// We always expect the `Object` field to be present.
 	if !hasObj {
-		return nil, fmt.Errorf("Read: Can only list by object")
+		return nil, errors.New("Read: Can only list by object")
 	}
 
 	// Users are what we are going to enumerate.
 	if hasUser {
-		return nil, fmt.Errorf("Read: Listing by user not supported")
+		return nil, errors.New("Read: Listing by user not supported")
 	}
 
 	// Expect the relation to be present.
 	if !hasRelation {
-		return nil, fmt.Errorf("Read: Listing all objects without a relation not supported")
+		return nil, errors.New("Read: Listing all objects without a relation not supported")
 	}
 
 	// Validate the object. We expect the URL to be present.
 	entityTypeStr, entityURL, hasURL := strings.Cut(obj, ":")
 	if !hasURL {
-		return nil, fmt.Errorf("Read: Listing all entities of type not supported")
+		return nil, errors.New("Read: Listing all entities of type not supported")
 	}
 
 	entityType := entity.Type(entityTypeStr)
@@ -165,7 +166,7 @@ func (o *openfgaStore) Read(ctx context.Context, s string, key *openfgav1.TupleK
 		tupleKey.User = string(entity.TypeServer) + ":" + entity.ServerURL().String()
 
 	case entity.TypeInstance:
-		if !shared.ValueInSlice(entityType, []entity.Type{entity.TypeInstanceBackup, entity.TypeInstanceSnapshot}) {
+		if !slices.Contains([]entity.Type{entity.TypeInstanceBackup, entity.TypeInstanceSnapshot}, entityType) {
 			return nil, fmt.Errorf("Received unexpected query, entities of type %q do not have an instance relation", entityType)
 		}
 
@@ -177,7 +178,7 @@ func (o *openfgaStore) Read(ctx context.Context, s string, key *openfgav1.TupleK
 		tupleKey.User = string(entity.TypeInstance) + ":" + entity.InstanceURL(projectName, pathArgs[0]).String()
 
 	case entity.TypeStorageVolume:
-		if !shared.ValueInSlice(entityType, []entity.Type{entity.TypeStorageVolumeBackup, entity.TypeStorageVolumeSnapshot}) {
+		if !slices.Contains([]entity.Type{entity.TypeStorageVolumeBackup, entity.TypeStorageVolumeSnapshot}, entityType) {
 			return nil, fmt.Errorf("Received unexpected query, entities of type %q do not have an instance relation", entityType)
 		}
 
@@ -212,7 +213,7 @@ func (o *openfgaStore) ReadUserTuple(ctx context.Context, store string, tk *open
 	// Expect the User field to be present.
 	user := tk.GetUser()
 	if user == "" {
-		return nil, fmt.Errorf("ReadUserTuple: User field of tuple key must be provided")
+		return nil, errors.New("ReadUserTuple: User field of tuple key must be provided")
 	}
 
 	// Only allow `identity` for the User type.
@@ -324,14 +325,14 @@ func (o *openfgaStore) ensureCacheLoaded(ctx context.Context, cache *RequestCach
 func (o *openfgaStore) ReadUsersetTuples(ctx context.Context, store string, filter storage.ReadUsersetTuplesFilter, options storage.ReadUsersetTuplesOptions) (storage.TupleIterator, error) {
 	// Expect both an object and a relation.
 	if filter.Object == "" || filter.Relation == "" {
-		return nil, fmt.Errorf("ReadUsersetTuples: Filter must include both an object and a relation")
+		return nil, errors.New("ReadUsersetTuples: Filter must include both an object and a relation")
 	}
 
 	// Expect a URL to be present for the object. (E.g. we don't want to list all groups that have `can_view` on
 	// all projects, we should be checking for a specific entity).
 	entityTypeStr, entityURL, hasURL := strings.Cut(filter.Object, ":")
 	if !hasURL {
-		return nil, fmt.Errorf("ReadUsersetTuples: Listing all entities of type not supported")
+		return nil, errors.New("ReadUsersetTuples: Listing all entities of type not supported")
 	}
 
 	entityType := entity.Type(entityTypeStr)
@@ -346,7 +347,7 @@ func (o *openfgaStore) ReadUsersetTuples(ctx context.Context, store string, filt
 	}
 
 	// Get cache from context. If it is not present, we'll fall back to calling the database directly.
-	cache, err := request.GetCtxValue[*RequestCache](ctx, request.CtxOpenFGARequestCache)
+	cache, err := request.GetContextValue[*RequestCache](ctx, request.CtxOpenFGARequestCache)
 	if err != nil {
 		groups, err := o.getGroupsWithEntitlementOnEntityWithURL(ctx, auth.Entitlement(filter.Relation), entityType, u)
 		if err != nil {
@@ -528,13 +529,13 @@ func (o *openfgaStore) ReadStartingWithUser(ctx context.Context, store string, f
 
 	// Expect that there will be exactly one user filter.
 	if len(filter.UserFilter) != 1 {
-		return nil, fmt.Errorf("ReadStartingWithUser: Unexpected user filter list length")
+		return nil, errors.New("ReadStartingWithUser: Unexpected user filter list length")
 	}
 
 	// Expect that the user filter object has an entity type and a URL.
 	userTypeStr, userURL, ok := strings.Cut(filter.UserFilter[0].GetObject(), ":")
 	if !ok {
-		return nil, fmt.Errorf("ReadStartingWithUser: Must provide user reference")
+		return nil, errors.New("ReadStartingWithUser: Must provide user reference")
 	}
 
 	// Validate the user entity type.
@@ -559,7 +560,7 @@ func (o *openfgaStore) ReadStartingWithUser(ctx context.Context, store string, f
 	relationEntityType := entity.Type(filter.Relation)
 
 	// If the relation is "project" or "server", we are listing all resources under the project/server.
-	if shared.ValueInSlice(relationEntityType, []entity.Type{entity.TypeProject, entity.TypeServer, entity.TypeInstance, entity.TypeStorageVolume}) {
+	if slices.Contains([]entity.Type{entity.TypeProject, entity.TypeServer, entity.TypeInstance, entity.TypeStorageVolume}, relationEntityType) {
 		if filter.Relation != string(userEntityType) {
 			// Expect that the user entity type is expected for the relation.
 			return nil, fmt.Errorf("ReadStartingWithUser: Relation %q is not valid for entities of type %q", filter.Relation, userEntityType)
@@ -653,7 +654,7 @@ func (o *openfgaStore) ReadStartingWithUser(ctx context.Context, store string, f
 	entitlement := auth.Entitlement(filter.Relation)
 
 	// Get cache from context. If it is not present, we'll fall back to calling the database directly.
-	cache, err := request.GetCtxValue[*RequestCache](ctx, request.CtxOpenFGARequestCache)
+	cache, err := request.GetContextValue[*RequestCache](ctx, request.CtxOpenFGARequestCache)
 	if err != nil {
 		entityURLs, err := o.getEntitiesOfTypeWhereGroupHasEntitlement(ctx, entityType, groupName, entitlement)
 		if err != nil {
@@ -793,12 +794,12 @@ WHERE auth_groups_permissions.entitlement = ? AND auth_groups_permissions.entity
 
 // ReadPage is not implemented. It is not required for the functionality we need.
 func (*openfgaStore) ReadPage(ctx context.Context, store string, tk *openfgav1.TupleKey, opts storage.ReadPageOptions) ([]*openfgav1.Tuple, string, error) {
-	return nil, "", api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, "", api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // Write is not implemented, we should never be performing writes because we are reading directly from the cluster DB.
 func (*openfgaStore) Write(ctx context.Context, store string, d storage.Deletes, w storage.Writes) error {
-	return api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // WriteAuthorizationModel sets the model.
@@ -831,7 +832,7 @@ func (o *openfgaStore) ReadAuthorizationModels(ctx context.Context, store string
 		return []*openfgav1.AuthorizationModel{o.model}, "", nil
 	}
 
-	return nil, "", fmt.Errorf("Authorization model not set")
+	return nil, "", errors.New("Authorization model not set")
 }
 
 // MaxTuplesPerWrite returns -1 because we should never be writing to the store.
@@ -847,37 +848,37 @@ func (*openfgaStore) MaxTypesPerAuthorizationModel() int {
 
 // CreateStore returns a not implemented error, because there is only one store.
 func (*openfgaStore) CreateStore(ctx context.Context, store *openfgav1.Store) (*openfgav1.Store, error) {
-	return nil, api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // DeleteStore returns a not implemented error, because there is only one store.
 func (*openfgaStore) DeleteStore(ctx context.Context, id string) error {
-	return api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // GetStore returns a not implemented error, because there is only one store.
 func (*openfgaStore) GetStore(ctx context.Context, id string) (*openfgav1.Store, error) {
-	return nil, api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // ListStores returns a not implemented error, because there is only one store.
 func (*openfgaStore) ListStores(ctx context.Context, paginationOptions storage.ListStoresOptions) ([]*openfgav1.Store, string, error) {
-	return nil, "", api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, "", api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // WriteAssertions returns a not implemented error, because we do not need to use the assertions API.
 func (*openfgaStore) WriteAssertions(ctx context.Context, store, modelID string, assertions []*openfgav1.Assertion) error {
-	return api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // ReadAssertions returns a not implemented error, because we do not need to use the assertions API.
 func (*openfgaStore) ReadAssertions(ctx context.Context, store, modelID string) ([]*openfgav1.Assertion, error) {
-	return nil, api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // ReadChanges returns a not implemented error, because we do not need to use the read changes API.
 func (*openfgaStore) ReadChanges(ctx context.Context, store string, filter storage.ReadChangesFilter, options storage.ReadChangesOptions) ([]*openfgav1.TupleChange, string, error) {
-	return nil, "", api.StatusErrorf(http.StatusNotImplemented, "not implemented")
+	return nil, "", api.NewGenericStatusError(http.StatusNotImplemented)
 }
 
 // IsReady returns true.
