@@ -8,7 +8,7 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db/query"
+	"github.com/canonical/lxd/lxd/db/query"
 )
 
 func TestDump(t *testing.T) {
@@ -99,7 +99,7 @@ func TestDumpTableStoragePoolsConfig(t *testing.T) {
 );`, dump["storage_pools_config"][1])
 	data, err := query.GetTableData(context.Background(), tx, "storage_pools_config")
 	require.NoError(t, err)
-	assert.Equal(t, data, []string{"INSERT INTO storage_pools_config VALUES(1,1,NULL,'k','v');"})
+	assert.Equal(t, []string{"INSERT INTO storage_pools_config VALUES(1,1,NULL,'k','v');"}, data)
 }
 
 // Return a new transaction against an in-memory SQLite database populated with
