@@ -7,7 +7,7 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // Transaction executes the given function within a database transaction with a 10s context timeout.
@@ -43,7 +43,7 @@ func Transaction(ctx context.Context, db *sql.DB, f func(context.Context, *sql.T
 // succeeds the given error is returned, otherwise a new error that wraps it
 // gets generated and returned.
 func rollback(tx *sql.Tx, reason error) error {
-	err := Retry(tx.Rollback)
+	err := Retry(context.TODO(), func(_ context.Context) error { return tx.Rollback() })
 	if err != nil {
 		logger.Warnf("Failed to rollback transaction after error (%v): %v", reason, err)
 	}
