@@ -3,12 +3,15 @@ package schema
 import (
 	"context"
 	"database/sql"
+	"errors"
 	"fmt"
+	"net/http"
+	"slices"
 	"sort"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/db/query"
-	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // Schema captures the schema of a database in terms of a series of ordered
@@ -137,7 +140,7 @@ func (s *Schema) File(path string) {
 // initial version that the schema has been upgraded from.
 func (s *Schema) Ensure(db *sql.DB) (int, error) {
 	var current int
-	aborted := false
+	var gracefulAbortErr error
 	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		err := execFromFile(ctx, tx, s.path, s.hook)
 		if err != nil {
@@ -156,14 +159,13 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 
 		if s.check != nil {
 			err := s.check(ctx, current, tx)
-			if err == ErrGracefulAbort {
-				// Abort the update gracefully, committing what
-				// we've done so far.
-				aborted = true
-				return nil
-			}
-
 			if err != nil {
+				if api.StatusErrorCheck(err, http.StatusPreconditionFailed) {
+					// Abort the update gracefully, committing what we've done so far.
+					gracefulAbortErr = err
+					return nil
+				}
+
 				return err
 			}
 		}
@@ -188,8 +190,8 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 		return -1, err
 	}
 
-	if aborted {
-		return current, ErrGracefulAbort
+	if gracefulAbortErr != nil {
+		return current, gracefulAbortErr
 	}
 
 	return current, nil
@@ -302,11 +304,11 @@ func queryCurrentVersion(ctx context.Context, tx *sql.Tx) (int, error) {
 	}
 
 	// Fix bad upgrade code between 30 and 32
-	hasVersion := func(v int) bool { return shared.ValueInSlice(v, versions) }
+	hasVersion := func(v int) bool { return slices.Contains(versions, v) }
 	if hasVersion(30) && hasVersion(32) && !hasVersion(31) {
 		err = insertSchemaVersion(tx, 31)
 		if err != nil {
-			return -1, fmt.Errorf("failed to insert missing schema version 31")
+			return -1, errors.New("failed to insert missing schema version 31")
 		}
 
 		versions, err = selectSchemaVersions(ctx, tx)
@@ -326,7 +328,7 @@ func queryCurrentVersion(ctx context.Context, tx *sql.Tx) (int, error) {
 			// Insert the missing version.
 			err := insertSchemaVersion(tx, 38)
 			if err != nil {
-				return -1, fmt.Errorf("Failed to insert missing schema version 38")
+				return -1, errors.New("Failed to insert missing schema version 38")
 			}
 
 			versions = append(versions, 38)
@@ -404,7 +406,7 @@ func checkAllUpdatesAreApplied(ctx context.Context, tx *sql.Tx, updates []Update
 	}
 
 	if len(versions) == 0 {
-		return fmt.Errorf("expected schema table to contain at least one row")
+		return errors.New("expected schema table to contain at least one row")
 	}
 
 	err = checkSchemaVersionsHaveNoHoles(versions)
@@ -433,7 +435,7 @@ func formatSQL(statement string) string {
 			continue
 		}
 
-		lines[i] = strings.Replace(line, ", ", ",\n    ", -1)
+		lines[i] = strings.ReplaceAll(line, ", ", ",\n    ")
 	}
 
 	return strings.Join(lines, "\n")
