@@ -10,9 +10,9 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
 )
 
 func TestGetInstanceSnapshots(t *testing.T) {
@@ -46,7 +46,7 @@ func TestGetInstanceSnapshots(t *testing.T) {
 	assert.Equal(t, "snap1", s1.Name)
 	assert.Equal(t, "c1", s1.Instance)
 	assert.Equal(t, map[string]string{}, s1Config)
-	assert.Len(t, s1Devices, 0)
+	assert.Empty(t, s1Devices)
 
 	s2Config, err := cluster.GetInstanceSnapshotConfig(context.TODO(), tx.Tx(), snapshots[1].ID)
 	require.NoError(t, err)
