@@ -9,7 +9,7 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db"
 )
 
 // Addresses of all nodes with matching volume name are returned.
