@@ -1,7 +1,9 @@
 package device
 
 import (
-	"github.com/lxc/lxd/lxd/instance/instancetype"
+	"slices"
+
+	"github.com/canonical/lxd/lxd/instance/instancetype"
 )
 
 // instanceSupported is a helper function to check instance type is supported for validation.
@@ -12,11 +14,5 @@ func instanceSupported(instType instancetype.Type, supportedTypes ...instancetyp
 		return true
 	}
 
-	for _, supportedType := range supportedTypes {
-		if instType == supportedType {
-			return true
-		}
-	}
-
-	return false
+	return slices.Contains(supportedTypes, instType)
 }
