@@ -21,6 +21,7 @@ import (
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/warningtype"
 	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/device/filters"
 	"github.com/canonical/lxd/lxd/idmap"
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/instance/instancetype"
@@ -148,29 +149,26 @@ func (d *disk) sourceIsLocalPath(source string) bool {
 	return true
 }
 
-func (d *disk) sourceVolumeFields() (volumeName string, volumeType storageDrivers.VolumeType, dbVolumeType int, volumeTypeName string, err error) {
+func (d *disk) sourceVolumeFields() (volumeName string, volumeType storageDrivers.VolumeType, dbVolumeType cluster.StoragePoolVolumeType, err error) {
 	volumeName = d.config["source"]
 
 	if d.config["source.snapshot"] != "" {
 		volumeName = volumeName + shared.SnapshotDelimiter + d.config["source.snapshot"]
 	}
 
-	volumeTypeName = cluster.StoragePoolVolumeTypeNameCustom
+	volumeTypeName := cluster.StoragePoolVolumeTypeNameCustom
 	if d.config["source.type"] != "" {
 		volumeTypeName = d.config["source.type"]
 	}
 
-	dbVolumeType, err = storagePools.VolumeTypeNameToDBType(volumeTypeName)
+	dbVolumeType, err = cluster.StoragePoolVolumeTypeFromName(volumeTypeName)
 	if err != nil {
-		return volumeName, volumeType, dbVolumeType, volumeTypeName, err
+		return volumeName, volumeType, dbVolumeType, err
 	}
 
-	volumeType, err = storagePools.VolumeDBTypeToType(dbVolumeType)
-	if err != nil {
-		return volumeName, volumeType, dbVolumeType, volumeTypeName, err
-	}
+	volumeType = storagePools.VolumeDBTypeToType(dbVolumeType)
 
-	return volumeName, volumeType, dbVolumeType, volumeTypeName, nil
+	return volumeName, volumeType, dbVolumeType, nil
 }
 
 // Check that unshared custom storage block volumes are not added to profiles or
@@ -182,10 +180,10 @@ func (d *disk) checkBlockVolSharing(instanceType instancetype.Type, projectName
 	}
 
 	if instanceType == instancetype.Any {
-		return fmt.Errorf("Cannot add block volume to profiles if security.shared is false or unset")
+		return errors.New("Cannot add block volume to profiles if security.shared is false or unset")
 	}
 
-	err := storagePools.VolumeUsedByInstanceDevices(d.state, d.pool.Name(), projectName, volume, true, func(inst db.InstanceArgs, project api.Project, usedByDevices []string) error {
+	return storagePools.VolumeUsedByInstanceDevices(d.state, d.pool.Name(), projectName, volume, true, func(inst db.InstanceArgs, project api.Project, usedByDevices []string) error {
 		// Don't count the current instance.
 		if d.inst != nil && d.inst.Project().Name == inst.Project && d.inst.Name() == inst.Name {
 			return nil
@@ -206,13 +204,8 @@ func (d *disk) checkBlockVolSharing(instanceType instancetype.Type, projectName
 			}
 		}
 
-		return fmt.Errorf("Cannot add block volume to more than one instance if security.shared is false or unset")
+		return errors.New("Cannot add block volume to more than one instance if security.shared is false or unset")
 	})
-	if err != nil {
-		return err
-	}
-
-	return nil
 }
 
 // validateConfig checks the supplied config for correctness.
@@ -226,7 +219,7 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 	// These come from https://www.kernel.org/doc/Documentation/filesystems/sharedsubtree.txt
 	propagationTypes := []string{"", "private", "shared", "slave", "unbindable", "rshared", "rslave", "runbindable", "rprivate"}
 	validatePropagation := func(input string) error {
-		if !shared.ValueInSlice(d.config["bind"], propagationTypes) {
+		if !slices.Contains(propagationTypes, d.config["bind"]) {
 			return fmt.Errorf("Invalid propagation value. Must be one of: %s", strings.Join(propagationTypes, ", "))
 		}
 
@@ -412,6 +405,16 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 		//  condition: virtual machine
 		//  shortdesc: Bus for the device
 		"io.bus": validate.Optional(validate.IsOneOf("nvme", "virtio-blk", "virtio-scsi")),
+		// lxdmeta:generate(entities=device-disk; group=device-conf; key=io.threads)
+		// This option controls the `virtiofsd` thread pool size, which can help improve I/O performance. Only applies to virtiofs file system shares.
+		// In {config:option}`project-restricted:restricted` projects, it can only be used when {config:option}`project-restricted:restricted.virtual-machines.lowlevel` is set to `allow`.
+		// ---
+		//  type: integer
+		//  defaultdesc: `0`
+		//  required: no
+		//  condition: virtual machine
+		//  shortdesc: Thread pool for virtiofs file system shares
+		"io.threads": validate.Optional(validate.IsUint16),
 	}
 
 	err := d.config.Validate(rules)
@@ -420,55 +423,55 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 	}
 
 	if instConf.Type() == instancetype.Container && d.config["io.bus"] != "" {
-		return fmt.Errorf("IO bus configuration cannot be applied to containers")
+		return errors.New("IO bus configuration cannot be applied to containers")
 	}
 
 	if instConf.Type() == instancetype.Container && d.config["io.cache"] != "" {
-		return fmt.Errorf("IO cache configuration cannot be applied to containers")
+		return errors.New("IO cache configuration cannot be applied to containers")
 	}
 
 	if d.config["required"] != "" && d.config["optional"] != "" {
-		return fmt.Errorf(`Cannot use both "required" and deprecated "optional" properties at the same time`)
+		return errors.New(`Cannot use both "required" and deprecated "optional" properties at the same time`)
 	}
 
 	if d.config["source.snapshot"] != "" && (d.config["pool"] == "" || d.config["path"] == "/") {
-		return fmt.Errorf(`"source.snapshot" can only be used on storage volume disk devices`)
+		return errors.New(`"source.snapshot" can only be used on storage volume disk devices`)
 	}
 
 	if d.config["source.type"] != "" && d.config["pool"] == "" {
-		return fmt.Errorf(`"source.type" can only be used on storage volume disk devices`)
+		return errors.New(`"source.type" can only be used on storage volume disk devices`)
 	}
 
 	if d.config["source"] == "" && d.config["path"] != "/" {
-		return fmt.Errorf(`Non root disk devices require the "source" property`)
+		return errors.New(`Non root disk devices require the "source" property`)
 	}
 
 	if d.config["path"] == "/" && d.config["source"] != "" {
-		return fmt.Errorf(`Root disk entry may not have a "source" property set`)
+		return errors.New(`Root disk entry may not have a "source" property set`)
 	}
 
 	if d.config["path"] == "/" && d.config["pool"] == "" {
-		return fmt.Errorf(`Root disk entry must have a "pool" property set`)
+		return errors.New(`Root disk entry must have a "pool" property set`)
 	}
 
 	if d.config["size"] != "" && d.config["path"] != "/" {
-		return fmt.Errorf("Only the root disk may have a size quota")
+		return errors.New("Only the root disk may have a size quota")
 	}
 
 	if d.config["size.state"] != "" && d.config["path"] != "/" {
-		return fmt.Errorf("Only the root disk may have a migration size quota")
+		return errors.New("Only the root disk may have a migration size quota")
 	}
 
 	if d.config["recursive"] != "" && (d.config["path"] == "/" || !shared.IsDir(shared.HostPath(d.config["source"]))) {
-		return fmt.Errorf("The recursive option is only supported for additional bind-mounted paths")
+		return errors.New("The recursive option is only supported for additional bind-mounted paths")
 	}
 
 	if shared.IsTrue(d.config["recursive"]) && shared.IsTrue(d.config["readonly"]) {
-		return fmt.Errorf("Recursive read-only bind-mounts aren't currently supported by the kernel")
+		return errors.New("Recursive read-only bind-mounts aren't currently supported by the kernel")
 	}
 
 	// Check ceph options are only used when ceph or cephfs type source is specified.
-	if !(d.sourceIsCeph() || d.sourceIsCephFs()) && (d.config["ceph.cluster_name"] != "" || d.config["ceph.user_name"] != "") {
+	if !d.sourceIsCeph() && !d.sourceIsCephFs() && (d.config["ceph.cluster_name"] != "" || d.config["ceph.user_name"] != "") {
 		return fmt.Errorf("Invalid options ceph.cluster_name/ceph.user_name for source %q", d.config["source"])
 	}
 
@@ -491,7 +494,7 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 	srcPathIsAbs := filepath.IsAbs(d.config["source"])
 
 	if srcPathIsLocal && !srcPathIsAbs {
-		return fmt.Errorf("Source path must be absolute for local sources")
+		return errors.New("Source path must be absolute for local sources")
 	}
 
 	// Check that external disk source path exists. External disk sources have a non-empty "source" property
@@ -506,26 +509,26 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 	// Check if validating a storage volume disk.
 	if d.config["pool"] != "" {
 		if d.config["shift"] != "" {
-			return fmt.Errorf(`The "shift" property cannot be used with custom storage volumes (set "security.shifted=true" on the volume instead)`)
+			return errors.New(`The "shift" property cannot be used with custom storage volumes (set "security.shifted=true" on the volume instead)`)
 		}
 
 		if srcPathIsAbs {
-			return fmt.Errorf("Storage volumes cannot be specified as absolute paths")
+			return errors.New("Storage volumes cannot be specified as absolute paths")
 		}
 
 		var dbCustomVolume *db.StorageVolume
 		var storageProjectName string
 
 		// Check if validating an instance or a custom storage volume attached to a profile.
-		if (d.inst != nil && !d.inst.IsSnapshot()) || (d.inst == nil && instConf.Type() == instancetype.Any && !instancetype.IsRootDiskDevice(d.config)) {
+		if (d.inst != nil && !d.inst.IsSnapshot()) || (d.inst == nil && instConf.Type() == instancetype.Any && !filters.IsRootDisk(d.config)) {
 			d.pool, err = storagePools.LoadByName(d.state, d.config["pool"])
 			if err != nil {
 				return fmt.Errorf("Failed to get storage pool %q: %w", d.config["pool"], err)
 			}
 
 			// Non-root volume validation.
-			if !instancetype.IsRootDiskDevice(d.config) {
-				volumeName, volumeType, dbVolumeType, _, err := d.sourceVolumeFields()
+			if !filters.IsRootDisk(d.config) {
+				volumeName, volumeType, dbVolumeType, err := d.sourceVolumeFields()
 				if err != nil {
 					return err
 				}
@@ -577,28 +580,28 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 				}
 
 				if remoteInstance != nil && remoteInstance.ID != instConf.ID() {
-					return fmt.Errorf("Custom volume is already attached to an instance on a different cluster member")
+					return errors.New("Custom volume is already attached to an instance on a different cluster member")
 				}
 
 				// Check that block volumes are *only* attached to VM instances.
 				if dbCustomVolume.ContentType == cluster.StoragePoolVolumeContentTypeNameBlock {
 					if instConf.Type() == instancetype.Container {
-						return fmt.Errorf("Custom block volumes cannot be used on containers")
+						return errors.New("Custom block volumes cannot be used on containers")
 					}
 
 					if d.config["path"] != "" {
-						return fmt.Errorf("Custom block volumes cannot have a path defined")
+						return errors.New("Custom block volumes cannot have a path defined")
 					}
 				} else if dbCustomVolume.ContentType == cluster.StoragePoolVolumeContentTypeNameISO {
 					if instConf.Type() == instancetype.Container {
-						return fmt.Errorf("Custom ISO volumes cannot be used on containers")
+						return errors.New("Custom ISO volumes cannot be used on containers")
 					}
 
 					if d.config["path"] != "" {
-						return fmt.Errorf("Custom ISO volumes cannot have a path defined")
+						return errors.New("Custom ISO volumes cannot have a path defined")
 					}
 				} else if d.config["path"] == "" {
-					return fmt.Errorf("Custom filesystem volumes require a path to be defined")
+					return errors.New("Custom filesystem volumes require a path to be defined")
 				}
 			}
 
@@ -613,8 +616,8 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 			}
 
 			if len(initialConfig) > 0 {
-				if !instancetype.IsRootDiskDevice(d.config) {
-					return fmt.Errorf("Non-root disk device cannot contain initial.* configuration")
+				if !filters.IsRootDisk(d.config) {
+					return errors.New("Non-root disk device cannot contain initial.* configuration")
 				}
 
 				volumeType, err := storagePools.InstanceTypeToVolumeType(d.inst.Type())
@@ -643,19 +646,19 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 	// Restrict disks allowed when live-migratable.
 	if instConf.Type() == instancetype.VM && shared.IsTrue(instConf.ExpandedConfig()["migration.stateful"]) {
 		if d.config["path"] != "" && d.config["path"] != "/" {
-			return fmt.Errorf("Shared filesystem are incompatible with migration.stateful=true")
+			return errors.New("Shared filesystem are incompatible with migration.stateful=true")
 		}
 
 		if d.config["pool"] == "" {
-			return fmt.Errorf("Only LXD-managed disks are allowed with migration.stateful=true")
+			return errors.New("Only LXD-managed disks are allowed with migration.stateful=true")
 		}
 
 		if d.config["io.bus"] == "nvme" {
-			return fmt.Errorf("NVME disks aren't supported with migration.stateful=true")
+			return errors.New("NVME disks aren't supported with migration.stateful=true")
 		}
 
 		if d.config["path"] != "/" && d.pool != nil && !d.pool.Driver().Info().Remote {
-			return fmt.Errorf("Only additional disks coming from a shared storage pool are supported with migration.stateful=true")
+			return errors.New("Only additional disks coming from a shared storage pool are supported with migration.stateful=true")
 		}
 	}
 
@@ -702,7 +705,7 @@ func (d *disk) validateEnvironmentSourcePath() error {
 			}
 
 			if shared.IsTrue(d.config["shift"]) {
-				return fmt.Errorf(`The "shift" property cannot be used with a restricted source path`)
+				return errors.New(`The "shift" property cannot be used with a restricted source path`)
 			}
 
 			d.restrictedParentSourcePath = shared.HostPath(restrictedParentSourcePath)
@@ -754,7 +757,7 @@ func (d *disk) Register() error {
 			return err
 		}
 	} else if d.config["path"] != "/" && d.config["source"] != "" && d.config["pool"] != "" {
-		volumeName, _, dbVolumeType, volumeTypeName, err := d.sourceVolumeFields()
+		volumeName, _, dbVolumeType, err := d.sourceVolumeFields()
 		if err != nil {
 			return err
 		}
@@ -781,7 +784,7 @@ func (d *disk) Register() error {
 		} else {
 			_, err = d.pool.MountCustomVolume(storageProjectName, volumeName, nil)
 			if err != nil {
-				return fmt.Errorf(`Failed mounting storage volume "%s/%s": %w`, volumeTypeName, volumeName, err)
+				return fmt.Errorf(`Failed mounting storage volume "%s/%s": %w`, dbVolumeType, volumeName, err)
 			}
 		}
 	}
@@ -862,7 +865,7 @@ func (d *disk) startContainer() (*deviceConfig.RunConfig, error) {
 	defer revert.Fail()
 
 	// Deal with a rootfs.
-	if instancetype.IsRootDiskDevice(d.config) {
+	if filters.IsRootDisk(d.config) {
 		// Set the rootfs path.
 		rootfs := deviceConfig.RootFSEntryItem{
 			Path: d.inst.RootfsPath(),
@@ -899,7 +902,7 @@ func (d *disk) startContainer() (*deviceConfig.RunConfig, error) {
 		// If ownerShift is none and pool is specified then check whether the volume
 		// has owner shifting enabled, and if so enable shifting on this device too.
 		if ownerShift == deviceConfig.MountOwnerShiftNone && d.config["pool"] != "" {
-			volumeName, _, dbVolumeType, _, err := d.sourceVolumeFields()
+			volumeName, _, dbVolumeType, err := d.sourceVolumeFields()
 			if err != nil {
 				return nil, err
 			}
@@ -979,7 +982,7 @@ func (d *disk) startContainer() (*deviceConfig.RunConfig, error) {
 		// Instruct LXD to perform the mount.
 		runConf.Mounts = append(runConf.Mounts, deviceConfig.MountEntryItem{
 			DevName:    d.name,
-			DevSource:  DevSourcePath{Path: sourceDevPath},
+			DevSource:  deviceConfig.DevSourcePath{Path: sourceDevPath},
 			TargetPath: relativeDestPath,
 			FSType:     "none",
 			Opts:       options,
@@ -1003,10 +1006,9 @@ func (d *disk) vmVirtfsProxyHelperPaths() string {
 
 // vmVirtiofsdPaths returns the path for the socket and PID file to use with virtiofsd process.
 func (d *disk) vmVirtiofsdPaths() (sockPath string, pidPath string) {
-	sockPath = filepath.Join(d.inst.DevicesPath(), "virtio-fs."+filesystem.PathNameEncode(d.name)+".sock")
-	pidPath = filepath.Join(d.inst.DevicesPath(), "virtio-fs."+filesystem.PathNameEncode(d.name)+".pid")
+	prefix := filepath.Join(d.inst.DevicesPath(), "virtio-fs."+filesystem.PathNameEncode(d.name))
 
-	return sockPath, pidPath
+	return prefix + ".sock", prefix + ".pid"
 }
 
 func (d *disk) detectVMPoolMountOpts() []string {
@@ -1073,7 +1075,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 		}
 	}
 
-	if instancetype.IsRootDiskDevice(d.config) {
+	if filters.IsRootDisk(d.config) {
 		// Handle previous requests for setting new quotas.
 		err := d.applyDeferredQuota()
 		if err != nil {
@@ -1112,7 +1114,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 		// Encode the file descriptor and original isoPath into the DevPath field.
 		runConf.Mounts = []deviceConfig.MountEntryItem{
 			{
-				DevSource: DevSourceFD{FD: f.Fd(), Path: isoPath},
+				DevSource: deviceConfig.DevSourceFD{FD: f.Fd(), Path: isoPath},
 				DevName:   d.name,
 				FSType:    "iso9660",
 				Opts:      opts,
@@ -1129,7 +1131,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 			clusterName, userName := d.cephCreds()
 			runConf.Mounts = []deviceConfig.MountEntryItem{
 				{
-					DevSource: DevSourceRBD{
+					DevSource: deviceConfig.DevSourceRBD{
 						ClusterName: clusterName,
 						UserName:    userName,
 						PoolName:    fields[0],
@@ -1143,7 +1145,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 		} else {
 			// Default to block device or image file passthrough first.
 			mount := deviceConfig.MountEntryItem{
-				DevSource: DevSourcePath{Path: shared.HostPath(d.config["source"])},
+				DevSource: deviceConfig.DevSourcePath{Path: shared.HostPath(d.config["source"])},
 				DevName:   d.name,
 				Opts:      opts,
 				Limits:    diskLimits,
@@ -1155,7 +1157,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 			if d.config["pool"] != "" {
 				var revertFunc func()
 
-				volumeName, volumeType, dbVolumeType, _, err := d.sourceVolumeFields()
+				volumeName, volumeType, dbVolumeType, err := d.sourceVolumeFields()
 				if err != nil {
 					return nil, err
 				}
@@ -1174,7 +1176,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					return nil, fmt.Errorf("Failed loading custom volume: %w", err)
 				}
 
-				dbContentType, err := storagePools.VolumeContentTypeNameToContentType(dbVolume.ContentType)
+				dbContentType, err := cluster.StoragePoolVolumeContentTypeFromName(dbVolume.ContentType)
 				if err != nil {
 					return nil, err
 				}
@@ -1183,6 +1185,16 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					mount.FSType = "iso9660"
 				}
 
+				revertFunc, mountedPath, _, err := d.mountPoolVolume()
+				if err != nil {
+					return nil, diskSourceNotFoundError{msg: "Failed mounting volume", err: err}
+				}
+
+				mount.DevSource = deviceConfig.DevSourcePath{Path: mountedPath}
+				mount.Opts = append(mount.Opts, d.detectVMPoolMountOpts()...)
+
+				revert.Add(revertFunc)
+
 				// If the pool is ceph backed and a block device, don't mount it, instead pass config to QEMU instance
 				// to use the built in RBD support.
 				if d.pool.Driver().Info().Name == "ceph" && (dbContentType == cluster.StoragePoolVolumeContentTypeBlock || dbContentType == cluster.StoragePoolVolumeContentTypeISO) {
@@ -1199,57 +1211,40 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 						clusterName = storageDrivers.CephDefaultUser
 					}
 
-					contentType, err := storagePools.VolumeDBContentTypeToContentType(dbContentType)
-					if err != nil {
-						return nil, err
+					contentType := storagePools.VolumeDBContentTypeToContentType(dbContentType)
+
+					var volStorageName string
+					if dbVolume.Type == cluster.StoragePoolVolumeTypeNameCustom {
+						volStorageName = project.StorageVolume(storageProjectName, volumeName)
+					} else {
+						volStorageName = project.Instance(storageProjectName, volumeName)
 					}
 
-					projectStorageVolumeName := project.StorageVolume(d.inst.Project().Name, volumeName)
-
-					vol := d.pool.GetVolume(volumeType, contentType, projectStorageVolumeName, dbVolume.Config)
+					vol := d.pool.GetVolume(volumeType, contentType, volStorageName, dbVolume.Config)
 					rbdImageName, snapName := storageDrivers.CephGetRBDImageName(vol, false)
 
-					mount := deviceConfig.MountEntryItem{
-						DevSource: DevSourceRBD{
-							ClusterName: clusterName,
-							UserName:    userName,
-							PoolName:    poolName,
-							ImageName:   rbdImageName,
-							Snapshot:    snapName,
-						},
-						DevName: d.name,
-						Opts:    opts,
-						Limits:  diskLimits,
-					}
-
-					if dbContentType == cluster.StoragePoolVolumeContentTypeISO {
-						mount.FSType = "iso9660"
+					mount.DevSource = deviceConfig.DevSourceRBD{
+						ClusterName: clusterName,
+						UserName:    userName,
+						PoolName:    poolName,
+						ImageName:   rbdImageName,
+						Snapshot:    snapName,
 					}
 
 					runConf.Mounts = []deviceConfig.MountEntryItem{mount}
 
+					revert.Success()
 					return &runConf, nil
 				}
-
-				revertFunc, mountedPath, _, err := d.mountPoolVolume()
-				if err != nil {
-					return nil, diskSourceNotFoundError{msg: "Failed mounting volume", err: err}
-				}
-
-				mount.DevSource = DevSourcePath{Path: mountedPath}
-
-				revert.Add(revertFunc)
-
-				mount.Opts = append(mount.Opts, d.detectVMPoolMountOpts()...)
 			}
 
 			// If the source being added is a directory or cephfs share, then we will use the lxd-agent
 			// directory sharing feature to mount the directory inside the VM, and as such we need to
 			// indicate to the VM the target path to mount to.
-			pathSource, isPath := mount.DevSource.(DevSourcePath)
+			pathSource, isPath := mount.DevSource.(deviceConfig.DevSourcePath)
 			if (isPath && shared.IsDir(pathSource.Path)) || d.sourceIsCephFs() {
 				if d.config["path"] == "" {
-					return nil, fmt.Errorf(`Missing mount "path" setting`)
+					return nil, errors.New(`Missing mount "path" setting`)
 				}
 
 				// Mount the source in the instance devices directory.
@@ -1262,7 +1257,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					return nil, err
 				}
 
-				mount.DevSource = DevSourcePath{Path: mountedPath}
+				mount.DevSource = deviceConfig.DevSourcePath{Path: mountedPath}
 
 				revert.Add(revertFunc)
 
@@ -1283,6 +1278,17 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					rawIDMaps = diskAddRootUserNSEntry(rawIDMaps, 65534)
 				}
 
+				// Parse the io.threads setting.
+				var virtiofsdThreadPoolSize uint16
+				if d.config["io.threads"] != "" {
+					virtiofsdThreadPoolSizeRaw, err := strconv.ParseUint(d.config["io.threads"], 10, 16)
+					if err != nil {
+						return nil, fmt.Errorf("Failed parsing io.threads %q: %w", d.config["io.threads"], err)
+					}
+
+					virtiofsdThreadPoolSize = uint16(virtiofsdThreadPoolSizeRaw)
+				}
+
 				// Start virtiofsd for virtio-fs share. The lxd-agent prefers to use this over the
 				// virtfs-proxy-helper 9p share. The 9p share will only be used as a fallback.
 				err = func() error {
@@ -1290,7 +1296,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					logPath := filepath.Join(d.inst.LogPath(), "disk."+filesystem.PathNameEncode(d.name)+".log")
 					_ = os.Remove(logPath) // Remove old log if needed.
 
-					revertFunc, unixListener, err := DiskVMVirtiofsdStart(d.state.OS.KernelVersion, d.inst, sockPath, pidPath, logPath, mountedPath, rawIDMaps)
+					revertFunc, unixListener, err := DiskVMVirtiofsdStart(d.state.OS.KernelVersion, d.inst, sockPath, pidPath, logPath, mountedPath, rawIDMaps, virtiofsdThreadPoolSize)
 					if err != nil {
 						var errUnsupported UnsupportedError
 						if errors.As(err, &errUnsupported) {
@@ -1353,7 +1359,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 						runConf.PostHooks = append(runConf.PostHooks, unixListener.Close)
 
 						// Use 9p socket FD number as dev path so qemu can connect to the proxy.
-						mount.DevSource = DevSourceFD{FD: unixListener.Fd()}
+						mount.DevSource = deviceConfig.DevSourceFD{FD: unixListener.Fd()}
 
 						return nil
 					}()
@@ -1377,9 +1383,9 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					mount.FSType = "iso9660"
 				}
 
-				mount.DevSource = DevSourceFD{FD: f.Fd(), Path: pathSource.Path}
+				mount.DevSource = deviceConfig.DevSourceFD{FD: f.Fd(), Path: pathSource.Path}
 			} else {
-				return nil, fmt.Errorf("Unexpected DevSource for runConf.Mount; expected %T, got %T", DevSourcePath{}, mount.DevSource)
+				return nil, fmt.Errorf("Unexpected DevSource for runConf.Mount; expected %T, got %T", deviceConfig.DevSourcePath{}, mount.DevSource)
 			}
 
 			// Add successfully setup mount config to runConf.
@@ -1390,7 +1396,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 		return &runConf, nil
 	}
 
-	return nil, fmt.Errorf("Disk type not supported for VMs")
+	return nil, errors.New("Disk type not supported for VMs")
 }
 
 // postStart is run after the instance is started.
@@ -1408,7 +1414,7 @@ func (d *disk) postStart() error {
 
 // Update applies configuration changes to a started device.
 func (d *disk) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
-	if instancetype.IsRootDiskDevice(d.config) {
+	if filters.IsRootDisk(d.config) {
 		// Make sure we have a valid root disk device (and only one).
 		expandedDevices := d.inst.ExpandedDevices()
 		newRootDiskDeviceKey, _, err := instancetype.GetRootDiskDevice(expandedDevices.CloneNative())
@@ -1419,7 +1425,7 @@ func (d *disk) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 		// Retrieve the first old root disk device key, even if there are duplicates.
 		oldRootDiskDeviceKey := ""
 		for k, v := range oldDevices {
-			if instancetype.IsRootDiskDevice(v) {
+			if filters.IsRootDisk(v) {
 				oldRootDiskDeviceKey = k
 				break
 			}
@@ -1429,7 +1435,7 @@ func (d *disk) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 		oldRootDiskDevicePool := oldDevices[oldRootDiskDeviceKey]["pool"]
 		newRootDiskDevicePool := expandedDevices[newRootDiskDeviceKey]["pool"]
 		if oldRootDiskDevicePool != newRootDiskDevicePool {
-			return fmt.Errorf("The storage pool of the root disk can only be changed through move")
+			return errors.New("The storage pool of the root disk can only be changed through move")
 		}
 
 		// Deal with quota changes.
@@ -1577,11 +1583,7 @@ func (d *disk) applyQuota(remount bool) error {
 func (d *disk) generateLimits(runConf *deviceConfig.RunConfig) error {
 	// Disk throttle limits.
 	hasDiskLimits := false
-	for _, dev := range d.inst.ExpandedDevices() {
-		if dev["type"] != "disk" {
-			continue
-		}
-
+	for _, dev := range d.inst.ExpandedDevices().Filter(filters.IsDisk) {
 		if dev["limits.read"] != "" || dev["limits.write"] != "" || dev["limits.max"] != "" {
 			hasDiskLimits = true
 		}
@@ -1589,7 +1591,7 @@ func (d *disk) generateLimits(runConf *deviceConfig.RunConfig) error {
 
 	if hasDiskLimits {
 		if !d.state.OS.CGInfo.Supports(cgroup.Blkio, nil) {
-			return fmt.Errorf("Cannot apply disk limits as blkio cgroup controller is missing")
+			return errors.New("Cannot apply disk limits as blkio cgroup controller is missing")
 		}
 
 		diskLimits, err := d.getDiskLimits()
@@ -1642,7 +1644,7 @@ type cgroupWriter struct {
 
 // Get returns the cgroup's controller key.
 func (w *cgroupWriter) Get(version cgroup.Backend, controller string, key string) (string, error) {
-	return "", fmt.Errorf("This cgroup handler does not support reading")
+	return "", errors.New("This cgroup handler does not support reading")
 }
 
 // Set applies the cgroup's controller key value.
@@ -1670,10 +1672,10 @@ func (d *disk) mountPoolVolume() (func(), string, *storagePools.MountInfo, error
 	var mountInfo *storagePools.MountInfo
 
 	if filepath.IsAbs(d.config["source"]) {
-		return nil, "", nil, fmt.Errorf(`When the "pool" property is set "source" must specify the name of a volume, not a path`)
+		return nil, "", nil, errors.New(`When the "pool" property is set "source" must specify the name of a volume, not a path`)
 	}
 
-	volumeName, volumeType, dbVolumeType, volumeTypeName, err := d.sourceVolumeFields()
+	volumeName, volumeType, dbVolumeType, err := d.sourceVolumeFields()
 	if err != nil {
 		return nil, "", nil, err
 	}
@@ -1707,7 +1709,7 @@ func (d *disk) mountPoolVolume() (func(), string, *storagePools.MountInfo, error
 	} else {
 		mountInfo, err = d.pool.MountCustomVolume(storageProjectName, volumeName, nil)
 		if err != nil {
-			return nil, "", nil, fmt.Errorf(`Failed mounting storage volume "%s/%s" from storage pool %q: %w`, volumeTypeName, volumeName, d.pool.Name(), err)
+			return nil, "", nil, fmt.Errorf(`Failed mounting storage volume "%s/%s" from storage pool %q: %w`, dbVolumeType, volumeName, d.pool.Name(), err)
 		}
 
 		revert.Add(func() { _, _ = d.pool.UnmountCustomVolume(storageProjectName, volumeName, nil) })
@@ -1733,12 +1735,12 @@ func (d *disk) mountPoolVolume() (func(), string, *storagePools.MountInfo, error
 
 	if d.inst.Type() == instancetype.Container {
 		if dbVolume.ContentType != cluster.StoragePoolVolumeContentTypeNameFS {
-			return nil, "", nil, fmt.Errorf("Only filesystem volumes are supported for containers")
+			return nil, "", nil, errors.New("Only filesystem volumes are supported for containers")
 		}
 
 		err = d.storagePoolVolumeAttachShift(storageProjectName, d.pool.Name(), volumeName, dbVolumeType, srcPath)
 		if err != nil {
-			return nil, "", nil, fmt.Errorf(`Failed shifting storage volume "%s/%s" on storage pool %q: %w`, volumeTypeName, volumeName, d.pool.Name(), err)
+			return nil, "", nil, fmt.Errorf(`Failed shifting storage volume "%s/%s" on storage pool %q: %w`, dbVolumeType, volumeName, d.pool.Name(), err)
 		}
 	}
 
@@ -1945,7 +1947,7 @@ func (d *disk) localSourceOpen(srcPath string) (*os.File, error) {
 	return f, nil
 }
 
-func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName string, volumeType int, remapPath string) error {
+func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName string, volumeType cluster.StoragePoolVolumeType, remapPath string) error {
 	var err error
 	var dbVolume *db.StorageVolume
 	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -1956,7 +1958,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 		return err
 	}
 
-	poolVolumePut := dbVolume.StorageVolume.Writable()
+	poolVolumePut := dbVolume.Writable()
 
 	// Check if unmapped.
 	if shared.IsTrue(poolVolumePut.Config["security.unmapped"]) {
@@ -2050,7 +2052,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 					}
 
 					if err != nil {
-						return fmt.Errorf("Failed to retrieve idmap of container")
+						return errors.New("Failed to retrieve idmap of container")
 					}
 
 					if !nextIdmap.Equals(ctNextIdmap) {
@@ -2062,7 +2064,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 				// we can shift the storage volume.
 				// I'm not sure if we want some locking here.
 				if volumeUsedBy[0].Name() != d.inst.Name() {
-					return fmt.Errorf("Idmaps of container and storage volume are not identical")
+					return errors.New("Idmaps of container and storage volume are not identical")
 				}
 			}
 		}
@@ -2186,7 +2188,7 @@ func (d *disk) postStop() error {
 
 	// Check if pool-specific action should be taken to unmount custom volume disks.
 	if d.config["pool"] != "" && d.config["path"] != "/" {
-		volumeName, _, dbVolumeType, _, err := d.sourceVolumeFields()
+		volumeName, _, dbVolumeType, err := d.sourceVolumeFields()
 		if err != nil {
 			return err
 		}
@@ -2259,11 +2261,7 @@ func (d *disk) getDiskLimits() (map[string]diskBlockLimit, error) {
 
 	// Process all the limits
 	blockLimits := map[string][]diskBlockLimit{}
-	for devName, dev := range d.inst.ExpandedDevices() {
-		if dev["type"] != "disk" {
-			continue
-		}
-
+	for devName, dev := range d.inst.ExpandedDevices().Filter(filters.IsDisk) {
 		// Parse the user input
 		readBps, readIops, writeBps, writeIops, err := d.parseLimit(dev)
 		if err != nil {
@@ -2300,14 +2298,14 @@ func (d *disk) getDiskLimits() (map[string]diskBlockLimit, error) {
 		for _, block := range blocks {
 			blockStr := ""
 
-			if shared.ValueInSlice(block, validBlocks) {
+			if slices.Contains(validBlocks, block) {
 				// Straightforward entry (full block device)
 				blockStr = block
 			} else {
 				// Attempt to deal with a partition (guess its parent)
 				fields := strings.SplitN(block, ":", 2)
 				fields[1] = "0"
-				if shared.ValueInSlice(fields[0]+":"+fields[1], validBlocks) {
+				if slices.Contains(validBlocks, fields[0]+":"+fields[1]) {
 					blockStr = fields[0] + ":" + fields[1]
 				}
 			}
@@ -2465,7 +2463,7 @@ func (d *disk) getParentBlocks(path string) ([]string, error) {
 	}
 
 	if dev == nil {
-		return nil, fmt.Errorf("Couldn't find a match /proc/self/mountinfo entry")
+		return nil, errors.New("Couldn't find a match /proc/self/mountinfo entry")
 	}
 
 	// Handle the most simple case
@@ -2487,7 +2485,7 @@ func (d *disk) getParentBlocks(path string) ([]string, error) {
 		}
 
 		header := true
-		for _, line := range strings.Split(output, "\n") {
+		for line := range strings.SplitSeq(output, "\n") {
 			fields := strings.Fields(line)
 			if len(fields) < 5 {
 				continue
@@ -2544,7 +2542,7 @@ func (d *disk) getParentBlocks(path string) ([]string, error) {
 			devices = append(devices, fmt.Sprint(major, ":", minor))
 		}
 
-		for _, line := range strings.Split(output, "\n") {
+		for line := range strings.SplitSeq(output, "\n") {
 			fields := strings.Fields(line)
 			if len(fields) == 0 || fields[0] != "devid" {
 				continue
@@ -2637,8 +2635,8 @@ func (d *disk) generateVMConfigDrive() (string, error) {
 	// The added values are single quoted to prevent rendering `meta-data` unparseable.
 	// Single quotes included in the value itself are escaped by being replaced with `''`.
 	for key, value := range instanceConfig {
-		if strings.HasPrefix(key, "user.") && !shared.ValueInSlice(key, excludedKeys) {
-			metaDataBuilder.WriteString(key + ": '" + strings.Replace(value, "'", "''", -1) + "'\n")
+		if strings.HasPrefix(key, "user.") && !slices.Contains(excludedKeys, key) {
+			metaDataBuilder.WriteString(key + ": '" + strings.ReplaceAll(value, "'", "''") + "'\n")
 		}
 	}
 
