@@ -1,8 +1,10 @@
 package device
 
 import (
+	"errors"
 	"fmt"
 	"path/filepath"
+	"strconv"
 
 	deviceConfig "github.com/canonical/lxd/lxd/device/config"
 	pcidev "github.com/canonical/lxd/lxd/device/pci"
@@ -46,7 +48,7 @@ func (d *pci) validateConfig(instConf instance.ConfigReader) error {
 // validateEnvironment checks if the PCI device is available.
 func (d *pci) validateEnvironment() error {
 	if d.inst.Type() == instancetype.VM && shared.IsTrue(d.inst.ExpandedConfig()["migration.stateful"]) {
-		return fmt.Errorf("PCI devices cannot be used when migration.stateful is enabled")
+		return errors.New("PCI devices cannot be used when migration.stateful is enabled")
 	}
 
 	return validatePCIDevice(d.config["address"])
@@ -79,6 +81,11 @@ func (d *pci) Start() (*deviceConfig.RunConfig, error) {
 	saveData["last_state.pci.slot.name"] = pciDev.SlotName
 	saveData["last_state.pci.driver"] = pciDev.Driver
 
+	pciIOMMUGroup, err := pcidev.DeviceIOMMUGroup(saveData["last_state.pci.slot.name"])
+	if err != nil {
+		return nil, err
+	}
+
 	err = pcidev.DeviceDriverOverride(pciDev, "vfio-pci")
 	if err != nil {
 		return nil, fmt.Errorf("Failed to override IOMMU group driver: %w", err)
@@ -88,6 +95,7 @@ func (d *pci) Start() (*deviceConfig.RunConfig, error) {
 		[]deviceConfig.RunConfigItem{
 			{Key: "devName", Value: d.name},
 			{Key: "pciSlotName", Value: saveData["last_state.pci.slot.name"]},
+			{Key: "pciIOMMUGroup", Value: strconv.FormatUint(pciIOMMUGroup, 10)},
 		}...)
 
 	err = d.volatileSet(saveData)
@@ -98,6 +106,11 @@ func (d *pci) Start() (*deviceConfig.RunConfig, error) {
 	return &runConf, nil
 }
 
+// CanHotPlug returns whether the device can be managed whilst the instance is running.
+func (d *pci) CanHotPlug() bool {
+	return true
+}
+
 // Stop is run when the device is removed from the instance.
 func (d *pci) Stop() (*deviceConfig.RunConfig, error) {
 	runConf := deviceConfig.RunConfig{
