@@ -5,10 +5,10 @@ import (
 	"net"
 	"time"
 
-	"github.com/lxc/lxd/lxd/endpoints/listeners"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/endpoints/listeners"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // ClusterAddress returns the cluster address of the cluster endpoint, or an
@@ -39,7 +39,7 @@ func (e *Endpoints) ClusterUpdateAddress(address string) error {
 		return nil
 	}
 
-	logger.Infof("Update cluster address")
+	logger.Info("Update cluster address")
 
 	e.mu.Lock()
 	defer e.mu.Unlock()
@@ -62,7 +62,7 @@ func (e *Endpoints) ClusterUpdateAddress(address string) error {
 		var err error
 		var listener net.Listener
 
-		for i := 0; i < 10; i++ { // Ten retries over a second seems reasonable.
+		for range 10 { // Ten retries over a second seems reasonable.
 			listener, err = net.Listen("tcp", address)
 			if err == nil {
 				break
