@@ -5,7 +5,7 @@ import (
 	"net"
 	"regexp"
 
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 type networkServerErrorLogWriter struct {
@@ -13,7 +13,7 @@ type networkServerErrorLogWriter struct {
 }
 
 // Regex for the log we want to ignore.
-var unwantedLogRegex = regexp.MustCompile(`^http: TLS handshake error from ([^\[:]+?|\[([^\]]+?)\]):[0-9]+: .+write: connection reset by peer$`)
+var unwantedLogRegex = regexp.MustCompile(`^http: TLS handshake error from ([^\[:]+?|\[([^\]]+?)\]):[0-9]+: .+: connection reset by peer$`)
 
 func (d networkServerErrorLogWriter) Write(p []byte) (int, error) {
 	strippedLog := d.stripLog(p)
@@ -27,7 +27,7 @@ func (d networkServerErrorLogWriter) Write(p []byte) (int, error) {
 
 func (d networkServerErrorLogWriter) stripLog(p []byte) string {
 	// Strip the beginning of the log until we reach "http:".
-	for string(p[0:5]) != "http:" && len(p) > 0 {
+	for len(p) > 5 && string(p[0:5]) != "http:" {
 		p = bytes.TrimLeftFunc(p, func(r rune) bool {
 			return r != 'h'
 		})
