@@ -53,7 +53,7 @@ func (e *Endpoints) PprofUpdateAddress(address string) error {
 		return nil
 	}
 
-	logger.Infof("Update pprof address")
+	logger.Info("Update pprof address")
 
 	e.mu.Lock()
 	defer e.mu.Unlock()
@@ -71,7 +71,7 @@ func (e *Endpoints) PprofUpdateAddress(address string) error {
 		var err error
 		var listener net.Listener
 
-		for i := 0; i < 10; i++ { // Ten retries over a second seems reasonable.
+		for range 10 { // Ten retries over a second seems reasonable.
 			listener, err = net.Listen("tcp", address)
 			if err == nil {
 				break
