@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"net/http"
+	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/auth"
@@ -14,7 +15,6 @@ import (
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/state"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/logger"
@@ -52,14 +52,9 @@ func (r *eventsServe) String() string {
 }
 
 func eventsSocket(s *state.State, r *http.Request, w http.ResponseWriter) error {
-	// Detect project mode.
-	projectName := request.QueryParam(r, "project")
-	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
-
-	if allProjects && projectName != "" {
-		return api.StatusErrorf(http.StatusBadRequest, "Cannot specify a project when requesting all projects")
-	} else if !allProjects && projectName == "" {
-		projectName = api.ProjectDefaultName
+	projectName, allProjects, err := request.ProjectParams(r)
+	if err != nil {
+		return err
 	}
 
 	if !allProjects && projectName != api.ProjectDefaultName {
@@ -97,7 +92,7 @@ func eventsSocket(s *state.State, r *http.Request, w http.ResponseWriter) error
 	if len(types) == 1 && types[0] == "" {
 		types = []string{}
 		for _, entry := range eventTypes {
-			if !canViewPrivilegedEvents && shared.ValueInSlice(entry, privilegedEventTypes) {
+			if !canViewPrivilegedEvents && slices.Contains(privilegedEventTypes, entry) {
 				continue
 			}
 
@@ -107,12 +102,12 @@ func eventsSocket(s *state.State, r *http.Request, w http.ResponseWriter) error
 
 	// Validate event types.
 	for _, entry := range types {
-		if !shared.ValueInSlice(entry, eventTypes) {
+		if !slices.Contains(eventTypes, entry) {
 			return api.StatusErrorf(http.StatusBadRequest, "%q isn't a supported event type", entry)
 		}
 	}
 
-	if shared.ValueInSlice(api.EventTypeLogging, types) && !canViewPrivilegedEvents {
+	if slices.Contains(types, api.EventTypeLogging) && !canViewPrivilegedEvents {
 		return api.StatusErrorf(http.StatusForbidden, "Forbidden")
 	}
 
@@ -122,13 +117,11 @@ func eventsSocket(s *state.State, r *http.Request, w http.ResponseWriter) error
 	// Get the current local serverName and store it for the events.
 	// We do that now to avoid issues with changes to the name and to limit
 	// the number of DB access to just one per connection.
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		if isClusterNotification(r) {
-			ctx := r.Context()
-
-			// Try and match cluster member certificate fingerprint to member name.
-			fingerprint, found := ctx.Value(request.CtxUsername).(string)
-			if found {
+			reqInfo := request.GetContextInfo(r.Context())
+			if reqInfo != nil && reqInfo.Username != "" {
+				fingerprint := reqInfo.Username
 				cert, err := cluster.GetCertificateByFingerprintPrefix(context.Background(), tx.Tx(), fingerprint)
 				if err != nil {
 					return fmt.Errorf("Failed matching client certificate to cluster member: %w", err)
