@@ -5,7 +5,7 @@ import (
 
 	"github.com/sirupsen/logrus"
 
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // LoggingServer controls what server to use for messages coming from the logger.
@@ -42,7 +42,7 @@ func logContextMap(ctx logrus.Fields) map[string]string {
 	ctxMap := map[string]string{}
 
 	for k, v := range ctx {
-		ctxMap[k] = fmt.Sprintf("%v", v)
+		ctxMap[k] = fmt.Sprint(v)
 	}
 
 	return ctxMap
