@@ -3,7 +3,7 @@ package firewall
 import (
 	"net"
 
-	drivers "github.com/lxc/lxd/lxd/firewall/drivers"
+	"github.com/canonical/lxd/lxd/firewall/drivers"
 )
 
 // Firewall represents a LXD firewall.
@@ -11,7 +11,7 @@ type Firewall interface {
 	String() string
 	Compat() (bool, error)
 
-	NetworkSetup(networkName string, opts drivers.Opts) error
+	NetworkSetup(networkName string, ip4Address net.IP, ip6Address net.IP, opts drivers.Opts) error
 	NetworkClear(networkName string, delete bool, ipVersions []uint) error
 	NetworkApplyACLRules(networkName string, rules []drivers.ACLRule) error
 	NetworkApplyForwards(networkName string, rules []drivers.AddressForward) error
@@ -24,4 +24,7 @@ type Firewall interface {
 
 	InstanceSetupRPFilter(projectName string, instanceName string, deviceName string, hostName string) error
 	InstanceClearRPFilter(projectName string, instanceName string, deviceName string) error
+
+	InstanceSetupNetPrio(projectName string, instanceName string, deviceName string, netPrio uint32) error
+	InstanceClearNetPrio(projectName string, instanceName string, deviceName string) error
 }
