@@ -1,17 +1,17 @@
 package drivers
 
 import (
-	"fmt"
+	"errors"
 )
 
 // ErrInvalidFunction is the "Invalid function" error.
-var ErrInvalidFunction = fmt.Errorf("Invalid function")
+var ErrInvalidFunction = errors.New("Invalid function")
 
 // ErrUnknownDriver is the "Unknown driver" error.
-var ErrUnknownDriver = fmt.Errorf("Unknown driver")
+var ErrUnknownDriver = errors.New("Unknown driver")
 
 // ErrWatchExists is the "Watch already exists" error.
-var ErrWatchExists = fmt.Errorf("Watch already exists")
+var ErrWatchExists = errors.New("Watch already exists")
 
 // ErrInvalidPath is the "Invalid path" error.
 type ErrInvalidPath struct {
