@@ -4,8 +4,6 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"net/http"
-	"path/filepath"
 	"strings"
 	"sync"
 	"time"
@@ -68,16 +66,16 @@ func instanceCreateAsEmpty(s *state.State, args db.InstanceArgs) (instance.Insta
 }
 
 // instanceImageTransfer transfers an image from another cluster node.
-func instanceImageTransfer(s *state.State, r *http.Request, projectName string, hash string, nodeAddress string) error {
+func instanceImageTransfer(ctx context.Context, s *state.State, projectName string, hash string, nodeAddress string) error {
 	logger.Debugf("Transferring image %q from node %q", hash, nodeAddress)
-	client, err := cluster.Connect(nodeAddress, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(ctx, nodeAddress, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return err
 	}
 
 	client = client.UseProject(projectName)
 
-	err = imageImportFromNode(filepath.Join(s.OS.VarDir, "images"), client, hash)
+	err = imageImportFromNode(s.ImagesStoragePath(), client, hash)
 	if err != nil {
 		return err
 	}
@@ -85,7 +83,7 @@ func instanceImageTransfer(s *state.State, r *http.Request, projectName string,
 	return nil
 }
 
-func ensureImageIsLocallyAvailable(s *state.State, r *http.Request, img *api.Image, projectName string) error {
+func ensureImageIsLocallyAvailable(ctx context.Context, s *state.State, img *api.Image, projectName string) error {
 	// Check if the image is available locally or it's on another member.
 	// Ensure we are the only ones operating on this image. Otherwise another instance created at the same
 	// time may also arrive at the conclusion that the image doesn't exist on this cluster member and then
@@ -111,7 +109,7 @@ func ensureImageIsLocallyAvailable(s *state.State, r *http.Request, img *api.Ima
 
 	if memberAddress != "" {
 		// The image is available from another node, let's try to import it.
-		err = instanceImageTransfer(s, r, projectName, img.Fingerprint, memberAddress)
+		err = instanceImageTransfer(ctx, s, projectName, img.Fingerprint, memberAddress)
 		if err != nil {
 			return fmt.Errorf("Failed transferring image %q from %q: %w", img.Fingerprint, memberAddress, err)
 		}
@@ -204,7 +202,7 @@ func instanceCreateFromImage(s *state.State, img *api.Image, args db.InstanceArg
 	return nil
 }
 
-func instanceRebuildFromImage(s *state.State, r *http.Request, inst instance.Instance, img *api.Image, op *operations.Operation) error {
+func instanceRebuildFromImage(ctx context.Context, s *state.State, inst instance.Instance, img *api.Image, op *operations.Operation) error {
 	// Validate the type of the image matches the type of the instance.
 	imgType, err := instancetype.New(img.Type)
 	if err != nil {
@@ -215,7 +213,7 @@ func instanceRebuildFromImage(s *state.State, r *http.Request, inst instance.Ins
 		return fmt.Errorf("Requested image's type %q doesn't match instance type %q", imgType, inst.Type())
 	}
 
-	err = ensureImageIsLocallyAvailable(s, r, img, inst.Project().Name)
+	err = ensureImageIsLocallyAvailable(ctx, s, img, inst.Project().Name)
 	if err != nil {
 		return err
 	}
@@ -239,12 +237,13 @@ func instanceRebuildFromEmpty(inst instance.Instance, op *operations.Operation)
 
 // instanceCreateAsCopyOpts options for copying an instance.
 type instanceCreateAsCopyOpts struct {
-	sourceInstance       instance.Instance // Source instance.
-	targetInstance       db.InstanceArgs   // Configuration for new instance.
-	instanceOnly         bool              // Only copy the instance and not it's snapshots.
-	refresh              bool              // Refresh an existing target instance.
-	applyTemplateTrigger bool              // Apply deferred TemplateTriggerCopy.
-	allowInconsistent    bool              // Ignore some copy errors
+	sourceInstance           instance.Instance // Source instance.
+	targetInstance           db.InstanceArgs   // Configuration for new instance.
+	instanceOnly             bool              // Only copy the instance and not it's snapshots.
+	refresh                  bool              // Refresh an existing target instance.
+	applyTemplateTrigger     bool              // Apply deferred TemplateTriggerCopy.
+	allowInconsistent        bool              // Ignore some copy errors
+	overrideSnapshotProfiles bool              // Copy the target instance profiles to the instance snapshots
 }
 
 // instanceCreateAsCopy create a new instance by copying from an existing instance.
@@ -403,6 +402,11 @@ func instanceCreateAsCopy(s *state.State, opts instanceCreateAsCopyOpts, op *ope
 				CreationDate: srcSnap.CreationDate(),
 			}
 
+			// Fix target profiles
+			if opts.overrideSnapshotProfiles {
+				snapInstArgs.Profiles = opts.targetInstance.Profiles
+			}
+
 			// Create the snapshots.
 			_, snapInstOp, cleanup, err := instance.CreateInternal(s, snapInstArgs, true)
 			if err != nil {
@@ -504,13 +508,7 @@ func autoCreateInstanceSnapshots(ctx context.Context, s *state.State, instances
 			return err
 		}
 
-		expiry, err := shared.GetExpiry(time.Now(), inst.ExpandedConfig()["snapshots.expiry"])
-		if err != nil {
-			l.Error("Error getting snapshots.expiry date")
-			return err
-		}
-
-		err = inst.Snapshot(snapshotName, expiry, false)
+		err = inst.Snapshot(snapshotName, nil, false)
 		if err != nil {
 			l.Error("Error creating snapshot", logger.Ctx{"snapshot": snapshotName, "err": err})
 			return err
@@ -547,10 +545,10 @@ func pruneExpiredInstanceSnapshots(ctx context.Context, snapshots []instance.Ins
 	return nil
 }
 
-func pruneExpiredAndAutoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
+func pruneExpiredAndAutoCreateInstanceSnapshotsTask(stateFunc func() *state.State) (task.Func, task.Schedule) {
 	// `f` creates new scheduled instance snapshots and then, prune the expired ones
 	f := func(ctx context.Context) {
-		s := d.State()
+		s := stateFunc()
 		var instances, expiredSnapshotInstances []instance.Instance
 
 		// Get list of expired instance snapshots for this local member.
@@ -673,7 +671,7 @@ func pruneExpiredAndAutoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.
 				return pruneExpiredInstanceSnapshots(ctx, expiredSnapshotInstances)
 			}
 
-			op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.SnapshotsExpire, nil, nil, opRun, nil, nil, nil)
+			op, err := operations.OperationCreate(context.Background(), s, "", operations.OperationClassTask, operationtype.SnapshotsExpire, nil, nil, opRun, nil, nil)
 			if err != nil {
 				logger.Error("Failed creating instance snapshots expiry operation", logger.Ctx{"err": err})
 			} else {
@@ -699,7 +697,7 @@ func pruneExpiredAndAutoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.
 				return autoCreateInstanceSnapshots(ctx, s, instances)
 			}
 
-			op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.SnapshotCreate, nil, nil, opRun, nil, nil, nil)
+			op, err := operations.OperationCreate(context.Background(), s, "", operations.OperationClassTask, operationtype.SnapshotCreate, nil, nil, opRun, nil, nil)
 			if err != nil {
 				logger.Error("Failed creating scheduled instance snapshot operation", logger.Ctx{"err": err})
 			} else {
@@ -774,5 +772,5 @@ func instanceOperationLock(ctx context.Context, projectName string, instanceName
 	l.Debug("Acquiring lock for instance")
 	defer l.Debug("Lock acquired for instance")
 
-	return locking.Lock(ctx, fmt.Sprintf("InstanceOperation_%s", project.Instance(projectName, instanceName)))
+	return locking.Lock(ctx, "InstanceOperation_"+project.Instance(projectName, instanceName))
 }
