@@ -7,6 +7,7 @@ import (
 	"net/http"
 	"os"
 	"path/filepath"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -15,6 +16,7 @@ import (
 	"github.com/google/uuid"
 	"golang.org/x/sys/unix"
 
+	agentAPI "github.com/canonical/lxd/lxd-agent/api"
 	"github.com/canonical/lxd/lxd/backup"
 	"github.com/canonical/lxd/lxd/db"
 	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
@@ -329,7 +331,7 @@ func (d *common) VolatileSet(changes map[string]string) error {
 	// Quick check.
 	for key := range changes {
 		if !strings.HasPrefix(key, instancetype.ConfigVolatilePrefix) {
-			return fmt.Errorf("Only volatile keys can be modified with VolatileSet")
+			return errors.New("Only volatile keys can be modified with VolatileSet")
 		}
 	}
 
@@ -337,11 +339,11 @@ func (d *common) VolatileSet(changes map[string]string) error {
 	if !d.volatileSetPersistDisable {
 		var err error
 		if d.isSnapshot {
-			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(_ context.Context, tx *db.ClusterTx) error {
 				return tx.UpdateInstanceSnapshotConfig(d.id, changes)
 			})
 		} else {
-			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(_ context.Context, tx *db.ClusterTx) error {
 				return tx.UpdateInstanceConfig(d.id, changes)
 			})
 		}
@@ -486,12 +488,14 @@ func (d *common) deviceVolatileReset(devName string, oldConfig, newConfig device
 func (d *common) deviceVolatileGetFunc(devName string) func() map[string]string {
 	return func() map[string]string {
 		volatile := make(map[string]string)
-		prefix := fmt.Sprintf("volatile.%s.", devName)
+		prefix := "volatile." + devName + "."
 		for k, v := range d.localConfig {
-			if strings.HasPrefix(k, prefix) {
-				volatile[strings.TrimPrefix(k, prefix)] = v
+			after, ok := strings.CutPrefix(k, prefix)
+			if ok {
+				volatile[after] = v
 			}
 		}
+
 		return volatile
 	}
 }
@@ -509,6 +513,26 @@ func (d *common) deviceVolatileSetFunc(devName string) func(save map[string]stri
 	}
 }
 
+// postMigrateSendCommon handles common instance post-migration steps.
+func (d *common) postMigrateSendCommon(inst instance.Instance, clusterMoveSourceName string) error {
+	// Perform post-migration device cleanup.
+	for devName, devConfig := range d.ExpandedDevices() {
+		dev, err := d.deviceLoad(inst, devName, devConfig)
+		if err != nil {
+			logger.Error("Failed to load device %q during post-migration steps on source: %v", logger.Ctx{"devName": devName, "err": err})
+		}
+
+		if dev != nil {
+			err = dev.PostMigrateSend(clusterMoveSourceName)
+			if err != nil {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
 // expandConfig applies the config of each profile in order, followed by the local config.
 func (d *common) expandConfig() error {
 	var globalConfigDump map[string]any
@@ -576,7 +600,7 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 		}
 	} else {
 		if inst.IsFrozen() {
-			err = fmt.Errorf("Instance is not running")
+			err = errors.New("Instance is not running")
 			op.Done(err)
 			return err
 		}
@@ -620,7 +644,7 @@ func (d *common) rebuildCommon(inst instance.Instance, img *api.Image, op *opera
 	delete(instLocalConfig, "volatile.base_image")
 	if img != nil {
 		for k, v := range img.Properties {
-			instLocalConfig[fmt.Sprintf("image.%s", k)] = v
+			instLocalConfig["image."+k] = v
 		}
 
 		instLocalConfig["volatile.base_image"] = img.Fingerprint
@@ -691,10 +715,23 @@ func (d *common) runHooks(hooks []func() error) error {
 }
 
 // snapshot handles the common part of the snapshoting process.
-func (d *common) snapshotCommon(inst instance.Instance, name string, expiry time.Time, stateful bool) error {
+func (d *common) snapshotCommon(inst instance.Instance, name string, expiry *time.Time, stateful bool) error {
 	revert := revert.New()
 	defer revert.Fail()
 
+	snapshotCreationDate := time.Now().UTC()
+
+	// If the expiry is unset, retrieve it from the instance's expanded config.
+	if expiry == nil {
+		// Use the snapshot creation date as reference for an exact expiry date.
+		instanceSnapshotExpiry, err := shared.GetExpiry(snapshotCreationDate, inst.ExpandedConfig()["snapshots.expiry"])
+		if err != nil {
+			return err
+		}
+
+		expiry = &instanceSnapshotExpiry
+	}
+
 	// Setup the arguments.
 	args := db.InstanceArgs{
 		Project:      inst.Project().Name,
@@ -707,7 +744,8 @@ func (d *common) snapshotCommon(inst instance.Instance, name string, expiry time
 		Name:         inst.Name() + shared.SnapshotDelimiter + name,
 		Profiles:     inst.Profiles(),
 		Stateful:     stateful,
-		ExpiryDate:   expiry,
+		ExpiryDate:   *expiry,
+		CreationDate: snapshotCreationDate,
 	}
 
 	// Create the snapshot.
@@ -811,7 +849,7 @@ func (d *common) isRunningStatusCode(statusCode api.StatusCode) bool {
 // isStartableStatusCode returns an error if the status code means the instance cannot be started currently.
 func (d *common) isStartableStatusCode(statusCode api.StatusCode) error {
 	if d.isRunningStatusCode(statusCode) {
-		return fmt.Errorf("The instance is already running")
+		return errors.New("The instance is already running")
 	}
 
 	// If the instance process exists but is crashed, don't allow starting until its been cleaned up, as it
@@ -831,7 +869,7 @@ func (d *common) getStartupSnapNameAndExpiry(inst instance.Instance) (string, *t
 	}
 
 	triggers := strings.Split(schedule, ", ")
-	if !shared.ValueInSlice("@startup", triggers) {
+	if !slices.Contains(triggers, "@startup") {
 		return "", nil, nil
 	}
 
@@ -877,7 +915,7 @@ func (d *common) maasUpdate(inst instance.Instance, oldDevices map[string]map[st
 
 	// See if we're connected to MAAS
 	if d.state.MAAS == nil {
-		return fmt.Errorf("Can't perform the operation because MAAS is currently unavailable")
+		return errors.New("Can't perform the operation because MAAS is currently unavailable")
 	}
 
 	exists, err := d.state.MAAS.DefinedContainer(d)
@@ -963,7 +1001,7 @@ func (d *common) maasRename(inst instance.Instance, newName string) error {
 	}
 
 	if d.state.MAAS == nil {
-		return fmt.Errorf("Can't perform the operation because MAAS is currently unavailable")
+		return errors.New("Can't perform the operation because MAAS is currently unavailable")
 	}
 
 	exists, err := d.state.MAAS.DefinedContainer(d)
@@ -995,7 +1033,7 @@ func (d *common) maasDelete(inst instance.Instance) error {
 	}
 
 	if d.state.MAAS == nil {
-		return fmt.Errorf("Can't perform the operation because MAAS is currently unavailable")
+		return errors.New("Can't perform the operation because MAAS is currently unavailable")
 	}
 
 	exists, err := d.state.MAAS.DefinedContainer(d)
@@ -1168,7 +1206,7 @@ func (d *common) recordLastState() error {
 	d.expandedConfig["volatile.last_state.power"] = instance.PowerStateRunning
 
 	// Database updates
-	return d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	return d.state.DB.Cluster.Transaction(context.TODO(), func(_ context.Context, tx *db.ClusterTx) error {
 		// Record power state.
 		err = tx.UpdateInstancePowerState(d.id, instance.PowerStateRunning)
 		if err != nil {
@@ -1250,7 +1288,7 @@ func (d *common) needsNewInstanceID(changedConfig []string, oldExpandedDevices d
 		"user.user-data",
 		"user.network-config",
 	} {
-		if shared.ValueInSlice(key, changedConfig) {
+		if slices.Contains(changedConfig, key) {
 			return true
 		}
 	}
@@ -1292,13 +1330,13 @@ func (d *common) needsNewInstanceID(changedConfig []string, oldExpandedDevices d
 	newNames := getNICNames(d.expandedDevices)
 
 	for _, entry := range oldNames {
-		if !shared.ValueInSlice(entry, newNames) {
+		if !slices.Contains(newNames, entry) {
 			return true
 		}
 	}
 
 	for _, entry := range newNames {
-		if !shared.ValueInSlice(entry, oldNames) {
+		if !slices.Contains(oldNames, entry) {
 			return true
 		}
 	}
@@ -1348,7 +1386,7 @@ func (d *common) getParentStoragePool() (string, error) {
 	}
 
 	if parentStoragePool == "" {
-		return "", fmt.Errorf("Instance's root device is missing the pool property")
+		return "", errors.New("Instance's root device is missing the pool property")
 	}
 
 	return parentStoragePool, nil
@@ -1360,7 +1398,7 @@ func (d *common) deviceLoad(inst instance.Instance, deviceName string, rawConfig
 	var err error
 
 	// Create copy of config and load some fields from volatile if device is nic or infiniband.
-	if shared.ValueInSlice(rawConfig["type"], []string{"nic", "infiniband"}) {
+	if slices.Contains([]string{"nic", "infiniband"}, rawConfig["type"]) {
 		configCopy, err = inst.FillNetworkDevice(deviceName, rawConfig)
 		if err != nil {
 			return nil, err
@@ -1387,7 +1425,7 @@ func (d *common) deviceAdd(dev device.Device, instanceRunning bool) error {
 	l.Debug("Adding device")
 
 	if instanceRunning && !dev.CanHotPlug() {
-		return fmt.Errorf("Device cannot be added when instance is running")
+		return errors.New("Device cannot be added when instance is running")
 	}
 
 	return dev.Add()
@@ -1399,7 +1437,7 @@ func (d *common) deviceRemove(dev device.Device, instanceRunning bool) error {
 	l.Debug("Removing device")
 
 	if instanceRunning && !dev.CanHotPlug() {
-		return fmt.Errorf("Device cannot be removed when instance is running")
+		return errors.New("Device cannot be removed when instance is running")
 	}
 
 	return dev.Remove()
@@ -1482,7 +1520,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 
 	dm, ok := inst.(deviceManager)
 	if !ok {
-		return nil, fmt.Errorf("Instance is not compatible with deviceManager interface")
+		return nil, errors.New("Instance is not compatible with deviceManager interface")
 	}
 
 	// Remove devices in reverse order to how they were added.
@@ -1507,7 +1545,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 				}
 
 				devlxdEvents = append(devlxdEvents, map[string]any{
-					"action": "removed",
+					"action": agentAPI.DeviceRemoved,
 					"name":   entry.Name,
 					"config": entry.Config,
 				})
@@ -1575,7 +1613,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 			revert.Add(func() { _ = dm.deviceStop(dev, instanceRunning, "") })
 
 			event := map[string]any{
-				"action": "added",
+				"action": agentAPI.DeviceAdded,
 				"name":   entry.Name,
 				"config": entry.Config,
 			}
@@ -1639,7 +1677,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 					}
 
 					devlxdEvents = append(devlxdEvents, map[string]any{
-						"action": "updated",
+						"action": agentAPI.DeviceUpdated,
 						"name":   entry.Name,
 						"config": entry.Config,
 					})
@@ -1745,14 +1783,14 @@ func (d *common) checkRootVolumeNotInUse() error {
 		return err
 	}
 
-	err = storagePools.VolumeUsedByProfileDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, func(profileID int64, profile api.Profile, p api.Project, usedByDevices []string) error {
+	err = storagePools.VolumeUsedByProfileDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, func(_ int64, _ api.Profile, _ api.Project, _ []string) error {
 		return fmt.Errorf(`"%s/%s" is attached to a profile`, rootVolume.Type, rootVolume.Name)
 	})
 	if err != nil {
 		return err
 	}
 
-	err = storagePools.VolumeUsedByInstanceDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, false, func(inst db.InstanceArgs, p api.Project, usedByDevices []string) error {
+	err = storagePools.VolumeUsedByInstanceDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, false, func(inst db.InstanceArgs, _ api.Project, _ []string) error {
 		if inst.Name == d.Name() && inst.Project == d.Project().Name {
 			return nil
 		}
