@@ -2,6 +2,7 @@ package drivers
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -127,7 +128,7 @@ func (d *qemu) getQemuMemoryMetrics() (metrics.MemoryMetrics, error) {
 
 		// Extract the before last (value) and last (unit) fields
 		fields := strings.Split(line, "\t")
-		value := strings.Replace(fields[len(fields)-1], " ", "", -1)
+		value := strings.ReplaceAll(fields[len(fields)-1], " ", "")
 
 		// Feed the result to units.ParseByteSizeString to get an int value
 		valueBytes, err := units.ParseByteSizeString(value)
@@ -140,7 +141,7 @@ func (d *qemu) getQemuMemoryMetrics() (metrics.MemoryMetrics, error) {
 	}
 
 	if memRSS == -1 {
-		return out, fmt.Errorf("Couldn't find VM memory usage")
+		return out, errors.New("Couldn't find VM memory usage")
 	}
 
 	// Get max memory usage.
