@@ -28,6 +28,9 @@ import (
 // HookStart hook used when instance has started.
 const HookStart = "onstart"
 
+// HookStartHost hook used when instance is fully ready to be started.
+const HookStartHost = "onstarthost"
+
 // HookStopNS hook used when instance has stopped but before namespaces have been destroyed.
 const HookStopNS = "onstopns"
 
@@ -90,7 +93,7 @@ type Instance interface {
 
 	// Snapshots & migration & backups.
 	Restore(source Instance, stateful bool) error
-	Snapshot(name string, expiry time.Time, stateful bool) error
+	Snapshot(name string, expiry *time.Time, stateful bool) error
 	Snapshots() ([]Instance, error)
 	Backups() ([]backup.InstanceBackup, error)
 	UpdateBackupFile() error
