@@ -5,11 +5,13 @@ import (
 	"context"
 	"crypto/rand"
 	"database/sql"
+	"errors"
 	"fmt"
 	"math/big"
 	"net/http"
 	"os"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -57,11 +59,11 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 
 	for k, v := range config {
 		if instanceType == instancetype.Any && !expanded && strings.HasPrefix(k, instancetype.ConfigVolatilePrefix) {
-			return fmt.Errorf("Volatile keys can only be set on instances")
+			return errors.New("Volatile keys can only be set on instances")
 		}
 
 		if instanceType == instancetype.Any && !expanded && strings.HasPrefix(k, "image.") {
-			return fmt.Errorf("Image keys can only be set on instances")
+			return errors.New("Image keys can only be set on instances")
 		}
 
 		err := validConfigKey(sysOS, k, v, instanceType)
@@ -77,11 +79,11 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 	isDenyCompat := shared.IsTrue(config["security.syscalls.deny_compat"])
 
 	if rawSeccomp && (isAllow || isDeny || isDenyDefault || isDenyCompat) {
-		return fmt.Errorf("raw.seccomp is mutually exclusive with security.syscalls*")
+		return errors.New("raw.seccomp is mutually exclusive with security.syscalls*")
 	}
 
 	if isAllow && (isDeny || isDenyDefault || isDenyCompat) {
-		return fmt.Errorf("security.syscalls.allow is mutually exclusive with security.syscalls.deny*")
+		return errors.New("security.syscalls.allow is mutually exclusive with security.syscalls.deny*")
 	}
 
 	_, err := seccomp.SyscallInterceptMountFilter(config)
@@ -90,7 +92,7 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 	}
 
 	if expanded && (shared.IsFalseOrEmpty(config["security.privileged"])) && sysOS.IdmapSet == nil {
-		return fmt.Errorf("LXD doesn't have a uid/gid allocation. In this mode, only privileged containers are supported")
+		return errors.New("LXD doesn't have a uid/gid allocation. In this mode, only privileged containers are supported")
 	}
 
 	unprivOnly := os.Getenv("LXD_UNPRIVILEGED_ONLY")
@@ -103,16 +105,16 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 		}
 
 		if shared.IsTrue(config["security.privileged"]) {
-			return fmt.Errorf("LXD was configured to only allow unprivileged containers")
+			return errors.New("LXD was configured to only allow unprivileged containers")
 		}
 	}
 
 	if shared.IsTrue(config["security.privileged"]) && shared.IsTrue(config["nvidia.runtime"]) {
-		return fmt.Errorf("nvidia.runtime is incompatible with privileged containers")
+		return errors.New("nvidia.runtime is incompatible with privileged containers")
 	}
 
 	if sysOS.InUbuntuCore() && shared.IsTrue(config["nvidia.runtime"]) {
-		return fmt.Errorf("nvidia.runtime is incompatible with Ubuntu Core")
+		return errors.New("nvidia.runtime is incompatible with Ubuntu Core")
 	}
 
 	// Validate pinning strategy when limits.cpu specifies static pinning.
@@ -120,7 +122,7 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 	cpuLimit := config["limits.cpu"]
 	err = validate.IsStaticCPUPinning(cpuLimit)
 	if err == nil && !expanded && cpuPinStrategy == "auto" {
-		return fmt.Errorf(`CPU pinning specified, but pinning strategy is set to "auto"`)
+		return errors.New(`CPU pinning specified, but pinning strategy is set to "auto"`)
 	}
 
 	return nil
@@ -135,7 +137,7 @@ func validConfigKey(os *sys.OS, key string, value string, instanceType instancet
 	// Check if the key is a valid prefix and whether or not it requires a subkey.
 	knownPrefixes := append(instancetype.ConfigKeyPrefixesAny, instancetype.ConfigKeyPrefixesContainer...)
 	if strings.HasSuffix(key, ".") {
-		if !(key == instancetype.ConfigVolatilePrefix || shared.ValueInSlice(key, knownPrefixes)) {
+		if key != instancetype.ConfigVolatilePrefix && !slices.Contains(knownPrefixes, key) {
 			// Not a known prefix.
 			return fmt.Errorf("Unknown configuration key: %q", key)
 		}
@@ -156,7 +158,7 @@ func validConfigKey(os *sys.OS, key string, value string, instanceType instancet
 		}
 
 		_, existsAny := instancetype.InstanceConfigKeysAny[key]
-		if !(exists || existsAny) {
+		if !exists && !existsAny {
 			return fmt.Errorf("%q isn't supported for %q", key, instanceType)
 		}
 	}
@@ -214,7 +216,7 @@ func lxcParseRawLXC(line string) (key string, val string, err error) {
 }
 
 func lxcValidConfig(rawLxc string) error {
-	for _, line := range strings.Split(rawLxc, "\n") {
+	for line := range strings.SplitSeq(rawLxc, "\n") {
 		key, _, err := lxcParseRawLXC(line)
 		if err != nil {
 			return err
@@ -233,19 +235,19 @@ func lxcValidConfig(rawLxc string) error {
 
 		// block some keys
 		if key == "lxc.logfile" || key == "lxc.log.file" {
-			return fmt.Errorf("Setting lxc.logfile is not allowed")
+			return errors.New("Setting lxc.logfile is not allowed")
 		}
 
 		if key == "lxc.syslog" || key == "lxc.log.syslog" {
-			return fmt.Errorf("Setting lxc.log.syslog is not allowed")
+			return errors.New("Setting lxc.log.syslog is not allowed")
 		}
 
 		if key == "lxc.ephemeral" {
-			return fmt.Errorf("Setting lxc.ephemeral is not allowed")
+			return errors.New("Setting lxc.ephemeral is not allowed")
 		}
 
 		if strings.HasPrefix(key, "lxc.prlimit.") {
-			return fmt.Errorf(`Process limits should be set via ` +
+			return errors.New(`Process limits should be set via ` +
 				`"limits.kernel.[limit name]" and not ` +
 				`directly via "lxc.prlimit.[limit name]"`)
 		}
@@ -255,12 +257,12 @@ func lxcValidConfig(rawLxc string) error {
 			fields := strings.Split(key, ".")
 
 			// lxc.net.X.ipv4.address or lxc.net.X.ipv6.address
-			if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "address" {
+			if len(fields) == 5 && slices.Contains([]string{"ipv4", "ipv6"}, fields[3]) && fields[4] == "address" {
 				continue
 			}
 
 			// lxc.net.X.ipv4.gateway or lxc.net.X.ipv6.gateway
-			if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "gateway" {
+			if len(fields) == 5 && slices.Contains([]string{"ipv4", "ipv6"}, fields[3]) && fields[4] == "gateway" {
 				continue
 			}
 
@@ -280,7 +282,7 @@ func AllowedUnprivilegedOnlyMap(rawIdmap string) error {
 
 	for _, ent := range rawMaps {
 		if ent.Hostid == 0 {
-			return fmt.Errorf("Cannot map root user into container as LXD was configured to only allow unprivileged containers")
+			return errors.New("Cannot map root user into container as LXD was configured to only allow unprivileged containers")
 		}
 	}
 
@@ -378,7 +380,7 @@ func LoadByProjectAndName(s *state.State, projectName string, instanceName strin
 
 	inst, err := Load(s, args, *p)
 	if err != nil {
-		return nil, fmt.Errorf("Failed to load instance: %w", err)
+		return nil, fmt.Errorf("Failed loading instance: %w", err)
 	}
 
 	return inst, nil
@@ -431,14 +433,14 @@ func LoadFromBackup(s *state.State, projectName string, instancePath string) (In
 
 	// Stop instance.Load() from expanding profile config from DB, and apply expanded config from
 	// backup file to local config. This way we can still see the devices even if DB not available.
-	instDBArgs.Config = backupConf.Container.ExpandedConfig
-	instDBArgs.Devices = deviceConfig.NewDevices(backupConf.Container.ExpandedDevices)
+	instDBArgs.Config = backupConf.Instance.ExpandedConfig
+	instDBArgs.Devices = deviceConfig.NewDevices(backupConf.Instance.ExpandedDevices)
 
 	// Set Node field to local node.
 	instDBArgs.Node = s.ServerName
 
 	p := api.Project{
-		Name: backupConf.Container.Project,
+		Name: backupConf.Instance.Project,
 	}
 
 	inst, err := Load(s, *instDBArgs, p)
@@ -513,7 +515,7 @@ func ResolveImage(ctx context.Context, tx *db.ClusterTx, projectName string, sou
 
 	if source.Properties != nil {
 		if source.Server != "" {
-			return "", fmt.Errorf("Property match is only supported for local images")
+			return "", errors.New("Property match is only supported for local images")
 		}
 
 		hashes, err := tx.GetImagesFingerprints(ctx, projectName, false)
@@ -551,10 +553,10 @@ func ResolveImage(ctx context.Context, tx *db.ClusterTx, projectName string, sou
 			return image.Fingerprint, nil
 		}
 
-		return "", fmt.Errorf("No matching image could be found")
+		return "", errors.New("No matching image could be found")
 	}
 
-	return "", fmt.Errorf("Must specify one of alias, fingerprint or properties for init from image")
+	return "", errors.New("Must specify one of alias, fingerprint or properties for init from image")
 }
 
 // SuitableArchitectures returns a slice of architecture ids based on an instance create request.
@@ -563,7 +565,7 @@ func ResolveImage(ctx context.Context, tx *db.ClusterTx, projectName string, sou
 // A nil list indicates that we can't tell at this stage, typically for private images.
 func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx, projectName string, sourceInst *cluster.Instance, sourceImageRef string, req api.InstancesPost) ([]int, error) {
 	// Handle cases where the architecture is already provided.
-	if shared.ValueInSlice(req.Source.Type, []string{api.SourceTypeConversion, api.SourceTypeMigration, api.SourceTypeNone}) && req.Architecture != "" {
+	if slices.Contains([]string{api.SourceTypeConversion, api.SourceTypeMigration, api.SourceTypeNone}, req.Source.Type) && req.Architecture != "" {
 		id, err := osarch.ArchitectureId(req.Architecture)
 		if err != nil {
 			return nil, err
@@ -573,7 +575,7 @@ func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx
 	}
 
 	// For migration and conversion, an architecture must be specified in the req.
-	if shared.ValueInSlice(req.Source.Type, []string{api.SourceTypeConversion, api.SourceTypeMigration}) && req.Architecture == "" {
+	if slices.Contains([]string{api.SourceTypeConversion, api.SourceTypeMigration}, req.Source.Type) && req.Architecture == "" {
 		return nil, api.StatusErrorf(http.StatusBadRequest, "An architecture must be specified in migration or conversion requests")
 	}
 
@@ -613,7 +615,7 @@ func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx
 
 			var err error
 			var remote lxd.ImageServer
-			if shared.ValueInSlice(req.Source.Protocol, []string{"", "lxd"}) {
+			if slices.Contains([]string{"", "lxd"}, req.Source.Protocol) {
 				// Remote LXD image server.
 				remote, err = lxd.ConnectPublicLXD(req.Source.Server, &lxd.ConnectionArgs{
 					TLSServerCert: req.Source.Certificate,
@@ -704,7 +706,7 @@ func CreateInternal(s *state.State, args db.InstanceArgs, clearLogDir bool) (Ins
 			return err
 		})
 		if err != nil {
-			return nil, nil, nil, fmt.Errorf("Failed to get default profile for new instance")
+			return nil, nil, nil, errors.New("Failed to get default profile for new instance")
 		}
 	}
 
@@ -764,8 +766,8 @@ func CreateInternal(s *state.State, args db.InstanceArgs, clearLogDir bool) (Ins
 		return nil, nil, nil, err
 	}
 
-	if !shared.ValueInSlice(args.Architecture, s.OS.Architectures) {
-		return nil, nil, nil, fmt.Errorf("Requested architecture isn't supported by this host")
+	if !slices.Contains(s.OS.Architectures, args.Architecture) {
+		return nil, nil, nil, errors.New("Requested architecture isn't supported by this host")
 	}
 
 	var profiles []string
@@ -782,12 +784,12 @@ func CreateInternal(s *state.State, args db.InstanceArgs, clearLogDir bool) (Ins
 
 	checkedProfiles := map[string]bool{}
 	for _, profile := range args.Profiles {
-		if !shared.ValueInSlice(profile.Name, profiles) {
+		if !slices.Contains(profiles, profile.Name) {
 			return nil, nil, nil, fmt.Errorf("Requested profile %q doesn't exist", profile.Name)
 		}
 
 		if checkedProfiles[profile.Name] {
-			return nil, nil, nil, fmt.Errorf("Duplicate profile found in request")
+			return nil, nil, nil, errors.New("Duplicate profile found in request")
 		}
 
 		checkedProfiles[profile.Name] = true
@@ -1002,7 +1004,7 @@ func NextSnapshotName(s *state.State, inst Instance, defaultPattern string) (str
 
 	count := strings.Count(pattern, "%d")
 	if count > 1 {
-		return "", fmt.Errorf("Snapshot pattern may contain '%%d' only once")
+		return "", errors.New("Snapshot pattern may contain '%%d' only once")
 	} else if count == 1 {
 		var i int
 
@@ -1032,7 +1034,7 @@ func NextSnapshotName(s *state.State, inst Instance, defaultPattern string) (str
 
 	// Append '-0', '-1', etc. if the actual pattern/snapshot name already exists
 	if snapshotExists {
-		pattern = fmt.Sprintf("%s-%%d", pattern)
+		pattern = pattern + "-%d"
 
 		var i int
 
