@@ -7,6 +7,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/canonical/lxd/lxd/device/filters"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/units"
@@ -61,7 +62,7 @@ func ValidName(instanceName string, isSnapshot bool) error {
 // ValidSnapName validates a snnapshot instance name which must not include the instance prefix.
 func ValidSnapName(snapshotName string) error {
 	if snapshotName == "" {
-		return fmt.Errorf("Invalid instance snapshot name, cannot be empty")
+		return errors.New("Invalid instance snapshot name, cannot be empty")
 	}
 
 	if snapshotName == ".." {
@@ -75,22 +76,8 @@ func ValidSnapName(snapshotName string) error {
 	return nil
 }
 
-// IsRootDiskDevice returns true if the given device representation is configured as root disk for
-// an instance. It typically get passed a specific entry of api.Instance.Devices.
-func IsRootDiskDevice(device map[string]string) bool {
-	// Root disk devices also need a non-empty "pool" property, but we can't check that here
-	// because this function is used with clients talking to older servers where there was no
-	// concept of a storage pool, and also it is used for migrating from old to new servers.
-	// The validation of the non-empty "pool" property is done inside the disk device itself.
-	if device["type"] == "disk" && device["path"] == "/" && device["source"] == "" {
-		return true
-	}
-
-	return false
-}
-
 // ErrNoRootDisk means there is no root disk device found.
-var ErrNoRootDisk = fmt.Errorf("No root device could be found")
+var ErrNoRootDisk = errors.New("No root device could be found")
 
 // GetRootDiskDevice returns the instance device that is configured as root disk.
 // Returns the device name and device config map.
@@ -99,9 +86,9 @@ func GetRootDiskDevice(devices map[string]map[string]string) (string, map[string
 	var dev map[string]string
 
 	for n, d := range devices {
-		if IsRootDiskDevice(d) {
+		if filters.IsRootDisk(d) {
 			if devName != "" {
-				return "", nil, fmt.Errorf("More than one root device found")
+				return "", nil, errors.New("More than one root device found")
 			}
 
 			devName = n
@@ -125,7 +112,9 @@ var HugePageSizeSuffix = [...]string{"64KB", "1MB", "2MB", "1GB"}
 // InstanceConfigKeysAny is a map of config key to validator. (keys applying to containers AND virtual machines).
 var InstanceConfigKeysAny = map[string]func(value string) error{
 	// lxdmeta:generate(entities=instance; group=boot; key=boot.autostart)
-	// If set to `false`, restore the last state.
+	// If set to `true`, the instance will always be auto-started, unless `security.protection.start` is also enabled.
+	// If set to `false`, the instance will not be started on LXD start up.
+	// If this option is not set, the instance will be restored to its last known state.
 	// ---
 	//  type: bool
 	//  liveupdate: no
@@ -310,7 +299,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 		}
 
 		if num == 0 {
-			return fmt.Errorf("Memory limit can't be 0")
+			return errors.New("Memory limit can't be 0")
 		}
 
 		return nil
@@ -416,12 +405,8 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 
 	// lxdmeta:generate(entities=instance; group=miscellaneous; key=ubuntu_pro.guest_attach)
 	// Indicate whether the guest should auto-attach Ubuntu Pro at start up.
-	// The allowed values are `off`, `on`, and `available`.
-	// If set to `off`, it will not be possible for the Ubuntu Pro client in the guest to obtain guest token via `devlxd`.
-	// If set to `available`, attachment via guest token is possible but will not be performed automatically by the Ubuntu Pro client in the guest at startup.
-	// If set to `on`, attachment will be performed automatically by the Ubuntu Pro client in the guest at startup.
-	// To allow guest attachment, the host must be an Ubuntu machine that is Pro attached, and guest attachment must be enabled via the Pro client.
-	// To do this, run `pro config set lxd_guest_attach=on`.
+	//
+	// See {ref}`instances-ubuntu-pro-attach` for more information.
 	// ---
 	// type: string
 	// liveupdate: no
@@ -938,6 +923,74 @@ var InstanceConfigKeysContainer = map[string]func(value string) error{
 	//  shortdesc: Whether to handle the `sysinfo` system call
 	"security.syscalls.intercept.sysinfo": validate.Optional(validate.IsBool),
 
+	// lxdmeta:generate(entities=instance; group=security; key=security.delegate_bpf)
+	// This option enables BPF functionality delegation mechanism (using BPF Token).
+	//
+	// Note: `security.delegate_bpf.cmd_types`, `security.delegate_bpf.map_types`,
+	// `security.delegate_bpf.prog_types`, `security.delegate_bpf.attach_types`
+	// need to be configured depending on BPF workload in the container.
+	//
+	// See {ref}`bpf-delegation-token` for more information.
+	//
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: no
+	//  condition: unprivileged container
+	//  shortdesc: Whether to enable eBPF delegation using BPF Token mechanism
+	"security.delegate_bpf": validate.Optional(validate.IsBool),
+
+	// lxdmeta:generate(entities=instance; group=security; key=security.delegate_bpf.cmd_types)
+	// Which eBPF commands to allow with delegation mechanism. Syntax follows a kernel one for `delegate_cmds`
+	// bpffs mount option. A number (bitmask) or `:`-separated list of commands to allow can be specified.
+	// For example, `prog_load:map_create` allows eBPF programs loading and eBPF maps creation.
+	// Notice: `security.delegate_bpf.prog_types` and `security.delegate_bpf.map_types` still need to
+	// be configured accordingly.
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: no
+	//  condition: unprivileged container
+	//  shortdesc: Which eBPF commands to allow with delegation mechanism
+	"security.delegate_bpf.cmd_types": validate.Optional(validate.IsBPFDelegationOption("cmds")),
+
+	// lxdmeta:generate(entities=instance; group=security; key=security.delegate_bpf.map_types)
+	// Which eBPF maps to allow with delegation mechanism. Syntax follows a kernel one for `delegate_maps`
+	// bpffs mount option. A number (bitmask) or `:`-separated list of map types to allow can be specified.
+	// For example, `ringbuf` allows `BPF_MAP_TYPE_RINGBUF` map.
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: no
+	//  condition: unprivileged container
+	//  shortdesc: Which eBPF maps to allow with delegation mechanism
+	"security.delegate_bpf.map_types": validate.Optional(validate.IsBPFDelegationOption("maps")),
+
+	// lxdmeta:generate(entities=instance; group=security; key=security.delegate_bpf.prog_types)
+	// Which eBPF program types to allow with delegation mechanism. Syntax follows a kernel one for `delegate_progs`
+	// bpffs mount option. A number (bitmask) or `:`-separated list of program types to allow can be specified.
+	// For example, `socket_filter` allows `BPF_PROG_TYPE_SOCKET_FILTER` program type.
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: no
+	//  condition: unprivileged container
+	//  shortdesc: Which eBPF program types to allow with delegation mechanism
+	"security.delegate_bpf.prog_types": validate.Optional(validate.IsBPFDelegationOption("progs")),
+
+	// lxdmeta:generate(entities=instance; group=security; key=security.delegate_bpf.attach_types)
+	// Which eBPF program attachment types to allow with delegation mechanism. Syntax follows
+	// a kernel one for `delegate_attachs` bpffs mount option.
+	// A number (bitmask) or `:`-separated list of attachment types to allow can be specified.
+	// For example, `cgroup_inet_ingress` allows `BPF_CGROUP_INET_INGRESS` attachment type.
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: no
+	//  condition: unprivileged container
+	//  shortdesc: Which eBPF attach types to allow with delegation mechanism
+	"security.delegate_bpf.attach_types": validate.Optional(validate.IsBPFDelegationOption("attachs")),
+
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.last_state.idmap)
 	// The UID/GID map that has been applied to the container's underlying storage.
 	// This is usually set for containers created on older kernels that don't
