@@ -7,6 +7,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net/http"
 	"net/url"
 
@@ -90,7 +91,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(fmt.Errorf("Invalid instance name"))
+		return response.BadRequest(errors.New("Invalid instance name"))
 	}
 
 	// Flag indicating whether the node running the instance is offline.
@@ -102,7 +103,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 
 	target := request.QueryParam(r, "target")
 	if !s.ServerClustered && target != "" {
-		return response.BadRequest(fmt.Errorf("Target only allowed when clustered"))
+		return response.BadRequest(errors.New("Target only allowed when clustered"))
 	}
 
 	// A POST to /instances/<name>?target=<member> is meant to be used to
@@ -170,7 +171,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 	// and we'll either forward the request or load the instance.
 	if target == "" || !sourceNodeOffline {
 		// Handle requests targeted to an instance on a different node.
-		resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
+		resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -180,7 +181,8 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		}
 	} else if sourceNodeOffline {
 		// If a target was specified, forward the request to the relevant node.
-		resp := forwardedResponseIfTargetIsRemote(s, r)
+		target := request.QueryParam(r, "target")
+		resp := forwardedResponseToNode(r.Context(), s, target)
 		if resp != nil {
 			return resp
 		}
@@ -211,7 +213,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 
 			var targetGroupName string
 
-			targetMemberInfo, targetGroupName, err = limits.CheckTarget(ctx, s.Authorizer, r, tx, targetProject, target, allMembers)
+			targetMemberInfo, targetGroupName, err = limits.CheckTarget(ctx, s.Authorizer, tx, targetProject, target, allMembers)
 			if err != nil {
 				return err
 			}
@@ -278,7 +280,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if targetMemberInfo.IsOffline(s.GlobalConfig.OfflineThreshold()) {
-			return response.BadRequest(fmt.Errorf("Target cluster member is offline"))
+			return response.BadRequest(errors.New("Target cluster member is offline"))
 		}
 	}
 
@@ -332,12 +334,12 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 
 			// Setup the instance move operation.
 			run := func(op *operations.Operation) error {
-				return instancePostMigration(s, inst, req.Name, req.Pool, req.Project, req.Config, req.Devices, req.Profiles, req.InstanceOnly, req.Live, req.AllowInconsistent, op)
+				return instancePostMigration(s, inst, req, op)
 			}
 
 			resources := map[string][]api.URL{}
 			resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", name)}
-			op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil, r)
+			op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil)
 			if err != nil {
 				return response.InternalError(err)
 			}
@@ -359,11 +361,11 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			if len(backups) > 0 {
-				return response.BadRequest(fmt.Errorf("Instance has backups"))
+				return response.BadRequest(errors.New("Instance has backups"))
 			}
 
 			run := func(op *operations.Operation) error {
-				return migrateInstance(s, r, inst, targetMemberInfo.Name, req, op)
+				return migrateInstance(r.Context(), s, inst, targetMemberInfo.Name, req, op)
 			}
 
 			resources := map[string][]api.URL{}
@@ -373,7 +375,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 				resources["containers"] = resources["instances"]
 			}
 
-			op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil, r)
+			op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil)
 			if err != nil {
 				return response.InternalError(err)
 			}
@@ -406,7 +408,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 
 		if req.Target != nil {
 			// Push mode.
-			op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil, r)
+			op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask, operationtype.InstanceMigrate, resources, nil, run, nil, nil)
 			if err != nil {
 				return response.InternalError(err)
 			}
@@ -415,7 +417,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		// Pull mode.
-		op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.InstanceMigrate, resources, ws.Metadata(), run, cancel, ws.Connect, r)
+		op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassWebsocket, operationtype.InstanceMigrate, resources, ws.Metadata(), run, cancel, ws.Connect)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -446,7 +448,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceRename, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask, operationtype.InstanceRename, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -455,18 +457,18 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 }
 
 // Move an instance.
-func instancePostMigration(s *state.State, inst instance.Instance, newName string, newPool string, newProject string, config map[string]string, devices map[string]map[string]string, profiles []string, instanceOnly bool, stateful bool, allowInconsistent bool, op *operations.Operation) error {
+func instancePostMigration(s *state.State, inst instance.Instance, req api.InstancePost, op *operations.Operation) error {
 	if inst.IsSnapshot() {
-		return fmt.Errorf("Instance snapshots cannot be moved between pools")
+		return errors.New("Instance snapshots cannot be moved between pools")
 	}
 
-	if newProject == "" {
-		newProject = inst.Project().Name
+	if req.Project == "" {
+		req.Project = inst.Project().Name
 	}
 
 	statefulStart := false
 	if inst.IsRunning() {
-		if !stateful {
+		if !req.Live {
 			return api.StatusErrorf(http.StatusBadRequest, "Instance must be stopped to move between pools statelessly")
 		}
 
@@ -479,33 +481,29 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 
 	// Copy config from instance to avoid modifying it.
 	localConfig := make(map[string]string)
-	for k, v := range inst.LocalConfig() {
-		localConfig[k] = v
-	}
+	maps.Copy(localConfig, inst.LocalConfig())
 
 	// Set user defined configuration entries.
-	for k, v := range config {
-		localConfig[k] = v
-	}
+	maps.Copy(localConfig, req.Config)
 
 	// Get instance local devices and then set user defined devices.
 	localDevices := inst.LocalDevices().Clone()
-	for devName, dev := range devices {
+	for devName, dev := range req.Devices {
 		localDevices[devName] = dev
 	}
 
 	// Apply previous profiles, if provided profiles are nil.
-	if profiles == nil {
-		profiles = make([]string, 0, len(inst.Profiles()))
+	if req.Profiles == nil {
+		req.Profiles = make([]string, 0, len(inst.Profiles()))
 		for _, p := range inst.Profiles() {
-			profiles = append(profiles, p.Name)
+			req.Profiles = append(req.Profiles, p.Name)
 		}
 	}
 
 	apiProfiles := []api.Profile{}
-	if len(profiles) > 0 {
+	if len(req.Profiles) > 0 {
 		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			profiles, err := dbCluster.GetProfilesIfEnabled(ctx, tx.Tx(), newProject, profiles)
+			profiles, err := dbCluster.GetProfilesIfEnabled(ctx, tx.Tx(), req.Project, req.Profiles)
 			if err != nil {
 				return err
 			}
@@ -574,19 +572,19 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 	}
 
 	// Set specific storage pool for the instance, if provided.
-	if newPool != "" {
-		rootDev["pool"] = newPool
+	if req.Pool != "" {
+		rootDev["pool"] = req.Pool
 		localDevices[rootDevKey] = rootDev
 	}
 
 	// Specify the target instance config with the new name.
 	args := db.InstanceArgs{
-		Name:         newName,
+		Name:         req.Name,
 		BaseImage:    localConfig["volatile.base_image"],
 		Config:       localConfig,
 		Devices:      localDevices,
 		Profiles:     apiProfiles,
-		Project:      newProject,
+		Project:      req.Project,
 		Type:         inst.Type(),
 		Architecture: inst.Architecture(),
 		Description:  inst.Description(),
@@ -598,7 +596,7 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 	// the copy of the instance on the new pool with a temporary name that is different from the source to
 	// avoid conflicts. Then after the source instance has been deleted we will rename the new instance back
 	// to the original name.
-	if newName == inst.Name() && newProject == inst.Project().Name {
+	if req.Name == inst.Name() && req.Project == inst.Project().Name {
 		args.Name, err = instance.MoveTemporaryName(inst)
 		if err != nil {
 			return err
@@ -607,16 +605,28 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 
 	// Copy instance to new target instance.
 	targetInst, err := instanceCreateAsCopy(s, instanceCreateAsCopyOpts{
-		sourceInstance:       inst,
-		targetInstance:       args,
-		instanceOnly:         instanceOnly,
-		applyTemplateTrigger: false, // Don't apply templates when moving.
-		allowInconsistent:    allowInconsistent,
+		sourceInstance:           inst,
+		targetInstance:           args,
+		instanceOnly:             req.InstanceOnly,
+		applyTemplateTrigger:     false, // Don't apply templates when moving.
+		allowInconsistent:        req.AllowInconsistent,
+		overrideSnapshotProfiles: req.OverrideSnapshotProfiles,
 	}, op)
 	if err != nil {
 		return err
 	}
 
+	// Update any permissions relating to the old instance to point to the new instance before it is deleted.
+	// Warnings relating to the old instance will be deleted.
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		q := `UPDATE auth_groups_permissions SET entity_id = ? WHERE entity_type = ? AND entity_id = ?`
+		_, err = tx.Tx().ExecContext(ctx, q, targetInst.ID(), dbCluster.EntityType(entity.TypeInstance), inst.ID())
+		return err
+	})
+	if err != nil {
+		return fmt.Errorf("Failed to copy instance permissions: %w", err)
+	}
+
 	// Delete original instance.
 	err = inst.Delete(true)
 	if err != nil {
@@ -624,8 +634,8 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 	}
 
 	// Rename copy from temporary name to original name if needed.
-	if newName == inst.Name() && newProject == inst.Project().Name {
-		err = targetInst.Rename(newName, false) // Don't apply templates when moving.
+	if req.Name == inst.Name() && req.Project == inst.Project().Name {
+		err = targetInst.Rename(req.Name, false) // Don't apply templates when moving.
 		if err != nil {
 			return err
 		}
@@ -641,14 +651,15 @@ func instancePostMigration(s *state.State, inst instance.Instance, newName strin
 	return nil
 }
 
-// Move a non-ceph instance to another cluster node. Source and target members must be online.
-func instancePostClusteringMigrate(s *state.State, r *http.Request, srcPool storagePools.Pool, srcInst instance.Instance, newInstName string, srcMember db.NodeInfo, newMember db.NodeInfo, stateful bool, allowInconsistent bool) (func(op *operations.Operation) error, error) {
+// Migrate an instance to another cluster node (supports both local and remote storage).
+// Source and target members must be online.
+func instancePostClusteringMigrate(ctx context.Context, s *state.State, srcPool storagePools.Pool, srcInst instance.Instance, newInstName string, srcMember db.NodeInfo, newMember db.NodeInfo, stateful bool, allowInconsistent bool) (func(op *operations.Operation) error, error) {
 	srcMemberOffline := srcMember.IsOffline(s.GlobalConfig.OfflineThreshold())
 
 	// Make sure that the source member is online if we end up being called from another member after a
 	// redirection due to the source member being offline.
 	if srcMemberOffline {
-		return nil, fmt.Errorf("The cluster member hosting the instance is offline")
+		return nil, errors.New("The cluster member hosting the instance is offline")
 	}
 
 	// Save the original value of the "volatile.apply_template" config key,
@@ -679,7 +690,7 @@ func instancePostClusteringMigrate(s *state.State, r *http.Request, srcPool stor
 		// Connect to the destination member, i.e. the member to migrate the instance to.
 		// Use the notify argument to indicate to the destination that we are moving an instance between
 		// cluster members.
-		dest, err := cluster.Connect(newMember.Address, networkCert, s.ServerCert(), r, true)
+		dest, err := cluster.Connect(ctx, newMember.Address, networkCert, s.ServerCert(), true)
 		if err != nil {
 			return fmt.Errorf("Failed to connect to destination server %q: %w", newMember.Address, err)
 		}
@@ -748,7 +759,7 @@ func instancePostClusteringMigrate(s *state.State, r *http.Request, srcPool stor
 			return nil
 		}
 
-		srcOp, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.InstanceMigrate, resources, srcMigration.Metadata(), run, cancel, srcMigration.Connect, r)
+		srcOp, err := operations.OperationCreate(ctx, s, projectName, operations.OperationClassWebsocket, operationtype.InstanceMigrate, resources, srcMigration.Metadata(), run, cancel, srcMigration.Connect)
 		if err != nil {
 			return err
 		}
@@ -887,15 +898,15 @@ func instancePostClusteringMigrate(s *state.State, r *http.Request, srcPool stor
 
 // instancePostClusteringMigrateWithRemoteStorage handles moving a remote shared storage instance from a source member that is offline.
 // This function must be run on the target cluster member to move the instance to.
-func instancePostClusteringMigrateWithRemoteStorage(s *state.State, r *http.Request, srcPool storagePools.Pool, srcInst instance.Instance, newInstName string, newMember db.NodeInfo, stateful bool) (func(op *operations.Operation) error, error) {
+func instancePostClusteringMigrateWithRemoteStorage(s *state.State, srcPool storagePools.Pool, srcInst instance.Instance, newInstName string, newMember db.NodeInfo) (func(op *operations.Operation) error, error) {
 	// Sense checks to avoid unexpected behaviour.
 	if !srcPool.Driver().Info().Remote {
-		return nil, fmt.Errorf("Source instance's storage pool is not remote shared storage")
+		return nil, errors.New("Source instance's storage pool is not remote shared storage")
 	}
 
 	// Check this function is only run on the target member.
 	if s.ServerName != newMember.Name {
-		return nil, fmt.Errorf("Remote shared storage instance move when source member is offline must be run on target member")
+		return nil, errors.New("Remote shared storage instance move when source member is offline must be run on target member")
 	}
 
 	// Check we can convert the instance to the volume types needed.
@@ -951,10 +962,10 @@ func instancePostClusteringMigrateWithRemoteStorage(s *state.State, r *http.Requ
 	return run, nil
 }
 
-func migrateInstance(s *state.State, r *http.Request, inst instance.Instance, targetNode string, req api.InstancePost, op *operations.Operation) error {
+func migrateInstance(ctx context.Context, s *state.State, inst instance.Instance, targetNode string, req api.InstancePost, op *operations.Operation) error {
 	// If target isn't the same as the instance's location.
 	if targetNode == inst.Location() {
-		return fmt.Errorf("Target must be different than instance's current location")
+		return errors.New("Target must be different than instance's current location")
 	}
 
 	var err error
@@ -987,7 +998,7 @@ func migrateInstance(s *state.State, r *http.Request, inst instance.Instance, ta
 
 	// Only use instancePostClusteringMigrateWithRemoteStorage when source member is offline and storage location is remote.
 	if srcMember.IsOffline(s.GlobalConfig.OfflineThreshold()) && srcPool.Driver().Info().Remote {
-		f, err := instancePostClusteringMigrateWithRemoteStorage(s, r, srcPool, inst, req.Name, newMember, req.Live)
+		f, err := instancePostClusteringMigrateWithRemoteStorage(s, srcPool, inst, req.Name, newMember)
 		if err != nil {
 			return err
 		}
@@ -995,7 +1006,7 @@ func migrateInstance(s *state.State, r *http.Request, inst instance.Instance, ta
 		return f(op)
 	}
 
-	f, err := instancePostClusteringMigrate(s, r, srcPool, inst, req.Name, srcMember, newMember, req.Live, req.AllowInconsistent)
+	f, err := instancePostClusteringMigrate(ctx, s, srcPool, inst, req.Name, srcMember, newMember, req.Live, req.AllowInconsistent)
 	if err != nil {
 		return err
 	}
