@@ -3,6 +3,7 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -68,7 +69,7 @@ func instanceRebuildPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(fmt.Errorf("Invalid instance name"))
+		return response.BadRequest(errors.New("Invalid instance name"))
 	}
 
 	instanceType, err := urlInstanceTypeDetect(r)
@@ -77,7 +78,7 @@ func instanceRebuildPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(s, r, targetProjectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, targetProjectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -132,7 +133,7 @@ func instanceRebuildPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if inst.IsRunning() {
-		return response.BadRequest(fmt.Errorf("Instance must be stopped to be rebuilt"))
+		return response.BadRequest(errors.New("Instance must be stopped to be rebuilt"))
 	}
 
 	run := func(op *operations.Operation) error {
@@ -141,17 +142,17 @@ func instanceRebuildPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if req.Source.Server != "" {
-			sourceImage, err = ensureDownloadedImageFitWithinBudget(s, r, op, *targetProject, sourceImageRef, req.Source, inst.Type().String())
+			sourceImage, err = ensureDownloadedImageFitWithinBudget(r.Context(), s, op, *targetProject, sourceImageRef, req.Source, inst.Type().String())
 			if err != nil {
 				return err
 			}
 		}
 
 		if sourceImage == nil {
-			return fmt.Errorf("Image not provided for instance rebuild")
+			return errors.New("Image not provided for instance rebuild")
 		}
 
-		return instanceRebuildFromImage(s, r, inst, sourceImage, op)
+		return instanceRebuildFromImage(r.Context(), s, inst, sourceImage, op)
 	}
 
 	resources := map[string][]api.URL{}
@@ -161,7 +162,7 @@ func instanceRebuildPost(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, targetProject.Name, operations.OperationClassTask, operationtype.InstanceRebuild, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(r.Context(), s, targetProject.Name, operations.OperationClassTask, operationtype.InstanceRebuild, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
