@@ -32,7 +32,7 @@ func (suite *containerTestSuite) TestContainer_ProfilesDefault() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
@@ -70,7 +70,7 @@ func (suite *containerTestSuite) TestContainer_ProfilesMulti() {
 		return err
 	})
 
-	suite.Req.Nil(err, "Failed to create the unprivileged profile.")
+	suite.Req.NoError(err, "Failed to create the unprivileged profile.")
 	defer func() {
 		_ = suite.d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return cluster.DeleteProfile(ctx, tx.Tx(), "default", "unprivileged")
@@ -84,7 +84,7 @@ func (suite *containerTestSuite) TestContainer_ProfilesMulti() {
 
 		return err
 	})
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	args := db.InstanceArgs{
 		Type:      instancetype.Container,
@@ -94,7 +94,7 @@ func (suite *containerTestSuite) TestContainer_ProfilesMulti() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
@@ -127,15 +127,15 @@ func (suite *containerTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
 
 		return err
 	})
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	suite.True(c.IsPrivileged(), "This container should be privileged.")
 
 	out, _, err := c.Render()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	state, ok := out.(*api.Instance)
 	suite.Req.True(ok)
@@ -167,39 +167,39 @@ func (suite *containerTestSuite) TestContainer_LoadFromDB() {
 
 		return err
 	})
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	// Create the container
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
 	poolName, err := c.StoragePool()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	pool, err := storagePools.LoadByName(state, poolName)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		_, err = tx.CreateStoragePoolVolume(ctx, c.Project().Name, c.Name(), "", cluster.StoragePoolVolumeContentTypeFS, pool.ID(), nil, cluster.StoragePoolVolumeContentTypeFS, time.Now())
+		_, err = tx.CreateStoragePoolVolume(ctx, c.Project().Name, c.Name(), "", cluster.StoragePoolVolumeTypeContainer, pool.ID(), nil, cluster.StoragePoolVolumeContentTypeFS, time.Now())
 
 		return err
 	})
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	// Load the container and trigger initLXC()
 	c2, err := instance.LoadByProjectAndName(state, "default", "testFoo")
 	c2.IsRunning()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	hostInterfaces, _ := net.Interfaces()
 
 	apiC1, etagC1, err := c.RenderFull(hostInterfaces)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	apiC2, etagC2, err := c2.RenderFull(hostInterfaces)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	suite.Equal(etagC1, etagC2)
 	suite.Exactly(
@@ -218,7 +218,7 @@ func (suite *containerTestSuite) TestContainer_Path_Regular() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
@@ -235,7 +235,7 @@ func (suite *containerTestSuite) TestContainer_LogPath() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
@@ -251,10 +251,10 @@ func (suite *containerTestSuite) TestContainer_IsPrivileged_Privileged() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	suite.Req.True(c.IsPrivileged(), "This container should be privileged.")
-	suite.Req.Nil(c.Delete(true), "Failed to delete the container.")
+	suite.Req.NoError(c.Delete(true), "Failed to delete the container.")
 }
 
 func (suite *containerTestSuite) TestContainer_AddRoutedNicValidation() {
@@ -273,7 +273,7 @@ func (suite *containerTestSuite) TestContainer_AddRoutedNicValidation() {
 
 		return err
 	})
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	args := db.InstanceArgs{
 		Type:     instancetype.Container,
@@ -297,7 +297,7 @@ func (suite *containerTestSuite) TestContainer_AddRoutedNicValidation() {
 		},
 		Name: "testFoo",
 	}, true)
-	suite.Req.NoError(err, fmt.Errorf("Adding multiple routed with gateway mode ['none'] should succeed. "))
+	suite.Req.NoError(err, "Adding multiple routed with gateway mode ['none'] should succeed.")
 
 	eth0["ipv6.gateway"] = "auto"
 	eth1["ipv6.gateway"] = ""
@@ -312,7 +312,7 @@ func (suite *containerTestSuite) TestContainer_AddRoutedNicValidation() {
 		Name: "testFoo",
 	}, true)
 	suite.Req.Error(err,
-		fmt.Errorf("Adding multiple routed nic devices with any gateway mmode ['auto',''] should throw error. "))
+		"Adding multiple routed nic devices with any gateway mode ['auto',''] should throw error.")
 
 	err = c.Update(db.InstanceArgs{
 		Type:     instancetype.Container,
@@ -325,7 +325,7 @@ func (suite *containerTestSuite) TestContainer_AddRoutedNicValidation() {
 		Name: "testFoo",
 	}, true)
 	suite.Req.NoError(err,
-		fmt.Errorf("Adding multiple nic devices with unicque nictype ['routed'] should throw error. "))
+		"Adding multiple nic devices with unique nictype ['routed'] should throw error.")
 }
 
 func (suite *containerTestSuite) TestContainer_IsPrivileged_Unprivileged() {
@@ -337,10 +337,10 @@ func (suite *containerTestSuite) TestContainer_IsPrivileged_Unprivileged() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	suite.Req.False(c.IsPrivileged(), "This container should be unprivileged.")
-	suite.Req.Nil(c.Delete(true), "Failed to delete the container.")
+	suite.Req.NoError(c.Delete(true), "Failed to delete the container.")
 }
 
 func (suite *containerTestSuite) TestContainer_Rename() {
@@ -351,11 +351,11 @@ func (suite *containerTestSuite) TestContainer_Rename() {
 	}
 
 	c, op, _, err := instance.CreateInternal(suite.d.State(), args, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c.Delete(true) }()
 
-	suite.Req.Nil(c.Rename("testFoo2", true), "Failed to rename the container.")
+	suite.Req.NoError(c.Rename("testFoo2", true), "Failed to rename the container.")
 	suite.Req.Equal(shared.VarPath("containers", "testFoo2"), c.Path())
 }
 
@@ -367,7 +367,7 @@ func (suite *containerTestSuite) TestContainer_findIdmap_isolated() {
 			"security.idmap.isolated": "true",
 		},
 	}, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c1.Delete(true) }()
 
@@ -378,24 +378,24 @@ func (suite *containerTestSuite) TestContainer_findIdmap_isolated() {
 			"security.idmap.isolated": "true",
 		},
 	}, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c2.Delete(true) }()
 
 	map1, err := c1.(instance.Container).NextIdmap()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	map2, err := c2.(instance.Container).NextIdmap()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	host := suite.d.os.IdmapSet.Idmap[0]
 
-	for i := 0; i < 2; i++ {
+	for i := range 2 {
 		suite.Req.Equal(host.Hostid+65536, map1.Idmap[i].Hostid, "hostids don't match %d", i)
 		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(int64(65536), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
-	for i := 0; i < 2; i++ {
+	for i := range 2 {
 		suite.Req.Equal(host.Hostid+65536*2, map2.Idmap[i].Hostid, "hostids don't match")
 		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
@@ -410,7 +410,7 @@ func (suite *containerTestSuite) TestContainer_findIdmap_mixed() {
 			"security.idmap.isolated": "false",
 		},
 	}, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c1.Delete(true) }()
 
@@ -421,24 +421,24 @@ func (suite *containerTestSuite) TestContainer_findIdmap_mixed() {
 			"security.idmap.isolated": "true",
 		},
 	}, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c2.Delete(true) }()
 
 	map1, err := c1.(instance.Container).NextIdmap()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	map2, err := c2.(instance.Container).NextIdmap()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	host := suite.d.os.IdmapSet.Idmap[0]
 
-	for i := 0; i < 2; i++ {
+	for i := range 2 {
 		suite.Req.Equal(host.Hostid, map1.Idmap[i].Hostid, "hostids don't match %d", i)
 		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(host.Maprange, map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
-	for i := 0; i < 2; i++ {
+	for i := range 2 {
 		suite.Req.Equal(host.Hostid+65536, map2.Idmap[i].Hostid, "hostids don't match")
 		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
@@ -454,12 +454,12 @@ func (suite *containerTestSuite) TestContainer_findIdmap_raw() {
 			"raw.idmap":               "both 1000 1000",
 		},
 	}, true)
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 	op.Done(nil)
 	defer func() { _ = c1.Delete(true) }()
 
 	map1, err := c1.(instance.Container).NextIdmap()
-	suite.Req.Nil(err)
+	suite.Req.NoError(err)
 
 	host := suite.d.os.IdmapSet.Idmap[0]
 
@@ -486,7 +486,7 @@ func (suite *containerTestSuite) TestContainer_findIdmap_maxed() {
 	maps := []*idmap.IdmapSet{}
 
 	instances := make([]instance.Instance, 0, 7)
-	for i := 0; i < 7; i++ {
+	for i := range 7 {
 		c, op, _, err := instance.CreateInternal(suite.d.State(), db.InstanceArgs{
 			Type: instancetype.Container,
 			Name: fmt.Sprintf("isol-%d", i),
@@ -497,17 +497,17 @@ func (suite *containerTestSuite) TestContainer_findIdmap_maxed() {
 
 		/* we should fail if there are no ids left */
 		if i == 6 {
-			suite.Req.NotNil(err)
+			suite.Req.Error(err)
 			return
 		}
 
-		suite.Req.Nil(err)
+		suite.Req.NoError(err)
 
 		op.Done(nil)
 		instances = append(instances, c)
 
 		m, err := c.(instance.Container).NextIdmap()
-		suite.Req.Nil(err)
+		suite.Req.NoError(err)
 
 		maps = append(maps, m)
 	}
@@ -526,7 +526,7 @@ func (suite *containerTestSuite) TestContainer_findIdmap_maxed() {
 
 	for _, c := range instances {
 		err := c.Delete(true)
-		suite.Req.NotNil(err)
+		suite.Req.Error(err)
 	}
 }
 
