@@ -44,7 +44,7 @@ import (
 	"github.com/canonical/lxd/shared/version"
 )
 
-func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *operations.Operation, p api.Project, imgAlias string, source api.InstanceSource, imgType string) (*api.Image, error) {
+func ensureDownloadedImageFitWithinBudget(ctx context.Context, s *state.State, op *operations.Operation, p api.Project, imgAlias string, source api.InstanceSource, imgType string) (*api.Image, error) {
 	var autoUpdate bool
 	var err error
 	if p.Config["images.auto_update_cached"] != "" {
@@ -62,7 +62,7 @@ func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *o
 		return nil, err
 	}
 
-	imgDownloaded, err := ImageDownload(r, s, op, &ImageDownloadArgs{
+	imgDownloaded, err := ImageDownload(ctx, s, op, &ImageDownloadArgs{
 		Server:       source.Server,
 		Protocol:     source.Protocol,
 		Certificate:  source.Certificate,
@@ -83,9 +83,9 @@ func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *o
 	return imgDownloaded, nil
 }
 
-func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []api.Profile, img *api.Image, imgAlias string, req *api.InstancesPost) response.Response {
+func createFromImage(ctx context.Context, s *state.State, p api.Project, profiles []api.Profile, img *api.Image, imgAlias string, req *api.InstancesPost) response.Response {
 	if s.DB.Cluster.LocalNodeIsEvacuated() {
-		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		return response.Forbidden(errors.New("Cluster member is evacuated"))
 	}
 
 	dbType, err := instancetype.New(string(req.Type))
@@ -108,17 +108,17 @@ func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []
 		}
 
 		if req.Source.Server != "" {
-			img, err = ensureDownloadedImageFitWithinBudget(s, r, op, p, imgAlias, req.Source, string(req.Type))
+			img, err = ensureDownloadedImageFitWithinBudget(ctx, s, op, p, imgAlias, req.Source, string(req.Type))
 			if err != nil {
 				return err
 			}
 		} else if img != nil {
-			err := ensureImageIsLocallyAvailable(s, r, img, args.Project)
+			err := ensureImageIsLocallyAvailable(ctx, s, img, args.Project)
 			if err != nil {
 				return err
 			}
 		} else {
-			return fmt.Errorf("Image not provided for instance creation")
+			return errors.New("Image not provided for instance creation")
 		}
 
 		args.Architecture, err = osarch.ArchitectureId(img.Architecture)
@@ -142,7 +142,7 @@ func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, p.Name, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(ctx, s, p.Name, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -150,9 +150,9 @@ func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []
 	return operations.OperationResponse(op)
 }
 
-func createFromNone(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
+func createFromNone(ctx context.Context, s *state.State, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
 	if s.DB.Cluster.LocalNodeIsEvacuated() {
-		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		return response.Forbidden(errors.New("Cluster member is evacuated"))
 	}
 
 	dbType, err := instancetype.New(string(req.Type))
@@ -182,7 +182,7 @@ func createFromNone(s *state.State, r *http.Request, projectName string, profile
 		args.Architecture = architecture
 	}
 
-	run := func(op *operations.Operation) error {
+	run := func(_ *operations.Operation) error {
 		// Actually create the instance.
 		_, err := instanceCreateAsEmpty(s, args)
 		if err != nil {
@@ -199,7 +199,7 @@ func createFromNone(s *state.State, r *http.Request, projectName string, profile
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(ctx, s, projectName, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -207,19 +207,17 @@ func createFromNone(s *state.State, r *http.Request, projectName string, profile
 	return operations.OperationResponse(op)
 }
 
-func createFromMigration(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
-	// The request can be nil (see `clusterCopyContainerInternal`).
-	if r != nil {
-		// If it isn't nil, get the protocol.
-		protocol, err := request.GetCtxValue[string](r.Context(), request.CtxProtocol)
-		if err != nil {
-			return response.SmartError(fmt.Errorf("Failed to check request origin: %w", err))
+func createFromMigration(ctx context.Context, s *state.State, projectName string, profiles []api.Profile, req *api.InstancesPost, isClusterNotification bool) response.Response {
+	reqInfo := request.GetContextInfo(ctx)
+	if reqInfo != nil {
+		if reqInfo.Protocol == "" {
+			return response.SmartError(errors.New("Failed to check request origin: Protocol not set in request context"))
 		}
 
 		// If the protocol is not auth.AuthenticationMethodCluster (e.g. not an internal request) and the node has been
 		// evacuated, reject the request.
-		if s.DB.Cluster.LocalNodeIsEvacuated() && protocol != auth.AuthenticationMethodCluster {
-			return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		if s.DB.Cluster.LocalNodeIsEvacuated() && reqInfo.Protocol != auth.AuthenticationMethodCluster {
+			return response.Forbidden(errors.New("Cluster member is evacuated"))
 		}
 	}
 
@@ -248,9 +246,9 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 
 	// Decide if this is an internal cluster move request.
 	var clusterMoveSourceName string
-	if r != nil && isClusterNotification(r) {
+	if isClusterNotification {
 		if req.Source.Source == "" {
-			return response.BadRequest(fmt.Errorf("Source instance name must be provided for cluster member move"))
+			return response.BadRequest(errors.New("Source instance name must be provided for cluster member move"))
 		}
 
 		clusterMoveSourceName = req.Source.Source
@@ -266,7 +264,7 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 
 			if clusterMoveSourceName != "" {
 				// Cluster move doesn't allow renaming as part of migration so fail here.
-				return response.SmartError(fmt.Errorf("Cluster move doesn't allow renaming"))
+				return response.SmartError(errors.New("Cluster move doesn't allow renaming"))
 			}
 
 			req.Source.Refresh = false
@@ -364,12 +362,12 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 
 	var op *operations.Operation
 	if push {
-		op, err = operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.InstanceCreate, resources, sink.Metadata(), run, nil, sink.Connect, r)
+		op, err = operations.OperationCreate(ctx, s, projectName, operations.OperationClassWebsocket, operationtype.InstanceCreate, resources, sink.Metadata(), run, nil, sink.Connect)
 		if err != nil {
 			return response.InternalError(err)
 		}
 	} else {
-		op, err = operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil, r)
+		op, err = operations.OperationCreate(ctx, s, projectName, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -381,9 +379,9 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 
 // createFromConversion receives the root disk (container FS or VM block volume) from the client and creates an
 // instance from it. Conversion options also allow the uploaded image to be converted into a raw format.
-func createFromConversion(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
+func createFromConversion(ctx context.Context, s *state.State, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
 	if s.DB.Cluster.LocalNodeIsEvacuated() {
-		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		return response.Forbidden(errors.New("Cluster member is evacuated"))
 	}
 
 	// Validate migration mode.
@@ -476,7 +474,7 @@ func createFromConversion(s *state.State, r *http.Request, projectName string, p
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.InstanceCreate, resources, sink.Metadata(), run, nil, sink.Connect, r)
+	op, err := operations.OperationCreate(ctx, s, projectName, operations.OperationClassWebsocket, operationtype.InstanceCreate, resources, sink.Metadata(), run, nil, sink.Connect)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -485,13 +483,13 @@ func createFromConversion(s *state.State, r *http.Request, projectName string, p
 	return operations.OperationResponse(op)
 }
 
-func createFromCopy(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
+func createFromCopy(ctx context.Context, s *state.State, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
 	if s.DB.Cluster.LocalNodeIsEvacuated() {
-		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		return response.Forbidden(errors.New("Cluster member is evacuated"))
 	}
 
 	if req.Source.Source == "" {
-		return response.BadRequest(fmt.Errorf("Must specify a source instance"))
+		return response.BadRequest(errors.New("Must specify a source instance"))
 	}
 
 	sourceProject := req.Source.Project
@@ -522,12 +520,12 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 
 			if sourcePoolName != destPoolName {
 				// Redirect to migration
-				return clusterCopyContainerInternal(s, r, source, projectName, profiles, req)
+				return clusterCopyContainerInternal(ctx, s, source, projectName, profiles, req)
 			}
 
 			var pool *api.StoragePool
 
-			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 				_, pool, _, err = tx.GetStoragePoolInAnyState(ctx, sourcePoolName)
 
 				return err
@@ -539,7 +537,7 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 
 			if pool.Driver != "ceph" {
 				// Redirect to migration
-				return clusterCopyContainerInternal(s, r, source, projectName, profiles, req)
+				return clusterCopyContainerInternal(ctx, s, source, projectName, profiles, req)
 			}
 		}
 	}
@@ -598,7 +596,7 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 	}
 
 	if dbType != instancetype.Any && dbType != source.Type() {
-		return response.BadRequest(fmt.Errorf("Instance type should not be specified or should match source type"))
+		return response.BadRequest(errors.New("Instance type should not be specified or should match source type"))
 	}
 
 	args := db.InstanceArgs{
@@ -621,10 +619,11 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 			sourceInstance: source,
 			targetInstance: args,
 			// We keep the ContainerOnly for backward compatibility.
-			instanceOnly:         req.Source.InstanceOnly || req.Source.ContainerOnly, //nolint:staticcheck,unused
-			refresh:              req.Source.Refresh,
-			applyTemplateTrigger: true,
-			allowInconsistent:    req.Source.AllowInconsistent,
+			instanceOnly:             req.Source.InstanceOnly || req.Source.ContainerOnly, //nolint:staticcheck,unused
+			refresh:                  req.Source.Refresh,
+			applyTemplateTrigger:     true,
+			allowInconsistent:        req.Source.AllowInconsistent,
+			overrideSnapshotProfiles: req.Source.OverrideSnapshotProfiles,
 		}, op)
 		if err != nil {
 			return err
@@ -647,7 +646,7 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(s, targetProject, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(ctx, s, targetProject, operations.OperationClassTask, operationtype.InstanceCreate, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -659,8 +658,10 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	revert := revert.New()
 	defer revert.Fail()
 
+	backupsPath := s.BackupsStoragePath()
+
 	// Create temporary file to store uploaded backup data.
-	backupFile, err := os.CreateTemp(shared.VarPath("backups"), backup.WorkingDirPrefix+"_")
+	backupFile, err := os.CreateTemp(backupsPath, backup.WorkingDirPrefix+"_")
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -690,7 +691,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		decomArgs := append(decomArgs, backupFile.Name())
 
 		// Create temporary file to store the decompressed tarball in.
-		tarFile, err := os.CreateTemp(shared.VarPath("backups"), backup.WorkingDirPrefix+"_decompress_")
+		tarFile, err := os.CreateTemp(backupsPath, backup.WorkingDirPrefix+"_decompress_")
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -698,7 +699,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		defer func() { _ = os.Remove(tarFile.Name()) }()
 
 		// Decompress to tarFile temporary file.
-		err = archive.ExtractWithFds(decomArgs[0], decomArgs[1:], nil, nil, s.OS, tarFile)
+		err = archive.ExtractWithFds(s, decomArgs[0], decomArgs[1:], nil, nil, tarFile)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -718,7 +719,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	}
 
 	logger.Debug("Reading backup file info")
-	bInfo, err := backup.GetInfo(backupFile, s.OS, backupFile.Name())
+	bInfo, err := backup.GetInfo(s, backupFile, backupFile.Name())
 	if err != nil {
 		return response.BadRequest(err)
 	}
@@ -727,7 +728,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		return response.BadRequest(errors.New("Backup config is missing"))
 	}
 
-	if bInfo.Config.Container == nil {
+	if bInfo.Config.Instance == nil {
 		return response.BadRequest(errors.New("Instance definition in backup config is missing"))
 	}
 
@@ -735,10 +736,10 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	var req api.InstancesPost
 	err = s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
 		req = api.InstancesPost{
-			InstancePut: bInfo.Config.Container.Writable(),
+			InstancePut: bInfo.Config.Instance.Writable(),
 			Name:        bInfo.Name,
 			Source:      api.InstanceSource{}, // Only relevant for "copy" or "migration", but may not be nil.
-			Type:        api.InstanceType(bInfo.Config.Container.Type),
+			Type:        api.InstanceType(bInfo.Config.Instance.Type),
 		}
 
 		return limits.AllowInstanceCreation(ctx, s.GlobalConfig, tx, projectName, req)
@@ -759,15 +760,20 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		bInfo.Name = instanceName
 	}
 
+	rootVol, err := bInfo.Config.RootVolume()
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed getting the root volume: %w", err))
+	}
+
 	// Override the volume's UUID.
 	// Normally a volume (and its snapshots) gets a new UUID if their config doesn't already have
 	// a `volatile.uuid` field during creation of the volume's record in the DB.
 	// When importing a backup we have to ensure to not pass the backup volume's UUID when
 	// calling the actual backend functions for the target volume that perform some preliminary validation checks.
-	bInfo.Config.Volume.Config["volatile.uuid"] = uuid.New().String()
+	rootVol.Config["volatile.uuid"] = uuid.New().String()
 
 	// Override the volume snapshot's UUID.
-	for _, snapshot := range bInfo.Config.VolumeSnapshots {
+	for _, snapshot := range rootVol.Snapshots {
 		snapshot.Config["volatile.uuid"] = uuid.New().String()
 	}
 
@@ -821,7 +827,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	// Copy reverter so far so we can use it inside run after this function has finished.
 	runRevert := revert.Clone()
 
-	run := func(op *operations.Operation) error {
+	run := func(_ *operations.Operation) error {
 		defer func() { _ = backupFile.Close() }()
 		defer runRevert.Fail()
 
@@ -847,7 +853,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 
 		runRevert.Add(revertHook)
 
-		err = internalImportFromBackup(s, bInfo.Project, bInfo.Name, instanceName != "", devices)
+		err = internalImportFromBackup(context.TODO(), s, bInfo.Project, bInfo.Name, instanceName != "", devices)
 		if err != nil {
 			return fmt.Errorf("Failed importing backup: %w", err)
 		}
@@ -877,7 +883,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	resources := map[string][]api.URL{}
 	resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", bInfo.Name)}
 
-	op, err := operations.OperationCreate(s, bInfo.Project, operations.OperationClassTask, operationtype.BackupRestore, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(r.Context(), s, bInfo.Project, operations.OperationClassTask, operationtype.BackupRestore, resources, nil, run, nil, nil)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -915,7 +921,7 @@ func setupInstanceArgs(s *state.State, instType instancetype.Type, projectName s
 	}
 
 	if storagePool == "" {
-		return "", nil, response.BadRequest(fmt.Errorf("Can't find a storage pool for the instance to use"))
+		return "", nil, response.BadRequest(errors.New("Can't find a storage pool for the instance to use"))
 	}
 
 	if localRootDiskDeviceKey == "" && storagePoolProfile == "" {
@@ -931,7 +937,7 @@ func setupInstanceArgs(s *state.State, instType instancetype.Type, projectName s
 		// Make sure that we do not overwrite a device the user is currently using
 		// under the name "root".
 		rootDevName := "root"
-		for i := 0; i < 100; i++ {
+		for i := range 100 {
 			if args.Devices[rootDevName] == nil {
 				break
 			}
@@ -960,6 +966,7 @@ func setupInstanceArgs(s *state.State, instType instancetype.Type, projectName s
 //	---
 //	consumes:
 //	  - application/json
+//	  - application/octet-stream
 //	produces:
 //	  - application/json
 //	parameters:
@@ -1103,7 +1110,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			// Check if the given target is allowed and try to resolve the right member or group
-			targetMemberInfo, targetGroupName, err = limits.CheckTarget(ctx, s.Authorizer, r, tx, targetProject, target, allMembers)
+			targetMemberInfo, targetGroupName, err = limits.CheckTarget(ctx, s.Authorizer, tx, targetProject, target, allMembers)
 			if err != nil {
 				return err
 			}
@@ -1170,7 +1177,6 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 		if len(req.Profiles) > 0 {
 			profileFilters := make([]dbCluster.ProfileFilter, 0, len(req.Profiles))
 			for _, profileName := range req.Profiles {
-				profileName := profileName
 				profileFilters = append(profileFilters, dbCluster.ProfileFilter{
 					Project: &profileProject,
 					Name:    &profileName,
@@ -1223,12 +1229,12 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 			for {
 				i++
 				req.Name = petname.Generate(2, "-")
-				if !shared.ValueInSlice(req.Name, names) {
+				if !slices.Contains(names, req.Name) {
 					break
 				}
 
 				if i > 100 {
-					return fmt.Errorf("Couldn't generate a new unique name after 100 tries")
+					return errors.New("Couldn't generate a new unique name after 100 tries")
 				}
 			}
 
@@ -1331,7 +1337,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if targetMemberInfo != nil && targetMemberInfo.Address != "" && targetMemberInfo.Name != s.ServerName {
-		client, err := cluster.Connect(targetMemberInfo.Address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+		client, err := cluster.Connect(r.Context(), targetMemberInfo.Address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -1351,15 +1357,15 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 
 	switch req.Source.Type {
 	case api.SourceTypeImage:
-		return createFromImage(s, r, *targetProject, profiles, sourceImage, sourceImageRef, &req)
+		return createFromImage(r.Context(), s, *targetProject, profiles, sourceImage, sourceImageRef, &req)
 	case api.SourceTypeNone:
-		return createFromNone(s, r, targetProjectName, profiles, &req)
+		return createFromNone(r.Context(), s, targetProjectName, profiles, &req)
 	case api.SourceTypeMigration:
-		return createFromMigration(s, r, targetProjectName, profiles, &req)
+		return createFromMigration(r.Context(), s, targetProjectName, profiles, &req, clusterNotification)
 	case api.SourceTypeConversion:
-		return createFromConversion(s, r, targetProjectName, profiles, &req)
+		return createFromConversion(r.Context(), s, targetProjectName, profiles, &req)
 	case api.SourceTypeCopy:
-		return createFromCopy(s, r, targetProjectName, profiles, &req)
+		return createFromCopy(r.Context(), s, targetProjectName, profiles, &req)
 	default:
 		return response.BadRequest(fmt.Errorf("Unknown source type %s", req.Source.Type))
 	}
@@ -1426,7 +1432,7 @@ func instanceFindStoragePool(s *state.State, projectName string, req *api.Instan
 		})
 		if err != nil {
 			if response.IsNotFoundError(err) {
-				return "", "", "", nil, response.BadRequest(fmt.Errorf("This LXD instance does not have any storage pools configured"))
+				return "", "", "", nil, response.BadRequest(errors.New("This LXD instance does not have any storage pools configured"))
 			}
 
 			return "", "", "", nil, response.SmartError(err)
@@ -1440,16 +1446,14 @@ func instanceFindStoragePool(s *state.State, projectName string, req *api.Instan
 	return storagePool, storagePoolProfile, localRootDiskDeviceKey, localRootDiskDevice, nil
 }
 
-func clusterCopyContainerInternal(s *state.State, r *http.Request, source instance.Instance, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
-	name := req.Source.Source
-
+func clusterCopyContainerInternal(ctx context.Context, s *state.State, source instance.Instance, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
 	// Locate the source of the container
 	var nodeAddress string
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		// Load source node.
-		nodeAddress, err = tx.GetNodeAddressOfInstance(ctx, projectName, name, source.Type())
+		nodeAddress, err = tx.GetNodeAddressOfInstance(ctx, source.Project().Name, source.Name(), source.Type())
 		if err != nil {
 			return fmt.Errorf("Failed to get address of instance's member: %w", err)
 		}
@@ -1461,11 +1465,11 @@ func clusterCopyContainerInternal(s *state.State, r *http.Request, source instan
 	}
 
 	if nodeAddress == "" {
-		return response.BadRequest(fmt.Errorf("The source instance is currently offline"))
+		return response.BadRequest(errors.New("The source instance is currently offline"))
 	}
 
 	// Connect to the container source
-	client, err := cluster.Connect(nodeAddress, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(ctx, nodeAddress, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1528,7 +1532,7 @@ func clusterCopyContainerInternal(s *state.State, r *http.Request, source instan
 	req.Source.Project = ""
 
 	// Run the migration
-	return createFromMigration(s, nil, projectName, profiles, req)
+	return createFromMigration(context.Background(), s, projectName, profiles, req, false)
 }
 
 // instanceCreateFinish finalizes the creation process of an instance by starting it based on
