@@ -1,7 +1,9 @@
 package ip
 
 import (
-	"github.com/lxc/lxd/shared"
+	"context"
+
+	"github.com/canonical/lxd/shared"
 )
 
 // Addr represents arguments for address protocol manipulation.
@@ -14,7 +16,7 @@ type Addr struct {
 
 // Add adds new protocol address.
 func (a *Addr) Add() error {
-	_, err := shared.RunCommand("ip", a.Family, "addr", "add", "dev", a.DevName, a.Address)
+	_, err := shared.RunCommandContext(context.TODO(), "ip", a.Family, "addr", "add", "dev", a.DevName, a.Address)
 	if err != nil {
 		return err
 	}
@@ -34,7 +36,7 @@ func (a *Addr) Flush() error {
 		cmd = append(cmd, "scope", a.Scope)
 	}
 
-	_, err := shared.RunCommand("ip", cmd...)
+	_, err := shared.RunCommandContext(context.TODO(), "ip", cmd...)
 	if err != nil {
 		return err
 	}
