@@ -5,11 +5,14 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net/http"
 	"net/url"
 	"reflect"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -20,6 +23,7 @@ import (
 
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
 )
 
 // This is a modified version of https://github.com/grafana/loki/blob/v1.6.1/pkg/promtail/client/.
@@ -55,8 +59,7 @@ type entry struct {
 type Client struct {
 	cfg     config
 	client  *http.Client
-	quit    chan struct{}
-	once    sync.Once
+	cancel  cancel.Canceller
 	entries chan entry
 	wg      sync.WaitGroup
 }
@@ -80,7 +83,7 @@ func NewClient(ctx context.Context, u *url.URL, username string, password string
 		},
 		client:  &http.Client{},
 		entries: make(chan entry),
-		quit:    make(chan struct{}),
+		cancel:  cancel.New(),
 	}
 
 	if caCert != "" {
@@ -118,11 +121,7 @@ func (c *Client) run() {
 	batch := newBatch()
 
 	minWaitCheckFrequency := 10 * time.Millisecond
-	maxWaitCheckFrequency := c.cfg.batchWait / 10
-
-	if maxWaitCheckFrequency < minWaitCheckFrequency {
-		maxWaitCheckFrequency = minWaitCheckFrequency
-	}
+	maxWaitCheckFrequency := max(c.cfg.batchWait/10, minWaitCheckFrequency)
 
 	maxWaitCheck := time.NewTicker(maxWaitCheckFrequency)
 
@@ -132,7 +131,7 @@ func (c *Client) run() {
 
 	for {
 		select {
-		case <-c.quit:
+		case <-c.cancel.Done():
 			return
 
 		case e := <-c.entries:
@@ -161,7 +160,7 @@ func (c *Client) run() {
 }
 
 func (c *Client) checkLoki(ctx context.Context) error {
-	req, err := http.NewRequest("GET", c.cfg.url.String()+"/ready", nil)
+	req, err := http.NewRequest(http.MethodGet, c.cfg.url.String()+"/ready", nil)
 	if err != nil {
 		return err
 	}
@@ -171,7 +170,7 @@ func (c *Client) checkLoki(ctx context.Context) error {
 
 	resp, err := c.client.Do(req)
 	if err != nil {
-		return fmt.Errorf("failed to connect to Loki")
+		return errors.New("failed to connect to Loki")
 	}
 
 	defer resp.Body.Close()
@@ -202,7 +201,7 @@ func (c *Client) sendBatch(batch *batch) {
 
 	var status int
 
-	for i := 0; i < 30; i++ {
+	for range 30 {
 		ctx, cancel := context.WithTimeout(context.Background(), c.cfg.timeout)
 		status, err = c.send(ctx, buf)
 		cancel()
@@ -218,7 +217,7 @@ func (c *Client) sendBatch(batch *batch) {
 
 		// Retry every 10s, but exit if Stop() is called.
 		select {
-		case <-c.quit:
+		case <-c.cancel.Done():
 			return
 		case <-time.After(c.cfg.timeout):
 		}
@@ -226,7 +225,7 @@ func (c *Client) sendBatch(batch *batch) {
 }
 
 func (c *Client) send(ctx context.Context, buf []byte) (int, error) {
-	req, err := http.NewRequest("POST", c.cfg.url.String()+"/loki/api/v1/push", bytes.NewReader(buf))
+	req, err := http.NewRequest(http.MethodPost, c.cfg.url.String()+"/loki/api/v1/push", bytes.NewReader(buf))
 	if err != nil {
 		return -1, err
 	}
@@ -259,13 +258,13 @@ func (c *Client) send(ctx context.Context, buf []byte) (int, error) {
 
 // Stop the client.
 func (c *Client) Stop() {
-	c.once.Do(func() { close(c.quit) })
+	c.cancel.Cancel()
 	c.wg.Wait()
 }
 
 // HandleEvent handles the event received from the internal event listener.
 func (c *Client) HandleEvent(event api.Event) {
-	if !shared.ValueInSlice(event.Type, c.cfg.types) {
+	if !slices.Contains(c.cfg.types, event.Type) {
 		return
 	}
 
@@ -289,7 +288,8 @@ func (c *Client) HandleEvent(event api.Event) {
 
 	context := make(map[string]string)
 
-	if event.Type == api.EventTypeLifecycle {
+	switch event.Type {
+	case api.EventTypeLifecycle:
 		lifecycleEvent := api.EventLifecycle{}
 
 		err := json.Unmarshal(event.Metadata, &lifecycleEvent)
@@ -310,9 +310,7 @@ func (c *Client) HandleEvent(event api.Event) {
 		context["action"] = lifecycleEvent.Action
 		context["source"] = lifecycleEvent.Source
 
-		for k, v := range buildNestedContext("context", lifecycleEvent.Context) {
-			context[k] = v
-		}
+		maps.Copy(context, buildNestedContext("context", lifecycleEvent.Context))
 
 		if lifecycleEvent.Requestor != nil {
 			context["requester-address"] = lifecycleEvent.Requestor.Address
@@ -322,7 +320,7 @@ func (c *Client) HandleEvent(event api.Event) {
 
 		// Add key-value pairs as labels but don't override any labels.
 		for k, v := range context {
-			if shared.ValueInSlice(k, c.cfg.labels) {
+			if slices.Contains(c.cfg.labels, k) {
 				_, ok := entry.labels[k]
 				if !ok {
 					// Label names may not contain any hyphens.
@@ -340,7 +338,7 @@ func (c *Client) HandleEvent(event api.Event) {
 		}
 
 		entry.Line = messagePrefix + lifecycleEvent.Action
-	} else if event.Type == api.EventTypeLogging || event.Type == api.EventTypeOVN {
+	case api.EventTypeLogging, api.EventTypeOVN:
 		logEvent := api.EventLogging{}
 
 		err := json.Unmarshal(event.Metadata, &logEvent)
@@ -368,13 +366,11 @@ func (c *Client) HandleEvent(event api.Event) {
 		// log message itself.
 		context["level"] = logEvent.Level
 
-		for k, v := range buildNestedContext("context", tmpContext) {
-			context[k] = v
-		}
+		maps.Copy(context, buildNestedContext("context", tmpContext))
 
 		// Add key-value pairs as labels but don't override any labels.
 		for k, v := range context {
-			if shared.ValueInSlice(k, c.cfg.labels) {
+			if slices.Contains(c.cfg.labels, k) {
 				_, ok := entry.labels[k]
 				if !ok {
 					entry.labels[k] = v
