@@ -84,7 +84,7 @@ func (c *cmdGlobal) rawArgs(cmd *cobra.Command) []string {
 func main() {
 	// daemon command (main)
 	daemonCmd := cmdDaemon{}
-	app := daemonCmd.Command()
+	app := daemonCmd.command()
 	app.SilenceUsage = true
 	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
@@ -112,7 +112,7 @@ func main() {
 
 	// activateifneeded sub-command
 	activateifneededCmd := cmdActivateifneeded{global: &globalCmd}
-	app.AddCommand(activateifneededCmd.Command())
+	app.AddCommand(activateifneededCmd.command())
 
 	// callhook sub-command
 	callhookCmd := cmdCallhook{global: &globalCmd}
@@ -124,27 +124,27 @@ func main() {
 
 	// forkdns sub-command
 	forkDNSCmd := cmdForkDNS{global: &globalCmd}
-	app.AddCommand(forkDNSCmd.Command())
+	app.AddCommand(forkDNSCmd.command())
 
 	// forkexec sub-command
 	forkexecCmd := cmdForkexec{global: &globalCmd}
-	app.AddCommand(forkexecCmd.Command())
+	app.AddCommand(forkexecCmd.command())
 
 	// forkfile sub-command
 	forkfileCmd := cmdForkfile{global: &globalCmd}
-	app.AddCommand(forkfileCmd.Command())
+	app.AddCommand(forkfileCmd.command())
 
 	// forklimits sub-command
 	forklimitsCmd := cmdForklimits{global: &globalCmd}
-	app.AddCommand(forklimitsCmd.Command())
+	app.AddCommand(forklimitsCmd.command())
 
 	// forkmigrate sub-command
 	forkmigrateCmd := cmdForkmigrate{global: &globalCmd}
-	app.AddCommand(forkmigrateCmd.Command())
+	app.AddCommand(forkmigrateCmd.command())
 
 	// forksyscall sub-command
 	forksyscallCmd := cmdForksyscall{global: &globalCmd}
-	app.AddCommand(forksyscallCmd.Command())
+	app.AddCommand(forksyscallCmd.command())
 
 	// forksyscallgo sub-command
 	forksyscallgoCmd := cmdForksyscallgo{global: &globalCmd}
@@ -152,7 +152,7 @@ func main() {
 
 	// forkcoresched sub-command
 	forkcoreschedCmd := cmdForkcoresched{global: &globalCmd}
-	app.AddCommand(forkcoreschedCmd.Command())
+	app.AddCommand(forkcoreschedCmd.command())
 
 	// forkmount sub-command
 	forkmountCmd := cmdForkmount{global: &globalCmd}
@@ -168,19 +168,19 @@ func main() {
 
 	// forkstart sub-command
 	forkstartCmd := cmdForkstart{global: &globalCmd}
-	app.AddCommand(forkstartCmd.Command())
+	app.AddCommand(forkstartCmd.command())
 
 	// forkuevent sub-command
 	forkueventCmd := cmdForkuevent{global: &globalCmd}
-	app.AddCommand(forkueventCmd.Command())
+	app.AddCommand(forkueventCmd.command())
 
 	// forkzfs sub-command
 	forkzfsCmd := cmdForkZFS{global: &globalCmd}
-	app.AddCommand(forkzfsCmd.Command())
+	app.AddCommand(forkzfsCmd.command())
 
 	// import sub-command
 	importCmd := cmdImport{global: &globalCmd}
-	app.AddCommand(importCmd.Command())
+	app.AddCommand(importCmd.command())
 
 	// init sub-command
 	initCmd := cmdInit{global: &globalCmd}
@@ -188,7 +188,7 @@ func main() {
 
 	// manpage sub-command
 	manpageCmd := cmdManpage{global: &globalCmd}
-	app.AddCommand(manpageCmd.Command())
+	app.AddCommand(manpageCmd.command())
 
 	// migratedumpsuccess sub-command
 	migratedumpsuccessCmd := cmdMigratedumpsuccess{global: &globalCmd}
@@ -196,11 +196,11 @@ func main() {
 
 	// netcat sub-command
 	netcatCmd := cmdNetcat{global: &globalCmd}
-	app.AddCommand(netcatCmd.Command())
+	app.AddCommand(netcatCmd.command())
 
 	// recover sub-command
 	recoverCmd := cmdRecover{global: &globalCmd}
-	app.AddCommand(recoverCmd.Command())
+	app.AddCommand(recoverCmd.command())
 
 	// shutdown sub-command
 	shutdownCmd := cmdShutdown{global: &globalCmd}
