@@ -1,7 +1,9 @@
 package main
 
 import (
+	"context"
 	"encoding/pem"
+	"errors"
 	"fmt"
 	"os"
 
@@ -68,21 +70,21 @@ func (c *cmdInit) Command() *cobra.Command {
 func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	if c.flagAuto && c.flagPreseed {
-		return fmt.Errorf("Can't use --auto and --preseed together")
+		return errors.New("Can't use --auto and --preseed together")
 	}
 
 	if c.flagMinimal && c.flagPreseed {
-		return fmt.Errorf("Can't use --minimal and --preseed together")
+		return errors.New("Can't use --minimal and --preseed together")
 	}
 
 	if c.flagMinimal && c.flagAuto {
-		return fmt.Errorf("Can't use --minimal and --auto together")
+		return errors.New("Can't use --minimal and --auto together")
 	}
 
 	if !c.flagAuto && (c.flagNetworkAddress != "" || c.flagNetworkPort != -1 ||
 		c.flagStorageBackend != "" || c.flagStorageDevice != "" ||
 		c.flagStorageLoopSize != -1 || c.flagStoragePool != "") {
-		return fmt.Errorf("Configuration flags require --auto")
+		return errors.New("Configuration flags require --auto")
 	}
 
 	if c.flagDump && (c.flagAuto || c.flagMinimal ||
@@ -90,7 +92,7 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 		c.flagNetworkPort != -1 || c.flagStorageBackend != "" ||
 		c.flagStorageDevice != "" || c.flagStorageLoopSize != -1 ||
 		c.flagStoragePool != "") {
-		return fmt.Errorf("Can't use --dump with other flags")
+		return errors.New("Can't use --dump with other flags")
 	}
 
 	// Connect to LXD
@@ -173,7 +175,7 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 			config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
 
 			// Cluster certificate
-			cert, err := shared.GetRemoteCertificate("https://"+config.Cluster.ClusterAddress, version.UserAgent)
+			cert, err := shared.GetRemoteCertificate(context.Background(), "https://"+config.Cluster.ClusterAddress, version.UserAgent)
 			if err != nil {
 				fmt.Printf("Error connecting to existing cluster member %q: %v\n", clusterAddress, err)
 				continue
@@ -190,7 +192,7 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		if config.Cluster.ClusterCertificate == "" {
-			return fmt.Errorf("Unable to connect to any of the cluster members specified in join token")
+			return errors.New("Unable to connect to any of the cluster members specified in join token")
 		}
 	}
 
