@@ -3,10 +3,12 @@ package main
 import (
 	"context"
 	"encoding/pem"
+	"errors"
 	"fmt"
 	"net"
 	"os"
 	"os/exec"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -44,7 +46,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 	}
 
 	// Clustering
-	err := c.askClustering(&config, d, server)
+	err := c.askClustering(&config, server)
 	if err != nil {
 		return nil, err
 	}
@@ -58,7 +60,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 		}
 
 		// MAAS
-		err = c.askMAAS(&config, d)
+		err = c.askMAAS(&config)
 		if err != nil {
 			return nil, err
 		}
@@ -70,7 +72,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 		}
 
 		// Daemon config
-		err = c.askDaemon(&config, d, server)
+		err = c.askDaemon(&config, server)
 		if err != nil {
 			return nil, err
 		}
@@ -106,7 +108,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 	return &config, nil
 }
 
-func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
+func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) error {
 	clustering, err := c.global.asker.AskBool("Would you like to use LXD clustering? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
@@ -131,8 +133,8 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			address := util.CanonicalNetworkAddress(value, shared.HTTPSDefaultPort)
 
 			host, _, _ := net.SplitHostPort(address)
-			if shared.ValueInSlice(host, []string{"", "[::]", "0.0.0.0"}) {
-				return fmt.Errorf("Invalid IP address or DNS name")
+			if slices.Contains([]string{"", "[::]", "0.0.0.0"}, host) {
+				return errors.New("Invalid IP address or DNS name")
 			}
 
 			if err == nil {
@@ -170,7 +172,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 
 			// Root is required to access the certificate files
 			if os.Geteuid() != 0 {
-				return fmt.Errorf("Joining an existing cluster requires root privileges")
+				return errors.New("Joining an existing cluster requires root privileges")
 			}
 
 			var joinToken *api.ClusterMemberJoinToken
@@ -199,7 +201,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 				config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
 
 				// Cluster certificate
-				cert, err := shared.GetRemoteCertificate("https://"+config.Cluster.ClusterAddress, version.UserAgent)
+				cert, err := shared.GetRemoteCertificate(context.Background(), "https://"+config.Cluster.ClusterAddress, version.UserAgent)
 				if err != nil {
 					fmt.Printf("Error connecting to existing cluster member %q: %v\n", clusterAddress, err)
 					continue
@@ -216,7 +218,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			}
 
 			if config.Cluster.ClusterCertificate == "" {
-				return fmt.Errorf("Unable to connect to any of the cluster members specified in join token")
+				return errors.New("Unable to connect to any of the cluster members specified in join token")
 			}
 
 			// Pass the raw join token.
@@ -229,7 +231,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			}
 
 			if !clusterWipeMember {
-				return fmt.Errorf("User aborted configuration")
+				return errors.New("User aborted configuration")
 			}
 
 			// Connect to existing cluster
@@ -291,7 +293,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 	return nil
 }
 
-func (c *cmdInit) askMAAS(config *api.InitPreseed, d lxd.InstanceServer) error {
+func (c *cmdInit) askMAAS(config *api.InitPreseed) error {
 	maas, err := c.global.asker.AskBool("Would you like to connect to a MAAS server? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
@@ -465,7 +467,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 							return fmt.Errorf("The auto-detected underlay (%s) isn't a /16 or /24, please specify manually", subnet.String())
 						}
 
-						return fmt.Errorf("The underlay subnet must be a /16 or a /24")
+						return errors.New("The underlay subnet must be a /16 or a /24")
 					}
 
 					return nil
@@ -523,7 +525,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 
 		// IPv4
 		net.Config["ipv4.address"], err = c.global.asker.AskString("What IPv4 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if shared.ValueInSlice(value, []string{"auto", "none"}) {
+			if slices.Contains([]string{"auto", "none"}, value) {
 				return nil
 			}
 
@@ -533,18 +535,18 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !shared.ValueInSlice(net.Config["ipv4.address"], []string{"auto", "none"}) {
+		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv4.address"]) {
 			netIPv4UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv4 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv4.nat"] = fmt.Sprint(netIPv4UseNAT)
+			net.Config["ipv4.nat"] = strconv.FormatBool(netIPv4UseNAT)
 		}
 
 		// IPv6
 		net.Config["ipv6.address"], err = c.global.asker.AskString("What IPv6 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if shared.ValueInSlice(value, []string{"auto", "none"}) {
+			if slices.Contains([]string{"auto", "none"}, value) {
 				return nil
 			}
 
@@ -554,13 +556,13 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !shared.ValueInSlice(net.Config["ipv6.address"], []string{"auto", "none"}) {
+		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv6.address"]) {
 			netIPv6UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv6 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv6.nat"] = fmt.Sprint(netIPv6UseNAT)
+			net.Config["ipv6.nat"] = strconv.FormatBool(netIPv6UseNAT)
 		}
 
 		// Add the new network
@@ -618,7 +620,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 
 	if len(availableBackends) == 0 {
 		if poolType != util.PoolTypeAny {
-			return fmt.Errorf("No storage backends available")
+			return errors.New("No storage backends available")
 		}
 
 		return fmt.Errorf("No %s storage backends available", poolType)
@@ -630,11 +632,11 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 	}
 
 	defaultStorage := "dir"
-	if backingFs == "btrfs" && shared.ValueInSlice("btrfs", availableBackends) {
+	if backingFs == "btrfs" && slices.Contains(availableBackends, "btrfs") {
 		defaultStorage = "btrfs"
-	} else if shared.ValueInSlice("zfs", availableBackends) {
+	} else if slices.Contains(availableBackends, "zfs") {
 		defaultStorage = "zfs"
-	} else if shared.ValueInSlice("btrfs", availableBackends) {
+	} else if slices.Contains(availableBackends, "btrfs") {
 		defaultStorage = "btrfs"
 	}
 
@@ -675,7 +677,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if len(availableBackends) > 1 {
 			defaultBackend := defaultStorage
 			if poolType == util.PoolTypeRemote {
-				if shared.ValueInSlice("ceph", availableBackends) {
+				if slices.Contains(availableBackends, "ceph") {
 					defaultBackend = "ceph"
 				} else {
 					defaultBackend = availableBackends[0] // Default to first remote driver.
@@ -733,7 +735,8 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		}
 
 		if poolCreate {
-			if pool.Driver == "ceph" {
+			switch pool.Driver {
+			case "ceph":
 				// Ask for the name of the cluster
 				pool.Config["ceph.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -751,7 +754,8 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 				if err != nil {
 					return err
 				}
-			} else if pool.Driver == "cephfs" {
+
+			case "cephfs":
 				// Ask for the name of the cluster
 				pool.Config["cephfs.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPHfs cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -763,7 +767,8 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 				if err != nil {
 					return err
 				}
-			} else {
+
+			default:
 				useEmptyBlockDev, err := c.global.asker.AskBool("Would you like to use an existing empty block device (e.g. a disk or partition)? (yes/no) [default=no]: ", "no")
 				if err != nil {
 					return err
@@ -788,14 +793,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 					}
 
 					/* choose 5 GiB < x < 30GiB, where x is 20% of the disk size */
-					defaultSize := uint64(st.Frsize) * st.Blocks / (1024 * 1024 * 1024) / 5
-					if defaultSize > 30 {
-						defaultSize = 30
-					}
-
-					if defaultSize < 5 {
-						defaultSize = 5
-					}
+					defaultSize := max(min(uint64(st.Frsize)*st.Blocks/(1024*1024*1024)/5, 30), 5)
 
 					pool.Config["size"], err = c.global.asker.AskString(
 						fmt.Sprintf("Size in GiB of the new loop device (1GiB minimum) [default=%dGiB]: ", defaultSize),
@@ -809,7 +807,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 							}
 
 							if result < 1 {
-								return fmt.Errorf("Minimum size is 1GiB")
+								return errors.New("Minimum size is 1GiB")
 							}
 
 							return nil
@@ -866,7 +864,7 @@ and make sure that your user can see and run the "thin_check" command before run
 				}
 
 				if !lvmContinueNoThin {
-					return fmt.Errorf("The LVM thin provisioning tools couldn't be found on the system")
+					return errors.New("The LVM thin provisioning tools couldn't be found on the system")
 				}
 
 				pool.Config["lvm.use_thinpool"] = "false"
@@ -880,7 +878,7 @@ and make sure that your user can see and run the "thin_check" command before run
 	return nil
 }
 
-func (c *cmdInit) askDaemon(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
+func (c *cmdInit) askDaemon(config *api.InitPreseed, server *api.Server) error {
 	// Detect lack of uid/gid
 	idmapset, err := idmap.DefaultIdmapSet("", "")
 	if (err != nil || len(idmapset.Idmap) == 0 || idmapset.Usable() != nil) && shared.RunningInUserNS() {
@@ -935,7 +933,7 @@ they otherwise would.
 				netAddr = "[" + netAddr + "]"
 			}
 
-			netPort, err := c.global.asker.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, fmt.Sprintf("%d", shared.HTTPSDefaultPort), func(netPort int64) error {
+			netPort, err := c.global.asker.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, strconv.Itoa(shared.HTTPSDefaultPort), func(netPort int64) error {
 				address := util.CanonicalNetworkAddressFromAddressAndPort(netAddr, netPort, shared.HTTPSDefaultPort)
 
 				if err == nil {
