@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -9,7 +10,7 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/client"
+	"github.com/canonical/lxd/client"
 )
 
 type cmdShutdown struct {
@@ -19,6 +20,7 @@ type cmdShutdown struct {
 	flagTimeout int
 }
 
+// Command returns a cobra.Command object representing the "shutdown" command.
 func (c *cmdShutdown) Command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = "shutdown"
@@ -39,6 +41,7 @@ func (c *cmdShutdown) Command() *cobra.Command {
 	return cmd
 }
 
+// Run executes the "shutdown" command.
 func (c *cmdShutdown) Run(cmd *cobra.Command, args []string) error {
 	connArgs := &lxd.ConnectionArgs{
 		SkipGetServer: true,
@@ -63,10 +66,15 @@ func (c *cmdShutdown) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		// Request shutdown, this shouldn't return until daemon has stopped so use a large request timeout.
-		httpTransport := httpClient.Transport.(*http.Transport)
+		httpTransport, ok := httpClient.Transport.(*http.Transport)
+		if !ok {
+			chResult <- errors.New("httpClient.Transport is not *http.Transport")
+			return
+		}
+
 		httpTransport.ResponseHeaderTimeout = 3600 * time.Second
 
-		_, _, err = d.RawQuery("PUT", fmt.Sprintf("/internal/shutdown?%s", v.Encode()), nil, "")
+		_, _, err = d.RawQuery(http.MethodPut, "/internal/shutdown?"+v.Encode(), nil, "")
 		if err != nil {
 			chResult <- err
 			return
