@@ -2,14 +2,16 @@ package main
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
+	"net/http"
 	"os"
+	"slices"
 
 	"github.com/spf13/cobra"
 
 	"github.com/canonical/lxd/client"
-	"github.com/canonical/lxd/shared"
 	cli "github.com/canonical/lxd/shared/cmd"
 )
 
@@ -66,16 +68,16 @@ func (c *cmdSQL) run(cmd *cobra.Command, args []string) error {
 			return nil
 		}
 
-		return fmt.Errorf("Missing required arguments")
+		return errors.New("Missing required arguments")
 	}
 
 	database := args[0]
 	query := args[1]
 
-	if !shared.ValueInSlice(database, []string{"local", "global"}) {
+	if !slices.Contains([]string{"local", "global"}, database) {
 		_ = cmd.Help()
 
-		return fmt.Errorf("Invalid database type")
+		return errors.New("Invalid database type")
 	}
 
 	if query == "-" {
@@ -104,7 +106,7 @@ func (c *cmdSQL) run(cmd *cobra.Command, args []string) error {
 			url += "&schema=1"
 		}
 
-		response, _, err := d.RawQuery("GET", url, nil, "")
+		response, _, err := d.RawQuery(http.MethodGet, url, nil, "")
 		if err != nil {
 			return fmt.Errorf("failed to request dump: %w", err)
 		}
@@ -124,7 +126,7 @@ func (c *cmdSQL) run(cmd *cobra.Command, args []string) error {
 		Query:    query,
 	}
 
-	response, _, err := d.RawQuery("POST", "/internal/sql", data, "")
+	response, _, err := d.RawQuery(http.MethodPost, "/internal/sql", data, "")
 	if err != nil {
 		return err
 	}
