@@ -3,6 +3,7 @@ package acl
 import (
 	"context"
 	"fmt"
+	"slices"
 
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/db/cluster"
@@ -84,7 +85,7 @@ func Exists(s *state.State, projectName string, name ...string) error {
 	checkedACLNames := make(map[string]struct{}, len(name))
 
 	for _, aclName := range name {
-		if !shared.ValueInSlice(aclName, existingACLNames) {
+		if !slices.Contains(existingACLNames, aclName) {
 			return fmt.Errorf("Network ACL %q does not exist", aclName)
 		}
 
@@ -125,7 +126,7 @@ func UsedBy(s *state.State, aclProjectName string, usageFunc func(ctx context.Co
 			netACLNames := shared.SplitNTrimSpace(network.Config["security.acls"], ",", -1, true)
 			matchedACLNames := []string{}
 			for _, netACLName := range netACLNames {
-				if shared.ValueInSlice(netACLName, matchACLNames) {
+				if slices.Contains(matchACLNames, netACLName) {
 					matchedACLNames = append(matchedACLNames, netACLName)
 				}
 			}
@@ -218,7 +219,7 @@ func UsedBy(s *state.State, aclProjectName string, usageFunc func(ctx context.Co
 			for _, rule := range aclInfo.Ingress {
 				for _, subject := range shared.SplitNTrimSpace(rule.Source, ",", -1, true) {
 					// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
-					if shared.ValueInSlice(subject, matchACLNames) && !shared.ValueInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
+					if slices.Contains(matchACLNames, subject) && !slices.Contains(matchedACLNames, subject) && subject != aclInfo.Name {
 						matchedACLNames = append(matchedACLNames, subject)
 					}
 				}
@@ -228,7 +229,7 @@ func UsedBy(s *state.State, aclProjectName string, usageFunc func(ctx context.Co
 			for _, rule := range aclInfo.Egress {
 				for _, subject := range shared.SplitNTrimSpace(rule.Destination, ",", -1, true) {
 					// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
-					if shared.ValueInSlice(subject, matchACLNames) && !shared.ValueInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
+					if slices.Contains(matchACLNames, subject) && !slices.Contains(matchedACLNames, subject) && subject != aclInfo.Name {
 						matchedACLNames = append(matchedACLNames, subject)
 					}
 				}
@@ -292,7 +293,7 @@ func isInUseByDevice(d deviceConfig.Device, matchACLNames ...string) []string {
 	}
 
 	for _, nicACLName := range shared.SplitNTrimSpace(d["security.acls"], ",", -1, true) {
-		if shared.ValueInSlice(nicACLName, matchACLNames) {
+		if slices.Contains(matchACLNames, nicACLName) {
 			matchedACLNames = append(matchedACLNames, nicACLName)
 		}
 	}
@@ -321,7 +322,7 @@ func NetworkUsage(s *state.State, aclProjectName string, aclNames []string, aclN
 				return fmt.Errorf("Failed to load network %q: %w", nicConfig["network"], err)
 			}
 
-			if shared.ValueInSlice(network.Type, supportedNetTypes) {
+			if slices.Contains(supportedNetTypes, network.Type) {
 				_, found := aclNets[network.Name]
 				if !found {
 					aclNets[network.Name] = NetworkACLUsage{
@@ -334,7 +335,7 @@ func NetworkUsage(s *state.State, aclProjectName string, aclNames []string, aclN
 			}
 
 		case *api.Network:
-			if shared.ValueInSlice(u.Type, supportedNetTypes) {
+			if slices.Contains(supportedNetTypes, u.Type) {
 				_, found := aclNets[u.Name]
 				if !found {
 					networkID, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, u.Name)
