@@ -2,14 +2,18 @@ package network
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"maps"
 	"net"
 	"os"
+	"slices"
 	"strconv"
 	"strings"
 	"unicode"
 
 	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/bgp"
 	"github.com/canonical/lxd/lxd/cluster"
 	"github.com/canonical/lxd/lxd/cluster/request"
 	"github.com/canonical/lxd/lxd/db"
@@ -126,9 +130,7 @@ func (n *common) validate(config map[string]string, driverRules map[string]func(
 	rules := n.validationRules()
 
 	// Merge driver specific rules into common rules.
-	for field, validator := range driverRules {
-		rules[field] = validator
-	}
+	maps.Copy(rules, driverRules)
 
 	// Run the validator against each field.
 	for k, validator := range rules {
@@ -207,6 +209,105 @@ func (n *common) validateZoneNames(config map[string]string) error {
 	return nil
 }
 
+// validateRoutes checks that ip routes are compatible with existing forwards and load balancers.
+func (n *common) validateRoutes(config map[string]string) error {
+	var (
+		routesListIPv4 []*net.IPNet
+		routesListIPv6 []*net.IPNet
+
+		forwards      map[string]map[string][]string
+		loadBalancers map[string]map[string][]string
+
+		err error
+	)
+
+	if config["ipv4.routes"] != "" {
+		routesListIPv4, err = shared.ParseNetworks(config["ipv4.routes"])
+		if err != nil {
+			return fmt.Errorf("Failed parsing ipv4.routes: %w", err)
+		}
+	}
+
+	if config["ipv6.routes"] != "" {
+		routesListIPv6, err = shared.ParseNetworks(config["ipv6.routes"])
+		if err != nil {
+			return fmt.Errorf("Failed parsing ipv6.routes: %w", err)
+		}
+	}
+
+	// Get all listen addresses for all dependent OVN networks.
+	// OVN networks do not support per-member forwards.
+	err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		forwards, err = tx.GetProjectNetworkForwardListenAddressesByUplink(ctx, n.name, false)
+		if err != nil {
+			return fmt.Errorf("Failed to get listen addresses of OVN network forwards: %w", err)
+		}
+
+		loadBalancers, err = tx.GetProjectNetworkLoadBalancerListenAddressesByUplink(ctx, n.name, false)
+		if err != nil {
+			return fmt.Errorf("Failed to get listen addresses of OVN load balancers: %w", err)
+		}
+
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+
+	checkListenAddressesOVN := func(project string, network string, ips []string) error {
+		for _, ip := range ips {
+			var routesKey string
+
+			netIP := net.ParseIP(ip)
+			routeExists := false
+
+			if netIP.To4() != nil {
+				// Listen address is IPv4.
+				for _, routes := range routesListIPv4 {
+					if routes.Contains(netIP) {
+						routeExists = true
+					}
+				}
+
+				routesKey = "ipv4.routes"
+			} else {
+				// Listen address is IPv6.
+				for _, routes := range routesListIPv6 {
+					if routes.Contains(netIP) {
+						routeExists = true
+					}
+				}
+
+				routesKey = "ipv6.routes"
+			}
+
+			if !routeExists {
+				return fmt.Errorf("%q is missing network for listener %q of network %q in project %q", routesKey, ip, network, project)
+			}
+		}
+
+		return nil
+	}
+
+	// Check that IP routes are provided for already existing OVN forwards and load balancers.
+	for _, listenAddresses := range []map[string]map[string][]string{forwards, loadBalancers} {
+		for project, networks := range listenAddresses {
+			for network, ips := range networks {
+				if n.name == network && n.project == project {
+					continue // Skip forwards set up on the uplink.
+				}
+
+				err = checkListenAddressesOVN(project, network, ips)
+				if err != nil {
+					return err
+				}
+			}
+		}
+	}
+
+	return nil
+}
+
 // ValidateName validates network name.
 func (n *common) ValidateName(name string) error {
 	err := validate.IsURLSegmentSafe(name)
@@ -320,7 +421,7 @@ func (n *common) DHCPv6Subnet() *net.IPNet {
 func (n *common) DHCPv4Ranges() []shared.IPRange {
 	dhcpRanges := make([]shared.IPRange, 0)
 	if n.config["ipv4.dhcp.ranges"] != "" {
-		for _, r := range strings.Split(n.config["ipv4.dhcp.ranges"], ",") {
+		for r := range strings.SplitSeq(n.config["ipv4.dhcp.ranges"], ",") {
 			parts := strings.SplitN(strings.TrimSpace(r), "-", 2)
 			if len(parts) == 2 {
 				startIP := net.ParseIP(parts[0])
@@ -340,7 +441,7 @@ func (n *common) DHCPv4Ranges() []shared.IPRange {
 func (n *common) DHCPv6Ranges() []shared.IPRange {
 	dhcpRanges := make([]shared.IPRange, 0)
 	if n.config["ipv6.dhcp.ranges"] != "" {
-		for _, r := range strings.Split(n.config["ipv6.dhcp.ranges"], ",") {
+		for r := range strings.SplitSeq(n.config["ipv6.dhcp.ranges"], ",") {
 			parts := strings.SplitN(strings.TrimSpace(r), "-", 2)
 			if len(parts) == 2 {
 				startIP := net.ParseIP(parts[0])
@@ -356,6 +457,16 @@ func (n *common) DHCPv6Ranges() []shared.IPRange {
 	return dhcpRanges
 }
 
+// Evacuate is invoked on a network in case its parent cluster member gets evacuated.
+func (n *common) Evacuate() error {
+	return nil
+}
+
+// Restore is invoked on a network in case its parent cluster member gets restored.
+func (n *common) Restore() error {
+	return nil
+}
+
 // update the internal config variables, and if not cluster notification, notifies all nodes and updates database.
 func (n *common) update(applyNetwork api.NetworkPut, targetNode string, clientType request.ClientType) error {
 	// Update internal config before database has been updated (so that if update is a notification we apply
@@ -377,7 +488,7 @@ func (n *common) update(applyNetwork api.NetworkPut, targetNode string, clientTy
 			sendNetwork.Config = make(map[string]string)
 			for k, v := range applyNetwork.Config {
 				// Don't forward node specific keys (these will be merged in on recipient node).
-				if shared.ValueInSlice(k, db.NodeSpecificNetworkConfig) {
+				if slices.Contains(db.NodeSpecificNetworkConfig, k) {
 					continue
 				}
 
@@ -421,18 +532,14 @@ func (n *common) configChanged(newNetwork api.NetworkPut) (bool, []string, api.N
 
 	// Diff the configurations.
 	changedKeys := []string{}
-	dbUpdateNeeded := false
-
-	if newNetwork.Description != n.description {
-		dbUpdateNeeded = true
-	}
+	dbUpdateNeeded := newNetwork.Description != n.description
 
 	for k, v := range oldNetwork.Config {
 		if v != newNetwork.Config[k] {
 			dbUpdateNeeded = true
 
 			// Add non-user changed key to list of changed keys.
-			if !strings.HasPrefix(k, "user.") && !shared.ValueInSlice(k, changedKeys) {
+			if !strings.HasPrefix(k, "user.") && !slices.Contains(changedKeys, k) {
 				changedKeys = append(changedKeys, k)
 			}
 		}
@@ -443,7 +550,7 @@ func (n *common) configChanged(newNetwork api.NetworkPut) (bool, []string, api.N
 			dbUpdateNeeded = true
 
 			// Add non-user changed key to list of changed keys.
-			if !strings.HasPrefix(k, "user.") && !shared.ValueInSlice(k, changedKeys) {
+			if !strings.HasPrefix(k, "user.") && !slices.Contains(changedKeys, k) {
 				changedKeys = append(changedKeys, k)
 			}
 		}
@@ -671,7 +778,7 @@ func (n *common) bgpClearPeers(config map[string]string) error {
 		// Remove the peer.
 		fields := strings.Split(peer, ",")
 		err := n.state.BGP.RemovePeer(net.ParseIP(fields[0]))
-		if err != nil {
+		if err != nil && !errors.Is(err, bgp.ErrPeerNotFound) {
 			return err
 		}
 	}
@@ -687,7 +794,7 @@ func (n *common) bgpSetupPeers(oldConfig map[string]string) error {
 
 	// Remove old peers.
 	for _, peer := range oldPeers {
-		if shared.ValueInSlice(peer, newPeers) {
+		if slices.Contains(newPeers, peer) {
 			continue
 		}
 
@@ -701,7 +808,7 @@ func (n *common) bgpSetupPeers(oldConfig map[string]string) error {
 
 	// Add new peers.
 	for _, peer := range newPeers {
-		if shared.ValueInSlice(peer, oldPeers) {
+		if slices.Contains(oldPeers, peer) {
 			continue
 		}
 
@@ -781,7 +888,7 @@ func (n *common) bgpSetupPrefixes(oldConfig map[string]string) error {
 					return err
 				}
 			}
-		} else if !shared.ValueInSlice(n.config[fmt.Sprintf("ipv%d.address", ipVersion)], []string{"", "none"}) {
+		} else if !slices.Contains([]string{"", "none"}, n.config[fmt.Sprintf("ipv%d.address", ipVersion)]) {
 			// If network has NAT disabled, then export the network's subnet if specified.
 			netAddress := n.config[fmt.Sprintf("ipv%d.address", ipVersion)]
 			_, subnet, err := net.ParseCIDR(netAddress)
@@ -809,7 +916,7 @@ func (n *common) bgpGetPeers(config map[string]string) []string {
 		}
 
 		fields := strings.Split(k, ".")
-		if !shared.ValueInSlice(fields[2], peerNames) {
+		if !slices.Contains(peerNames, fields[2]) {
 			peerNames = append(peerNames, fields[2])
 		}
 	}
@@ -855,7 +962,7 @@ func (n *common) projectUplinkIPQuotaAvailable(ctx context.Context, tx *db.Clust
 // forwardValidate validates the forward request.
 func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwardPut) ([]*forwardPortMap, error) {
 	if listenAddress == nil {
-		return nil, fmt.Errorf("Invalid listen address")
+		return nil, errors.New("Invalid listen address")
 	}
 
 	if listenAddress.IsUnspecified() {
@@ -900,17 +1007,17 @@ func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwar
 
 	if forward.Config["target_address"] != "" {
 		if defaultTargetAddress == nil {
-			return nil, fmt.Errorf("Invalid default target address")
+			return nil, errors.New("Invalid default target address")
 		}
 
 		defaultTargetIsIP4 := defaultTargetAddress.To4() != nil
 		if listenIsIP4 != defaultTargetIsIP4 {
-			return nil, fmt.Errorf("Cannot mix IP versions in listen address and default target address")
+			return nil, errors.New("Cannot mix IP versions in listen address and default target address")
 		}
 
 		// Check default target address is within network's subnet.
 		if netSubnet != nil && !SubnetContainsIP(netSubnet, defaultTargetAddress) {
-			return nil, fmt.Errorf("Default target address is not within the network subnet")
+			return nil, errors.New("Default target address is not within the network subnet")
 		}
 	}
 
@@ -926,7 +1033,7 @@ func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwar
 	// Maps portSpecID to a portMap struct.
 	portMaps := make([]*forwardPortMap, 0, len(forward.Ports))
 	for portSpecID, portSpec := range forward.Ports {
-		if !shared.ValueInSlice(portSpec.Protocol, validPortProcols) {
+		if !slices.Contains(validPortProcols, portSpec.Protocol) {
 			return nil, fmt.Errorf("Invalid port protocol in port specification %d, protocol must be one of: %s", portSpecID, strings.Join(validPortProcols, ", "))
 		}
 
@@ -969,7 +1076,7 @@ func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwar
 				return nil, fmt.Errorf("Invalid listen port in port specification %d: %w", portSpecID, err)
 			}
 
-			for i := int64(0); i < portRange; i++ {
+			for i := range portRange {
 				port := portFirst + i
 				_, found := listenPorts[portSpec.Protocol][port]
 				if found {
@@ -994,7 +1101,7 @@ func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwar
 					return nil, fmt.Errorf("Invalid target port in port specification %d", portSpecID)
 				}
 
-				for i := int64(0); i < portRange; i++ {
+				for i := range portRange {
 					port := portFirst + i
 					portMap.target.ports = append(portMap.target.ports, uint64(port))
 				}
@@ -1175,7 +1282,7 @@ func (n *common) getExternalSubnetInUse(ctx context.Context, tx *db.ClusterTx, u
 // loadBalancerValidate validates the load balancer request.
 func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkLoadBalancerPut) ([]*loadBalancerPortMap, error) {
 	if listenAddress == nil {
-		return nil, fmt.Errorf("Invalid listen address")
+		return nil, errors.New("Invalid listen address")
 	}
 
 	listenIsIP4 := listenAddress.To4() != nil
@@ -1256,7 +1363,7 @@ func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkL
 				return nil, fmt.Errorf("Invalid backend port specification %d in backend specification %d: %w", portSpecID, backendSpecID, err)
 			}
 
-			for i := int64(0); i < portRange; i++ {
+			for i := range portRange {
 				port := portFirst + i
 				target.ports = append(target.ports, uint64(port))
 			}
@@ -1268,7 +1375,7 @@ func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkL
 	// Check ports config.
 	portMaps := make([]*loadBalancerPortMap, 0, len(forward.Ports))
 	for portSpecID, portSpec := range forward.Ports {
-		if !shared.ValueInSlice(portSpec.Protocol, validPortProcols) {
+		if !slices.Contains(validPortProcols, portSpec.Protocol) {
 			return nil, fmt.Errorf("Invalid port protocol in port specification %d, protocol must be one of: %s", portSpecID, strings.Join(validPortProcols, ", "))
 		}
 
@@ -1290,7 +1397,7 @@ func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkL
 				return nil, fmt.Errorf("Invalid listen port in port specification %d: %w", portSpecID, err)
 			}
 
-			for i := int64(0); i < portRange; i++ {
+			for i := range portRange {
 				port := portFirst + i
 				_, found := listenPorts[portSpec.Protocol][port]
 				if found {
@@ -1443,7 +1550,7 @@ func (n *common) peerValidate(peerName string, peer *api.NetworkPeerPut) error {
 		return err
 	}
 
-	if shared.ValueInSlice(peerName, acl.ReservedNetworkSubects) {
+	if slices.Contains(acl.ReservedNetworkSubects, peerName) {
 		return fmt.Errorf("Name cannot be one of the reserved network subjects: %v", acl.ReservedNetworkSubects)
 	}
 
