@@ -2,9 +2,12 @@ package network
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"maps"
 	"net"
 	"net/http"
+	"slices"
 	"strconv"
 
 	"github.com/canonical/lxd/lxd/cluster/request"
@@ -209,9 +212,7 @@ func (n *physical) Validate(config map[string]string) error {
 		return err
 	}
 
-	for k, v := range bgpRules {
-		rules[k] = v
-	}
+	maps.Copy(rules, bgpRules)
 
 	// Validate the configuration.
 	err = n.validate(config, rules)
@@ -219,21 +220,43 @@ func (n *physical) Validate(config map[string]string) error {
 		return err
 	}
 
+	// Check that ipv4.routes and ipv6.routes contain the routes for existing OVN network
+	// forwards and load balancers.
+	err = n.validateRoutes(config)
+	if err != nil {
+		return err
+	}
+
 	return nil
 }
 
 // checkParentUse checks if parent is already in use by another network or instance device.
-func (n *physical) checkParentUse(ourConfig map[string]string) (bool, error) {
-	// Get all managed networks across all projects.
+// Returns an error if parent is already in use or the check has failed.
+func (n *physical) checkParentUse(ourConfig map[string]string) error {
 	var err error
-	var projectNetworks map[string]map[int64]api.Network
+	var projectNetworks map[string]map[int64]api.Network // All managed networks across all projects.
+	var nodesNetworksParent map[int64]map[string]string  // Node IDs mapped to networks and their node-specific parent.
 
 	err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Get all managed networks across all projects.
+		// If in clustered mode, returned network configs are for the current node.
 		projectNetworks, err = tx.GetCreatedNetworks(ctx)
-		return err
+		if err != nil {
+			return fmt.Errorf("Failed to load all networks: %w", err)
+		}
+
+		// Get parent interfaces for all networks on all nodes.
+		if n.state.ServerClustered && len(n.nodes) > 1 {
+			nodesNetworksParent, err = tx.GetNetworksNodeParent(ctx)
+			if err != nil {
+				return fmt.Errorf("Failed to load node-specific configs: %w", err)
+			}
+		}
+
+		return nil
 	})
 	if err != nil {
-		return false, fmt.Errorf("Failed to load all networks: %w", err)
+		return err
 	}
 
 	for projectName, networks := range projectNetworks {
@@ -251,13 +274,33 @@ func (n *physical) checkParentUse(ourConfig map[string]string) (bool, error) {
 				// If either network doesn't specify a vlan, or both specify same vlan,
 				// then we can't use this parent.
 				if (network.Config["vlan"] == "" || ourConfig["vlan"] == "") || network.Config["vlan"] == ourConfig["vlan"] {
-					return true, nil
+					return fmt.Errorf("Parent interface %q in use by another network", ourConfig["parent"])
 				}
 			}
 		}
 	}
 
-	return false, nil
+	currNodeID := n.state.DB.Cluster.GetNodeID()
+
+	// Check that parent interfaces on other nodes are not already in use.
+	for nodeID, networksParent := range nodesNetworksParent {
+		if nodeID == currNodeID {
+			continue // Skip the current node, it has been checked already.
+		}
+
+		parentsInUse := make(map[string]struct{})
+
+		for _, parent := range networksParent {
+			_, alreadyInUse := parentsInUse[parent]
+			if alreadyInUse {
+				return fmt.Errorf("Parent interface %q in use by another network on cluster member %q", parent, n.nodes[nodeID].Name)
+			}
+
+			parentsInUse[parent] = struct{}{}
+		}
+	}
+
+	return nil
 }
 
 // Create checks whether the referenced parent interface is used by other networks or instance devices, as we
@@ -267,14 +310,10 @@ func (n *physical) Create(clientType request.ClientType) error {
 
 	// We only need to check in the database once, not on every clustered node.
 	if clientType == request.ClientTypeNormal {
-		inUse, err := n.checkParentUse(n.config)
+		err := n.checkParentUse(n.config)
 		if err != nil {
 			return err
 		}
-
-		if inUse {
-			return fmt.Errorf("Parent interface %q in use by another network", n.config["parent"])
-		}
 	}
 
 	return nil
@@ -289,7 +328,7 @@ func (n *physical) Delete(clientType request.ClientType) error {
 		return err
 	}
 
-	return n.common.delete()
+	return n.delete()
 }
 
 // Rename renames a network.
@@ -297,7 +336,7 @@ func (n *physical) Rename(newName string) error {
 	n.logger.Debug("Rename", logger.Ctx{"newName": newName})
 
 	// Rename common steps.
-	err := n.common.rename(newName)
+	err := n.rename(newName)
 	if err != nil {
 		return err
 	}
@@ -363,7 +402,7 @@ func (n *physical) setup(oldConfig map[string]string) error {
 	// Record if we created this device or not (if we have not already recorded that we created it previously),
 	// so it can be removed on stop. This way we won't overwrite the setting on LXD restart.
 	if shared.IsFalseOrEmpty(n.config["volatile.last_state.created"]) {
-		n.config["volatile.last_state.created"] = fmt.Sprint(created)
+		n.config["volatile.last_state.created"] = strconv.FormatBool(created)
 		err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.UpdateNetwork(ctx, n.project, n.name, n.description, n.config)
 		})
@@ -372,10 +411,14 @@ func (n *physical) setup(oldConfig map[string]string) error {
 		}
 	}
 
+	nodeEvacuated := n.state.DB.Cluster.LocalNodeIsEvacuated()
+
 	// Setup BGP.
-	err = n.bgpSetup(oldConfig)
-	if err != nil {
-		return err
+	if !nodeEvacuated {
+		err = n.bgpSetup(oldConfig)
+		if err != nil {
+			return err
+		}
 	}
 
 	revert.Success()
@@ -424,12 +467,28 @@ func (n *physical) Stop() error {
 	return nil
 }
 
+// Evacuate the network by clearing BGP.
+func (n *physical) Evacuate() error {
+	n.logger.Debug("Evacuate")
+
+	// Clear BGP.
+	return n.bgpClear(n.config)
+}
+
+// Restore the network by setting up BGP.
+func (n *physical) Restore() error {
+	n.logger.Debug("Restore")
+
+	// Setup BGP.
+	return n.bgpSetup(nil)
+}
+
 // Update updates the network. Accepts notification boolean indicating if this update request is coming from a
 // cluster notification, in which case do not update the database, just apply local changes needed.
 func (n *physical) Update(newNetwork api.NetworkPut, targetNode string, clientType request.ClientType) error {
 	n.logger.Debug("Update", logger.Ctx{"clientType": clientType, "newNetwork": newNetwork})
 
-	dbUpdateNeeded, changedKeys, oldNetwork, err := n.common.configChanged(newNetwork)
+	dbUpdateNeeded, changedKeys, oldNetwork, err := n.configChanged(newNetwork)
 	if err != nil {
 		return err
 	}
@@ -442,30 +501,26 @@ func (n *physical) Update(newNetwork api.NetworkPut, targetNode string, clientTy
 	// pending, then don't apply the new settings to the node, just to the database record (ready for the
 	// actual global create request to be initiated).
 	if n.Status() == api.NetworkStatusPending || n.LocalStatus() == api.NetworkStatusPending {
-		return n.common.update(newNetwork, targetNode, clientType)
+		return n.update(newNetwork, targetNode, clientType)
 	}
 
 	revert := revert.New()
 	defer revert.Fail()
 
-	hostNameChanged := shared.ValueInSlice("vlan", changedKeys) || shared.ValueInSlice("parent", changedKeys)
+	hostNameChanged := slices.Contains(changedKeys, "vlan") || slices.Contains(changedKeys, "parent")
 
 	// We only need to check in the database once, not on every clustered node.
 	if clientType == request.ClientTypeNormal {
 		if hostNameChanged {
 			isUsed, err := n.IsUsed()
 			if isUsed || err != nil {
-				return fmt.Errorf("Cannot update network parent interface when in use")
+				return errors.New("Cannot update network parent interface when in use")
 			}
 
-			inUse, err := n.checkParentUse(newNetwork.Config)
+			err = n.checkParentUse(newNetwork.Config)
 			if err != nil {
 				return err
 			}
-
-			if inUse {
-				return fmt.Errorf("Parent interface %q in use by another network", newNetwork.Config["parent"])
-			}
 		}
 	}
 
@@ -482,18 +537,25 @@ func (n *physical) Update(newNetwork api.NetworkPut, targetNode string, clientTy
 	// Define a function which reverts everything.
 	revert.Add(func() {
 		// Reset changes to all nodes and database.
-		_ = n.common.update(oldNetwork, targetNode, clientType)
+		_ = n.update(oldNetwork, targetNode, clientType)
 	})
 
 	// Apply changes to all nodes and databse.
-	err = n.common.update(newNetwork, targetNode, clientType)
+	err = n.update(newNetwork, targetNode, clientType)
 	if err != nil {
 		return err
 	}
 
-	err = n.setup(oldNetwork.Config)
-	if err != nil {
-		return err
+	if !hostNameChanged {
+		err = n.setup(oldNetwork.Config)
+		if err != nil {
+			return err
+		}
+	} else {
+		err = n.setup(nil)
+		if err != nil {
+			return err
+		}
 	}
 
 	revert.Success()
@@ -502,7 +564,7 @@ func (n *physical) Update(newNetwork api.NetworkPut, targetNode string, clientTy
 	// Do this after the network has been successfully updated so that a failure to notify a dependent network
 	// doesn't prevent the network itself from being updated.
 	if clientType == request.ClientTypeNormal && len(changedKeys) > 0 {
-		n.common.notifyDependentNetworks(changedKeys)
+		n.notifyDependentNetworks(changedKeys)
 	}
 
 	return nil
