@@ -4,7 +4,7 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"fmt"
+	"errors"
 	"net/http"
 	"net/url"
 	"time"
@@ -69,6 +69,11 @@ var networkACLLogCmd = APIEndpoint{
 //      description: Project name
 //      type: string
 //      example: default
+//    - in: query
+//      name: all-projects
+//      description: Retrieve network ACLs from all projects
+//      type: boolean
+//      example: true
 //  responses:
 //    "200":
 //      description: API endpoints
@@ -118,6 +123,11 @@ var networkACLLogCmd = APIEndpoint{
 //	    description: Project name
 //	    type: string
 //	    example: default
+//	  - in: query
+//	    name: all-projects
+//	    description: Retrieve network ACLs from all projects
+//	    type: boolean
+//	    example: true
 //	responses:
 //	  "200":
 //	    description: API endpoints
@@ -149,25 +159,51 @@ var networkACLLogCmd = APIEndpoint{
 func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	requestProjectName := request.ProjectParam(r)
-	effectiveProjectName, _, err := project.NetworkProject(s.DB.Cluster, requestProjectName)
+	requestProjectName, allProjects, err := request.ProjectParams(r)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
+	var effectiveProjectName string
+	if !allProjects {
+		// Project specific requests require an effective project, when "features.networks" is enabled this is the requested project, otherwise it is the default project.
+		effectiveProjectName, _, err = project.NetworkProject(s.DB.Cluster, requestProjectName)
+		if err != nil {
+			return response.SmartError(err)
+		}
+
+		// If the request is project specific, then set effective project name in the request context so that the authorizer can generate the correct URL.
+		reqInfo := request.GetContextInfo(r.Context())
+		reqInfo.EffectiveProjectName = effectiveProjectName
+	}
+
 	recursion := util.IsRecursionRequest(r)
 	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeNetworkACL, true)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	var aclNames []string
-
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	var aclNames map[string][]string
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		// Get list of Network ACLs.
-		aclNames, err = tx.GetNetworkACLs(ctx, effectiveProjectName)
+		if allProjects {
+			// Get list of Network ACLs across all projects.
+			aclNames, err = tx.GetNetworkACLsAllProjects(ctx)
+			if err != nil {
+				return err
+			}
+		} else {
+			// Get list of Network ACLs.
+			acls, err := tx.GetNetworkACLs(ctx, effectiveProjectName)
+			if err != nil {
+				return err
+			}
+
+			// ACL names should be mapped to the requested project for project specific requests.
+			aclNames = map[string][]string{}
+			aclNames[requestProjectName] = acls
+		}
 
 		return err
 	})
@@ -175,7 +211,6 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 		return response.InternalError(err)
 	}
 
-	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
 	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeNetworkACL)
 	if err != nil {
 		return response.SmartError(err)
@@ -184,25 +219,34 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 	resultString := []string{}
 	resultMap := []*api.NetworkACL{}
 	urlToNetworkACL := make(map[*api.URL]auth.EntitlementReporter)
-	for _, aclName := range aclNames {
-		if !userHasPermission(entity.NetworkACLURL(requestProjectName, aclName)) {
-			continue
-		}
-
-		if !recursion {
-			resultString = append(resultString, fmt.Sprintf("/%s/network-acls/%s", version.APIVersion, aclName))
-		} else {
-			netACL, err := acl.LoadByName(s, effectiveProjectName, aclName)
-			if err != nil {
+	for projectName, acls := range aclNames {
+		for _, aclName := range acls {
+			if !userHasPermission(entity.NetworkACLURL(projectName, aclName)) {
 				continue
 			}
 
-			netACLInfo := netACL.Info()
-			netACLInfo.UsedBy, _ = netACL.UsedBy() // Ignore errors in UsedBy, will return nil.
-			netACLInfo.UsedBy = project.FilterUsedBy(s.Authorizer, r, netACLInfo.UsedBy)
+			if !recursion {
+				resultString = append(resultString, api.NewURL().Path(version.APIVersion, "network-acls", aclName).String())
+			} else {
+				var netACL acl.NetworkACL
+				if !allProjects {
+					netACL, err = acl.LoadByName(s, effectiveProjectName, aclName)
+				} else {
+					netACL, err = acl.LoadByName(s, projectName, aclName)
+				}
 
-			resultMap = append(resultMap, netACLInfo)
-			urlToNetworkACL[entity.NetworkACLURL(requestProjectName, aclName)] = netACLInfo
+				if err != nil {
+					return response.SmartError(err)
+				}
+
+				netACLInfo := netACL.Info()
+				netACLInfo.UsedBy, _ = netACL.UsedBy() // Ignore errors in UsedBy, will return nil.
+				netACLInfo.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, netACLInfo.UsedBy)
+				netACLInfo.Project = projectName
+
+				resultMap = append(resultMap, netACLInfo)
+				urlToNetworkACL[entity.NetworkACLURL(requestProjectName, aclName)] = netACLInfo
+			}
 		}
 	}
 
@@ -270,7 +314,7 @@ func networkACLsPost(d *Daemon, r *http.Request) response.Response {
 
 	_, err = acl.LoadByName(s, projectName, req.Name)
 	if err == nil {
-		return response.BadRequest(fmt.Errorf("The network ACL already exists"))
+		return response.BadRequest(errors.New("The network ACL already exists"))
 	}
 
 	err = acl.Create(s, projectName, &req)
@@ -283,7 +327,7 @@ func networkACLsPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	lc := lifecycle.NetworkACLCreated.Event(netACL, request.CreateRequestor(r), nil)
+	lc := lifecycle.NetworkACLCreated.Event(netACL, request.CreateRequestor(r.Context()), nil)
 	s.Events.SendLifecycle(projectName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -336,7 +380,7 @@ func networkACLDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	s.Events.SendLifecycle(projectName, lifecycle.NetworkACLDeleted.Event(netACL, request.CreateRequestor(r), nil))
+	s.Events.SendLifecycle(projectName, lifecycle.NetworkACLDeleted.Event(netACL, request.CreateRequestor(r.Context()), nil))
 
 	return response.EmptySyncResponse
 }
@@ -410,7 +454,7 @@ func networkACLGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	info.UsedBy = project.FilterUsedBy(s.Authorizer, r, info.UsedBy)
+	info.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, info.UsedBy)
 	if len(withEntitlements) > 0 {
 		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeNetworkACL, withEntitlements, map[*api.URL]auth.EntitlementReporter{entity.NetworkACLURL(projectName, aclName): info})
 		if err != nil {
@@ -541,7 +585,7 @@ func networkACLPut(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	s.Events.SendLifecycle(projectName, lifecycle.NetworkACLUpdated.Event(netACL, request.CreateRequestor(r), nil))
+	s.Events.SendLifecycle(projectName, lifecycle.NetworkACLUpdated.Event(netACL, request.CreateRequestor(r.Context()), nil))
 
 	return response.EmptySyncResponse
 }
@@ -610,7 +654,7 @@ func networkACLPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	lc := lifecycle.NetworkACLRenamed.Event(netACL, request.CreateRequestor(r), logger.Ctx{"old_name": aclName})
+	lc := lifecycle.NetworkACLRenamed.Event(netACL, request.CreateRequestor(r.Context()), logger.Ctx{"old_name": aclName})
 	s.Events.SendLifecycle(projectName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
