@@ -2,6 +2,8 @@ package main
 
 import (
 	"context"
+	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -57,82 +59,59 @@ var operationWebsocket = APIEndpoint{
 	Get: APIEndpointAction{Handler: operationWebsocketGet, AllowUntrusted: true},
 }
 
-// waitForOperations waits for operations to finish.
-// There's a timeout for console/exec operations that when reached will shut down the instances forcefully.
-func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdownTimeout time.Duration) {
-	timeout := time.After(consoleShutdownTimeout)
+// pendingInstanceOperations returns a map of instance URLs to operations that are currently running.
+// This is used to determine if an instance is busy and should not be shut down immediately.
+func pendingInstanceOperations() (map[string]*operations.Operation, error) {
+	// Get all the operations
+	ops := operations.Clone()
+	res := make(map[string]*operations.Operation)
 
-	defer func() {
-		_ = cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			err := dbCluster.DeleteOperations(ctx, tx.Tx(), cluster.GetNodeID())
-			if err != nil {
-				logger.Error("Failed cleaning up operations")
-			}
+	for _, op := range ops {
+		if op.Status() != api.Running || op.Class() == operations.OperationClassToken {
+			continue
+		}
 
-			return nil
-		})
-	}()
+		_, opAPI, err := op.Render()
+		if err != nil {
+			return nil, fmt.Errorf("Failed to render an operation while listing all operations: %w", err)
+		}
 
-	// Check operation status every second.
-	tick := time.NewTicker(time.Second)
-	defer tick.Stop()
-
-	var i int
-	for {
-		// Get all the operations
-		ops := operations.Clone()
-
-		var runningOps, execConsoleOps int
-		for _, op := range ops {
-			if op.Status() != api.Running || op.Class() == operations.OperationClassToken {
+		// If the current operations has a hold on some resources, we keep track of them in the `resourceMap`.
+		// This is used to mark instances and storage volumes as busy to avoid shutting them down / unmounting them prematurely.
+		// This avoids the situation where a single very long running operation can block the shutdown of unrelated instances and the unmount of unrelated storage volumes.
+		for resourceName, resourceEntries := range opAPI.Resources {
+			if resourceName != "instances" {
 				continue
 			}
 
-			runningOps++
+			for _, rawURL := range resourceEntries {
+				u := api.NewURL()
+				parsedURL, err := u.Parse(rawURL)
+				if err != nil {
+					logger.Warn("Failed to parse raw URL", logger.Ctx{"rawURL": rawURL, "err": err})
+					continue
+				}
 
-			opType := op.Type()
-			if opType == operationtype.CommandExec || opType == operationtype.ConsoleShow {
-				execConsoleOps++
+				entityType, projectName, location, pathArgs, err := entity.ParseURL(*parsedURL)
+				if err != nil {
+					logger.Warn("Failed to parse URL into a LXD entity", logger.Ctx{"url": parsedURL.String(), "err": err})
+					continue
+				}
+
+				if entityType == entity.TypeInstance {
+					entityURL, err := entityType.URL(projectName, location, pathArgs...)
+					if err != nil {
+						logger.Warn("Failed to generate entity URL", logger.Ctx{"entityType": entityType, "projectName": projectName, "location": location, "pathArgs": pathArgs, "err": err})
+						continue
+					}
+
+					res[entityURL.String()] = op
+				}
 			}
-
-			_, opAPI, err := op.Render()
-			if err != nil {
-				logger.Warn("Failed to render operation", logger.Ctx{"operation ID": op.ID(), "operation class": op.Class().String(), "operation status": op.Status().String(), "err": err})
-			} else if opAPI.MayCancel {
-				_, _ = op.Cancel()
-			}
-		}
-
-		// No more running operations left. Exit function.
-		if runningOps == 0 {
-			logger.Info("All running operations finished, shutting down")
-			return
-		}
-
-		// Print log message every minute.
-		if i%60 == 0 {
-			logger.Infof("Waiting for %d operation(s) to finish", runningOps)
-		}
-
-		i++
-
-		select {
-		case <-timeout:
-			// We wait up to core.shutdown_timeout minutes for exec/console operations to finish.
-			// If there are still running operations, we continue shutdown which will stop any running
-			// instances and terminate the operations.
-			if execConsoleOps > 0 {
-				logger.Info("Shutdown timeout reached, continuing with shutdown")
-			}
-
-			return
-		case <-ctx.Done():
-			// Return here, and ignore any running operations.
-			logger.Info("Forcing shutdown, ignoring running operations")
-			return
-		case <-tick.C:
 		}
 	}
+
+	return res, nil
 }
 
 // API functions
@@ -206,7 +185,7 @@ func operationGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if len(ops) > 1 {
-			return fmt.Errorf("More than one operation matches")
+			return errors.New("More than one operation matches")
 		}
 
 		operation := ops[0]
@@ -218,12 +197,12 @@ func operationGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(r.Context(), address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	return response.ForwardedResponse(client, r)
+	return response.ForwardedResponse(client)
 }
 
 // swagger:operation DELETE /1.0/operations/{id} operations operation_delete
@@ -307,7 +286,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(err)
 		}
 
-		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(op, request.CreateRequestor(r), nil))
+		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(op, request.CreateRequestor(r.Context()), nil))
 
 		return response.EmptySyncResponse
 	}
@@ -326,7 +305,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if len(ops) > 1 {
-			return fmt.Errorf("More than one operation matches")
+			return errors.New("More than one operation matches")
 		}
 
 		operation := ops[0]
@@ -338,16 +317,16 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(r.Context(), address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	return response.ForwardedResponse(client, r)
+	return response.ForwardedResponse(client)
 }
 
 // operationCancel cancels an operation that exists on any member.
-func operationCancel(s *state.State, r *http.Request, projectName string, op *api.Operation) error {
+func operationCancel(ctx context.Context, s *state.State, projectName string, op *api.Operation) error {
 	// Check if operation is local and if so, cancel it.
 	localOp, _ := operations.OperationGetInternal(op.ID)
 	if localOp != nil {
@@ -358,7 +337,7 @@ func operationCancel(s *state.State, r *http.Request, projectName string, op *ap
 			}
 		}
 
-		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(localOp, request.CreateRequestor(r), nil))
+		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(localOp, request.CreateRequestor(ctx), nil))
 
 		return nil
 	}
@@ -378,7 +357,7 @@ func operationCancel(s *state.State, r *http.Request, projectName string, op *ap
 		}
 
 		if len(ops) > 1 {
-			return fmt.Errorf("More than one operation matches")
+			return errors.New("More than one operation matches")
 		}
 
 		operation := ops[0]
@@ -390,7 +369,7 @@ func operationCancel(s *state.State, r *http.Request, projectName string, op *ap
 		return err
 	}
 
-	client, err := cluster.Connect(memberAddress, s.Endpoints.NetworkCert(), s.ServerCert(), r, true)
+	client, err := cluster.Connect(ctx, memberAddress, s.Endpoints.NetworkCert(), s.ServerCert(), true)
 	if err != nil {
 		return fmt.Errorf("Failed to connect to %q: %w", memberAddress, err)
 	}
@@ -509,16 +488,9 @@ func operationCancel(s *state.State, r *http.Request, projectName string, op *ap
 func operationsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	projectName := request.QueryParam(r, "project")
-	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
-	recursion := util.IsRecursionRequest(r)
-
-	if allProjects && projectName != "" {
-		return response.SmartError(
-			api.StatusErrorf(http.StatusBadRequest, "Cannot specify a project when requesting all projects"),
-		)
-	} else if !allProjects && projectName == "" {
-		projectName = api.ProjectDefaultName
+	projectName, allProjects, err := request.ProjectParams(r)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
 	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanViewOperations, entity.TypeProject)
@@ -587,6 +559,8 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		return body, nil
 	}
 
+	recursion := util.IsRecursionRequest(r)
+
 	// Check if called from a cluster node.
 	if isClusterNotification(r) {
 		// Only return the local data.
@@ -686,7 +660,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		// Connect to the remote server. Use notify=true to only get local operations on remote member.
-		client, err := cluster.Connect(memberAddress, networkCert, s.ServerCert(), r, true)
+		client, err := cluster.Connect(r.Context(), memberAddress, networkCert, s.ServerCert(), true)
 		if err != nil {
 			return response.SmartError(fmt.Errorf("Failed connecting to member %q: %w", memberAddress, err))
 		}
@@ -721,7 +695,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 			if recursion {
 				md[status] = append(md[status].([]*api.Operation), &op)
 			} else {
-				md[status] = append(md[status].([]string), fmt.Sprintf("/1.0/operations/%s", op.ID))
+				md[status] = append(md[status].([]string), "/1.0/operations/"+op.ID)
 			}
 		}
 	}
@@ -730,7 +704,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 }
 
 // operationsGetByType gets all operations for a project and type.
-func operationsGetByType(s *state.State, r *http.Request, projectName string, opType operationtype.Type) ([]*api.Operation, error) {
+func operationsGetByType(ctx context.Context, s *state.State, projectName string, opType operationtype.Type) ([]*api.Operation, error) {
 	ops := make([]*api.Operation, 0)
 
 	// Get local operations for project.
@@ -814,7 +788,7 @@ func operationsGetByType(s *state.State, r *http.Request, projectName string, op
 		}
 
 		// Connect to the remote server. Use notify=true to only get local operations on remote member.
-		client, err := cluster.Connect(memberAddress, networkCert, serverCert, r, true)
+		client, err := cluster.Connect(ctx, memberAddress, networkCert, serverCert, true)
 		if err != nil {
 			return nil, fmt.Errorf("Failed connecting to member %q: %w", memberAddress, err)
 		}
@@ -939,10 +913,8 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 
 	secret := r.FormValue("secret")
 
-	trusted, err := request.GetCtxValue[bool](r.Context(), request.CtxTrusted)
-	if err != nil {
-		return response.SmartError(fmt.Errorf("Failed to get authentication status: %w", err))
-	}
+	reqInfo := request.GetContextInfo(r.Context())
+	trusted := reqInfo != nil && reqInfo.Trusted
 
 	if !trusted && secret == "" {
 		return response.Forbidden(nil)
@@ -1017,7 +989,7 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if len(ops) > 1 {
-			return fmt.Errorf("More than one operation matches")
+			return errors.New("More than one operation matches")
 		}
 
 		operation := ops[0]
@@ -1029,12 +1001,12 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(r.Context(), address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	return response.ForwardedResponse(client, r)
+	return response.ForwardedResponse(client)
 }
 
 // swagger:operation GET /1.0/operations/{id}/websocket?public operations operation_websocket_get_untrusted
@@ -1108,7 +1080,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 	// Then check if the query is from an operation on another node, and, if so, forward it
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return response.BadRequest(fmt.Errorf("Missing websocket secret"))
+		return response.BadRequest(errors.New("Missing websocket secret"))
 	}
 
 	var address string
@@ -1124,7 +1096,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if len(ops) > 1 {
-			return fmt.Errorf("More than one operation matches")
+			return errors.New("More than one operation matches")
 		}
 
 		operation := ops[0]
@@ -1136,7 +1108,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
+	client, err := cluster.Connect(r.Context(), address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1149,9 +1121,9 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 	return operations.ForwardedOperationWebSocket(id, source)
 }
 
-func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
+func autoRemoveOrphanedOperationsTask(stateFunc func() *state.State) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
-		s := d.State()
+		s := stateFunc()
 
 		leaderInfo, err := s.LeaderInfo()
 		if err != nil {
@@ -1172,7 +1144,7 @@ func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 			return autoRemoveOrphanedOperations(ctx, s)
 		}
 
-		op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.RemoveOrphanedOperations, nil, nil, opRun, nil, nil, nil)
+		op, err := operations.OperationCreate(context.Background(), s, "", operations.OperationClassTask, operationtype.RemoveOrphanedOperations, nil, nil, opRun, nil, nil)
 		if err != nil {
 			logger.Error("Failed creating remove orphaned operations operation", logger.Ctx{"err": err})
 			return
@@ -1230,3 +1202,73 @@ func autoRemoveOrphanedOperations(ctx context.Context, s *state.State) error {
 
 	return nil
 }
+
+// operationWaitPost represents the fields of a request to register a dummy operation.
+type operationWaitPost struct {
+	Duration  string                    `json:"duration" yaml:"duration"`
+	OpClass   operations.OperationClass `json:"op_class" yaml:"op_class"`
+	OpType    operationtype.Type        `json:"op_type" yaml:"op_type"`
+	Resources map[string][]string       `json:"resources" yaml:"resources"`
+}
+
+// operationWaitHandler creates a dummy operation that waits for a specified duration.
+func operationWaitHandler(d *Daemon, r *http.Request) response.Response {
+	// Extract the entity URL and duration from the request.
+	req := operationWaitPost{}
+	err := json.NewDecoder(r.Body).Decode(&req)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Parse the duration.
+	duration, err := time.ParseDuration(req.Duration)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Extract and validate resources
+	var resources map[string][]api.URL
+	if req.Resources != nil {
+		resources = make(map[string][]api.URL)
+		for resourceType, entityURLs := range req.Resources {
+			for _, entityURL := range entityURLs {
+				parsedURL, err := url.Parse(entityURL)
+				if err != nil {
+					return response.BadRequest(err)
+				}
+
+				_, _, _, _, err = entity.ParseURL(*parsedURL)
+				if err != nil {
+					return response.BadRequest(err)
+				}
+
+				resources[resourceType] = append(resources[resourceType], api.URL{URL: *parsedURL})
+			}
+		}
+	}
+
+	if req.OpType == operationtype.Unknown {
+		return response.BadRequest(fmt.Errorf("Invalid operation type %q", req.OpType))
+	}
+
+	run := func(op *operations.Operation) error {
+		// Just sleep for the duration.
+		time.Sleep(duration)
+		return nil
+	}
+
+	var onConnect func(op *operations.Operation, r *http.Request, w http.ResponseWriter) error
+	if req.OpClass == operations.OperationClassWebsocket {
+		onConnect = func(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
+			// Do nothing
+			return nil
+		}
+	}
+
+	op, err := operations.OperationCreate(r.Context(), d.State(), "", req.OpClass, req.OpType, resources, nil, run, nil, onConnect)
+	if err != nil {
+		return response.InternalError(err)
+	}
+
+	return operations.OperationResponse(op)
+}
