@@ -2,10 +2,13 @@ package main
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"io/fs"
 	"net/http"
 	"os"
 	"path/filepath"
+	"slices"
 	"sort"
 	"strings"
 	"time"
@@ -97,6 +100,8 @@ var patches = []patch{
 	{name: "entity_type_identity_certificate_split", stage: patchPreLoadClusterConfig, run: patchSplitIdentityCertificateEntityTypes},
 	{name: "storage_unset_powerflex_sdt_setting", stage: patchPostDaemonStorage, run: patchUnsetPowerFlexSDTSetting},
 	{name: "oidc_groups_claim_scope", stage: patchPreLoadClusterConfig, run: patchOIDCGroupsClaimScope},
+	{name: "remove_backupsimages_symlinks", stage: patchPostDaemonStorage, run: patchRemoveBackupsImagesSymlinks},
+	{name: "move_images_storage", stage: patchPostDaemonStorage, run: patchMoveBackupsImagesStorage},
 }
 
 type patch struct {
@@ -150,7 +155,7 @@ func patchesApply(d *Daemon, stage patchStage) error {
 			continue
 		}
 
-		if shared.ValueInSlice(patch.name, appliedPatches) {
+		if slices.Contains(appliedPatches, patch.name) {
 			continue
 		}
 
@@ -193,7 +198,7 @@ func selectedPatchClusterMember(s *state.State) (bool, error) {
 	}
 
 	if len(clusterMembers) == 0 {
-		return false, fmt.Errorf("Clustered but no member found")
+		return false, errors.New("Clustered but no member found")
 	}
 
 	// Sort the cluster members by name.
@@ -319,7 +324,7 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 			continue
 		}
 
-		logger.Infof("Trusted server certificates found in trust store for all cluster members")
+		logger.Info("Trusted server certificates found in trust store for all cluster members")
 		break
 	}
 
@@ -738,20 +743,21 @@ func patchNetworkOVNEnableNAT(name string, d *Daemon) error {
 
 // Moves backups from shared.VarPath("backups") to shared.VarPath("backups", "instances").
 func patchMoveBackupsInstances(name string, d *Daemon) error {
-	if !shared.PathExists(shared.VarPath("backups")) {
+	backupsPathBase := d.State().BackupsStoragePath()
+	if !shared.PathExists(backupsPathBase) {
 		return nil // Nothing to do, no backups directory.
 	}
 
-	backupsPath := shared.VarPath("backups", "instances")
+	backupsPath := filepath.Join(backupsPathBase, "instances")
 
 	err := os.MkdirAll(backupsPath, 0700)
 	if err != nil {
 		return fmt.Errorf("Failed creating instances backup directory %q: %w", backupsPath, err)
 	}
 
-	backups, err := os.ReadDir(shared.VarPath("backups"))
+	backups, err := os.ReadDir(backupsPathBase)
 	if err != nil {
-		return fmt.Errorf("Failed listing existing backup directory %q: %w", shared.VarPath("backups"), err)
+		return fmt.Errorf("Failed listing existing backup directory %q: %w", backupsPathBase, err)
 	}
 
 	for _, backupDir := range backups {
@@ -759,7 +765,7 @@ func patchMoveBackupsInstances(name string, d *Daemon) error {
 			continue // Don't try and move our new instances directory or temporary directories.
 		}
 
-		oldPath := shared.VarPath("backups", backupDir.Name())
+		oldPath := filepath.Join(backupsPathBase, backupDir.Name())
 		newPath := filepath.Join(backupsPath, backupDir.Name())
 		logger.Debugf("Moving backup from %q to %q", oldPath, newPath)
 		err = os.Rename(oldPath, newPath)
@@ -923,7 +929,7 @@ func patchZfsSetContentTypeUserProperty(name string, d *Daemon) error {
 
 			zfsVolName := fmt.Sprintf("%s/%s/%s", poolName, storageDrivers.VolumeTypeCustom, project.StorageVolume(vol.Project, vol.Name))
 
-			_, err = shared.RunCommandContext(d.shutdownCtx, "zfs", "set", fmt.Sprintf("lxd:content_type=%s", vol.ContentType), zfsVolName)
+			_, err = shared.RunCommandContext(d.shutdownCtx, "zfs", "set", "lxd:content_type="+vol.ContentType, zfsVolName)
 			if err != nil {
 				logger.Debug("Failed setting lxd:content_type property", logger.Ctx{"name": zfsVolName, "err": err})
 			}
@@ -1008,7 +1014,7 @@ func patchStorageZfsUnsetInvalidBlockSettingsV2(_ string, d *Daemon) error {
 		return err
 	}
 
-	var volType int
+	var volType dbCluster.StoragePoolVolumeType
 
 	for pool, volumes := range poolVolumes {
 		for _, vol := range volumes {
@@ -1038,12 +1044,12 @@ func patchStorageZfsUnsetInvalidBlockSettingsV2(_ string, d *Daemon) error {
 				continue
 			}
 
-			if vol.Type == dbCluster.StoragePoolVolumeTypeNameVM {
+			switch vol.Type {
+			case dbCluster.StoragePoolVolumeTypeNameVM:
 				volType = volTypeVM
-			} else if vol.Type == dbCluster.StoragePoolVolumeTypeNameCustom {
+			case dbCluster.StoragePoolVolumeTypeNameCustom:
 				volType = volTypeCustom
-			} else {
-				// Should not happen.
+			default:
 				continue
 			}
 
@@ -1295,7 +1301,7 @@ func patchStorageRenameCustomISOBlockVolumesV2(name string, d *Daemon) error {
 			// We need to use ContentTypeBlock here in order for the driver to figure out the correct (old) location.
 			oldVol := storageDrivers.NewVolume(p.Driver(), p.Name(), storageDrivers.VolumeTypeCustom, storageDrivers.ContentTypeBlock, project.StorageVolume(vol.Project, vol.Name), nil, nil)
 
-			err = p.Driver().RenameVolume(oldVol, fmt.Sprintf("%s.iso", oldVol.Name()), nil)
+			err = p.Driver().RenameVolume(oldVol, oldVol.Name()+".iso", nil)
 			if err != nil {
 				return fmt.Errorf("Failed to rename volume %q in pool %q: %w", oldVol.Name(), p.Name(), err)
 			}
@@ -1460,18 +1466,18 @@ func patchOIDCGroupsClaimScope(_ string, d *Daemon) error {
 		}
 
 		// Get the groups claim and scopes (these will just be the default values at the time of the patch)
-		_, _, scopes, _, groupsClaim := globalConfig.OIDCServer()
+		_, _, _, scopes, _, groupsClaim := globalConfig.OIDCServer()
 
 		// If the groups claim is not set, or this patch was already run on another member and the groups claim is
 		// already present in the list of scopes, then there is nothing to do.
-		if groupsClaim == "" || shared.ValueInSlice(groupsClaim, scopes) {
+		if groupsClaim == "" || slices.Contains(scopes, groupsClaim) {
 			return nil
 		}
 
 		// Add the groups claim as an additional scope.
 		// The groups claim still needs to be set to extract group values from the token claims or userinfo.
 		oidcScopes := append(scopes, groupsClaim)
-		_, err = globalConfig.Patch(map[string]any{
+		_, err = globalConfig.Patch(tx, map[string]any{
 			"oidc.scopes": strings.Join(oidcScopes, " "),
 		})
 
@@ -1484,4 +1490,82 @@ func patchOIDCGroupsClaimScope(_ string, d *Daemon) error {
 	return nil
 }
 
+// Remove shared.VarPath("backups") and shared.VarPath("images") symlinks.
+func patchRemoveBackupsImagesSymlinks(_ string, d *Daemon) error {
+	dirs := []string{
+		shared.VarPath("backups"),
+		shared.VarPath("images"),
+	}
+
+	for _, dir := range dirs {
+		info, err := os.Lstat(dir)
+		if err != nil {
+			if errors.Is(err, fs.ErrNotExist) {
+				continue // Nothing to do, symlink doesn't exist
+			}
+
+			return fmt.Errorf("Failed to call Lstat() on %q: %w", dir, err)
+		}
+
+		if info.Mode()&os.ModeSymlink != 0 {
+			// Remove the symlink.
+			err = os.Remove(dir)
+			if err != nil {
+				return fmt.Errorf("Failed to delete storage symlink at %q: %w", dir, err)
+			}
+		}
+	}
+
+	return nil
+}
+
+// If storage.images_volume is set, move images into an `images` subfolder.
+func patchMoveBackupsImagesStorage(name string, d *Daemon) error {
+	moveStorage := func(destPath string) error {
+		sourcePath, dirName := filepath.Split(destPath)
+
+		err := os.MkdirAll(destPath, 0700)
+		if err != nil {
+			return fmt.Errorf("Failed creating directory %q: %w", destPath, err)
+		}
+
+		items, err := os.ReadDir(sourcePath)
+		if err != nil {
+			return fmt.Errorf("Failed listing existing directory %q: %w", sourcePath, err)
+		}
+
+		for _, item := range items {
+			if item.Name() == dirName {
+				continue // Don't try and move our new directory.
+			}
+
+			oldPath := filepath.Join(sourcePath, item.Name())
+			newPath := filepath.Join(destPath, item.Name())
+			logger.Debugf("Moving backup from %q to %q", oldPath, newPath)
+			err = os.Rename(oldPath, newPath)
+			if err != nil {
+				return fmt.Errorf("Failed moving file from %q to %q: %w", oldPath, newPath, err)
+			}
+		}
+
+		return nil
+	}
+
+	if d.localConfig.StorageImagesVolume() != "" {
+		err := moveStorage(d.State().ImagesStoragePath())
+		if err != nil {
+			return err
+		}
+	}
+
+	if d.localConfig.StorageBackupsVolume() != "" {
+		err := moveStorage(d.State().BackupsStoragePath())
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 // Patches end here
