@@ -2,6 +2,7 @@ package limits
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"net/http"
 	"slices"
@@ -99,14 +100,10 @@ func AllowInstanceCreation(ctx context.Context, globalConfig *clusterConfig.Conf
 	instance.SetWritable(req.InstancePut)
 	info.Instances = append(info.Instances, instance)
 
+	// Allow stripping volatile keys if dealing with a copy or migration.
+	strip := slices.Contains([]string{api.SourceTypeCopy, api.SourceTypeMigration}, req.Source.Type)
+
 	// Special case restriction checks on volatile.* keys.
-	strip := false
-
-	if shared.ValueInSlice(req.Source.Type, []string{api.SourceTypeCopy, api.SourceTypeMigration}) {
-		// Allow stripping volatile keys if dealing with a copy or migration.
-		strip = true
-	}
-
 	err = checkRestrictionsOnVolatileConfig(
 		info.Project, instanceType, req.Name, req.Config, map[string]string{}, strip)
 	if err != nil {
@@ -213,7 +210,7 @@ func checkRestrictionsOnVolatileConfig(project api.Project, instanceType instanc
 
 	// Checker for safe volatile keys.
 	isSafeKey := func(key string) bool {
-		if shared.ValueInSlice(key, []string{"volatile.apply_template", "volatile.base_image", "volatile.last_state.power"}) {
+		if slices.Contains([]string{"volatile.apply_template", "volatile.base_image", "volatile.last_state.power"}, key) {
 			return true
 		}
 
@@ -495,7 +492,7 @@ func parseHostIDMapRange(isUID bool, isGID bool, listValue string) ([]idmap.Idma
 // instances and profiles.
 func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profiles []api.Profile) error {
 	containerConfigChecks := map[string]func(value string) error{}
-	devicesChecks := map[string]func(value map[string]string) error{}
+	devicesChecks := map[string][]func(value map[string]string) error{}
 
 	allowContainerLowLevel := false
 	allowVMLowLevel := false
@@ -518,7 +515,7 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 					disabled := shared.IsFalseOrEmpty(instanceValue)
 
 					if restrictionValue != "allow" && !disabled {
-						return fmt.Errorf("Container syscall interception is forbidden")
+						return errors.New("Container syscall interception is forbidden")
 					}
 
 					return nil
@@ -527,7 +524,7 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 		case "restricted.containers.nesting":
 			containerConfigChecks["security.nesting"] = func(instanceValue string) error {
 				if restrictionValue == "block" && shared.IsTrue(instanceValue) {
-					return fmt.Errorf("Container nesting is forbidden")
+					return errors.New("Container nesting is forbidden")
 				}
 
 				return nil
@@ -541,7 +538,7 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 		case "restricted.containers.privilege":
 			containerConfigChecks["security.privileged"] = func(instanceValue string) error {
 				if restrictionValue != "allow" && shared.IsTrue(instanceValue) {
-					return fmt.Errorf("Privileged containers are forbidden")
+					return errors.New("Privileged containers are forbidden")
 				}
 
 				return nil
@@ -549,7 +546,7 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 
 			containerConfigChecks["security.idmap.isolated"] = func(instanceValue string) error {
 				if restrictionValue == "isolated" && shared.IsFalseOrEmpty(instanceValue) {
-					return fmt.Errorf("Non-isolated containers are forbidden")
+					return errors.New("Non-isolated containers are forbidden")
 				}
 
 				return nil
@@ -560,87 +557,97 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 				allowVMLowLevel = true
 			}
 
-		case "restricted.devices.unix-char":
-			devicesChecks["unix-char"] = func(device map[string]string) error {
-				if restrictionValue != "allow" {
-					return fmt.Errorf("Unix character devices are forbidden")
+			// Add check for valid usage of io.threads setting.
+			devicesChecks["disk"] = append(devicesChecks["disk"], func(device map[string]string) error {
+				_, ioThreadsUsed := device["io.threads"]
+				if ioThreadsUsed && !allowVMLowLevel {
+					return errors.New(`Use of low-level "io.threads" disk option forbidden`)
 				}
 
 				return nil
-			}
+			})
+
+		case "restricted.devices.unix-char":
+			devicesChecks["unix-char"] = append(devicesChecks["unix-char"], func(device map[string]string) error {
+				if restrictionValue != "allow" {
+					return errors.New("Unix character devices are forbidden")
+				}
+
+				return nil
+			})
 
 		case "restricted.devices.unix-block":
-			devicesChecks["unix-block"] = func(device map[string]string) error {
+			devicesChecks["unix-block"] = append(devicesChecks["unix-block"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("Unix block devices are forbidden")
+					return errors.New("Unix block devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.unix-hotplug":
-			devicesChecks["unix-hotplug"] = func(device map[string]string) error {
+			devicesChecks["unix-hotplug"] = append(devicesChecks["unix-hotplug"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("Unix hotplug devices are forbidden")
+					return errors.New("Unix hotplug devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.infiniband":
-			devicesChecks["infiniband"] = func(device map[string]string) error {
+			devicesChecks["infiniband"] = append(devicesChecks["infiniband"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("Infiniband devices are forbidden")
+					return errors.New("Infiniband devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.gpu":
-			devicesChecks["gpu"] = func(device map[string]string) error {
+			devicesChecks["gpu"] = append(devicesChecks["gpu"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("GPU devices are forbidden")
+					return errors.New("GPU devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.usb":
-			devicesChecks["usb"] = func(device map[string]string) error {
+			devicesChecks["usb"] = append(devicesChecks["usb"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("USB devices are forbidden")
+					return errors.New("USB devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.pci":
-			devicesChecks["pci"] = func(device map[string]string) error {
+			devicesChecks["pci"] = append(devicesChecks["pci"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("PCI devices are forbidden")
+					return errors.New("PCI devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.proxy":
-			devicesChecks["proxy"] = func(device map[string]string) error {
+			devicesChecks["proxy"] = append(devicesChecks["proxy"], func(device map[string]string) error {
 				if restrictionValue != "allow" {
-					return fmt.Errorf("Proxy devices are forbidden")
+					return errors.New("Proxy devices are forbidden")
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.nic":
-			devicesChecks["nic"] = func(device map[string]string) error {
+			devicesChecks["nic"] = append(devicesChecks["nic"], func(device map[string]string) error {
 				// Check if the NICs are allowed at all.
 				switch restrictionValue {
 				case "block":
-					return fmt.Errorf("Network devices are forbidden")
+					return errors.New("Network devices are forbidden")
 				case "managed":
 					if device["network"] == "" {
-						return fmt.Errorf("Only managed network devices are allowed")
+						return errors.New("Only managed network devices are allowed")
 					}
 				}
 
@@ -648,19 +655,19 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 				// restricted.devices.nic and restricted.networks.access settings.
 				if device["network"] != "" {
 					if !project.NetworkAllowed(proj.Config, device["network"], true) {
-						return fmt.Errorf("Network not allowed in project")
+						return errors.New("Network not allowed in project")
 					}
 				} else if device["parent"] != "" {
 					if !project.NetworkAllowed(proj.Config, device["parent"], false) {
-						return fmt.Errorf("Network not allowed in project")
+						return errors.New("Network not allowed in project")
 					}
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.devices.disk":
-			devicesChecks["disk"] = func(device map[string]string) error {
+			devicesChecks["disk"] = append(devicesChecks["disk"], func(device map[string]string) error {
 				// The root device is always allowed.
 				if device["path"] == "/" && device["pool"] != "" {
 					return nil
@@ -673,10 +680,10 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 
 				switch restrictionValue {
 				case "block":
-					return fmt.Errorf("Disk devices are forbidden")
+					return errors.New("Disk devices are forbidden")
 				case "managed":
 					if device["pool"] == "" {
-						return fmt.Errorf("Attaching disks not backed by a pool is forbidden")
+						return errors.New("Attaching disks not backed by a pool is forbidden")
 					}
 
 				case "allow":
@@ -689,7 +696,7 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 				}
 
 				return nil
-			}
+			})
 
 		case "restricted.idmap.uid":
 			var err error
@@ -770,16 +777,19 @@ func checkInstanceRestrictions(proj api.Project, instances []api.Instance, profi
 		}
 
 		for name, device := range devices {
-			check, ok := devicesChecks[device["type"]]
+			checks, ok := devicesChecks[device["type"]]
 			if !ok {
 				continue
 			}
 
-			err := check(device)
-			if err != nil {
-				return fmt.Errorf("Invalid device %q on %s %q of project %q: %w", name, entityTypeLabel, entityName, proj.Name, err)
+			for _, check := range checks {
+				err := check(device)
+				if err != nil {
+					return fmt.Errorf("Invalid device %q on %s %q of project %q: %w", name, entityTypeLabel, entityName, proj.Name, err)
+				}
 			}
 		}
+
 		return nil
 	}
 
@@ -862,22 +872,15 @@ var allowableIntercept = []string{
 
 // Return true if a low-level container option is forbidden.
 func isContainerLowLevelOptionForbidden(key string) bool {
-	if strings.HasPrefix(key, "security.syscalls.intercept") && !shared.ValueInSlice(key, allowableIntercept) {
+	if strings.HasPrefix(key, "security.syscalls.intercept") && !slices.Contains(allowableIntercept, key) {
 		return true
 	}
 
-	if shared.ValueInSlice(key, []string{
-		"boot.host_shutdown_timeout",
-		"linux.kernel_modules",
-		"linux.kernel_modules.load",
-		"raw.apparmor",
-		"raw.idmap",
-		"raw.lxc",
-		"raw.seccomp",
-		"security.devlxd.images",
-		"security.idmap.base",
-		"security.idmap.size",
-	}) {
+	if strings.HasPrefix(key, "security.delegate_bpf") {
+		return true
+	}
+
+	if slices.Contains([]string{"boot.host_shutdown_timeout", "linux.kernel_modules", "linux.kernel_modules.load", "raw.apparmor", "raw.idmap", "raw.lxc", "raw.seccomp", "security.devlxd.images", "security.idmap.base", "security.idmap.size"}, key) {
 		return true
 	}
 
@@ -886,12 +889,7 @@ func isContainerLowLevelOptionForbidden(key string) bool {
 
 // Return true if a low-level VM option is forbidden.
 func isVMLowLevelOptionForbidden(key string) bool {
-	return shared.ValueInSlice(key, []string{
-		"boot.host_shutdown_timeout",
-		"limits.memory.hugepages",
-		"raw.idmap",
-		"raw.qemu",
-	})
+	return slices.Contains([]string{"boot.host_shutdown_timeout", "limits.memory.hugepages", "raw.idmap", "raw.qemu"}, key)
 }
 
 // AllowInstanceUpdate returns an error if any project-specific limit or
@@ -1056,7 +1054,7 @@ func checkUplinkUse(ctx context.Context, tx *db.ClusterTx, projectName string, c
 		}
 
 		if hasIPV4Limits && invalidIPV4Quota || hasIPV6Limits && invalidIPV6Quota {
-			return fmt.Errorf("Uplink IP limit is below current number of used uplink addresses")
+			return errors.New("Uplink IP limit is below current number of used uplink addresses")
 		}
 	}
 
@@ -1068,7 +1066,7 @@ func checkUplinkUse(ctx context.Context, tx *db.ClusterTx, projectName string, c
 	allowedNets := shared.SplitNTrimSpace(config["restricted.networks.uplinks"], ",", -1, false)
 
 	for network := range uplinksInUseSet {
-		if !shared.ValueInSlice(network, allowedNets) {
+		if !slices.Contains(allowedNets, network) {
 			return fmt.Errorf("Restrictions cannot be enforced as project is already using %q as uplink", network)
 		}
 	}
@@ -1520,7 +1518,7 @@ func getInstanceLimits(instance api.Instance, keys []string, skipUnset bool, sto
 var aggregateLimitConfigValueParsers = map[string]func(string) (int64, error){
 	"limits.memory": func(value string) (int64, error) {
 		if strings.HasSuffix(value, "%") {
-			return -1, fmt.Errorf("Value can't be a percentage")
+			return -1, errors.New("Value can't be a percentage")
 		}
 
 		return units.ParseByteSizeString(value)
@@ -1535,7 +1533,7 @@ var aggregateLimitConfigValueParsers = map[string]func(string) (int64, error){
 	},
 	"limits.cpu": func(value string) (int64, error) {
 		if strings.Contains(value, ",") || strings.Contains(value, "-") {
-			return -1, fmt.Errorf("CPUs can't be pinned if project limits are used")
+			return -1, errors.New("CPUs can't be pinned if project limits are used")
 		}
 
 		limit, err := strconv.Atoi(value)
@@ -1555,10 +1553,10 @@ var aggregateLimitConfigValuePrinters = map[string]func(int64) string{
 		return units.GetByteSizeStringIEC(limit, 1)
 	},
 	"limits.processes": func(limit int64) string {
-		return fmt.Sprintf("%d", limit)
+		return strconv.FormatInt(limit, 10)
 	},
 	"limits.cpu": func(limit int64) string {
-		return fmt.Sprintf("%d", limit)
+		return strconv.FormatInt(limit, 10)
 	},
 	"limits.disk": func(limit int64) string {
 		return units.GetByteSizeStringIEC(limit, 1)
@@ -1584,10 +1582,10 @@ func projectHasRestriction(project *api.Project, restrictionKey string, blockVal
 }
 
 // CheckClusterTargetRestriction check if user is allowed to use cluster member targeting.
-func CheckClusterTargetRestriction(authorizer auth.Authorizer, r *http.Request, project *api.Project, targetFlag string) error {
+func CheckClusterTargetRestriction(ctx context.Context, authorizer auth.Authorizer, project *api.Project, targetFlag string) error {
 	if projectHasRestriction(project, "restricted.cluster.target", "block") && targetFlag != "" {
 		// Allow server administrators to move instances around even when restricted (node evacuation, ...)
-		err := authorizer.CheckPermission(r.Context(), entity.ServerURL(), auth.EntitlementCanOverrideClusterTargetRestriction)
+		err := authorizer.CheckPermission(ctx, entity.ServerURL(), auth.EntitlementCanOverrideClusterTargetRestriction)
 		if err != nil && auth.IsDeniedError(err) {
 			return api.StatusErrorf(http.StatusForbidden, "This project doesn't allow cluster member targeting")
 		} else if err != nil {
@@ -1640,7 +1638,7 @@ func AllowClusterMember(p *api.Project, member *db.NodeInfo) error {
 
 	if shared.IsTrue(p.Config["restricted"]) && len(clusterGroupsAllowed) > 0 {
 		for _, memberGroupName := range member.Groups {
-			if shared.ValueInSlice(memberGroupName, clusterGroupsAllowed) {
+			if slices.Contains(clusterGroupsAllowed, memberGroupName) {
 				return nil
 			}
 		}
@@ -1660,7 +1658,7 @@ func AllowClusterGroup(p *api.Project, groupName string) error {
 		return nil
 	}
 
-	if len(clusterGroupsAllowed) > 0 && !shared.ValueInSlice(groupName, clusterGroupsAllowed) {
+	if len(clusterGroupsAllowed) > 0 && !slices.Contains(clusterGroupsAllowed, groupName) {
 		return fmt.Errorf("Project isn't allowed to use this cluster group: %q", groupName)
 	}
 
@@ -1712,11 +1710,11 @@ func CheckTargetGroup(ctx context.Context, tx *db.ClusterTx, p *api.Project, gro
 // If target is a cluster member and is found in allMembers it returns the resolved node information object.
 // If target is a cluster group it returns the cluster group name.
 // In case of error, neither node information nor cluster group name gets returned.
-func CheckTarget(ctx context.Context, authorizer auth.Authorizer, r *http.Request, tx *db.ClusterTx, p *api.Project, target string, allMembers []db.NodeInfo) (*db.NodeInfo, string, error) {
+func CheckTarget(ctx context.Context, authorizer auth.Authorizer, tx *db.ClusterTx, p *api.Project, target string, allMembers []db.NodeInfo) (*db.NodeInfo, string, error) {
 	targetMemberName, targetGroupName := shared.TargetDetect(target)
 
 	// Check manual cluster member targeting restrictions.
-	err := CheckClusterTargetRestriction(authorizer, r, p, target)
+	err := CheckClusterTargetRestriction(ctx, authorizer, p, target)
 	if err != nil {
 		return nil, "", err
 	}
