@@ -3,18 +3,19 @@ package resources
 import (
 	"bufio"
 	"encoding/csv"
+	"errors"
 	"fmt"
 	"io"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 
 	"github.com/jaypipes/pcidb"
 	"golang.org/x/sys/unix"
 
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 )
 
@@ -26,7 +27,7 @@ func loadNvidiaProc() (map[string]*api.ResourcesGPUCardNvidia, error) {
 
 	gpusPath := filepath.Join(procDriverNvidia, "gpus")
 	if !sysfsExists(gpusPath) {
-		return nil, fmt.Errorf("No NVIDIA GPU proc driver")
+		return nil, errors.New("No NVIDIA GPU proc driver")
 	}
 
 	// List the GPUs from /proc
@@ -70,8 +71,8 @@ func loadNvidiaProc() (map[string]*api.ResourcesGPUCardNvidia, error) {
 			}
 
 			if key == "Device Minor" {
-				nvidiaCard.CardName = fmt.Sprintf("nvidia%s", value)
-				nvidiaCard.CardDevice = fmt.Sprintf("195:%s", value)
+				nvidiaCard.CardName = "nvidia" + value
+				nvidiaCard.CardDevice = "195:" + value
 			}
 		}
 
@@ -133,8 +134,8 @@ func loadNvidiaContainer() (map[string]*api.ResourcesGPUCardNvidia, error) {
 				Model:        record[2],
 				UUID:         record[4],
 				Architecture: record[6],
-				CardName:     fmt.Sprintf("nvidia%s", record[1]),
-				CardDevice:   fmt.Sprintf("195:%s", record[1]),
+				CardName:     "nvidia" + record[1],
+				CardDevice:   "195:" + record[1],
 			}
 		}
 	}
@@ -264,7 +265,7 @@ func gpuAddDeviceInfo(devicePath string, nvidiaCards map[string]*api.ResourcesGP
 		if ok {
 			card.Nvidia = nvidia
 		} else {
-			nvidia, ok := nvidiaCards[fmt.Sprintf("0000%s", card.PCIAddress)]
+			nvidia, ok := nvidiaCards["0000"+card.PCIAddress]
 			if ok {
 				card.Nvidia = nvidia
 			}
@@ -287,10 +288,10 @@ func gpuAddDeviceInfo(devicePath string, nvidiaCards map[string]*api.ResourcesGP
 			entryName := entry.Name()
 			entryPath := filepath.Join(drmPath, entryName)
 
-			if strings.HasPrefix(entryName, "card") {
+			after, ok := strings.CutPrefix(entryName, "card")
+			if ok {
 				// Get the card ID
-				idStr := strings.TrimPrefix(entryName, "card")
-				id, err := strconv.ParseUint(idStr, 10, 64)
+				id, err := strconv.ParseUint(after, 10, 64)
 				if err != nil {
 					return fmt.Errorf("Failed to parse card number: %w", err)
 				}
@@ -478,7 +479,7 @@ func GetGPU() (*api.ResourcesGPU, error) {
 				card.PCIAddress = pciAddr
 
 				// Skip devices we already know about
-				if shared.ValueInSlice(card.PCIAddress, pciKnown) {
+				if slices.Contains(pciKnown, card.PCIAddress) {
 					continue
 				}
 
@@ -526,7 +527,7 @@ func GetGPU() (*api.ResourcesGPU, error) {
 			devicePath := filepath.Join(sysBusPci, entryName)
 
 			// Skip devices we already know about
-			if shared.ValueInSlice(entryName, pciKnown) {
+			if slices.Contains(pciKnown, entryName) {
 				continue
 			}
 
