@@ -2,11 +2,14 @@ package rsync
 
 import (
 	"bytes"
+	"context"
+	"errors"
 	"fmt"
 	"io"
 	"net"
 	"os"
 	"os/exec"
+	"slices"
 	"strings"
 	"time"
 
@@ -29,7 +32,7 @@ var RunWrapper func(cmd *exec.Cmd, source string, destination string) (func(), e
 // rsync is a wrapper for the rsync command which will respect RunWrapper.
 func rsync(args ...string) (string, error) {
 	if len(args) < 2 {
-		return "", fmt.Errorf("rsync call expects a minimum of two arguments (source and destination)")
+		return "", errors.New("rsync call expects a minimum of two arguments (source and destination)")
 	}
 
 	// Setup the command.
@@ -61,8 +64,7 @@ func rsync(args ...string) (string, error) {
 	return stdout.String(), nil
 }
 
-// LocalCopy copies a directory using rsync (with the --devices option).
-func LocalCopy(source string, dest string, bwlimit string, xattrs bool, rsyncArgs ...string) (string, error) {
+func runRsync(source string, dest string, bwlimit string, xattrs bool, rsyncArgs ...string) (string, error) {
 	err := os.MkdirAll(dest, 0755)
 	if err != nil {
 		return "", err
@@ -101,7 +103,7 @@ func LocalCopy(source string, dest string, bwlimit string, xattrs bool, rsyncArg
 
 	args = append(args,
 		rsyncVerbosity,
-		shared.AddSlash(source),
+		source,
 		dest)
 
 	msg, err := rsync(args...)
@@ -122,6 +124,16 @@ func LocalCopy(source string, dest string, bwlimit string, xattrs bool, rsyncArg
 	return msg, nil
 }
 
+// LocalCopy copies a directory using rsync (with the --devices option).
+func LocalCopy(source string, dest string, bwlimit string, xattrs bool, rsyncArgs ...string) (string, error) {
+	return runRsync(shared.AddSlash(source), dest, bwlimit, xattrs, rsyncArgs...)
+}
+
+// CopyFile copies a single file using rsync (with the --devices option).
+func CopyFile(source string, dest string, bwlimit string, xattrs bool, rsyncArgs ...string) (string, error) {
+	return runRsync(strings.TrimSuffix(source, "/"), dest, bwlimit, xattrs, rsyncArgs...)
+}
+
 // Send sets up the sending half of an rsync, to recursively send the
 // directory pointed to by path over the websocket.
 func Send(name string, path string, conn io.ReadWriteCloser, tracker *ioprogress.ProgressTracker, features []string, bwlimit string, execPath string, rsyncArgs ...string) error {
@@ -139,7 +151,7 @@ func Send(name string, path string, conn io.ReadWriteCloser, tracker *ioprogress
 	 * stdin/stdout, but that also seemed messy. In any case, this seems to
 	 * work just fine.
 	 */
-	auds := fmt.Sprintf("@lxd/%s", uuid.New().String())
+	auds := "@lxd/" + uuid.New().String()
 	// We simply copy a part of the uuid if it's longer than the allowed
 	// maximum. That should be safe enough for our purposes.
 	if len(auds) > linux.ABSTRACT_UNIX_SOCK_LEN-1 {
@@ -411,34 +423,33 @@ func Recv(path string, conn io.ReadWriteCloser, tracker *ioprogress.ProgressTrac
 
 func rsyncFeatureArgs(features []string) []string {
 	args := []string{}
-	if shared.ValueInSlice("xattrs", features) {
+	if slices.Contains(features, "xattrs") {
 		args = append(args, "--xattrs")
 		if AtLeast("3.1.3") {
 			args = append(args, "--filter=-x security.selinux")
 		}
 	}
 
-	if shared.ValueInSlice("delete", features) {
+	if slices.Contains(features, "delete") {
 		args = append(args, "--delete")
 	}
 
-	if shared.ValueInSlice("compress", features) {
-		args = append(args, "--compress")
-		args = append(args, "--compress-level=2")
+	if slices.Contains(features, "compress") {
+		args = append(args, "--compress", "--compress-level=2")
 	}
 
 	return args
 }
 
 // AtLeast compares the local version to a minimum version.
-func AtLeast(min string) bool {
+func AtLeast(minimum string) bool {
 	// Parse the current version.
-	out, err := shared.RunCommand("rsync", "--version")
+	out, err := shared.RunCommandContext(context.TODO(), "rsync", "--version")
 	if err != nil {
 		return false
 	}
 
-	fields := strings.Split(strings.Split(out, "\n")[0], "  ")
+	fields := strings.Split(strings.SplitN(out, "\n", 2)[0], "  ")
 	if len(fields) < 3 {
 		return false
 	}
@@ -451,7 +462,7 @@ func AtLeast(min string) bool {
 	}
 
 	// Load minium version.
-	minVer, err := version.NewDottedVersion(min)
+	minVer, err := version.NewDottedVersion(minimum)
 	if err != nil {
 		return false
 	}
