@@ -3,6 +3,7 @@ package main
 import (
 	"context"
 	"fmt"
+	"slices"
 	"sync"
 	"sync/atomic"
 	"time"
@@ -16,7 +17,6 @@ import (
 	storagePools "github.com/canonical/lxd/lxd/storage"
 	storageDrivers "github.com/canonical/lxd/lxd/storage/drivers"
 	"github.com/canonical/lxd/lxd/warnings"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/logger"
@@ -48,11 +48,11 @@ func readStoragePoolDriversCache() ([]api.ServerStorageDriverInfo, map[string]st
 
 func storageStartup(s *state.State) error {
 	// Update the storage drivers supported and used cache in api_1.0.go.
-	storagePoolDriversCacheUpdate(s)
+	storagePoolDriversCacheUpdate(s.ShutdownCtx, s)
 
 	var poolNames []string
 
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		poolNames, err = tx.GetCreatedStoragePoolNames(ctx)
@@ -62,6 +62,11 @@ func storageStartup(s *state.State) error {
 	if err != nil {
 		if response.IsNotFoundError(err) {
 			logger.Debug("No existing storage pools detected")
+
+			// There aren't any storage pools.
+			// Unblock all the waitready callers using the --storage flag.
+			s.StorageReady.Cancel()
+
 			return nil
 		}
 
@@ -90,7 +95,7 @@ func storageStartup(s *state.State) error {
 		_, err = pool.Mount()
 		if err != nil {
 			logger.Error("Failed mounting storage pool", logger.Ctx{"pool": poolName, "err": err})
-			_ = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			_ = s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
 				return tx.UpsertWarningLocalNode(ctx, "", entity.TypeStoragePool, int(pool.ID()), warningtype.StoragePoolUnvailable, err.Error())
 			})
 
@@ -152,19 +157,62 @@ func storageStartup(s *state.State) error {
 					}
 
 					if len(initPools) <= 0 {
+						// All storage pools are ready now after performing some retries.
+						// This unblocks any waitready caller using the --storage flag.
+						s.StorageReady.Cancel()
+
 						return // Our job here is done.
 					}
 				}
 			}
 		}()
 	} else {
+		// All storage pools are ready.
+		// This unblocks any waitready caller using the --storage flag.
+		s.StorageReady.Cancel()
+
 		logger.Info("All storage pools initialized")
 	}
 
 	return nil
 }
 
-func storagePoolDriversCacheUpdate(s *state.State) {
+func storageStop(s *state.State) {
+	if s.DB.Cluster == nil {
+		logger.Warn("Skipping storage stop due to global database not being available")
+		return
+	}
+
+	logger.Info("Stopping storage pools")
+
+	var err error
+	var pools []string
+
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		pools, err = tx.GetStoragePoolNames(ctx)
+
+		return err
+	})
+	if err != nil && !response.IsNotFoundError(err) {
+		logger.Error("Failed to get storage pools", logger.Ctx{"err": err})
+	}
+
+	for _, poolName := range pools {
+		pool, err := storagePools.LoadByName(s, poolName)
+		if err != nil {
+			logger.Error("Failed to get storage pool", logger.Ctx{"pool": poolName, "err": err})
+			continue
+		}
+
+		_, err = pool.Unmount()
+		if err != nil {
+			logger.Error("Unable to unmount storage pool", logger.Ctx{"pool": poolName, "err": err})
+			continue
+		}
+	}
+}
+
+func storagePoolDriversCacheUpdate(ctx context.Context, s *state.State) {
 	// Get a list of all storage drivers currently in use
 	// on this LXD instance. Only do this when we do not already have done
 	// this once to avoid unnecessarily querying the db. All subsequent
@@ -177,7 +225,7 @@ func storagePoolDriversCacheUpdate(s *state.State) {
 
 	var drivers []string
 
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		drivers, err = tx.GetStorageDrivers(ctx)
@@ -201,7 +249,7 @@ func storagePoolDriversCacheUpdate(s *state.State) {
 			Remote:  entry.Remote,
 		})
 
-		if shared.ValueInSlice(entry.Name, drivers) {
+		if slices.Contains(drivers, entry.Name) {
 			usedDrivers[entry.Name] = entry.Version
 		}
 	}
