@@ -3,12 +3,14 @@ package drivers
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	"os"
 	"os/exec"
 	"regexp"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -101,6 +103,15 @@ func (d *ceph) osdDeletePool() error {
 	return nil
 }
 
+func (d *ceph) roundUpTo512(a int64) int64 {
+	result := (a / 512) * 512
+	if a%512 != 0 {
+		result += 512
+	}
+
+	return result
+}
+
 // rbdCreateVolume creates an RBD storage volume.
 // Note that the default set of features is intentionally limited
 // by passing --image-feature explicitly. This is done to ensure that
@@ -131,6 +142,9 @@ func (d *ceph) rbdCreateVolume(vol Volume, size string) error {
 		cmd = append(cmd, "--data-pool", d.config["ceph.osd.data_pool_name"])
 	}
 
+	// Ceph allows writing only to images of size in multiples of 512B
+	sizeBytes = d.roundUpTo512(sizeBytes)
+
 	cmd = append(cmd,
 		"--size", fmt.Sprintf("%dB", sizeBytes),
 		"create",
@@ -180,7 +194,7 @@ func (d *ceph) rbdMapVolume(vol Volume) (string, error) {
 
 	idx := strings.Index(devPath, "/dev/rbd")
 	if idx < 0 {
-		return "", fmt.Errorf("Failed to detect mapped device path")
+		return "", errors.New("Failed to detect mapped device path")
 	}
 
 	devPath = strings.TrimSpace(devPath[idx:])
@@ -527,7 +541,7 @@ func (d *ceph) rbdGetVolumeParent(vol Volume) (string, error) {
 
 	idx = strings.Index(msg, "\n")
 	if idx == -1 {
-		return "", fmt.Errorf("Unexpected parsing error")
+		return "", errors.New("Unexpected parsing error")
 	}
 
 	msg = msg[:idx]
@@ -587,12 +601,12 @@ func (d *ceph) rbdListVolumeSnapshots(vol Volume) ([]string, error) {
 	for _, v := range data {
 		_, ok := v["name"]
 		if !ok {
-			return []string{}, fmt.Errorf("No \"name\" property found")
+			return []string{}, errors.New("No \"name\" property found")
 		}
 
 		name, ok := v["name"].(string)
 		if !ok {
-			return []string{}, fmt.Errorf("\"name\" property did not have string type")
+			return []string{}, errors.New("\"name\" property did not have string type")
 		}
 
 		name = strings.TrimSpace(name)
@@ -907,7 +921,7 @@ func (d *ceph) deleteVolumeSnapshot(vol Volume, snapshotName string) (int, error
 			return -1, err
 		}
 
-		newSnapshotName := fmt.Sprintf("zombie_snapshot_%s", uuid.New().String())
+		newSnapshotName := "zombie_snapshot_" + uuid.New().String()
 		err = d.rbdRenameVolumeSnapshot(vol, snapshotName, newSnapshotName)
 		if err != nil {
 			return -1, err
@@ -925,7 +939,7 @@ func (d *ceph) parseParent(parent string) (Volume, string, error) {
 
 	idx := strings.Index(parent, "/")
 	if idx == -1 {
-		return vol, "", fmt.Errorf("Pool delimiter not found")
+		return vol, "", errors.New("Pool delimiter not found")
 	}
 
 	slider := parent[(idx + 1):]
@@ -995,7 +1009,7 @@ func (d *ceph) parseParent(parent string) (Volume, string, error) {
 func (d *ceph) parseClone(clone string) (poolName string, volumeType string, volumeName string, err error) {
 	idx := strings.Index(clone, "/")
 	if idx == -1 {
-		return "", "", "", fmt.Errorf("Unexpected parsing error")
+		return "", "", "", errors.New("Unexpected parsing error")
 	}
 
 	slider := clone[(idx + 1):]
@@ -1011,7 +1025,7 @@ func (d *ceph) parseClone(clone string) (poolName string, volumeType string, vol
 
 	idxType := strings.Index(slider, "_")
 	if idxType == -1 {
-		return "", "", "", fmt.Errorf("Unexpected parsing error")
+		return "", "", "", errors.New("Unexpected parsing error")
 	}
 
 	if idx == len("zombie_") {
@@ -1022,13 +1036,13 @@ func (d *ceph) parseClone(clone string) (poolName string, volumeType string, vol
 
 	idx = strings.Index(slider, "_")
 	if idx == -1 {
-		return "", "", "", fmt.Errorf("Unexpected parsing error")
+		return "", "", "", errors.New("Unexpected parsing error")
 	}
 
 	volumeName = slider
 	idx = strings.Index(volumeName, "_")
 	if idx == -1 {
-		return "", "", "", fmt.Errorf("Unexpected parsing error")
+		return "", "", "", errors.New("Unexpected parsing error")
 	}
 
 	volumeName = volumeName[(idx + 1):]
@@ -1110,7 +1124,7 @@ func (d *ceph) getRBDMappedDevPath(vol Volume, mapIfMissing bool) (bool, string,
 			if len(rbdNameParts) == 2 && rbdNameParts[1] == devSnapName {
 				return false, fmt.Sprintf("/dev/rbd%d", idx), nil // We found a match.
 			}
-		} else if shared.ValueInSlice(devSnapName, []string{"-", ""}) {
+		} else if slices.Contains([]string{"-", ""}, devSnapName) {
 			// Volume is not a snapshot and neither is this device.
 			return false, fmt.Sprintf("/dev/rbd%d", idx), nil // We found a match.
 		}
