@@ -2,11 +2,14 @@ package drivers
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"net/url"
 	"os/exec"
 	"regexp"
+	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/backup"
@@ -65,14 +68,12 @@ func (d *common) validatePool(config map[string]string, driverRules map[string]f
 	rules := d.commonRules.PoolRules()
 
 	// Merge driver specific rules into common rules.
-	for field, validator := range driverRules {
-		rules[field] = validator
-	}
+	maps.Copy(rules, driverRules)
 
 	// Add to pool volume configuration options as volume.* options.
 	// These will be used as default configuration options for volume.
 	for volRule, volValidator := range volumeRules {
-		rules[fmt.Sprintf("volume.%s", volRule)] = volValidator
+		rules["volume."+volRule] = volValidator
 	}
 
 	// Run the validator against each field.
@@ -114,13 +115,7 @@ func (d *common) fillVolumeConfig(vol *Volume, excludedKeys ...string) error {
 
 		volKey := strings.TrimPrefix(k, "volume.")
 
-		isExcluded := false
-		for _, excludedKey := range excludedKeys {
-			if excludedKey == volKey {
-				isExcluded = true
-				break
-			}
-		}
+		isExcluded := slices.Contains(excludedKeys, volKey)
 
 		if isExcluded {
 			continue
@@ -165,9 +160,7 @@ func (d *common) validateVolume(vol Volume, driverRules map[string]func(value st
 	rules := d.commonRules.VolumeRules(vol)
 
 	// Merge driver specific rules into common rules.
-	for field, validator := range driverRules {
-		rules[field] = validator
-	}
+	maps.Copy(rules, driverRules)
 
 	// Run the validator against each field.
 	for k, validator := range rules {
@@ -204,7 +197,7 @@ func (d *common) validateVolume(vol Volume, driverRules map[string]func(value st
 
 	// Check that security.unmapped and security.shifted are not set together.
 	if shared.IsTrue(vol.config["security.unmapped"]) && shared.IsTrue(vol.config["security.shifted"]) {
-		return fmt.Errorf("security.unmapped and security.shifted are mutually exclusive")
+		return errors.New("security.unmapped and security.shifted are mutually exclusive")
 	}
 
 	return nil
@@ -251,9 +244,7 @@ func (d *common) Logger() logger.Logger {
 // Config returns the storage pool config (as a copy, so not modifiable).
 func (d *common) Config() map[string]string {
 	confCopy := make(map[string]string, len(d.config))
-	for k, v := range d.config {
-		confCopy[k] = v
-	}
+	maps.Copy(confCopy, d.config)
 
 	return confCopy
 }
@@ -302,9 +293,10 @@ func (d *common) moveGPTAltHeader(devPath string) error {
 	if ok {
 		exitError, ok := runErr.Unwrap().(*exec.ExitError)
 		if ok {
-			// sgdisk manpage says exit status 3 means:
-			// "Non-GPT disk detected and no -g option, but operation requires a write action".
-			if exitError.ExitCode() == 3 {
+			// sgdisk exit code 3 = “Non-GPT  disk  detected and no -g option, but operation requires a write action”
+			// exit code 2 = “an error occurred while reading the partition table" (e.g. no GPT present or header CRC mismatch)
+			// Treat both as non-error for raw/MBR images, since we only relocate a GPT alternative header if one exists.
+			if exitError.ExitCode() == 2 || exitError.ExitCode() == 3 {
 				return nil // Non-error as non-GPT disk specified.
 			}
 		}
@@ -473,14 +465,14 @@ func (d *common) CheckVolumeSnapshots(vol Volume, snapVols []Volume, op *operati
 
 	// Check if the provided list of volume snapshots matches the ones from storage.
 	for _, wantedSnapshotName := range wantedSnapshotNames {
-		if !shared.ValueInSlice(wantedSnapshotName, storageSnapshotNames) {
+		if !slices.Contains(storageSnapshotNames, wantedSnapshotName) {
 			return fmt.Errorf("Snapshot %q expected but not in storage", wantedSnapshotName)
 		}
 	}
 
 	// Check if the snapshots in storage match the ones from the provided list.
 	for _, storageSnapshotName := range storageSnapshotNames {
-		if !shared.ValueInSlice(storageSnapshotName, wantedSnapshotNames) {
+		if !slices.Contains(wantedSnapshotNames, storageSnapshotName) {
 			return fmt.Errorf("Snapshot %q in storage but not expected", storageSnapshotName)
 		}
 	}
@@ -502,7 +494,7 @@ func (d *common) RenameVolumeSnapshot(snapVol Volume, newSnapshotName string, op
 func (d *common) ValidateBucket(bucket Volume) error {
 	projectName, bucketName := project.StorageVolumeParts(bucket.name)
 	if projectName == "" {
-		return fmt.Errorf("Project prefix missing in bucket volume name")
+		return errors.New("Project prefix missing in bucket volume name")
 	}
 
 	match, err := regexp.MatchString(`^[a-z0-9][\-\.a-z0-9]{2,62}$`, bucketName)
@@ -511,7 +503,7 @@ func (d *common) ValidateBucket(bucket Volume) error {
 	}
 
 	if !match {
-		return fmt.Errorf("Bucket name must be between 3 and 63 lowercase letters, numbers, periods or hyphens and must start with a letter or number")
+		return errors.New("Bucket name must be between 3 and 63 lowercase letters, numbers, periods or hyphens and must start with a letter or number")
 	}
 
 	return nil
@@ -540,12 +532,12 @@ func (d *common) UpdateBucket(bucket Volume, changedConfig map[string]string) er
 // ValidateBucketKey validates the supplied bucket key config.
 func (d *common) ValidateBucketKey(keyName string, creds S3Credentials, roleName string) error {
 	if keyName == "" {
-		return fmt.Errorf("Key name is required")
+		return errors.New("Key name is required")
 	}
 
 	validRoles := []string{"admin", "read-only"}
-	if !shared.ValueInSlice(roleName, validRoles) {
-		return fmt.Errorf("Invalid key role")
+	if !slices.Contains(validRoles, roleName) {
+		return errors.New("Invalid key role")
 	}
 
 	return nil
