@@ -2,11 +2,13 @@ package drivers
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"net/http"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -149,7 +151,7 @@ func (d *lvm) Create() error {
 				return err
 			}
 
-			d.config["size"] = fmt.Sprint(defaultSize) + "GiB"
+			d.config["size"] = strconv.FormatUint(defaultSize, 10) + "GiB"
 		}
 
 		size, err := units.ParseByteSizeString(d.config["size"])
@@ -202,7 +204,7 @@ func (d *lvm) Create() error {
 
 		// Size is invalid as the physical device is already sized.
 		if d.config["size"] != "" && !d.usesThinpool() {
-			return fmt.Errorf("Cannot specify size when using an existing physical device for non-thin pool")
+			return errors.New("Cannot specify size when using an existing physical device for non-thin pool")
 		}
 
 		if d.config["lvm.vg_name"] == "" {
@@ -212,7 +214,7 @@ func (d *lvm) Create() error {
 		d.config["source"] = d.config["lvm.vg_name"]
 
 		if !shared.IsBlockdevPath(srcPath) {
-			return fmt.Errorf("Custom loop file locations are not supported")
+			return errors.New("Custom loop file locations are not supported")
 		}
 
 		// Wipe if requested.
@@ -247,11 +249,11 @@ func (d *lvm) Create() error {
 
 		// Size is invalid as the volume group is already sized.
 		if d.config["size"] != "" && !d.usesThinpool() {
-			return fmt.Errorf("Cannot specify size when using an existing volume group for non-thin pool")
+			return errors.New("Cannot specify size when using an existing volume group for non-thin pool")
 		}
 
 		if d.config["lvm.vg_name"] != "" && d.config["lvm.vg_name"] != d.config["source"] {
-			return fmt.Errorf("Invalid combination of source and lvm.vg_name properties")
+			return errors.New("Invalid combination of source and lvm.vg_name properties")
 		}
 
 		d.config["lvm.vg_name"] = d.config["source"]
@@ -266,12 +268,12 @@ func (d *lvm) Create() error {
 			return fmt.Errorf("The requested volume group %q does not exist", d.config["lvm.vg_name"])
 		}
 	} else {
-		return fmt.Errorf("Invalid source property")
+		return errors.New("Invalid source property")
 	}
 
 	// This is an internal error condition which should never be hit.
 	if d.config["lvm.vg_name"] == "" {
-		return fmt.Errorf("No name for volume group detected")
+		return errors.New("No name for volume group detected")
 	}
 
 	// Used to track the result of checking whether the thin pool exists during the existing volume group empty
@@ -316,7 +318,7 @@ func (d *lvm) Create() error {
 			}
 
 			// Check the tags on the volume group to check it is not already being used by LXD.
-			if shared.ValueInSlice(lvmVgPoolMarker, vgTags) {
+			if slices.Contains(vgTags, lvmVgPoolMarker) {
 				return fmt.Errorf("Volume group %q is already used by LXD", d.config["lvm.vg_name"])
 			}
 		}
@@ -325,7 +327,7 @@ func (d *lvm) Create() error {
 		if !pvExists {
 			// This is an internal error condition which should never be hit.
 			if pvName == "" {
-				return fmt.Errorf("No name for physical volume detected")
+				return errors.New("No name for physical volume detected")
 			}
 
 			_, err := shared.TryRunCommand("pvcreate", pvName)
@@ -370,7 +372,7 @@ func (d *lvm) Create() error {
 				_ = d.removeLogicalVolume(d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
 			})
 		} else if d.config["size"] != "" {
-			return fmt.Errorf("Cannot specify size when using an existing thin pool")
+			return errors.New("Cannot specify size when using an existing thin pool")
 		}
 	}
 
@@ -462,7 +464,7 @@ func (d *lvm) Delete(op *operations.Operation) error {
 			d.logger.Debug("Volume group removed", logger.Ctx{"vg_name": d.config["lvm.vg_name"]})
 		} else {
 			// Otherwise just remove the lvmVgPoolMarker tag to indicate LXD no longer uses this VG.
-			if shared.ValueInSlice(lvmVgPoolMarker, vgTags) {
+			if slices.Contains(vgTags, lvmVgPoolMarker) {
 				_, err = shared.TryRunCommand("vgchange", "--deltag", lvmVgPoolMarker, d.config["lvm.vg_name"])
 				if err != nil {
 					return fmt.Errorf("Failed to remove marker tag on volume group for the lvm storage pool: %w", err)
@@ -517,7 +519,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  defaultdesc: name of the pool
 		//  shortdesc: Name of the volume group to create
 		//  scope: local
-		"lvm.vg_name": validate.IsAny,
+		"lvm.vg_name": validate.Optional(func(value string) error { return ValidPoolName(value) }),
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_name)
 		//
 		// ---
@@ -525,7 +527,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  defaultdesc: `LXDThinPool`
 		//  shortdesc: Thin pool where volumes are created
 		//  scope: local
-		"lvm.thinpool_name": validate.IsAny,
+		"lvm.thinpool_name": validate.Optional(func(value string) error { return ValidVolumeName(value) }),
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_metadata_size)
 		// By default, LVM calculates an appropriate size.
 		// ---
@@ -559,11 +561,11 @@ func (d *lvm) Validate(config map[string]string) error {
 
 	if shared.IsFalse(config["lvm.use_thinpool"]) {
 		if config["lvm.thinpool_name"] != "" {
-			return fmt.Errorf("The key lvm.use_thinpool cannot be set to false when lvm.thinpool_name is set")
+			return errors.New("The key lvm.use_thinpool cannot be set to false when lvm.thinpool_name is set")
 		}
 
 		if config["lvm.thinpool_metadata_size"] != "" {
-			return fmt.Errorf("The key lvm.use_thinpool cannot be set to false when lvm.thinpool_metadata_size is set")
+			return errors.New("The key lvm.use_thinpool cannot be set to false when lvm.thinpool_metadata_size is set")
 		}
 	}
 
@@ -574,22 +576,22 @@ func (d *lvm) Validate(config map[string]string) error {
 func (d *lvm) Update(changedConfig map[string]string) error {
 	_, changed := changedConfig["lvm.use_thinpool"]
 	if changed {
-		return fmt.Errorf("lvm.use_thinpool cannot be changed")
+		return errors.New("lvm.use_thinpool cannot be changed")
 	}
 
 	_, changed = changedConfig["lvm.thinpool_metadata_size"]
 	if changed {
-		return fmt.Errorf("lvm.thinpool_metadata_size cannot be changed")
+		return errors.New("lvm.thinpool_metadata_size cannot be changed")
 	}
 
 	_, changed = changedConfig["volume.lvm.stripes"]
 	if changed && d.usesThinpool() {
-		return fmt.Errorf("volume.lvm.stripes cannot be changed when using thin pool")
+		return errors.New("volume.lvm.stripes cannot be changed when using thin pool")
 	}
 
 	_, changed = changedConfig["volume.lvm.stripes.size"]
 	if changed && d.usesThinpool() {
-		return fmt.Errorf("volume.lvm.stripes.size cannot be changed when using thin pool")
+		return errors.New("volume.lvm.stripes.size cannot be changed when using thin pool")
 	}
 
 	if changedConfig["lvm.vg_name"] != "" {
@@ -616,7 +618,7 @@ func (d *lvm) Update(changedConfig map[string]string) error {
 		loopPath := loopFilePath(d.name)
 
 		if d.config["source"] != loopPath {
-			return fmt.Errorf("Cannot resize non-loopback pools")
+			return errors.New("Cannot resize non-loopback pools")
 		}
 
 		// Resize loop file
@@ -770,7 +772,7 @@ func (d *lvm) GetResources() (*api.ResourcesStoragePool, error) {
 
 		parts := strings.Split(strings.TrimSpace(out), ",")
 		if len(parts) < 2 {
-			return nil, fmt.Errorf("Unexpected output from vgs command")
+			return nil, errors.New("Unexpected output from vgs command")
 		}
 
 		total, err := strconv.ParseUint(parts[0], 10, 64)
