@@ -2,6 +2,7 @@ package drivers
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"net/http"
 	"os"
@@ -13,8 +14,8 @@ import (
 
 	"github.com/canonical/lxd/lxd/locking"
 	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/refcount"
 	"github.com/canonical/lxd/lxd/storage/block"
-	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
@@ -56,7 +57,7 @@ func (d *lvm) thinpoolName() string {
 // openLoopFile opens a loop device and returns the device path.
 func (d *lvm) openLoopFile(source string) (string, error) {
 	if source == "" {
-		return "", fmt.Errorf("No source property found for the storage pool")
+		return "", errors.New("No source property found for the storage pool")
 	}
 
 	if filepath.IsAbs(source) && !shared.IsBlockdevPath(source) {
@@ -75,7 +76,7 @@ func (d *lvm) openLoopFile(source string) (string, error) {
 		return loopDeviceName, nil
 	}
 
-	return "", fmt.Errorf("Source is not loop file")
+	return "", errors.New("Source is not loop file")
 }
 
 // isLVMNotFoundExitError checks whether the supplied error is an exit error from an LVM command
@@ -120,7 +121,7 @@ func (d *lvm) volumeGroupExists(vgName string) (bool, []string, error) {
 	}
 
 	output = strings.TrimSpace(output)
-	tags := strings.SplitN(output, ",", -1)
+	tags := strings.Split(output, ",")
 
 	return true, tags, nil
 }
@@ -274,7 +275,7 @@ func (d *lvm) createDefaultThinPool(lvmVersion, thinPoolName string, thinpoolSiz
 
 // lvmVersionIsAtLeast checks whether the installed version of LVM is at least the specific version.
 func (d *lvm) lvmVersionIsAtLeast(sTypeVersion string, versionString string) (bool, error) {
-	lvmVersionString := strings.Split(sTypeVersion, "/")[0]
+	lvmVersionString := strings.SplitN(sTypeVersion, "/", 2)[0]
 
 	lvmVersion, err := version.Parse(lvmVersionString)
 	if err != nil {
@@ -484,14 +485,15 @@ func (d *lvm) lvmFullVolumeName(volType VolumeType, contentType ContentType, vol
 	}
 
 	contentTypeSuffix := ""
-	if contentType == ContentTypeBlock {
+	switch contentType {
+	case ContentTypeBlock:
 		contentTypeSuffix = lvmBlockVolSuffix
-	} else if contentType == ContentTypeISO {
+	case ContentTypeISO:
 		contentTypeSuffix = lvmISOVolSuffix
 	}
 
 	// Escape the volume name to a name suitable for using as a logical volume.
-	lvName := strings.Replace(strings.Replace(volName, "-", lvmEscapedHyphen, -1), shared.SnapshotDelimiter, lvmSnapshotSeparator, -1)
+	lvName := strings.ReplaceAll(strings.ReplaceAll(volName, "-", lvmEscapedHyphen), shared.SnapshotDelimiter, lvmSnapshotSeparator)
 
 	return string(volType) + "_" + lvName + contentTypeSuffix
 }
@@ -709,7 +711,7 @@ func (d *lvm) thinPoolVolumeUsage(volDevPath string) (totalSize uint64, usedSize
 
 	parts := shared.SplitNTrimSpace(out, ",", -1, true)
 	if len(parts) < 3 {
-		return 0, 0, fmt.Errorf("Unexpected output from lvs command")
+		return 0, 0, errors.New("Unexpected output from lvs command")
 	}
 
 	totalSize, err = strconv.ParseUint(parts[0], 10, 64)
@@ -771,12 +773,33 @@ func (d *lvm) parseLogicalVolumeSnapshot(parent Volume, lvmVolName string) strin
 	// named volume that just has escaped "-" characters in it.
 	if strings.HasPrefix(lvmVolName, snapPrefix) && !strings.HasPrefix(lvmVolName, badPrefix) {
 		// Remove volume name prefix (including snapshot delimiter) and unescape snapshot name.
-		return strings.Replace(strings.TrimPrefix(lvmVolName, snapPrefix), lvmEscapedHyphen, "-", -1)
+		return strings.ReplaceAll(strings.TrimPrefix(lvmVolName, snapPrefix), lvmEscapedHyphen, "-")
 	}
 
 	return ""
 }
 
+func (d *lvm) activationRefCountName(vol Volume) string {
+	parentName := vol.Name()
+
+	// For non-thinpool volumes, activating an LV activates all of its snapshots
+	// (and vice versa). The activation ref counter should consider the parent
+	// and its snapshots to have the same activation.
+	if vol.IsSnapshot() && !d.usesThinpool() {
+		parentName, _, _ = api.GetParentAndSnapshotName(vol.Name())
+	}
+
+	return OperationLockName("Activate", vol.Pool(), vol.Type(), vol.ContentType(), parentName)
+}
+
+func (d *lvm) activationRefCountIncrement(vol Volume) uint {
+	return refcount.Increment(d.activationRefCountName(vol), 1)
+}
+
+func (d *lvm) activationRefCountDecrement(vol Volume) uint {
+	return refcount.Decrement(d.activationRefCountName(vol), 1)
+}
+
 // activateVolume activates an LVM logical volume if not already present. Returns true if activated, false if not.
 func (d *lvm) activateVolume(vol Volume) (bool, error) {
 	var volDevPath string
@@ -797,14 +820,26 @@ func (d *lvm) activateVolume(vol Volume) (bool, error) {
 
 		d.logger.Debug("Activated logical volume", logger.Ctx{"volName": vol.Name(), "dev": volDevPath})
 
+		d.activationRefCountIncrement(vol)
 		return true, nil
 	}
 
+	d.activationRefCountIncrement(vol)
 	return false, nil
 }
 
 // deactivateVolume deactivates an LVM logical volume if present. Returns true if deactivated, false if not.
 func (d *lvm) deactivateVolume(vol Volume) (bool, error) {
+	refCount := d.activationRefCountDecrement(vol)
+	if refCount > 0 {
+		d.logger.Debug("Skipping deactivate as in use", logger.Ctx{"volume": vol.Name(), "volume-type": vol.Type(), "content-type": vol.ContentType(), "refCount": refCount})
+
+		// Could return ErrInUse here, except it would imply that the volume itself
+		// is in use in more than one place; that's not the case. We're only
+		// guarding the deactivation of the block volume, not its use.
+		return false, nil
+	}
+
 	var volDevPath string
 
 	if d.usesThinpool() {
@@ -813,21 +848,12 @@ func (d *lvm) deactivateVolume(vol Volume) (bool, error) {
 		// Use parent for non-thinpool vols as deactivating the parent volume also activates its snapshots.
 		parent, _, _ := api.GetParentAndSnapshotName(vol.Name())
 		volDevPath = d.lvmDevPath(d.config["lvm.vg_name"], vol.volType, vol.contentType, parent)
-
-		if vol.IsSnapshot() {
-			parentVol := NewVolume(d, d.name, vol.volType, vol.contentType, parent, nil, d.config)
-
-			// If parent is in use then skip deactivating non-thinpool snapshot volume as it will fail.
-			if parentVol.MountInUse() || (parentVol.contentType == ContentTypeFS && filesystem.IsMountPoint(parentVol.MountPath())) {
-				return false, nil
-			}
-		}
 	}
 
 	if shared.PathExists(volDevPath) {
 		// Keep trying to deactivate a few times in case the device is still being flushed.
 		var err error
-		for i := 0; i < 20; i++ {
+		for i := range 20 {
 			_, err = shared.RunCommandContext(context.TODO(), "lvchange", "--activate", "n", "--ignoreactivationskip", volDevPath)
 			if err == nil {
 				break
