@@ -6,6 +6,7 @@ import (
 	"crypto/tls"
 	"encoding/base64"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -426,7 +427,7 @@ func (p *powerFlexClient) getVolumeID(name string) (string, error) {
 // getVolume returns the volume behind volumeID.
 func (p *powerFlexClient) getVolume(volumeID string) (*powerFlexVolume, error) {
 	var actualResponse powerFlexVolume
-	err := p.requestAuthenticated(http.MethodGet, fmt.Sprintf("/api/instances/Volume::%s", volumeID), nil, &actualResponse)
+	err := p.requestAuthenticated(http.MethodGet, "/api/instances/Volume::"+volumeID, nil, &actualResponse)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get volume: %q: %w", volumeID, err)
 	}
@@ -477,7 +478,7 @@ func (p *powerFlexClient) setVolumeSize(volumeID string, sizeGiB int64) error {
 		"sizeInGB": stringSize,
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Volume::%s/action/setVolumeSize", volumeID), body, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Volume::"+volumeID+"/action/setVolumeSize", body, nil)
 	if err != nil {
 		return fmt.Errorf("Failed to set volume size: %q: %w", volumeID, err)
 	}
@@ -491,7 +492,7 @@ func (p *powerFlexClient) overwriteVolume(volumeID string, snapshotID string) er
 		"srcVolumeId": snapshotID,
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Volume::%s/action/overwriteVolumeContent", volumeID), body, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Volume::"+volumeID+"/action/overwriteVolumeContent", body, nil)
 	if err != nil {
 		return fmt.Errorf("Failed to overwrite volume: %q: %w", volumeID, err)
 	}
@@ -517,7 +518,7 @@ func (p *powerFlexClient) createVolumeSnapshot(systemID string, volumeID string,
 		VolumeIDs []string `json:"volumeIdList"`
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/System::%s/action/snapshotVolumes", systemID), body, &actualResponse)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/System::"+systemID+"/action/snapshotVolumes", body, &actualResponse)
 	if err != nil {
 		powerFlexError, ok := err.(*powerFlexError)
 		if ok {
@@ -532,7 +533,7 @@ func (p *powerFlexClient) createVolumeSnapshot(systemID string, volumeID string,
 	}
 
 	if len(actualResponse.VolumeIDs) == 0 {
-		return "", fmt.Errorf("Response does not contain a single snapshot ID")
+		return "", errors.New("Response does not contain a single snapshot ID")
 	}
 
 	return actualResponse.VolumeIDs[0], nil
@@ -546,7 +547,7 @@ func (p *powerFlexClient) getVolumeSnapshots(volumeID string) ([]powerFlexVolume
 	}
 
 	var actualResponse []powerFlexVolume
-	err = p.requestAuthenticated(http.MethodGet, fmt.Sprintf("/api/instances/VTree::%s/relationships/Volume", volume.VTreeID), nil, &actualResponse)
+	err = p.requestAuthenticated(http.MethodGet, "/api/instances/VTree::"+volume.VTreeID+"/relationships/Volume", nil, &actualResponse)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get volume snapshots: %q: %w", volumeID, err)
 	}
@@ -570,7 +571,7 @@ func (p *powerFlexClient) deleteVolume(volumeID string, deleteMode string) error
 		"removeMode": deleteMode,
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Volume::%s/action/removeVolume", volumeID), body, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Volume::"+volumeID+"/action/removeVolume", body, nil)
 	if err != nil {
 		return fmt.Errorf("Failed to delete volume: %q: %w", volumeID, err)
 	}
@@ -676,7 +677,7 @@ func (p *powerFlexClient) createHost(hostName string, nqn string) (string, error
 
 // deleteHost deletes the host behind hostID.
 func (p *powerFlexClient) deleteHost(hostID string) error {
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Sdc::%s/action/removeSdc", hostID), nil, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Sdc::"+hostID+"/action/removeSdc", nil, nil)
 	if err != nil {
 		return fmt.Errorf("Failed to delete host: %w", err)
 	}
@@ -692,7 +693,7 @@ func (p *powerFlexClient) createHostVolumeMapping(hostID string, volumeID string
 		"allowMultipleMappings": "true",
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Volume::%s/action/addMappedHost", volumeID), body, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Volume::"+volumeID+"/action/addMappedHost", body, nil)
 	if err != nil {
 		return fmt.Errorf("Failed to create host volume mapping between %q and %q: %w", hostID, volumeID, err)
 	}
@@ -707,7 +708,7 @@ func (p *powerFlexClient) deleteHostVolumeMapping(hostID string, volumeID string
 		"hostId": hostID,
 	}
 
-	err := p.requestAuthenticated(http.MethodPost, fmt.Sprintf("/api/instances/Volume::%s/action/removeMappedHost", volumeID), body, nil)
+	err := p.requestAuthenticated(http.MethodPost, "/api/instances/Volume::"+volumeID+"/action/removeMappedHost", body, nil)
 	if err != nil {
 		powerFlexError, ok := err.(*powerFlexError)
 		if ok {
@@ -726,7 +727,7 @@ func (p *powerFlexClient) deleteHostVolumeMapping(hostID string, volumeID string
 // getHostVolumeMappings returns the volume mappings for the host behind hostID.
 func (p *powerFlexClient) getHostVolumeMappings(hostID string) ([]powerFlexVolume, error) {
 	var actualResponse []powerFlexVolume
-	err := p.requestAuthenticated(http.MethodGet, fmt.Sprintf("/api/instances/Sdc::%s/relationships/Volume", hostID), nil, &actualResponse)
+	err := p.requestAuthenticated(http.MethodGet, "/api/instances/Sdc::"+hostID+"/relationships/Volume", nil, &actualResponse)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get host volume mappings: %w", err)
 	}
@@ -1234,9 +1235,10 @@ func (d *powerflex) getVolumeName(vol Volume) (string, error) {
 	volName := base64.StdEncoding.EncodeToString(binUUID)
 
 	var suffix string
-	if vol.contentType == ContentTypeBlock {
+	switch vol.contentType {
+	case ContentTypeBlock:
 		suffix = powerFlexBlockVolSuffix
-	} else if vol.contentType == ContentTypeISO {
+	case ContentTypeISO:
 		suffix = powerFlexISOVolSuffix
 	}
 
@@ -1295,7 +1297,7 @@ func (d *powerflex) discover(ctx context.Context, targetAddresses ...string) ([]
 
 	// In case none of the target addresses returned any log records also return an error.
 	if len(discoveryLog.Records) == 0 {
-		return nil, fmt.Errorf("Failed to fetch a discovery log record from any of the target addresses")
+		return nil, errors.New("Failed to fetch a discovery log record from any of the target addresses")
 	}
 
 	return discoveryLog.Records, nil
