@@ -1,11 +1,13 @@
 package drivers
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"net/http"
 	"os"
 	"slices"
+	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
@@ -171,7 +173,7 @@ func (d *pure) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Ope
 
 // CreateVolumeFromBackup re-creates a volume from its exported state.
 func (d *pure) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
-	return genericVFSBackupUnpack(d, d.state.OS, vol, srcBackup.Snapshots, srcData, op)
+	return genericVFSBackupUnpack(d, d.state, vol, srcBackup.Snapshots, srcData, op)
 }
 
 // CreateVolumeFromCopy provides same-pool volume copying functionality.
@@ -303,7 +305,7 @@ func (d *pure) CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInco
 	// Finally, copy the source volume (or snapshot) into destination volume snapshots.
 	if srcVol.IsSnapshot() {
 		// Get snapshot parent volume name.
-		srcParentVol := srcVol.Volume.GetParent()
+		srcParentVol := srcVol.GetParent()
 		srcParentVolName, err := d.getVolumeName(srcParentVol)
 		if err != nil {
 			return err
@@ -533,7 +535,7 @@ func (d *pure) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 	// Finally, copy the source volume (or snapshot) into destination volume snapshots.
 	if srcVol.IsSnapshot() {
 		// Find snapshot parent volume.
-		srcParentVol := srcVol.Volume.GetParent()
+		srcParentVol := srcVol.GetParent()
 		srcParentVolName, err := d.getVolumeName(srcParentVol)
 		if err != nil {
 			return nil, err
@@ -728,7 +730,7 @@ func (d *pure) ValidateVolume(vol Volume, removeUnknownKeys bool) error {
 		remainder := sizeBytes % 512
 		if remainder > 0 {
 			sizeBytes = (sizeBytes/512 + 1) * 512
-			vol.SetConfigSize(fmt.Sprintf("%d", sizeBytes))
+			vol.SetConfigSize(strconv.FormatInt(sizeBytes, 10))
 		}
 	}
 
@@ -961,7 +963,8 @@ func (d *pure) MountVolume(vol Volume, op *operations.Operation) error {
 
 	revert.Add(cleanup)
 
-	if vol.contentType == ContentTypeFS {
+	switch vol.contentType {
+	case ContentTypeFS:
 		mountPath := vol.MountPath()
 		if !filesystem.IsMountPoint(mountPath) {
 			err = vol.EnsureMountPath()
@@ -979,14 +982,15 @@ func (d *pure) MountVolume(vol Volume, op *operations.Operation) error {
 			}
 
 			mountFlags, mountOptions := filesystem.ResolveMountOptions(strings.Split(vol.ConfigBlockMountOptions(), ","))
-			err = TryMount(volDevPath, mountPath, fsType, mountFlags, mountOptions)
+			err = TryMount(context.TODO(), volDevPath, mountPath, fsType, mountFlags, mountOptions)
 			if err != nil {
 				return err
 			}
 
 			d.logger.Debug("Mounted Pure Storage volume", logger.Ctx{"volName": vol.name, "dev": volDevPath, "path": mountPath, "options": mountOptions})
 		}
-	} else if vol.contentType == ContentTypeBlock {
+
+	case ContentTypeBlock:
 		// For VMs, mount the filesystem volume.
 		if vol.IsVMBlock() {
 			fsVol := vol.NewVMBlockFilesystemVolume()
