@@ -2,10 +2,12 @@ package drivers
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -64,14 +66,6 @@ func (d *zfs) load() error {
 		return fmt.Errorf("Error loading %q module: %w", "zfs", err)
 	}
 
-	// Validate the needed tools are present.
-	for _, tool := range []string{"zpool", "zfs"} {
-		_, err := exec.LookPath(tool)
-		if err != nil {
-			return fmt.Errorf("Required tool '%s' is missing", tool)
-		}
-	}
-
 	// Get the version information.
 	if zfsVersion == "" {
 		version, err := d.version()
@@ -82,6 +76,18 @@ func (d *zfs) load() error {
 		zfsVersion = version
 	}
 
+	// Validate the needed tools are present.
+	for _, tool := range []string{"zpool", "zfs"} {
+		_, err := exec.LookPath(tool)
+		if err != nil {
+			if shared.InSnap() {
+				return fmt.Errorf("Required tool %q is missing. The snap does not contain ZFS tools matching the module version (%q). Consider installing ZFS tools in the host and use 'snap set lxd zfs.external=true'", tool, zfsVersion)
+			}
+
+			return fmt.Errorf("Required tool %q is missing", tool)
+		}
+	}
+
 	// Decide whether we can use features added by 0.8.0.
 	ver080, err := version.Parse("0.8.0")
 	if err != nil {
@@ -164,11 +170,11 @@ func (d zfs) ensureInitialDatasets(warnOnExistingPolicyApplyError bool) error {
 
 	for _, dataset := range d.initialDatasets() {
 		properties := []string{"mountpoint=legacy"}
-		if shared.ValueInSlice(dataset, []string{"virtual-machines", "deleted/virtual-machines"}) {
+		if slices.Contains([]string{"virtual-machines", "deleted/virtual-machines"}, dataset) {
 			properties = append(properties, "volmode=none")
 		}
 
-		datasetPath := filepath.Join(d.config["zfs.pool_name"], dataset)
+		datasetPath := d.config["zfs.pool_name"] + "/" + dataset
 		exists, err := d.datasetExists(datasetPath)
 		if err != nil {
 			return err
@@ -220,7 +226,7 @@ func (d *zfs) FillConfig() error {
 				return err
 			}
 
-			d.config["size"] = fmt.Sprint(defaultSize) + "GiB"
+			d.config["size"] = strconv.FormatUint(defaultSize, 10) + "GiB"
 		}
 	} else if filepath.IsAbs(d.config["source"]) {
 		// Set default pool_name.
@@ -249,6 +255,9 @@ func (d *zfs) Create() error {
 	// Store the provided source as we are likely to be mangling it.
 	d.config["volatile.initial_source"] = d.config["source"]
 
+	revert := revert.New()
+	defer revert.Fail()
+
 	err := d.FillConfig()
 	if err != nil {
 		return err
@@ -258,7 +267,7 @@ func (d *zfs) Create() error {
 	if d.config["source"] == "" || d.config["source"] == loopPath {
 		// Validate pool_name.
 		if strings.Contains(d.config["zfs.pool_name"], "/") {
-			return fmt.Errorf("zfs.pool_name can't point to a dataset when source isn't set")
+			return errors.New("zfs.pool_name can't point to a dataset when source isn't set")
 		}
 
 		// Create the loop file itself.
@@ -272,6 +281,8 @@ func (d *zfs) Create() error {
 			return err
 		}
 
+		revert.Add(func() { _ = os.Remove(d.config["source"]) })
+
 		// Create the zpool.
 		_, err = shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], loopPath)
 		if err != nil {
@@ -288,12 +299,12 @@ func (d *zfs) Create() error {
 	} else if filepath.IsAbs(d.config["source"]) {
 		// Handle existing block devices.
 		if !shared.IsBlockdevPath(d.config["source"]) {
-			return fmt.Errorf("Custom loop file locations are not supported")
+			return errors.New("Custom loop file locations are not supported")
 		}
 
 		// Validate pool_name.
 		if strings.Contains(d.config["zfs.pool_name"], "/") {
-			return fmt.Errorf("zfs.pool_name can't point to a dataset when source isn't set")
+			return errors.New("zfs.pool_name can't point to a dataset when source isn't set")
 		}
 
 		// Wipe if requested.
@@ -331,7 +342,7 @@ func (d *zfs) Create() error {
 	} else {
 		// Validate pool_name.
 		if d.config["zfs.pool_name"] != d.config["source"] {
-			return fmt.Errorf("The source must match zfs.pool_name if specified")
+			return errors.New("The source must match zfs.pool_name if specified")
 		}
 
 		if strings.Contains(d.config["zfs.pool_name"], "/") {
@@ -366,10 +377,6 @@ func (d *zfs) Create() error {
 		}
 	}
 
-	// Setup revert in case of problems
-	revert := revert.New()
-	defer revert.Fail()
-
 	revert.Add(func() { _ = d.Delete(nil) })
 
 	// Apply our default configuration.
@@ -404,7 +411,7 @@ func (d *zfs) Delete(op *operations.Operation) error {
 	for _, dataset := range datasets {
 		dataset = strings.TrimPrefix(dataset, "/")
 
-		if shared.ValueInSlice(dataset, initialDatasets) {
+		if slices.Contains(initialDatasets, dataset) {
 			continue
 		}
 
@@ -489,7 +496,7 @@ func (d *zfs) Validate(config map[string]string) error {
 func (d *zfs) Update(changedConfig map[string]string) error {
 	_, ok := changedConfig["zfs.pool_name"]
 	if ok {
-		return fmt.Errorf("zfs.pool_name cannot be modified")
+		return errors.New("zfs.pool_name cannot be modified")
 	}
 
 	size, ok := changedConfig["size"]
@@ -498,7 +505,7 @@ func (d *zfs) Update(changedConfig map[string]string) error {
 		loopPath := loopFilePath(d.name)
 
 		if d.config["source"] != loopPath {
-			return fmt.Errorf("Cannot resize non-loopback pools")
+			return errors.New("Cannot resize non-loopback pools")
 		}
 
 		// Resize loop file
@@ -549,7 +556,7 @@ func (d *zfs) importPool() (bool, error) {
 	}
 
 	if exists {
-		return false, fmt.Errorf("ZFS zpool exists but dataset is missing")
+		return false, errors.New("ZFS zpool exists but dataset is missing")
 	}
 
 	// Import the pool.
@@ -576,7 +583,7 @@ func (d *zfs) importPool() (bool, error) {
 		return true, nil
 	}
 
-	return false, fmt.Errorf("ZFS zpool exists but dataset is missing")
+	return false, errors.New("ZFS zpool exists but dataset is missing")
 }
 
 // Mount mounts the storage pool.
@@ -727,7 +734,7 @@ func (d *zfs) patchDropBlockVolumeFilesystemExtension() error {
 		return fmt.Errorf("Failed listing images: %w", err)
 	}
 
-	for _, volume := range strings.Split(out, "\n") {
+	for volume := range strings.SplitSeq(out, "\n") {
 		fields := strings.SplitN(volume, poolName+"/images/", 2)
 
 		if len(fields) != 2 || fields[1] == "" {
