@@ -2,6 +2,7 @@ package drivers
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"os"
 	"strings"
@@ -34,9 +35,10 @@ func CephGetRBDImageName(vol Volume, zombie bool) (imageName string, snapName st
 		parentName = parentName + "_" + vol.ConfigBlockFilesystem()
 	}
 
-	if vol.contentType == ContentTypeBlock {
+	switch vol.contentType {
+	case ContentTypeBlock:
 		parentName = parentName + cephBlockVolSuffix
-	} else if vol.contentType == ContentTypeISO {
+	case ContentTypeISO:
 		parentName = parentName + cephISOVolSuffix
 	}
 
@@ -97,10 +99,10 @@ func CephMonitors(cluster string) ([]string, error) {
 			// doesn't take the version indication, trailing bits or supports those
 			// tuples, all of those effectively get stripped away to get a clean
 			// address list (with ports).
-			entries := strings.Split(fields[1], " ")
-			for _, entry := range entries {
-				servers := strings.Split(entry, ",")
-				for _, server := range servers {
+			entries := strings.SplitSeq(fields[1], " ")
+			for entry := range entries {
+				servers := strings.SplitSeq(entry, ",")
+				for server := range servers {
 					// Trim leading/trailing spaces.
 					server = strings.TrimSpace(server)
 
@@ -139,7 +141,7 @@ func CephMonitors(cluster string) ([]string, error) {
 	}
 
 	if len(cephMon) == 0 {
-		return nil, fmt.Errorf("Couldn't find a CEPH mon")
+		return nil, errors.New("Couldn't find a CEPH mon")
 	}
 
 	return cephMon, nil
@@ -174,7 +176,7 @@ func getCephKeyFromFile(path string) (string, error) {
 	}
 
 	if cephSecret == "" {
-		return "", fmt.Errorf("Couldn't find a keyring entry")
+		return "", errors.New("Couldn't find a keyring entry")
 	}
 
 	return cephSecret, nil
@@ -244,7 +246,7 @@ func CephKeyring(cluster string, client string) (string, error) {
 	}
 
 	if cephSecret == "" {
-		return "", fmt.Errorf("Couldn't find a keyring entry")
+		return "", errors.New("Couldn't find a keyring entry")
 	}
 
 	return cephSecret, nil
