@@ -4,6 +4,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"maps"
 	"os"
 
 	"github.com/canonical/lxd/lxd/locking"
@@ -150,7 +151,7 @@ func (v Volume) ExpandedConfig(key string) string {
 // Load the snapshot from the database instead if you want to access its own UUID.
 func (v Volume) NewSnapshot(snapshotName string) (Volume, error) {
 	if v.IsSnapshot() {
-		return Volume{}, fmt.Errorf("Cannot create a snapshot volume from a snapshot")
+		return Volume{}, errors.New("Cannot create a snapshot volume from a snapshot")
 	}
 
 	// Deep copy the volume's config.
@@ -350,7 +351,7 @@ func (v Volume) UnmountTask(task func(op *operations.Operation) error, keepBlock
 // Snapshots returns a list of snapshots for the volume (in no particular order).
 func (v Volume) Snapshots(op *operations.Operation) ([]Volume, error) {
 	if v.IsSnapshot() {
-		return nil, fmt.Errorf("Volume is a snapshot")
+		return nil, errors.New("Volume is a snapshot")
 	}
 
 	snapshots, err := v.driver.VolumeSnapshots(v, op)
@@ -579,15 +580,11 @@ func (v *Volume) GetParent() Volume {
 func (v Volume) Clone() Volume {
 	// Copy the config map to avoid internal modifications affecting external state.
 	newConfig := make(map[string]string, len(v.config))
-	for k, v := range v.config {
-		newConfig[k] = v
-	}
+	maps.Copy(newConfig, v.config)
 
 	// Copy the pool config map to avoid internal modifications affecting external state.
 	newPoolConfig := make(map[string]string, len(v.poolConfig))
-	for k, v := range v.poolConfig {
-		newPoolConfig[k] = v
-	}
+	maps.Copy(newPoolConfig, v.poolConfig)
 
 	return NewVolume(v.driver, v.pool, v.volType, v.contentType, v.name, newConfig, newPoolConfig)
 }
