@@ -167,7 +167,7 @@ import (
 	"github.com/canonical/lxd/shared"
 )
 
-var errNoDevice = fmt.Errorf("Couldn't find backing device for mountpoint")
+var errNoDevice = errors.New("Couldn't find backing device for mountpoint")
 
 func devForPath(path string) (string, error) {
 	// Get major/minor
