@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"os"
+	"slices"
 	"sort"
 
 	"github.com/canonical/lxd/lxd/db"
@@ -104,12 +105,13 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 		for _, vol := range volumes {
 			var u *api.URL
 
-			if shared.ValueInSlice(vol.Type, ignoreVolumeType) {
+			if slices.Contains(ignoreVolumeType, vol.Type) {
 				continue
 			}
 
 			// Generate URL for volume based on types that map to other entities.
-			if vol.Type == cluster.StoragePoolVolumeTypeNameContainer || vol.Type == cluster.StoragePoolVolumeTypeNameVM {
+			switch vol.Type {
+			case cluster.StoragePoolVolumeTypeNameContainer, cluster.StoragePoolVolumeTypeNameVM:
 				volName, snapName, isSnap := api.GetParentAndSnapshotName(vol.Name)
 				if isSnap {
 					u = api.NewURL().Path(version.APIVersion, "instances", volName, "snapshots", snapName).Project(vol.Project)
@@ -118,7 +120,7 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 				}
 
 				usedBy = append(usedBy, u.String())
-			} else if vol.Type == cluster.StoragePoolVolumeTypeNameImage {
+			case cluster.StoragePoolVolumeTypeNameImage:
 				imgProjectNames, err := tx.GetProjectsUsingImage(ctx, vol.Name)
 				if err != nil {
 					return fmt.Errorf("Failed loading projects using image %q: %w", vol.Name, err)
@@ -134,7 +136,8 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 					u = vol.URL(version.APIVersion)
 					usedBy = append(usedBy, u.String())
 				}
-			} else {
+
+			default:
 				u = vol.URL(version.APIVersion)
 				usedBy = append(usedBy, u.String())
 			}
