@@ -3,7 +3,9 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
+	"maps"
 	"net/http"
 	"net/url"
 	"slices"
@@ -26,7 +28,6 @@ import (
 	"github.com/canonical/lxd/lxd/state"
 	storagePools "github.com/canonical/lxd/lxd/storage"
 	"github.com/canonical/lxd/lxd/util"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/logger"
@@ -158,7 +159,7 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 	var poolNames []string
 	var hiddenPoolNames []string
 
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		// Load the pool names.
@@ -208,7 +209,7 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 			}
 
 			poolAPI := pool.ToAPI()
-			poolAPI.UsedBy = project.FilterUsedBy(s.Authorizer, r, poolUsedBy)
+			poolAPI.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, poolUsedBy)
 
 			if !hasEditPermission(entity.StoragePoolURL(poolName)) {
 				// Don't allow non-admins to see pool config as sensitive info can be stored there.
@@ -235,7 +236,7 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if len(withEntitlements) > 0 {
-		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeStorageVolume, withEntitlements, urlToPool)
+		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeStoragePool, withEntitlements, urlToPool)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -298,15 +299,15 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if req.Name == "" {
-		return response.BadRequest(fmt.Errorf("No name provided"))
+		return response.BadRequest(errors.New("No name provided"))
 	}
 
 	if strings.Contains(req.Name, "/") {
-		return response.BadRequest(fmt.Errorf("Storage pool names may not contain slashes"))
+		return response.BadRequest(errors.New("Storage pool names may not contain slashes"))
 	}
 
 	if req.Driver == "" {
-		return response.BadRequest(fmt.Errorf("No driver provided"))
+		return response.BadRequest(errors.New("No driver provided"))
 	}
 
 	if req.Config == nil {
@@ -320,7 +321,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 		ctx["target"] = targetNode
 	}
 
-	lc := lifecycle.StoragePoolCreated.Event(req.Name, request.CreateRequestor(r), ctx)
+	lc := lifecycle.StoragePoolCreated.Event(req.Name, request.CreateRequestor(r.Context()), ctx)
 	resp := response.SyncResponseLocation(true, nil, lc.Source)
 
 	clientType := clusterRequest.UserAgentClientType(r.Header.Get("User-Agent"))
@@ -336,7 +337,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 
 		var poolID int64
 
-		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
 
 			poolID, err = tx.GetStoragePoolID(ctx, req.Name)
@@ -347,7 +348,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 			return response.SmartError(err)
 		}
 
-		_, err = storagePoolCreateLocal(s, poolID, req, clientType)
+		_, err = storagePoolCreateLocal(r.Context(), s, poolID, req, clientType)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -359,7 +360,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 		// A targetNode was specified, let's just define the node's storage without actually creating it.
 		// The only legal key values for the storage config are the ones in NodeSpecificStorageConfig.
 		for key := range req.Config {
-			if !shared.ValueInSlice(key, db.NodeSpecificStorageConfig) {
+			if !slices.Contains(db.NodeSpecificStorageConfig, key) {
 				return response.SmartError(fmt.Errorf("Config key %q may not be used as member-specific key", key))
 			}
 		}
@@ -369,7 +370,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(err)
 		}
 
-		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.CreatePendingStoragePool(ctx, targetNode, req.Name, req.Driver, req.Config)
 		})
 		if err != nil {
@@ -385,7 +386,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 
 	var pool *api.StoragePool
 
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		// Load existing pool if exists, if not don't fail.
@@ -406,13 +407,13 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 	// No targetNode was specified and we're clustered or there is an existing partially created single node
 	// pool, either way finalize the config in the db and actually create the pool on all nodes in the cluster.
 	if count > 1 || (pool != nil && pool.Status != api.StoragePoolStatusCreated) {
-		err = storagePoolsPostCluster(s, pool, req, clientType)
+		err = storagePoolsPostCluster(r.Context(), s, pool, req, clientType)
 		if err != nil {
 			return response.InternalError(err)
 		}
 	} else {
 		// Create new single node storage pool.
-		err = storagePoolCreateGlobal(s, req, clientType)
+		err = storagePoolCreateGlobal(r.Context(), s, req, clientType)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -435,7 +436,7 @@ func storagePoolPartiallyCreated(pool *api.StoragePool) bool {
 	// If the pool has global config keys, then it has previously been created by having its global config
 	// inserted, and this means it is partialled created.
 	for key := range pool.Config {
-		if !shared.ValueInSlice(key, db.NodeSpecificStorageConfig) {
+		if !slices.Contains(db.NodeSpecificStorageConfig, key) {
 			return true
 		}
 	}
@@ -445,10 +446,10 @@ func storagePoolPartiallyCreated(pool *api.StoragePool) bool {
 
 // storagePoolsPostCluster handles creating storage pools after the per-node config records have been created.
 // Accepts an optional existing pool record, which will exist when performing subsequent re-create attempts.
-func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.StoragePoolsPost, clientType clusterRequest.ClientType) error {
+func storagePoolsPostCluster(ctx context.Context, s *state.State, pool *api.StoragePool, req api.StoragePoolsPost, clientType clusterRequest.ClientType) error {
 	// Check that no node-specific config key has been defined.
 	for key := range req.Config {
-		if shared.ValueInSlice(key, db.NodeSpecificStorageConfig) {
+		if slices.Contains(db.NodeSpecificStorageConfig, key) {
 			return fmt.Errorf("Config key %q is cluster member specific", key)
 		}
 	}
@@ -457,7 +458,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	if pool != nil {
 		// Check pool isn't already created.
 		if pool.Status == api.StoragePoolStatusCreated {
-			return fmt.Errorf("The storage pool is already created")
+			return errors.New("The storage pool is already created")
 		}
 
 		// Check the requested pool type matches the type created when adding the local member config.
@@ -469,7 +470,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	// Check that the pool is properly defined, fetch the node-specific configs and insert the global config.
 	var configs map[string]map[string]string
 	var poolID int64
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		// Check that the pool was defined at all. Must come before partially created checks.
@@ -481,7 +482,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 		// Check if any global config exists already, if so we should not create global config again.
 		if pool != nil && storagePoolPartiallyCreated(pool) {
 			if len(req.Config) > 0 {
-				return fmt.Errorf("Storage pool already partially created. Please do not specify any global config when re-running create")
+				return errors.New("Storage pool already partially created. Please do not specify any global config when re-running create")
 			}
 
 			logger.Debug("Skipping global storage pool create as global config already partially created", logger.Ctx{"pool": req.Name})
@@ -505,7 +506,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	})
 	if err != nil {
 		if response.IsNotFoundError(err) {
-			return fmt.Errorf("Pool not pending on any node (use --target <node> first)")
+			return errors.New("Pool not pending on any node (use --target <node> first)")
 		}
 
 		return err
@@ -521,11 +522,9 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	nodeReq := req
 
 	// Merge node specific config items into global config.
-	for key, value := range configs[s.ServerName] {
-		nodeReq.Config[key] = value
-	}
+	maps.Copy(nodeReq.Config, configs[s.ServerName])
 
-	updatedConfig, err := storagePoolCreateLocal(s, poolID, req, clientType)
+	updatedConfig, err := storagePoolCreateLocal(ctx, s, poolID, req, clientType)
 	if err != nil {
 		return err
 	}
@@ -545,14 +544,10 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 		// Clone fresh node config so we don't modify req.Config with this node's specific config which
 		// could result in it being sent to other nodes later.
 		nodeReq.Config = make(map[string]string, len(req.Config))
-		for k, v := range req.Config {
-			nodeReq.Config[k] = v
-		}
+		maps.Copy(nodeReq.Config, req.Config)
 
 		// Merge node specific config items into global config.
-		for key, value := range configs[member.Name] {
-			nodeReq.Config[key] = value
-		}
+		maps.Copy(nodeReq.Config, configs[member.Name])
 
 		err = client.CreateStoragePool(nodeReq)
 		if err != nil {
@@ -568,7 +563,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	}
 
 	// Finally update the storage pool state.
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 		return tx.StoragePoolCreated(req.Name)
 	})
 	if err != nil {
@@ -629,7 +624,8 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(s, r)
+	target := request.QueryParam(r, "target")
+	resp := forwardedResponseToNode(r.Context(), s, target)
 	if resp != nil {
 		return resp
 	}
@@ -644,10 +640,7 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	memberSpecific := false
-	if request.QueryParam(r, "target") != "" {
-		memberSpecific = true
-	}
+	memberSpecific := request.QueryParam(r, "target") != ""
 
 	var hiddenPoolNames []string
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -683,7 +676,7 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	poolAPI := pool.ToAPI()
-	poolAPI.UsedBy = project.FilterUsedBy(s.Authorizer, r, poolUsedBy)
+	poolAPI.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, poolUsedBy)
 
 	err = s.Authorizer.CheckPermission(r.Context(), entity.StoragePoolURL(poolName), auth.EntitlementCanEdit)
 	if err != nil && !auth.IsDeniedError(err) {
@@ -758,7 +751,8 @@ func storagePoolPut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(s, r)
+	target := request.QueryParam(r, "target")
+	resp := forwardedResponseToNode(r.Context(), s, target)
 	if resp != nil {
 		return resp
 	}
@@ -777,7 +771,7 @@ func storagePoolPut(d *Daemon, r *http.Request) response.Response {
 	targetNode := request.QueryParam(r, "target")
 
 	if targetNode == "" && pool.Status() != api.StoragePoolStatusCreated {
-		return response.BadRequest(fmt.Errorf("Cannot update storage pool global config when not in created state"))
+		return response.BadRequest(errors.New("Cannot update storage pool global config when not in created state"))
 	}
 
 	// Duplicate config for etag modification and generation.
@@ -813,7 +807,7 @@ func storagePoolPut(d *Daemon, r *http.Request) response.Response {
 		if targetNode == "" {
 			// If no target is specified, then ensure only non-node-specific config keys are changed.
 			for k := range req.Config {
-				if shared.ValueInSlice(k, db.NodeSpecificStorageConfig) {
+				if slices.Contains(db.NodeSpecificStorageConfig, k) {
 					return response.BadRequest(fmt.Errorf("Config key %q is cluster member specific", k))
 				}
 			}
@@ -822,7 +816,7 @@ func storagePoolPut(d *Daemon, r *http.Request) response.Response {
 
 			// If a target is specified, then ensure only node-specific config keys are changed.
 			for k, v := range req.Config {
-				if !shared.ValueInSlice(k, db.NodeSpecificStorageConfig) && curConfig[k] != v {
+				if !slices.Contains(db.NodeSpecificStorageConfig, k) && curConfig[k] != v {
 					return response.BadRequest(fmt.Errorf("Config key %q may not be used as cluster member specific key", k))
 				}
 			}
@@ -833,7 +827,7 @@ func storagePoolPut(d *Daemon, r *http.Request) response.Response {
 
 	response := doStoragePoolUpdate(s, pool, req, targetNode, clientType, r.Method, s.ServerClustered)
 
-	requestor := request.CreateRequestor(r)
+	requestor := request.CreateRequestor(r.Context())
 
 	ctx := logger.Ctx{}
 	if targetNode != "" {
@@ -902,7 +896,7 @@ func doStoragePoolUpdate(s *state.State, pool storagePools.Pool, req api.Storage
 		// node-specific network config with the submitted config to allow validation.
 		// This allows removal of non-node specific keys when they are absent from request config.
 		for k, v := range pool.Driver().Config() {
-			if shared.ValueInSlice(k, db.NodeSpecificStorageConfig) {
+			if slices.Contains(db.NodeSpecificStorageConfig, k) {
 				req.Config[k] = v
 			}
 		}
@@ -934,7 +928,7 @@ func doStoragePoolUpdate(s *state.State, pool storagePools.Pool, req api.Storage
 		sendPool.Config = make(map[string]string)
 		for k, v := range req.Config {
 			// Don't forward node specific keys (these will be merged in on recipient node).
-			if shared.ValueInSlice(k, db.NodeSpecificStorageConfig) {
+			if slices.Contains(db.NodeSpecificStorageConfig, k) {
 				continue
 			}
 
@@ -1005,7 +999,7 @@ func storagePoolDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if inUse {
-			return response.BadRequest(fmt.Errorf("The storage pool is currently in use"))
+			return response.BadRequest(errors.New("The storage pool is currently in use"))
 		}
 
 		// Get the cluster notifier
@@ -1072,12 +1066,12 @@ func storagePoolDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	err = dbStoragePoolDeleteAndUpdateCache(s, pool.Name())
+	err = dbStoragePoolDeleteAndUpdateCache(r.Context(), s, pool.Name())
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r)
+	requestor := request.CreateRequestor(r.Context())
 	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.StoragePoolDeleted.Event(pool.Name(), requestor, nil))
 
 	return response.EmptySyncResponse
