@@ -8,23 +8,27 @@ import (
 	"github.com/canonical/lxd/lxd/backup"
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/device/config"
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/state"
 	storagePools "github.com/canonical/lxd/lxd/storage"
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
 	"github.com/canonical/lxd/shared/revert"
 	"github.com/canonical/lxd/shared/version"
 )
 
-var supportedVolumeTypes = []int{cluster.StoragePoolVolumeTypeContainer, cluster.StoragePoolVolumeTypeVM, cluster.StoragePoolVolumeTypeCustom, cluster.StoragePoolVolumeTypeImage}
+var supportedVolumeTypes = []cluster.StoragePoolVolumeType{cluster.StoragePoolVolumeTypeContainer, cluster.StoragePoolVolumeTypeVM, cluster.StoragePoolVolumeTypeCustom, cluster.StoragePoolVolumeTypeImage}
 
-func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolName string, oldVol *api.StorageVolume, newPoolName string, newVol *api.StorageVolume) (revert.Hook, error) {
+func storagePoolVolumeUpdateUsers(ctx context.Context, s *state.State, projectName string, oldPoolName string, oldVol *api.StorageVolume, newPoolName string, newVol *api.StorageVolume) (revert.Hook, error) {
 	revert := revert.New()
 	defer revert.Fail()
 
-	// Update all instances that are using the volume with a local (non-expanded) device.
+	var instances []instance.Instance
+	var instancesOldArgs []db.InstanceArgs
+	var instancesNewDevices []config.Devices
+
+	// Get all instances that are using the volume with a local (non-expanded) device.
 	err := storagePools.VolumeUsedByInstanceDevices(s, oldPoolName, projectName, oldVol, false, func(dbInst db.InstanceArgs, project api.Project, usedByDevices []string) error {
 		inst, err := instance.Load(s, dbInst, project)
 		if err != nil {
@@ -42,11 +46,25 @@ func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolNam
 			}
 		}
 
+		instances = append(instances, inst)
+		instancesOldArgs = append(instancesOldArgs, dbInst)
+		instancesNewDevices = append(instancesNewDevices, newDevices)
+
+		return nil
+	})
+	if err != nil {
+		return nil, err
+	}
+
+	// Iterate over all instances and update their devices.
+	// Don't perform this within a transaction as the instance's Update will persist the updates to file.
+	// Furthermore this allows requesting further information from the database down the line.
+	for i, inst := range instances {
 		args := db.InstanceArgs{
 			Architecture: inst.Architecture(),
 			Description:  inst.Description(),
 			Config:       inst.LocalConfig(),
-			Devices:      newDevices,
+			Devices:      instancesNewDevices[i],
 			Ephemeral:    inst.IsEphemeral(),
 			Profiles:     inst.Profiles(),
 			Project:      inst.Project().Name,
@@ -56,20 +74,15 @@ func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolNam
 
 		err = inst.Update(args, false)
 		if err != nil {
-			return err
+			return nil, err
 		}
 
 		revert.Add(func() {
-			err := inst.Update(dbInst, false)
+			err := inst.Update(instancesOldArgs[i], false)
 			if err != nil {
-				logger.Error("Failed to revert instance update", logger.Ctx{"project": dbInst.Project, "instance": dbInst.Name, "error": err})
+				logger.Error("Failed to revert instance update", logger.Ctx{"project": instancesOldArgs[i].Project, "instance": instancesOldArgs[i].Name, "error": err})
 			}
 		})
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
 	}
 
 	// Update all profiles that are using the volume with a device.
@@ -86,7 +99,7 @@ func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolNam
 				newDevices[devName][key] = val
 			}
 
-			if shared.ValueInSlice(devName, usedByDevices) {
+			if slices.Contains(usedByDevices, devName) {
 				newDevices[devName]["pool"] = newPoolName
 				newDevices[devName]["source"] = newVol.Name
 			}
@@ -98,7 +111,7 @@ func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolNam
 			Devices:     newDevices,
 		}
 
-		err = doProfileUpdate(s, p, profile.Name, profileID, &profile, pUpdate)
+		err = doProfileUpdate(ctx, s, p, profile.Name, profileID, &profile, pUpdate)
 		if err != nil {
 			return err
 		}
@@ -110,7 +123,7 @@ func storagePoolVolumeUpdateUsers(s *state.State, projectName string, oldPoolNam
 				Devices:     profile.Devices,
 			}
 
-			err := doProfileUpdate(s, p, profile.Name, profileID, &profile, original)
+			err := doProfileUpdate(ctx, s, p, profile.Name, profileID, &profile, original)
 			if err != nil {
 				logger.Error("Failed reverting profile update", logger.Ctx{"project": p.Name, "profile": profile.Name, "error": err})
 			}
@@ -158,7 +171,7 @@ func storagePoolVolumeUsedByGet(s *state.State, requestProjectName string, vol *
 
 	// Pass false to expandDevices, as we only want to see instances directly using a volume, rather than their
 	// profiles using a volume.
-	err = storagePools.VolumeUsedByInstanceDevices(s, vol.Pool, vol.Project, &vol.StorageVolume, false, func(inst db.InstanceArgs, p api.Project, usedByDevices []string) error {
+	err = storagePools.VolumeUsedByInstanceDevices(s, vol.Pool, vol.Project, &vol.StorageVolume, false, func(inst db.InstanceArgs, _ api.Project, _ []string) error {
 		volumeUsedBy = append(volumeUsedBy, api.NewURL().Path(version.APIVersion, "instances", inst.Name).Project(inst.Project).String())
 		return nil
 	})
@@ -166,7 +179,7 @@ func storagePoolVolumeUsedByGet(s *state.State, requestProjectName string, vol *
 		return []string{}, err
 	}
 
-	err = storagePools.VolumeUsedByProfileDevices(s, vol.Pool, requestProjectName, &vol.StorageVolume, func(profileID int64, profile api.Profile, p api.Project, usedByDevices []string) error {
+	err = storagePools.VolumeUsedByProfileDevices(s, vol.Pool, requestProjectName, &vol.StorageVolume, func(_ int64, profile api.Profile, p api.Project, _ []string) error {
 		volumeUsedBy = append(volumeUsedBy, api.NewURL().Path(version.APIVersion, "profiles", profile.Name).Project(p.Name).String())
 		return nil
 	})
@@ -195,10 +208,10 @@ func storagePoolVolumeUsedByGet(s *state.State, requestProjectName string, vol *
 	return volumeUsedBy, nil
 }
 
-func storagePoolVolumeBackupLoadByName(s *state.State, projectName, poolName, backupName string) (*backup.VolumeBackup, error) {
+func storagePoolVolumeBackupLoadByName(ctx context.Context, s *state.State, projectName, poolName, backupName string) (*backup.VolumeBackup, error) {
 	var b db.StoragePoolVolumeBackup
 
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 		b, err = tx.GetStoragePoolVolumeBackup(ctx, projectName, poolName, backupName)
 		return err
