@@ -14,6 +14,7 @@ import (
 	"github.com/canonical/lxd/lxd/cgroup"
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/node"
 	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/lxd/util"
 	"github.com/canonical/lxd/shared"
@@ -76,11 +77,14 @@ type OS struct {
 	AppArmorStacked   bool
 	AppArmorStacking  bool
 	AppArmorFeatures  AppArmorFeaturesInfo
+	AppArmorCacheLoc  string
+	AppArmorCacheDir  string // Based on AppArmorCacheLoc, but may also point to a subdirectory influenced by features.
 
 	// Cgroup features
 	CGInfo cgroup.Info
 
 	// Kernel features
+	BPFToken                bool // BPFToken indicates support for BPF token delegation mechanism.
 	CloseRange              bool // CloseRange indicates support for the close_range syscall.
 	ContainerCoreScheduling bool // ContainerCoreScheduling indicates LXC and kernel support for core scheduling.
 	CoreScheduling          bool // CoreScheduling indicates support for core scheduling syscalls.
@@ -100,10 +104,13 @@ type OS struct {
 	LXCFeatures map[string]bool
 
 	// OS info
-	ReleaseInfo   map[string]string
-	KernelVersion version.DottedVersion
-	Uname         *shared.Utsname
-	BootTime      time.Time
+	ReleaseInfo map[string]string
+	Uname       *shared.Utsname
+	BootTime    time.Time
+
+	// Version info
+	KernelVersion   version.DottedVersion
+	AppArmorVersion *version.DottedVersion // AppArmorVersion is nil if AppArmorAvailable is false.
 }
 
 // DefaultOS returns a fresh uninitialized OS instance with default values.
@@ -209,7 +216,7 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 	}
 
 	btime := int64(0)
-	for _, line := range strings.Split(string(out), "\n") {
+	for line := range strings.SplitSeq(string(out), "\n") {
 		if !strings.HasPrefix(line, "btime ") {
 			continue
 		}
@@ -231,8 +238,8 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 }
 
 // InitStorage initialises the storage layer after it has been mounted.
-func (s *OS) InitStorage() error {
-	return s.initStorageDirs()
+func (s *OS) InitStorage(config *node.Config) error {
+	return s.initStorageDirs(config)
 }
 
 // InUbuntuCore returns true if we're running on Ubuntu Core.
@@ -247,3 +254,13 @@ func (s *OS) InUbuntuCore() bool {
 
 	return false
 }
+
+// GetUnixSocket returns the full path to the unix.socket file that this daemon is listening on. Used by tests.
+func (s *OS) GetUnixSocket() string {
+	path := os.Getenv("LXD_SOCKET")
+	if path != "" {
+		return path
+	}
+
+	return filepath.Join(s.VarDir, "unix.socket")
+}
