@@ -2,7 +2,7 @@ package template
 
 import (
 	"bytes"
-	"fmt"
+	"errors"
 	"io"
 	"os"
 	"path/filepath"
@@ -38,7 +38,7 @@ func (l ChrootLoader) Get(path string) (io.Reader, error) {
 
 	// Validate that we're under the expected prefix
 	if !strings.HasPrefix(path, basePath) {
-		return nil, fmt.Errorf("Attempting to access a file outside the instance")
+		return nil, errors.New("Attempting to access a file outside the instance")
 	}
 
 	// Open and read the file
