@@ -83,7 +83,7 @@ func autoRemoveExpiredTokens(ctx context.Context, s *state.State) {
 		return nil
 	}
 
-	op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.RemoveExpiredTokens, nil, nil, opRun, nil, nil, nil)
+	op, err := operations.OperationCreate(context.Background(), s, "", operations.OperationClassTask, operationtype.RemoveExpiredTokens, nil, nil, opRun, nil, nil)
 	if err != nil {
 		logger.Warn("Failed creating remove expired tokens operation", logger.Ctx{"err": err})
 		return
@@ -144,9 +144,11 @@ func getExpiredPendingIdentities(ctx context.Context, s *state.State) ([]cluster
 	return expiredPendingTLSIdentities, nil
 }
 
-func autoRemoveExpiredTokensTask(d *Daemon) (task.Func, task.Schedule) {
+func autoRemoveExpiredTokensTask(stateFunc func() *state.State) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
-		autoRemoveExpiredTokens(ctx, d.State())
+		s := stateFunc()
+
+		autoRemoveExpiredTokens(ctx, s)
 	}
 
 	return f, task.Every(time.Hour)
