@@ -2,10 +2,10 @@ package util
 
 import (
 	"fmt"
+	"maps"
+	"slices"
 	"sort"
 	"strings"
-
-	"github.com/canonical/lxd/shared"
 )
 
 // CompareConfigs compares two config maps and returns an error if they differ.
@@ -16,7 +16,7 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 
 	delta := []string{}
 	for key, value := range config1 {
-		if shared.ValueInSlice(key, exclude) {
+		if slices.Contains(exclude, key) {
 			continue
 		}
 
@@ -25,18 +25,12 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 		}
 	}
 	for key, value := range config2 {
-		if shared.ValueInSlice(key, exclude) {
+		if slices.Contains(exclude, key) {
 			continue
 		}
 
 		if config1[key] != value {
-			present := false
-			for i := range delta {
-				if delta[i] == key {
-					present = true
-					break
-				}
-			}
+			present := slices.Contains(delta, key)
 			if !present {
 				delta = append(delta, key)
 			}
@@ -54,9 +48,7 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 // CopyConfig creates a new map with a copy of the given config.
 func CopyConfig(config map[string]string) map[string]string {
 	newConfig := make(map[string]string, len(config))
-	for key, value := range config {
-		newConfig[key] = value
-	}
+	maps.Copy(newConfig, config)
 
 	return newConfig
 }
