@@ -285,9 +285,11 @@ type ClusterMemberStatePost struct {
 	Action string `json:"action" yaml:"action"`
 
 	// Override the configured evacuation mode.
+	// Valid modes for the "evacuate" action are "stop", "migrate", and "live-migrate".
+	// Valid modes for the "restore" action are "skip".
 	// Example: stop
 	//
-	// API extension: clustering_evacuate_mode
+	// API extension: clustering_evacuation_mode
 	Mode string `json:"mode" yaml:"mode"`
 }
 
@@ -321,6 +323,11 @@ type ClusterGroup struct {
 	// List of members in this group
 	// Example: ["node1", "node3"]
 	Members []string `json:"members" yaml:"members"`
+
+	// UsedBy is a list or LXD entity URLs that reference the cluster group.
+	//
+	// API extension: clustering_groups_used_by
+	UsedBy []string `json:"used_by" yaml:"used_by"`
 }
 
 // ClusterGroupPost represents the fields required to rename a cluster group.
