@@ -144,6 +144,16 @@ func TestURL(t *testing.T) {
 			expectedPathArgs:      []string{"my-instance"},
 			expectedErr:           nil,
 		},
+		{
+			name:                  "instances",
+			rawURL:                "/1.0/instances/my-instance?target=member01",
+			expectedNormalisedURL: "/1.0/instances/my-instance?project=default",
+			expectedEntityType:    TypeInstance,
+			expectedLocation:      "member01",
+			expectedProject:       api.ProjectDefaultName,
+			expectedPathArgs:      []string{"my-instance"},
+			expectedErr:           nil,
+		},
 		{
 			name:                  "instance backup",
 			rawURL:                "/1.0/instances/my-instance/backups/my-backup?project=my-project",
@@ -319,7 +329,7 @@ func TestURL(t *testing.T) {
 			}
 
 			normalisedURL, err := actualEntityType.URL(actualProject, actualLocation, actualPathArgs...)
-			assert.Equal(t, normalisedURL.String(), tt.expectedNormalisedURL)
+			assert.Equal(t, tt.expectedNormalisedURL, normalisedURL.String())
 			assert.NoError(t, err)
 		})
 	}
