@@ -9,9 +9,11 @@ import (
 	"encoding/gob"
 	"encoding/hex"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"hash"
 	"io"
+	"maps"
 	"net/http"
 	"net/url"
 	"os"
@@ -21,6 +23,7 @@ import (
 	"reflect"
 	"regexp"
 	"runtime"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -332,7 +335,7 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 		filetype = "file"
 	}
 
-	if !ValueInSlice(filetype, []string{"file", "symlink", "directory"}) {
+	if !slices.Contains([]string{"file", "symlink", "directory"}, filetype) {
 		return nil, fmt.Errorf("Invalid file type: %q", filetype)
 	}
 
@@ -344,7 +347,7 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 		write = "overwrite"
 	}
 
-	if !ValueInSlice(write, []string{"overwrite", "append"}) {
+	if !slices.Contains([]string{"overwrite", "append"}, write) {
 		return nil, fmt.Errorf("Invalid file write mode: %q", write)
 	}
 
@@ -354,13 +357,14 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 
 	modifyPermHeader := headers.Get("X-LXD-modify-perm")
 
+	modifyPermFields := []string{"uid", "gid", "mode"}
 	if modifyPermHeader != "" {
-		for _, perm := range strings.Split(modifyPermHeader, ",") {
+		for perm := range strings.SplitSeq(modifyPermHeader, ",") {
 			UIDModifyExisting = UIDModifyExisting || perm == "uid"
 			GIDModifyExisting = GIDModifyExisting || perm == "gid"
 			modeModifyExisting = modeModifyExisting || perm == "mode"
 
-			if !ValueInSlice(perm, []string{"uid", "gid", "mode"}) {
+			if !slices.Contains(modifyPermFields, perm) {
 				return nil, fmt.Errorf("Invalid modify-perm field: %q", perm)
 			}
 		}
@@ -424,7 +428,7 @@ func RandomCryptoString() (string, error) {
 	}
 
 	if n != len(buf) {
-		return "", fmt.Errorf("not enough random bytes read")
+		return "", errors.New("not enough random bytes read")
 	}
 
 	return hex.EncodeToString(buf), nil
@@ -593,7 +597,7 @@ func DirCopy(source string, dest string) error {
 	}
 
 	if !info.IsDir() {
-		return fmt.Errorf("source is not a directory")
+		return errors.New("source is not a directory")
 	}
 
 	// Remove dest if it already exists.
@@ -669,7 +673,7 @@ func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 			return nil
 		}
 
-		return fmt.Errorf("path exists but isn't a directory")
+		return errors.New("path exists but isn't a directory")
 	}
 
 	// Slow path: make sure parent exists and then call Mkdir for path.
@@ -720,17 +724,6 @@ func HasKey[K comparable, V any](key K, m map[K]V) bool {
 	return found
 }
 
-// ValueInSlice returns true if key is in list.
-func ValueInSlice[T comparable](key T, list []T) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-
-	return false
-}
-
 // StringPrefixInSlice returns true if any element in the list has the given prefix.
 func StringPrefixInSlice(key string, list []string) bool {
 	for _, entry := range list {
@@ -751,13 +744,13 @@ func RemoveElementsFromSlice[T comparable](list []T, elements ...T) []T {
 		for j := len(list) - 1; j >= 0; j-- {
 			if element == list[j] {
 				match = true
-				list = append(list[:j], list[j+1:]...)
+				list = slices.Delete(list, j, j+1)
 				break
 			}
 		}
 
 		if match {
-			elements = append(elements[:i], elements[i+1:]...)
+			elements = slices.Delete(elements, i, i+1)
 		}
 	}
 
@@ -780,7 +773,7 @@ func IsTrue(value string) bool {
 		return true
 	}
 
-	return ValueInSlice(strings.ToLower(value), []string{"true", "yes", "on"})
+	return slices.Contains([]string{"true", "yes", "on"}, strings.ToLower(value))
 }
 
 // IsTrueOrEmpty returns true if value is empty or if IsTrue() returns true.
@@ -794,7 +787,7 @@ func IsFalse(value string) bool {
 		return true
 	}
 
-	return ValueInSlice(strings.ToLower(value), []string{"false", "no", "off"})
+	return slices.Contains([]string{"false", "no", "off"}, strings.ToLower(value))
 }
 
 // IsFalseOrEmpty returns true if value is empty or if IsFalse() returns true.
@@ -903,7 +896,7 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 				}
 			}
 			if editor == "" {
-				return []byte{}, fmt.Errorf("No text editor found, please set the EDITOR environment variable")
+				return []byte{}, errors.New("No text editor found, please set the EDITOR environment variable")
 			}
 		}
 	}
@@ -970,7 +963,7 @@ func ParseMetadata(metadata any) (map[string]any, error) {
 	if s.Kind() == reflect.Map {
 		for _, k := range s.MapKeys() {
 			if k.Kind() != reflect.String {
-				return nil, fmt.Errorf("Invalid metadata provided (key isn't a string)")
+				return nil, errors.New("Invalid metadata provided (key isn't a string)")
 			}
 
 			newMetadata[k.String()] = s.MapIndex(k).Interface()
@@ -978,7 +971,7 @@ func ParseMetadata(metadata any) (map[string]any, error) {
 	} else if s.Kind() == reflect.Ptr && !s.Elem().IsValid() {
 		return nil, nil
 	} else {
-		return nil, fmt.Errorf("Invalid metadata provided (type isn't a map)")
+		return nil, errors.New("Invalid metadata provided (type isn't a map)")
 	}
 
 	return newMetadata, nil
@@ -996,7 +989,7 @@ func RemoveDuplicatesFromString(s string, sep string) string {
 
 	dup := sep + sep
 	for strings.Contains(s, dup) {
-		s = strings.Replace(s, dup, sep, -1)
+		s = strings.ReplaceAll(s, dup, sep)
 	}
 
 	return s
@@ -1134,7 +1127,7 @@ func TryRunCommand(name string, arg ...string) (string, error) {
 	var err error
 	var output string
 
-	for i := 0; i < 20; i++ {
+	for range 20 {
 		output, err = RunCommand(name, arg...)
 		if err == nil {
 			break
@@ -1219,7 +1212,7 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 	if ctx != nil {
 		req, err = http.NewRequestWithContext(ctx, "GET", url, nil)
 	} else {
-		req, err = http.NewRequest("GET", url, nil)
+		req, err = http.NewRequest(http.MethodGet, url, nil)
 	}
 
 	if err != nil {
@@ -1269,7 +1262,7 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 			return -1, err
 		}
 
-		result := fmt.Sprintf("%x", hashFunc.Sum(nil))
+		result := hex.EncodeToString(hashFunc.Sum(nil))
 		if result != hash {
 			return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
 		}
@@ -1330,25 +1323,52 @@ func (r *ReadSeeker) Seek(offset int64, whence int) (int64, error) {
 }
 
 // RenderTemplate renders a pongo2 template.
-func RenderTemplate(template string, ctx pongo2.Context) (string, error) {
-	// Load template from string
-	tpl, err := pongo2.FromString("{% autoescape off %}" + template + "{% endautoescape %}")
-	if err != nil {
-		return "", err
+func RenderTemplate(template string, ctx pongo2.Context) (output string, err error) {
+	defer func() {
+		// Capture panics in the pongo2 template rendering.
+		// This is to prevent the server from crashing due to a template error.
+		r := recover()
+		if r != nil {
+			err = fmt.Errorf("Panic while rendering template: %v", r)
+		}
+	}()
+
+	// Create custom TemplateSet
+	set := pongo2.NewSet("restricted", pongo2.DefaultLoader)
+
+	// Ban tags that could be used to access the host's filesystem.
+	for _, tag := range []string{"extends", "import", "include", "ssi"} {
+		err := set.BanTag(tag)
+		if err != nil {
+			return "", fmt.Errorf("Failed to ban tag %q: %w", tag, err)
+		}
 	}
 
-	// Get rendered template
-	ret, err := tpl.Execute(ctx)
-	if err != nil {
-		return ret, err
+	// Prevent unbounded recursion while rendering templates. Normal use should not
+	// require more than 1 or 2 levels of recursion.
+	for range 3 {
+		// Load template from string
+		tpl, err := set.FromString("{% autoescape off %}" + template + "{% endautoescape %}")
+		if err != nil {
+			return "", err
+		}
+
+		// Get rendered template
+		ret, err := tpl.Execute(ctx)
+		if err != nil {
+			return "", err
+		}
+
+		// Check if another pass is needed.
+		if !strings.Contains(ret, "{{") && !strings.Contains(ret, "{%") {
+			return ret, nil
+		}
+
+		// Prepare for another pass.
+		template = ret
 	}
 
-	// Looks like we're nesting templates so run pongo again
-	if strings.Contains(ret, "{{") || strings.Contains(ret, "{%") {
-		return RenderTemplate(ret, ctx)
-	}
-
-	return ret, err
+	return "", errors.New("Recursion limit reached while rendering template")
 }
 
 // GetExpiry returns the expiry date based on the reference date and a length of time.
@@ -1385,12 +1405,12 @@ func GetExpiry(refDate time.Time, s string) (time.Time, error) {
 	for _, value := range values {
 		fields := re.FindStringSubmatch(value)
 		if fields == nil {
-			return time.Time{}, fmt.Errorf("Invalid expiry expression")
+			return time.Time{}, errors.New("Invalid expiry expression")
 		}
 
 		if expiry[fields[2]] > 0 {
 			// We don't allow fields to be set multiple times
-			return time.Time{}, fmt.Errorf("Invalid expiry expression")
+			return time.Time{}, errors.New("Invalid expiry expression")
 		}
 
 		val, err := strconv.Atoi(fields[1])
@@ -1460,19 +1480,19 @@ func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 	}
 
 	if j.ServerName == "" {
-		return nil, fmt.Errorf("No server name in join token")
+		return nil, errors.New("No server name in join token")
 	}
 
 	if len(j.Addresses) < 1 {
-		return nil, fmt.Errorf("No cluster member addresses in join token")
+		return nil, errors.New("No cluster member addresses in join token")
 	}
 
 	if j.Secret == "" {
-		return nil, fmt.Errorf("No secret in join token")
+		return nil, errors.New("No secret in join token")
 	}
 
 	if j.Fingerprint == "" {
-		return nil, fmt.Errorf("No certificate fingerprint in join token")
+		return nil, errors.New("No certificate fingerprint in join token")
 	}
 
 	return &j, nil
@@ -1481,8 +1501,9 @@ func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 // TargetDetect returns either target node or group based on the provided prefix:
 // An invocation with `target=h1` returns "h1", "" and `target=@g1` returns "", "g1".
 func TargetDetect(target string) (targetNode string, targetGroup string) {
-	if strings.HasPrefix(target, "@") {
-		targetGroup = strings.TrimPrefix(target, "@")
+	after, found := strings.CutPrefix(target, "@")
+	if found {
+		targetGroup = after
 	} else {
 		targetNode = target
 	}
@@ -1499,9 +1520,7 @@ func ApplyDeviceOverrides(localDevices map[string]map[string]string, profileDevi
 		_, isLocalDevice := localDevices[deviceName]
 		if isLocalDevice {
 			// Apply overrides to local device.
-			for k, v := range deviceOverrides[deviceName] {
-				localDevices[deviceName][k] = v
-			}
+			maps.Copy(localDevices[deviceName], deviceOverrides[deviceName])
 		} else {
 			// Check device exists in expanded profile devices.
 			profileDeviceConfig, found := profileDevices[deviceName]
@@ -1509,9 +1528,7 @@ func ApplyDeviceOverrides(localDevices map[string]map[string]string, profileDevi
 				return nil, fmt.Errorf("Cannot override config for device %q: Device not found in profile devices", deviceName)
 			}
 
-			for k, v := range deviceOverrides[deviceName] {
-				profileDeviceConfig[k] = v
-			}
+			maps.Copy(profileDeviceConfig, deviceOverrides[deviceName])
 
 			localDevices[deviceName] = profileDeviceConfig
 		}
