@@ -8,6 +8,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"math"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -61,6 +62,15 @@ func GetPathMode(path string) (os.FileMode, error) {
 // SetSize sets the terminal size to the specified width and height for the given file descriptor.
 func SetSize(fd int, width int, height int) (err error) {
 	var dimensions [4]uint16
+
+	if width > math.MaxUint16 || height > math.MaxUint16 {
+		return fmt.Errorf("Width or height too large: %d %d", width, height)
+	}
+
+	if width < 0 || height < 0 {
+		return fmt.Errorf("Width and height must not be negative: %d %d", width, height)
+	}
+
 	dimensions[0] = uint16(height)
 	dimensions[1] = uint16(width)
 
@@ -99,7 +109,7 @@ func GetAllXattr(path string) (map[string]string, error) {
 }
 
 // ErrObjectFound indicates that the requested object was found.
-var ErrObjectFound = fmt.Errorf("Found requested object")
+var ErrObjectFound = errors.New("Found requested object")
 
 // LookupUUIDByBlockDevPath finds and returns the UUID of a block device by its path.
 func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
@@ -135,7 +145,7 @@ func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
 	}
 
 	if uuid == "" {
-		return "", fmt.Errorf("Failed to detect UUID")
+		return "", errors.New("Failed to detect UUID")
 	}
 
 	lastSlash := strings.LastIndex(uuid, "/")
@@ -203,7 +213,7 @@ func Uname() (*Utsname, error) {
 func intArrayToString(arr any) string {
 	slice := reflect.ValueOf(arr)
 	s := ""
-	for i := 0; i < slice.Len(); i++ {
+	for i := range slice.Len() {
 		val := slice.Index(i)
 		valInt := int64(-1)
 
@@ -314,7 +324,7 @@ func OpenPtyInDevpts(devptsFD int, uid, gid int64) (ptx *os.File, pty *os.File,
 		pty = os.NewFile(ptyFd, fmt.Sprintf("/dev/pts/%d", id))
 	} else {
 		if devptsFD >= 0 {
-			return nil, nil, fmt.Errorf("TIOCGPTPEER required but not available")
+			return nil, nil, errors.New("TIOCGPTPEER required but not available")
 		}
 
 		// Get the pty side.
@@ -475,9 +485,9 @@ func (w *execWrapper) Read(p []byte) (int, error) {
 			case err != nil:
 				opErr = err
 			case revents&unix.POLLERR > 0:
-				opErr = fmt.Errorf("Got POLLERR event")
+				opErr = errors.New("Got POLLERR event")
 			case revents&unix.POLLNVAL > 0:
-				opErr = fmt.Errorf("Got POLLNVAL event")
+				opErr = errors.New("Got POLLNVAL event")
 			case revents&(unix.POLLIN|unix.POLLPRI) > 0:
 				// If there is something to read then read it.
 				n, opErr = unix.Read(int(fd), p)
