@@ -11,6 +11,7 @@ import (
 	"os/exec"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -25,12 +26,7 @@ import (
 
 // stringInSlice checks whether the supplied string is present in the supplied slice.
 func stringInSlice(key string, list []string) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-	return false
+	return slices.Contains(list, key)
 }
 
 // Required returns function that runs one or more validators, all must pass without error.
@@ -78,6 +74,16 @@ func IsUint8(value string) error {
 	return nil
 }
 
+// IsUint16 validates whether the string can be converted to an uint16.
+func IsUint16(value string) error {
+	_, err := strconv.ParseUint(value, 10, 16)
+	if err != nil {
+		return fmt.Errorf("Invalid value for uint16 %q: %w", value, err)
+	}
+
+	return nil
+}
+
 // IsUint32 validates whether the string can be converted to an uint32.
 func IsUint32(value string) error {
 	_, err := strconv.ParseUint(value, 10, 32)
@@ -94,7 +100,7 @@ func ParseUint32Range(value string) (start uint32, rangeSize uint32, err error)
 	rangeParts := strings.SplitN(value, "-", 2)
 	rangeLen := len(rangeParts)
 	if rangeLen != 1 && rangeLen != 2 {
-		return 0, 0, fmt.Errorf("Range must contain a single number or start and end numbers")
+		return 0, 0, errors.New("Range must contain a single number or start and end numbers")
 	}
 
 	startNum, err := strconv.ParseUint(rangeParts[0], 10, 32)
@@ -186,7 +192,7 @@ func IsAny(value string) error {
 // IsListOf returns a validator for a comma separated list of values.
 func IsListOf(validator func(value string) error) func(value string) error {
 	return func(value string) error {
-		for _, v := range strings.Split(value, ",") {
+		for v := range strings.SplitSeq(value, ",") {
 			v = strings.TrimSpace(v)
 
 			err := validator(v)
@@ -202,7 +208,7 @@ func IsListOf(validator func(value string) error) func(value string) error {
 // IsNotEmpty requires a non-empty string.
 func IsNotEmpty(value string) error {
 	if value == "" {
-		return fmt.Errorf("Required value")
+		return errors.New("Required value")
 	}
 
 	return nil
@@ -222,7 +228,7 @@ func IsSize(value string) error {
 func IsDeviceID(value string) error {
 	match, _ := regexp.MatchString(`^[0-9a-f]{4}$`, value)
 	if !match {
-		return fmt.Errorf("Invalid value, must be four lower case hex characters")
+		return errors.New("Invalid value, must be four lower case hex characters")
 	}
 
 	return nil
@@ -232,17 +238,17 @@ func IsDeviceID(value string) error {
 func IsInterfaceName(value string) error {
 	// Validate the length.
 	if len(value) < 2 {
-		return fmt.Errorf("Network interface is too short (minimum 2 characters)")
+		return errors.New("Network interface is too short (minimum 2 characters)")
 	}
 
 	if len(value) > 15 {
-		return fmt.Errorf("Network interface is too long (maximum 15 characters)")
+		return errors.New("Network interface is too long (maximum 15 characters)")
 	}
 
 	// Validate the character set.
 	match, _ := regexp.MatchString(`^[-_a-zA-Z0-9.]+$`, value)
 	if !match {
-		return fmt.Errorf("Network interface contains invalid characters")
+		return errors.New("Network interface contains invalid characters")
 	}
 
 	return nil
@@ -254,7 +260,7 @@ func IsNetworkMAC(value string) error {
 
 	// Check is valid Ethernet MAC length and delimiter.
 	if err != nil || len(value) != 17 || strings.ContainsAny(value, "-.") {
-		return fmt.Errorf("Invalid MAC address, must be 6 bytes of hex separated by colons")
+		return errors.New("Invalid MAC address, must be 6 bytes of hex separated by colons")
 	}
 
 	return nil
@@ -298,7 +304,7 @@ func IsNetworkAddressCIDR(value string) error {
 func IsNetworkRange(value string) error {
 	ips := strings.SplitN(value, "-", 2)
 	if len(ips) != 2 {
-		return fmt.Errorf("IP range must contain start and end IP addresses")
+		return errors.New("IP range must contain start and end IP addresses")
 	}
 
 	startIP := net.ParseIP(ips[0])
@@ -312,11 +318,11 @@ func IsNetworkRange(value string) error {
 	}
 
 	if (startIP.To4() != nil) != (endIP.To4() != nil) {
-		return fmt.Errorf("Start and end IP addresses are not in same family")
+		return errors.New("Start and end IP addresses are not in same family")
 	}
 
 	if bytes.Compare(startIP, endIP) > 0 {
-		return fmt.Errorf("Start IP address must be before or equal to end IP address")
+		return errors.New("Start IP address must be before or equal to end IP address")
 	}
 
 	return nil
@@ -372,7 +378,7 @@ func IsNetworkAddressCIDRV4(value string) error {
 func IsNetworkRangeV4(value string) error {
 	ips := strings.SplitN(value, "-", 2)
 	if len(ips) != 2 {
-		return fmt.Errorf("IP range must contain start and end IP addresses")
+		return errors.New("IP range must contain start and end IP addresses")
 	}
 
 	for _, ip := range ips {
@@ -435,7 +441,7 @@ func IsNetworkAddressCIDRV6(value string) error {
 func IsNetworkRangeV6(value string) error {
 	ips := strings.SplitN(value, "-", 2)
 	if len(ips) != 2 {
-		return fmt.Errorf("IP range must contain start and end IP addresses")
+		return errors.New("IP range must contain start and end IP addresses")
 	}
 
 	for _, ip := range ips {
@@ -497,7 +503,7 @@ func IsNetworkPortRange(value string) error {
 	ports := strings.SplitN(value, "-", 2)
 	portsLen := len(ports)
 	if portsLen != 1 && portsLen != 2 {
-		return fmt.Errorf("Port range must contain either a single port or start and end port numbers")
+		return errors.New("Port range must contain either a single port or start and end port numbers")
 	}
 
 	startPort, err := strconv.ParseUint(ports[0], 10, 32)
@@ -534,7 +540,7 @@ func IsURLSegmentSafe(value string) error {
 func IsUUID(value string) error {
 	_, err := uuid.Parse(value)
 	if err != nil {
-		return fmt.Errorf("Invalid UUID")
+		return errors.New("Invalid UUID")
 	}
 
 	return nil
@@ -544,7 +550,7 @@ func IsUUID(value string) error {
 func IsPCIAddress(value string) error {
 	match, _ := regexp.MatchString(`^(?:[0-9a-fA-F]{4}:)?[0-9a-fA-F]{2}:[0-9a-fA-F]{2}\.[0-9a-fA-F]$`, value)
 	if !match {
-		return fmt.Errorf("Invalid PCI address")
+		return errors.New("Invalid PCI address")
 	}
 
 	return nil
@@ -581,14 +587,12 @@ func IsCron(aliases []string) func(value string) error {
 	return func(value string) error {
 		isValid := func(value string) error {
 			// Accept valid aliases.
-			for _, alias := range aliases {
-				if alias == value {
-					return nil
-				}
+			if slices.Contains(aliases, value) {
+				return nil
 			}
 
 			if len(strings.Split(value, " ")) != 5 {
-				return fmt.Errorf("Schedule must be of the form: <minute> <hour> <day-of-month> <month> <day-of-week>")
+				return errors.New("Schedule must be of the form: <minute> <hour> <day-of-month> <month> <day-of-week>")
 			}
 
 			_, err := cron.ParseStandard(value)
@@ -601,8 +605,8 @@ func IsCron(aliases []string) func(value string) error {
 
 		// Can be comma+space separated (just commas are valid cron pattern).
 		value = strings.ToLower(value)
-		triggers := strings.Split(value, ", ")
-		for _, trigger := range triggers {
+		triggers := strings.SplitSeq(value, ", ")
+		for trigger := range triggers {
 			err := isValid(trigger)
 			if err != nil {
 				return err
@@ -620,7 +624,7 @@ func IsListenAddress(allowDNS bool, allowWildcard bool, requirePort bool) func(v
 		host, _, err := net.SplitHostPort(value)
 		if err != nil {
 			if requirePort {
-				return fmt.Errorf("A port is required as part of the address")
+				return errors.New("A port is required as part of the address")
 			}
 
 			host = value
@@ -629,7 +633,7 @@ func IsListenAddress(allowDNS bool, allowWildcard bool, requirePort bool) func(v
 		// Validate wildcard.
 		if stringInSlice(host, []string{"", "::", "[::]", "0.0.0.0"}) {
 			if !allowWildcard {
-				return fmt.Errorf("Wildcard addresses aren't allowed")
+				return errors.New("Wildcard addresses aren't allowed")
 			}
 
 			return nil
@@ -642,7 +646,7 @@ func IsListenAddress(allowDNS bool, allowWildcard bool, requirePort bool) func(v
 		}
 
 		if !allowDNS {
-			return fmt.Errorf("DNS names not allowed in address")
+			return errors.New("DNS names not allowed in address")
 		}
 
 		_, err = net.LookupHost(host)
@@ -658,7 +662,7 @@ func IsListenAddress(allowDNS bool, allowWildcard bool, requirePort bool) func(v
 func IsX509Certificate(value string) error {
 	certBlock, _ := pem.Decode([]byte(value))
 	if certBlock == nil {
-		return fmt.Errorf("Invalid certificate")
+		return errors.New("Invalid certificate")
 	}
 
 	_, err := x509.ParseCertificate(certBlock.Bytes)
@@ -669,7 +673,7 @@ func IsX509Certificate(value string) error {
 // IsAbsFilePath checks if value is an absolute file path.
 func IsAbsFilePath(value string) error {
 	if !filepath.IsAbs(value) {
-		return fmt.Errorf("Must be absolute file path")
+		return errors.New("Must be absolute file path")
 	}
 
 	return nil
@@ -720,22 +724,22 @@ func ParseNetworkVLANRange(vlan string) (vlanRangeStart int, rangeSize int, err
 func IsHostname(name string) error {
 	// Validate length
 	if len(name) < 1 || len(name) > 63 {
-		return fmt.Errorf("Name must be 1-63 characters long")
+		return errors.New("Name must be 1-63 characters long")
 	}
 
 	// Validate first character
 	if strings.HasPrefix(name, "-") {
-		return fmt.Errorf(`Name must not start with "-" character`)
+		return errors.New(`Name must not start with "-" character`)
 	}
 
 	// Validate last character
 	if strings.HasSuffix(name, "-") {
-		return fmt.Errorf(`Name must not end with "-" character`)
+		return errors.New(`Name must not end with "-" character`)
 	}
 
 	_, err := strconv.ParseUint(name, 10, 64)
 	if err == nil {
-		return fmt.Errorf("Name cannot be a number")
+		return errors.New("Name cannot be a number")
 	}
 
 	match, err := regexp.MatchString(`^[\-a-zA-Z0-9]+$`, name)
@@ -744,7 +748,7 @@ func IsHostname(name string) error {
 	}
 
 	if !match {
-		return fmt.Errorf("Name can only contain alphanumeric and hyphen characters")
+		return errors.New("Name can only contain alphanumeric and hyphen characters")
 	}
 
 	return nil
@@ -754,11 +758,11 @@ func IsHostname(name string) error {
 // forward slash, hyphen, colon, underscore and full stop characters.
 func IsDeviceName(name string) error {
 	if len(name) < 1 || len(name) > 63 {
-		return fmt.Errorf("Name must be 1-63 characters long")
+		return errors.New("Name must be 1-63 characters long")
 	}
 
 	if string(name[0]) == "." {
-		return fmt.Errorf(`Name must not start with "." character`)
+		return errors.New(`Name must not start with "." character`)
 	}
 
 	match, err := regexp.MatchString(`^[\/\.\-:_a-zA-Z0-9]+$`, name)
@@ -767,7 +771,7 @@ func IsDeviceName(name string) error {
 	}
 
 	if !match {
-		return fmt.Errorf("Name can only contain alphanumeric, forward slash, hyphen, colon, underscore and full stop characters")
+		return errors.New("Name can only contain alphanumeric, forward slash, hyphen, colon, underscore and full stop characters")
 	}
 
 	return nil
@@ -776,7 +780,7 @@ func IsDeviceName(name string) error {
 // IsRequestURL checks value is a valid HTTP/HTTPS request URL.
 func IsRequestURL(value string) error {
 	if value == "" {
-		return fmt.Errorf("Empty URL")
+		return errors.New("Empty URL")
 	}
 
 	_, err := url.ParseRequestURI(value)
@@ -807,6 +811,11 @@ func IsCloudInitUserData(value string) error {
 // IsUserSSHKey checks value is a valid SSH public key for cloud-init.ssh-keys.
 // This should take the format {user}:{key}, and neither part can be empty.
 func IsUserSSHKey(value string) error {
+	// A "none" value can be used to avoid inheriting an SSH key from a profile.
+	if value == "none" {
+		return nil
+	}
+
 	user, key, _ := strings.Cut(value, ":")
 
 	if key == "" {
@@ -837,13 +846,13 @@ func IsValidCPUSet(value string) error {
 	// Validate the CPU set syntax.
 	match, _ := regexp.MatchString(`^(?:[0-9]+(?:[,-][0-9]+)?)(?:,[0-9]+(?:[,-][0-9]+)*)?$`, value)
 	if !match {
-		return fmt.Errorf("Invalid CPU limit syntax")
+		return errors.New("Invalid CPU limit syntax")
 	}
 
 	cpus := make(map[int64]int)
-	chunks := strings.Split(value, ",")
+	chunks := strings.SplitSeq(value, ",")
 
-	for _, chunk := range chunks {
+	for chunk := range chunks {
 		if strings.Contains(chunk, "-") {
 			// Range
 			fields := strings.SplitN(chunk, "-", 2)
@@ -878,7 +887,7 @@ func IsValidCPUSet(value string) error {
 	for i := range cpus {
 		// The CPU was specified more than once, e.g. 1-3,3.
 		if cpus[i] > 1 {
-			return fmt.Errorf("Cannot define CPU multiple times")
+			return errors.New("Cannot define CPU multiple times")
 		}
 	}
 
