@@ -441,6 +441,19 @@ var APIExtensions = []string{
 	"oidc_scopes",
 	"project_default_network_and_storage",
 	"client_cert_presence",
+	"clustering_groups_used_by",
+	"container_bpf_delegation",
+	"override_snapshot_profiles_on_copy",
+	"resources_device_fs_uuid",
+	"backup_metadata_version",
+	"storage_buckets_all_projects",
+	"network_acls_all_projects",
+	"networks_all_projects",
+	"clustering_restore_skip_mode",
+	"disk_io_threads_virtiofsd",
+	"oidc_client_secret",
+	"pci_hotplug",
+	"device_patch_removal",
 }
 
 // APIExtensionsCount returns the number of available API extensions.
