@@ -56,7 +56,7 @@ cleanup() {
 
     # Try to stop all the containers
     my_curl "$BASEURL/1.0/containers" | jq -r .metadata[] 2>/dev/null | while read -r line; do
-        wait_for my_curl -X PUT "$BASEURL$line/state" -d "{\"action\":\"stop\",\"force\":true}"
+        wait_for my_curl -X PUT "$BASEURL$line/state" --fail-with-body -H 'Content-Type: application/json' -d "{\"action\":\"stop\",\"force\":true}"
     done
 
     # kill the lxds which share our pgrp as parent
@@ -81,7 +81,8 @@ cleanup() {
 trap cleanup EXIT HUP INT TERM
 
 if ! command -v lxc > /dev/null; then
-    echo "==> Couldn't find lxc" && false
+    echo "==> Couldn't find lxc"
+    false
 fi
 
 spawn_lxd() {
