@@ -6,9 +6,9 @@ setup_clustering_bridge() {
   echo "==> Setup clustering bridge ${name}"
 
   ip link add "${name}" up type bridge
-  ip addr add 10.1.1.1/16 dev "${name}"
+  ip addr add 100.64.1.1/16 dev "${name}"
 
-  iptables -w -t nat -A POSTROUTING -s 10.1.0.0/16 -d 0.0.0.0/0 -j MASQUERADE
+  iptables -w -t nat -A POSTROUTING -s 100.64.0.0/16 -d 0.0.0.0/0 -j MASQUERADE
   echo 1 > /proc/sys/net/ipv4/ip_forward
 }
 
@@ -18,7 +18,7 @@ teardown_clustering_bridge() {
   if [ -e "/sys/class/net/${name}" ]; then
       echo "==> Teardown clustering bridge ${name}"
       echo 0 > /proc/sys/net/ipv4/ip_forward
-      iptables -w -t nat -D POSTROUTING -s 10.1.0.0/16 -d 0.0.0.0/0 -j MASQUERADE
+      iptables -w -t nat -D POSTROUTING -s 100.64.0.0/16 -d 0.0.0.0/0 -j MASQUERADE
       ip link del dev "${name}"
   fi
 }
@@ -67,7 +67,7 @@ EOF
 
   veth1="v${ns}1"
   veth2="v${ns}2"
-  nspid=$(cat "${TEST_DIR}/ns/${ns}/PID")
+  nspid=$(< "${TEST_DIR}/ns/${ns}/PID")
 
   ip link add "${veth1}" type veth peer name "${veth2}"
   ip link set "${veth2}" netns "${nspid}"
@@ -80,8 +80,8 @@ set -e
 ip link set dev lo up
 ip link set dev "${veth2}" name eth0
 ip link set eth0 up
-ip addr add "10.1.1.10${id}/16" dev eth0
-ip route add default via 10.1.1.1
+ip addr add "100.64.1.10${id}/16" dev eth0
+ip route add default via 100.64.1.1
 ip link add localBridge${id} type bridge
 EOF
 }
@@ -101,7 +101,7 @@ teardown_clustering_netns() {
         ip link del "${veth1}"
       fi
 
-      pid="$(cat "${TEST_DIR}/ns/${ns}/PID")"
+      pid="$(< "${TEST_DIR}/ns/${ns}/PID")"
       kill -9 "${pid}" 2>/dev/null || true
 
       umount -l "${TEST_DIR}/ns/${ns}/net" >/dev/null 2>&1 || true
@@ -133,11 +133,11 @@ spawn_lxd_and_bootstrap_cluster() {
 
     cat > "${LXD_DIR}/preseed.yaml" <<EOF
 config:
-  core.https_address: 10.1.1.101:8443
+  core.https_address: 100.64.1.101:8443
 EOF
     if [ "${port}" != "" ]; then
       cat >> "${LXD_DIR}/preseed.yaml" <<EOF
-  cluster.https_address: 10.1.1.101:${port}
+  cluster.https_address: 100.64.1.101:${port}
 EOF
     fi
     cat >> "${LXD_DIR}/preseed.yaml" <<EOF
@@ -230,15 +230,15 @@ spawn_lxd_and_join_cluster() {
     # If a custom cluster port was given, we need to first set the REST
     # API address.
     if [ "${port}" != "8443" ]; then
-      lxc config set core.https_address "10.1.1.10${index}:8443"
+      lxc config set core.https_address "100.64.1.10${index}:8443"
     fi
 
     cat > "${LXD_DIR}/preseed.yaml" <<EOF
 cluster:
   enabled: true
   server_name: node${index}
-  server_address: 10.1.1.10${index}:${port}
-  cluster_address: 10.1.1.10${target}:8443
+  server_address: 100.64.1.10${index}:${port}
+  cluster_address: 100.64.1.10${target}:8443
   cluster_certificate: "$cert"
   cluster_token: ${token}
   member_config:
@@ -301,3 +301,9 @@ respawn_lxd_cluster_member() {
 
   LXD_NETNS="${ns}" respawn_lxd "${LXD_DIR}" true
 }
+
+is_uuid_v4() {
+  # Case insensitive match for a v4 UUID. The third group must start with 4, and the fourth group must start with 8, 9,
+  # a, or b. This accounts for the version and variant. See https://datatracker.ietf.org/doc/html/rfc9562#name-uuid-version-4.
+  printf '%s' "${1}" | grep --ignore-case '^[0-9a-f]\{8\}-[0-9a-f]\{4\}-4[0-9a-f]\{3\}-[89ab][0-9a-f]\{3\}-[0-9a-f]\{12\}$'
+}
