@@ -1,4 +1,5 @@
-#!/bin/sh -eu
+#!/bin/bash
+set -eu
 
 echo "Checking that functional blocks are followed by newlines..."
 
@@ -24,4 +25,4 @@ do
   fi
 done
 
-exit $exit_code
+exit "${exit_code}"
