@@ -1,6 +1,5 @@
 test_storage_volume_recover() {
   LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD_IMPORT_DIR}"
   spawn_lxd "${LXD_IMPORT_DIR}" true
 
   poolName=$(lxc profile device get default root pool)
@@ -50,13 +49,13 @@ yes
 EOF
 
   # Ensure custom storage volume has been recovered.
-  lxc storage volume show "${poolName}" vol1 | grep -q 'content_type: block'
-  lxc storage volume show "${poolName}" vol2 | grep -q 'content_type: iso'
+  lxc storage volume show "${poolName}" vol1 | grep -xF 'content_type: block'
+  lxc storage volume show "${poolName}" vol2 | grep -xF 'content_type: iso'
 
   if [ "$poolDriver" = "zfs" ]; then
     # Ensure custom storage volumes have been recovered.
-    lxc storage volume show "${poolName}" vol3 | grep -q 'content_type: filesystem'
-    lxc storage volume show "${poolName}" vol4 | grep -q 'content_type: filesystem'
+    lxc storage volume show "${poolName}" vol3 | grep -xF 'content_type: filesystem'
+    lxc storage volume show "${poolName}" vol4 | grep -xF 'content_type: filesystem'
 
     # Cleanup
     lxc storage volume delete "${poolName}" vol3
@@ -72,7 +71,6 @@ EOF
 
 test_container_recover() {
   LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD_IMPORT_DIR}"
   spawn_lxd "${LXD_IMPORT_DIR}" true
 
   (
@@ -250,6 +248,9 @@ ceph.osd.pool_name=$(lxc storage get "${poolName}" ceph.osd.pool_name)
 ceph.user.name=$(lxc storage get "${poolName}" ceph.user.name)
 "
       ;;
+      *)
+        # nothing extra config needed
+      ;;
     esac
 
     lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM instances WHERE name='c1'"
@@ -628,12 +629,12 @@ _backup_export_with_project() {
   rm -rf "${LXD_DIR}/optimized" "${LXD_DIR}/non-optimized"
 
   # Check if hyphens cause issues when creating backups
-  lxc launch testimage c1-foo -d "${SMALL_ROOT_DISK}"
+  lxc init --empty c1-foo -d "${SMALL_ROOT_DISK}"
   lxc snapshot c1-foo
 
   lxc export c1-foo "${LXD_DIR}/c1-foo.tar.gz"
 
-  lxc delete --force c1-foo
+  lxc delete c1-foo
 
   # Cleanup exported tarballs
   rm -f "${LXD_DIR}"/c*.tar.gz
@@ -649,14 +650,16 @@ test_backup_rename() {
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
-  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
+  OUTPUT="$(! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
+  if ! echo "${OUTPUT}" | grep -F "Error: Instance backup not found" ; then
     echo "invalid rename response for missing container"
     false
   fi
 
   lxc init --empty c1 -d "${SMALL_ROOT_DISK}"
 
-  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
+  OUTPUT="$(! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
+  if ! echo "${OUTPUT}" | grep -F "Error: Instance backup not found" ; then
     echo "invalid rename response for missing backup"
     false
   fi
@@ -736,6 +739,7 @@ _backup_volume_export_with_project() {
 
   # Create file on the custom volume.
   echo foo | lxc file push - c1/mnt/test
+  LXC_LOCAL='' lxc_remote exec c1 -- sync /mnt/test
 
   # Snapshot the custom volume.
   lxc storage volume set "${custom_vol_pool}" testvol user.foo=test-snap0
@@ -743,6 +747,7 @@ _backup_volume_export_with_project() {
 
   # Change the content (the snapshot will contain the old value).
   echo bar | lxc file push - c1/mnt/test
+  LXC_LOCAL='' lxc_remote exec c1 -- sync /mnt/test
 
   lxc storage volume set "${custom_vol_pool}" testvol user.foo=test-snap1
   lxc storage volume snapshot "${custom_vol_pool}" testvol test-snap1
@@ -775,10 +780,10 @@ _backup_volume_export_with_project() {
   ls -l "${LXD_DIR}/non-optimized/backup/"
   [ -f "${LXD_DIR}/non-optimized/backup/index.yaml" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume" ]
-  [ "$(cat "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
+  [ "$(< "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
   [ ! -d "${LXD_DIR}/non-optimized/backup/volume-snapshots" ]
 
-  ! grep -q -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml" || false
+  ! grep -F -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml" || false
 
   rm -rf "${LXD_DIR}/non-optimized/"*
   rm "${LXD_DIR}/testvol.tar.gz"
@@ -810,11 +815,11 @@ _backup_volume_export_with_project() {
   ls -l "${LXD_DIR}/non-optimized/backup/"
   [ -f "${LXD_DIR}/non-optimized/backup/index.yaml" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume" ]
-  [ "$(cat "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
+  [ "$(< "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0" ]
-  [  "$(cat "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0/test")" = "foo" ]
+  [  "$(< "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0/test")" = "foo" ]
 
-  grep -q -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml"
+  grep -F -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml"
 
   rm -rf "${LXD_DIR}/non-optimized/"*
 
@@ -913,7 +918,8 @@ test_backup_volume_rename_delete() {
   # Create test volume.
   lxc storage volume create "${pool}" vol1 size=32MiB
 
-  if ! lxc query -X POST /1.0/storage-pools/"${pool}"/volumes/custom/vol1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Storage volume backup not found" ; then
+  OUTPUT="$(! lxc query -X POST /1.0/storage-pools/"${pool}"/volumes/custom/vol1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
+  if ! echo "${OUTPUT}" | grep -F "Error: Storage volume backup not found" ; then
     echo "invalid rename response for missing storage volume"
     false
   fi
@@ -969,13 +975,13 @@ test_backup_instance_uuid() {
   ensure_has_localhost_remote "${LXD_ADDR}"
 
   echo "==> Checking instance UUIDs during backup operation"
-  lxc launch testimage c1 -d "${SMALL_ROOT_DISK}"
+  lxc init --empty c1 -d "${SMALL_ROOT_DISK}"
   initialUUID=$(lxc config get c1 volatile.uuid)
   initialGenerationID=$(lxc config get c1 volatile.uuid.generation)
 
   # export and import should preserve the UUID and generation UUID
   lxc export c1 "${LXD_DIR}/c1.tar.gz"
-  lxc delete -f c1
+  lxc delete c1
   lxc import "${LXD_DIR}/c1.tar.gz"
 
   newUUID=$(lxc config get c1 volatile.uuid)
@@ -986,7 +992,7 @@ test_backup_instance_uuid() {
     false
   fi
 
-  lxc delete -f c1
+  lxc delete c1
 
   # Cleanup exported tarballs
   rm -f "${LXD_DIR}"/c*.tar.gz
@@ -1084,3 +1090,151 @@ test_backup_export_import_instance_only() {
   rm "${LXD_DIR}/c1.tar.gz"
   lxc delete -f c1
 }
+
+test_backup_metadata() {
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  # Fetch the least and most recent supported backup metadata version from the range.
+  lowest_version=$(lxc query /1.0 | jq -r .environment.backup_metadata_version_range[0])
+  highest_version=$(lxc query /1.0 | jq -r .environment.backup_metadata_version_range[1])
+
+  [ "$lowest_version" = "1" ]
+  [ "$highest_version" = "2" ]
+
+  tmpDir=$(mktemp -d -p "${TEST_DIR}" metadata-XXX)
+
+  # Create an instance with one snapshot.
+  lxc init testimage c1 -d "${SMALL_ROOT_DISK}"
+  lxc snapshot c1
+
+  # Attach a disk from another pool with one snapshot.
+  custom_vol_pool="lxdtest-$(basename "${LXD_DIR}")-dir"
+  lxc storage create "${custom_vol_pool}" dir
+  lxc storage volume create "${custom_vol_pool}" foo
+  lxc storage volume snapshot "${custom_vol_pool}" foo
+  lxc storage volume attach "${custom_vol_pool}" foo c1 path=/mnt
+  [ "$(lxc query "/1.0/instances/c1" | jq '.expanded_devices | map(select(.type=="disk")) | length')" = "2" ]
+
+  lxc start c1
+  backup_yaml_path="${LXD_DIR}/containers/c1/backup.yaml"
+  cat "${backup_yaml_path}"
+
+  # Test the containers backup config contains the latest format.
+  [ "$(yq -r '.snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r .version < "${backup_yaml_path}")" = "${highest_version}" ]
+  [ "$(yq -r '.volumes | length' < "${backup_yaml_path}")" = "2" ]
+  [ "$(yq -r '.volumes.[0].snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r '.volumes.[1].snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r '.pools | length' < "${backup_yaml_path}")" = "2" ]
+
+  # Test attaching the same vol a second time doesn't increase it's appearance in the backup config.
+  lxc storage volume attach "${custom_vol_pool}" foo c1 foo2 /mnt2
+  [ "$(lxc query "/1.0/instances/c1" | jq '.expanded_devices | map(select(.type=="disk")) | length')" = "3" ]
+  [ "$(yq -r '.volumes | length' < "${backup_yaml_path}")" = "2" ]
+  [ "$(yq -r '.volumes.[0].snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r '.volumes.[1].snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r '.pools | length' < "${backup_yaml_path}")" = "2" ]
+  lxc storage volume detach "${custom_vol_pool}" foo c1 foo2
+
+  # Test custom volume changes are reflected in the config file.
+  lxc storage volume set "${custom_vol_pool}" foo user.foo bar # test volume config update
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .config."user.foo"' < "${backup_yaml_path}")" = "bar" ]
+  lxc storage volume unset "${custom_vol_pool}" foo user.foo
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .config."user.foo"' < "${backup_yaml_path}")" = "null" ]
+  [ "$(yq -r '.volumes | length' < "${backup_yaml_path}")" = "2" ]
+  [ "$(yq -r '.pools | length' < "${backup_yaml_path}")" = "2" ]
+  lxc storage volume detach "${custom_vol_pool}" foo c1 # test detaching/attaching vol and its effects on the list of vols and pools
+  [ "$(yq -r '.volumes | length' < "${backup_yaml_path}")" = "1" ]
+  [ "$(yq -r '.pools | length' < "${backup_yaml_path}")" = "1" ]
+  lxc storage volume attach "${custom_vol_pool}" foo c1 path=/mnt
+  [ "$(yq -r '.volumes | length' < "${backup_yaml_path}")" = "2" ]
+  [ "$(yq -r '.pools | length' < "${backup_yaml_path}")" = "2" ]
+
+  # Test custom volume snapshots changes are reflected in the config file.
+  lxc storage volume snapshot "${custom_vol_pool}" foo # test snapshot creation
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots | length' < "${backup_yaml_path}")" = "2" ]
+  lxc storage volume rm "${custom_vol_pool}" foo/snap1
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots | length' < "${backup_yaml_path}")" = "1" ]
+  lxc storage volume rename "${custom_vol_pool}" foo/snap0 foo/snap00 # test snapshot rename
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap00") | .name' < "${backup_yaml_path}")" = "snap00" ]
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap0") | .name' < "${backup_yaml_path}")" = "" ]
+  lxc storage volume rename "${custom_vol_pool}" foo/snap00 foo/snap0
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap0") | .name' < "${backup_yaml_path}")" = "snap0" ]
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap00") | .name' < "${backup_yaml_path}")" = "" ]
+  lxc storage volume set "${custom_vol_pool}" foo/snap0 --property description bar # test snapshot update (only description can be updated on snaps)
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap0") | .description' < "${backup_yaml_path}")" = "bar" ]
+  lxc storage volume unset "${custom_vol_pool}" foo/snap0 --property description
+  [ "$(yq -r '.volumes.[] | select(.name == "foo" and .pool == "'"${custom_vol_pool}"'") | .snapshots.[] | select(.name == "snap0") | .description' < "${backup_yaml_path}")" = "" ]
+
+  lxc stop -f c1
+
+  # Export the instance without setting an export version.
+  # The server should implicitly pick its latest supported version.
+  lxc export c1 "${tmpDir}/c1.tar.gz"
+  tar -xzf "${tmpDir}/c1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
+
+  cat "${tmpDir}/backup/index.yaml"
+  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "${highest_version}" ]
+  [ "$(yq '.config.volumes | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq '.config.volumes.[0].snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq '.config.pools | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+
+  rm -rf "${tmpDir}/backup" "${tmpDir}/c1.tar.gz"
+
+  # Export the instance using the specified lowest export version.
+  # The server should used the provided version instead of its default.
+  lxc export c1 "${tmpDir}/c1.tar.gz" --export-version "${lowest_version}"
+  tar -xzf "${tmpDir}/c1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
+
+  cat "${tmpDir}/backup/index.yaml"
+  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
+  [ "$(yq .config.container < "${tmpDir}/backup/index.yaml")" != "null" ]
+  [ "$(yq .config.pool < "${tmpDir}/backup/index.yaml")" != "null" ]
+  [ "$(yq .config.volume < "${tmpDir}/backup/index.yaml")" != "null" ]
+  [ "$(yq '.config.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+
+  rm -rf "${tmpDir}/backup" "${tmpDir}/c1.tar.gz"
+  lxc delete -f c1
+
+  # Create a custom storage volume with one snapshot.
+  poolName=$(lxc profile device get default root pool)
+  lxc storage volume create "${poolName}" vol1 size=32MiB
+  lxc storage volume snapshot "${poolName}" vol1
+
+  # Export the custom storage volume without setting an export version.
+  # The server should implicitly pick its latest supported version.
+  lxc storage volume export "${poolName}" vol1 "${tmpDir}/vol1.tar.gz"
+  tar -xzf "${tmpDir}/vol1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
+
+  cat "${tmpDir}/backup/index.yaml"
+  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "${highest_version}" ]
+  [ "$(yq .config.instance < "${tmpDir}/backup/index.yaml")" = "null" ]
+  [ "$(yq '.config.volumes | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq '.config.volumes.[0].snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq '.config.pools | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+
+  rm -rf "${tmpDir}/backup" "${tmpDir}/vol1.tar.gz"
+
+  # Export the custom storage volume using the specified lowest export version.
+  # The server should used the provided version instead of its default.
+  lxc storage volume export "${poolName}" vol1 "${tmpDir}/vol1.tar.gz" --export-version "${lowest_version}"
+  tar -xzf "${tmpDir}/vol1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
+
+  cat "${tmpDir}/backup/index.yaml"
+  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
+  [ "$(yq .config.container < "${tmpDir}/backup/index.yaml")" = "null" ]
+  [ "$(yq .config.volume < "${tmpDir}/backup/index.yaml")" != "null" ]
+  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
+
+  rm -rf "${tmpDir}/backup" "${tmpDir}/vol1.tar.gz"
+  lxc storage volume delete "${poolName}" vol1
+  lxc storage volume delete "${custom_vol_pool}" foo
+  lxc storage delete "${custom_vol_pool}"
+
+  rm -rf "${tmpDir}"
+}
