@@ -14,6 +14,10 @@ test_basic_usage() {
   lxc image show "${sum}"
   lxc image alias create a/b/ "${sum}"
 
+  echo "Test using alias with slashes"
+  lxc init a/b/ c1 -d "${SMALL_ROOT_DISK}"
+  lxc delete c1
+
   # Ensure aliased image won't launch with vm flag set
   ! lxc launch a/b/ --vm || false
 
@@ -22,24 +26,24 @@ test_basic_usage() {
   # Test alias list filtering
   lxc image alias create foo "${sum}"
   lxc image alias create bar "${sum}"
-  lxc image alias list local: | grep -q foo
-  lxc image alias list local: | grep -q bar
-  lxc image alias list local: foo | grep -q -v bar
-  lxc image alias list local: "${sum}" | grep -q foo
-  lxc image alias list local: non-existent | grep -q -v non-existent
+  lxc image alias list local: | grep -wF foo
+  lxc image alias list local: | grep -wF bar
+  !  lxc image alias list local: foo | grep -wF bar || false
+  lxc image alias list local: "${sum}" | grep -wF foo
+  ! lxc image alias list local: non-existent | grep -wF non-existent || false
   lxc image alias delete foo
   lxc image alias delete bar
 
   lxc image alias create foo "${sum}"
   lxc image alias rename foo bar
-  lxc image alias list | grep -qv foo  # the old name is gone
+  ! lxc image alias list | grep -wF foo || false  # the old name is gone
   lxc image alias delete bar
 
   # Test image list output formats (table & json)
-  lxc image list --format table | grep -q testimage
+  lxc image list --format table | grep -wF testimage
   lxc image list --format json \
     | jq '.[]|select(.alias[0].name="testimage")' \
-    | grep -q '"name": "testimage"'
+    | grep -F '"name": "testimage"'
 
   # Test image delete
   lxc image delete testimage
@@ -51,8 +55,8 @@ test_basic_usage() {
   # Re-import the image
   mv "${LXD_DIR}/${sum}.tar.xz" "${LXD_DIR}/testimage.tar.xz"
   lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage user.foo=bar --public
-  lxc image show testimage | grep -qF "user.foo: bar"
-  lxc image show testimage | grep -qF "public: true"
+  [ "$(lxc image get-property testimage user.foo)" = "bar" ]
+  lxc image show testimage | grep -xF "public: true"
   lxc image delete testimage
   lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage
   rm "${LXD_DIR}/testimage.tar.xz"
@@ -104,6 +108,31 @@ test_basic_usage() {
   lxc list | grep foo | grep STOPPED
   lxc list fo | grep foo | grep STOPPED
 
+  echo "Invalid container names"
+  ! lxc init --empty ".." || false
+  # Escaping `\` multiple times due to `lxc` wrapper script munging the first layer
+  ! lxc init --empty "\\\\" || false
+  ! lxc init --empty "/" || false
+  ! lxc init --empty ";" || false
+
+  echo "Too small containers"
+  ! lxc init --empty c1 -c limits.memory=0 || false
+  ! lxc init --empty c1 -c limits.memory=0% || false
+
+  echo "Containers with snapshots"
+  lxc init testimage c1 -d "${SMALL_ROOT_DISK}"
+  lxc snapshot c1
+  # Invalid snapshot names
+  ! lxc snapshot c1 ".." || false
+  # Escaping `\` multiple times due to `lxc` wrapper script munging the first layer
+  ! lxc snapshot c1 "\\\\" || false
+  ! lxc snapshot c1 "/" || false
+  [ "$(lxc list -f csv -c S c1)" = "1" ]
+  lxc start c1
+  lxc snapshot c1
+  [ "$(lxc list -f csv -c S c1)" = "2" ]
+  lxc delete --force c1
+
   # Test list json format
   lxc list --format json | jq '.[]|select(.name="foo")' | grep '"name": "foo"'
 
@@ -111,7 +140,7 @@ test_basic_usage() {
   ! lxc list --columns=nsp --fast || false
 
   # Check volatile.apply_template is correct.
-  lxc config get foo volatile.apply_template | grep create
+  [ "$(lxc config get foo volatile.apply_template)" = "create" ]
 
   # Start the instance to clear apply_template.
   lxc start foo
@@ -121,14 +150,14 @@ test_basic_usage() {
   lxc move foo bar
 
   # Check volatile.apply_template is altered during rename.
-  lxc config get bar volatile.apply_template | grep rename
+  [ "$(lxc config get bar volatile.apply_template)" = "rename" ]
 
-  lxc list | grep -v foo
-  lxc list | grep bar
+  [ "$(lxc list -c n | grep -F foo)" = "" ]
+  [ "$(lxc list -c n | grep -F bar)" != "" ]
 
   lxc rename bar foo
-  lxc list | grep -v bar
-  lxc list | grep foo
+  [ "$(lxc list -c n | grep -F bar)" = "" ]
+  [ "$(lxc list -c n | grep -F foo)" != "" ]
   lxc rename foo bar
 
   # Test container copy
@@ -144,11 +173,11 @@ test_basic_usage() {
   # Test unprivileged container publish
   lxc publish bar --alias=foo-image prop1=val1
   lxc image show foo-image | grep val1
-  CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
+  ! CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" || false
   lxc image delete foo-image
 
   # Test container publish with existing alias
-  lxc publish bar --alias=foo-image --alias=foo-image2
+  lxc publish bar --alias=foo-image --alias=bar-image2
   lxc launch testimage baz
   # change the container filesystem so the resulting image is different
   lxc exec baz -- touch /somefile
@@ -158,33 +187,33 @@ test_basic_usage() {
   # publishing another image with same alias and '--reuse' flag should success
   lxc publish baz --alias=foo-image --reuse
   fooImage=$(lxc image list -cF -fcsv foo-image)
-  fooImage2=$(lxc image list -cF -fcsv foo-image2)
+  barImage2=$(lxc image list -cF -fcsv bar-image2)
   lxc delete baz
-  lxc image delete foo-image foo-image2
+  lxc image delete foo-image bar-image2
 
-  # the first image should have foo-image2 alias and the second imgae foo-image alias
-  if [ "$fooImage" = "$fooImage2" ]; then
-    echo "foo-image and foo-image2 aliases should be assigned to two different images"
+  # the first image should have bar-image2 alias and the second imgae foo-image alias
+  if [ "$fooImage" = "$barImage2" ]; then
+    echo "foo-image and bar-image2 aliases should be assigned to two different images"
     false
   fi
 
 
   # Test container publish with existing alias
-  lxc publish bar --alias=foo-image --alias=foo-image2
+  lxc publish bar --alias=foo-image --alias=bar-image2
   lxc launch testimage baz
   # change the container filesystem so the resulting image is different
   lxc exec baz -- touch /somefile
   lxc stop baz --force
   # publishing another image with same aliases
-  lxc publish baz --alias=foo-image --alias=foo-image2 --reuse
+  lxc publish baz --alias=foo-image --alias=bar-image2 --reuse
   fooImage=$(lxc image list -cF -fcsv foo-image)
-  fooImage2=$(lxc image list -cF -fcsv foo-image2)
+  barImage2=$(lxc image list -cF -fcsv bar-image2)
   lxc delete baz
   lxc image delete foo-image
 
-  # the second image should have foo-image and foo-image2 aliases and the first one should be removed
-  if [ "$fooImage" != "$fooImage2" ]; then
-    echo "foo-image and foo-image2 aliases should be assigned to the same image"
+  # the second image should have foo-image and bar-image2 aliases and the first one should be removed
+  if [ "$fooImage" != "$barImage2" ]; then
+    echo "foo-image and bar-image2 aliases should be assigned to the same image"
     false
   fi
 
@@ -192,7 +221,7 @@ test_basic_usage() {
   # Test image compression on publish
   lxc publish bar --alias=foo-image-compressed --compression=bzip2 prop=val1
   lxc image show foo-image-compressed | grep val1
-  CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
+  ! CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" || false
   lxc image delete foo-image-compressed
 
   # Test compression options
@@ -205,7 +234,7 @@ test_basic_usage() {
   lxc init testimage barpriv -p default -p priv
   lxc publish barpriv --alias=foo-image prop1=val1
   lxc image show foo-image | grep val1
-  CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
+  ! CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" || false
   lxc image delete foo-image
   lxc delete barpriv
   lxc profile delete priv
@@ -224,9 +253,9 @@ test_basic_usage() {
   fi
 
   # Test public images
-  lxc publish --public bar --alias=foo-image2
+  lxc publish --public bar --alias=bar-image2
   CERTNAME="client3" my_curl -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/"
-  lxc image delete foo-image2
+  lxc image delete bar-image2
 
   # Test invalid instance names
   ! lxc init testimage -abc || false
@@ -240,7 +269,7 @@ test_basic_usage() {
   lxc snapshot bar
   lxc publish bar/snap0 --alias foo
   lxc init foo bar2
-  lxc list | grep bar2
+  lxc list -c n | grep bar2
   lxc delete bar2
   lxc image delete foo
 
@@ -265,7 +294,7 @@ test_basic_usage() {
   echo "  cp: list" >> "${LXD_CONF}/config.yml"
   [ "$(lxc ls)" = "$(lxc cp)" ]
   #   7. User-defined aliases override commands and don't recurse
-  lxc init testimage foo
+  lxc init --empty foo
   LXC_CONFIG_SHOW=$(lxc config show foo --expanded)
   echo "  config show: config show --expanded" >> "${LXD_CONF}/config.yml"
   [ "$(lxc config show foo)" = "$LXC_CONFIG_SHOW" ]
@@ -286,12 +315,12 @@ test_basic_usage() {
   lxc delete -f "${RDNAME}"
 
   # Test "nonetype" container creation
-  wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers" \
+  wait_for "${LXD_ADDR}" my_curl -X POST --fail-with-body -H 'Content-Type: application/json' "https://${LXD_ADDR}/1.0/containers" \
         -d "{\"name\":\"nonetype\",\"source\":{\"type\":\"none\"}}"
   lxc delete nonetype
 
   # Test "nonetype" container creation with an LXC config
-  wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers" \
+  wait_for "${LXD_ADDR}" my_curl -X POST --fail-with-body -H 'Content-Type: application/json' "https://${LXD_ADDR}/1.0/containers" \
         -d "{\"name\":\"configtest\",\"config\":{\"raw.lxc\":\"lxc.hook.clone=/bin/true\"},\"source\":{\"type\":\"none\"}}"
   # shellcheck disable=SC2102
   [ "$(my_curl "https://${LXD_ADDR}/1.0/containers/configtest" | jq -r .metadata.config[\"raw.lxc\"])" = "lxc.hook.clone=/bin/true" ]
@@ -299,18 +328,17 @@ test_basic_usage() {
 
   # Test activateifneeded/shutdown
   LXD_ACTIVATION_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD_ACTIVATION_DIR}"
   spawn_lxd "${LXD_ACTIVATION_DIR}" true
   (
     set -e
     # shellcheck disable=SC2030
     LXD_DIR=${LXD_ACTIVATION_DIR}
     ensure_import_testimage
-    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has core.https_address set, activating..."
+    lxd activateifneeded --debug 2>&1 | grep -F "Daemon has core.https_address set, activating..."
     lxc config unset core.https_address --force-local
-    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
+    ! lxd activateifneeded --debug 2>&1 | grep -F "activating..." || false
     lxc init testimage autostart --force-local
-    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
+    ! lxd activateifneeded --debug 2>&1 | grep -F "activating..." || false
     lxc config set autostart boot.autostart true --force-local
 
     # Restart the daemon, this forces the global database to be dumped to disk.
@@ -318,28 +346,42 @@ test_basic_usage() {
     respawn_lxd "${LXD_DIR}" true
     lxc stop --force autostart --force-local
 
-    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has auto-started instances, activating..."
+    lxd activateifneeded --debug 2>&1 | grep -F "Daemon has auto-started instances, activating..."
 
     lxc config unset autostart boot.autostart --force-local
-    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
+
+    # Restart the daemon, this forces the global database to be dumped to disk.
+    shutdown_lxd "${LXD_DIR}"
+    respawn_lxd "${LXD_DIR}" true
+
+    ! lxd activateifneeded --debug 2>&1 | grep -F "activating..." || false
 
     lxc start autostart --force-local
-    PID=$(lxc info autostart --force-local | awk '/^PID:/ {print $2}')
+    PID="$(lxc list --force-local -f csv -c p autostart)"
     shutdown_lxd "${LXD_DIR}"
-    [ -d "/proc/${PID}" ] && false
 
-    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has auto-started instances, activating..."
+    # Stopping LXD should also stop the instances
+    ! [ -d "/proc/${PID}" ] || false
+
+    # `lxd activateifneeded` will error out due to LXD being stopped and not having any Unix socket to wake it up
+    # but it should also log something about the activation status
+    OUTPUT="$(! lxd activateifneeded --debug 2>&1 || false)"
+    echo "${OUTPUT}" | grep -F "Daemon has auto-started instances, activating..."
 
     # shellcheck disable=SC2031
     respawn_lxd "${LXD_DIR}" true
 
-    lxc list --force-local autostart | grep -q RUNNING
+    lxc list --force-local autostart | grep -wF RUNNING
 
     # Check for scheduled instance snapshots
     lxc stop --force autostart --force-local
     lxc config set autostart snapshots.schedule "* * * * *" --force-local
     shutdown_lxd "${LXD_DIR}"
-    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has scheduled instance snapshots, activating..."
+
+    # `lxd activateifneeded` will error out due to LXD being stopped and not having any Unix socket to wake it up
+    # but it should also log something about the activation status
+    OUTPUT="$(! lxd activateifneeded --debug 2>&1 || false)"
+    echo "${OUTPUT}" | grep -F "Daemon has scheduled instance snapshots, activating..."
 
     # shellcheck disable=SC2031
     respawn_lxd "${LXD_DIR}" true
@@ -352,7 +394,7 @@ test_basic_usage() {
     lxc storage volume create "${storage_pool}" vol --force-local
 
     shutdown_lxd "${LXD_DIR}"
-    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
+    ! lxd activateifneeded --debug 2>&1 | grep -F "activating..." || false
 
     # shellcheck disable=SC2031
     respawn_lxd "${LXD_DIR}" true
@@ -360,7 +402,11 @@ test_basic_usage() {
     lxc storage volume set "${storage_pool}" vol snapshots.schedule="* * * * *" --force-local
 
     shutdown_lxd "${LXD_DIR}"
-    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has scheduled volume snapshots, activating..."
+
+    # `lxd activateifneeded` will error out due to LXD being stopped and not having any Unix socket to wake it up
+    # but it should also log something about the activation status
+    OUTPUT="$(! lxd activateifneeded --debug 2>&1 || false)"
+    echo "${OUTPUT}" | grep -F "Daemon has scheduled volume snapshots, activating..."
 
     # shellcheck disable=SC2031
     respawn_lxd "${LXD_DIR}" true
@@ -377,7 +423,7 @@ test_basic_usage() {
   lxc list | grep foo | grep RUNNING
   lxc stop foo --force
 
-  if lxc info | grep -q 'unpriv_binfmt: "true"'; then
+  if lxc info | grep -F 'unpriv_binfmt: "true"'; then
     # Test binfmt_misc support
     lxc start foo
     lxc exec foo -- mount -t binfmt_misc none /proc/sys/fs/binfmt_misc
@@ -406,7 +452,7 @@ test_basic_usage() {
   lxc start foo
 
   # Test instance types
-  lxc launch testimage test-limits -t c0.5-m0.2
+  lxc init --empty test-limits -t c0.5-m0.2 -d "${SMALL_ROOT_DISK}"
   [ "$(lxc config get test-limits limits.cpu)" = "1" ]
   [ "$(lxc config get test-limits limits.cpu.allowance)" = "50%" ]
   [ "$(lxc config get test-limits limits.memory)" = "204MiB" ]
@@ -414,9 +460,9 @@ test_basic_usage() {
 
   # Test last_used_at field is working properly
   lxc init testimage last-used-at-test
-  lxc list last-used-at-test  --format json | jq -r '.[].last_used_at' | grep '1970-01-01T00:00:00Z'
+  [ "$(lxc list last-used-at-test --format json | jq -r '.[].last_used_at')" = "1970-01-01T00:00:00Z" ]
   lxc start last-used-at-test
-  lxc list last-used-at-test  --format json | jq -r '.[].last_used_at' | grep -v '1970-01-01T00:00:00Z'
+  [ "$(lxc list last-used-at-test --format json | jq -r '.[].last_used_at')" != "1970-01-01T00:00:00Z" ]
   lxc delete last-used-at-test --force
 
   # Test user, group and cwd
@@ -446,7 +492,7 @@ test_basic_usage() {
   lxc profile delete clash
 
   # check that we can get the return code for a non- wait-for-websocket exec
-  op=$(my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/exec" -d '{"command": ["echo", "test"], "environment": {}, "wait-for-websocket": false, "interactive": false}' | jq -r .operation)
+  op=$(my_curl -X POST --fail-with-body -H 'Content-Type: application/json' "https://${LXD_ADDR}/1.0/containers/foo/exec" -d '{"command": ["echo", "test"], "environment": {}, "wait-for-websocket": false, "interactive": false}' | jq -r .operation)
   [ "$(my_curl "https://${LXD_ADDR}${op}/wait" | jq -r .metadata.metadata.return)" != "null" ]
 
   # test file transfer
@@ -462,7 +508,7 @@ test_basic_usage() {
 
   # test lxc file edit doesn't change target file's owner and permissions
   echo "content" | lxc file push - foo/tmp/edit_test
-  lxc exec foo -- chown 55.55 /tmp/edit_test
+  lxc exec foo -- chown 55:55 /tmp/edit_test
   lxc exec foo -- chmod 555 /tmp/edit_test
   echo "new content" | lxc file edit foo/tmp/edit_test
   [ "$(lxc exec foo -- cat /tmp/edit_test)" = "new content" ]
@@ -484,8 +530,7 @@ test_basic_usage() {
 
   # FIXME: make this backend agnostic
   if [ "$lxd_backend" = "dir" ]; then
-    content=$(cat "${LXD_DIR}/containers/foo/rootfs/tmp/foo")
-    [ "${content}" = "foo" ]
+    [ "$(< "${LXD_DIR}/containers/foo/rootfs/tmp/foo")" = "foo" ]
   fi
 
   lxc launch testimage deleterunning
@@ -509,13 +554,13 @@ test_basic_usage() {
 
     if [ "${MAJOR}" -gt "1" ] || { [ "${MAJOR}" = "1" ] && [ "${MINOR}" -ge "2" ]; }; then
       aa_namespace="lxd-lxd-apparmor-test_<$(echo "${LXD_DIR}" | sed -e 's/\//-/g' -e 's/^.//')>"
-      aa-status | grep -q ":${aa_namespace}:unconfined" || aa-status | grep -qF ":${aa_namespace}://unconfined"
+      aa-status | grep -F ":${aa_namespace}:unconfined" || aa-status | grep -F ":${aa_namespace}://unconfined"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -qF ":${aa_namespace}:" || false
+      ! aa-status | grep -F ":${aa_namespace}:" || false
     else
       aa-status | grep "lxd-lxd-apparmor-test_<${LXD_DIR}>"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -qF "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
+      ! aa-status | grep -F "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
     fi
     lxc delete lxd-apparmor-test
     [ ! -f "${LXD_DIR}/security/apparmor/profiles/lxd-lxd-apparmor-test" ]
@@ -525,12 +570,12 @@ test_basic_usage() {
 
   if [ "$(awk '/^Seccomp:/ {print $2}' "/proc/self/status")" -eq "0" ]; then
     lxc launch testimage lxd-seccomp-test
-    init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+    init="$(lxc list -f csv -c p lxd-seccomp-test)"
     [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "2" ]
     lxc stop --force lxd-seccomp-test
     lxc config set lxd-seccomp-test security.syscalls.deny_default false
     lxc start lxd-seccomp-test
-    init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+    init="$(lxc list -f csv -c p lxd-seccomp-test)"
     [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "0" ]
     lxc delete --force lxd-seccomp-test
   else
@@ -565,9 +610,9 @@ test_basic_usage() {
   lxc publish --force c3 --alias=image3
   # Delete multiple images with lxc delete and confirm they're deleted
   lxc image delete local:image1 local:image2 local:image3
-  ! lxc image list | grep -q image1 || false
-  ! lxc image list | grep -q image2 || false
-  ! lxc image list | grep -q image3 || false
+  ! lxc image list | grep -wF image1 || false
+  ! lxc image list | grep -wF image2 || false
+  ! lxc image list | grep -wF image3 || false
   # Cleanup the containers
   lxc delete --force c1 c2 c3
 
@@ -592,13 +637,13 @@ test_basic_usage() {
   lxc delete --force c1 c2
 
   # Ephemeral
-  lxc launch testimage foo -e
-  OLD_INIT=$(lxc info foo | awk '/^PID:/ {print $2}')
+  lxc launch testimage foo --ephemeral
+  OLD_INIT="$(lxc list -f csv -c p foo)"
 
   REBOOTED="false"
 
   for _ in $(seq 60); do
-    NEW_INIT=$(lxc info foo | awk '/^PID:/ {print $2}' || true)
+    NEW_INIT="$(lxc list -f csv -c p foo)"
 
     # If init process is running, check if is old or new process.
     if [ -n "${NEW_INIT}" ]; then
@@ -620,27 +665,27 @@ test_basic_usage() {
 
   lxc restart -f foo
   lxc stop foo --force
-  ! lxc list | grep -q foo || false
+  ! lxc list | grep -wF foo || false
 
   # Test renaming/deletion of the default profile
   ! lxc profile rename default foobar || false
   ! lxc profile delete default || false
 
-  lxc init testimage c1
+  lxc init --empty c1
   result="$(! lxc config device override c1 root pool=bla 2>&1)"
   if ! echo "${result}" | grep "Error: Cannot update root disk device pool name"; then
     echo "Should fail device override because root disk device storage pool cannot be changed."
     false
   fi
 
-  lxc rm -f c1
+  lxc delete -f c1
 
   # Should fail to override root device storage pool when the new pool does not exist.
   ! lxc init testimage c1 -d root,pool=bla || false
 
   # Should succeed in overriding root device storage pool when the pool does exist and the override occurs at create time.
   lxc storage create bla dir
-  lxc init testimage c1 -d root,pool=bla
+  lxc init --empty c1 -d root,pool=bla
   lxc config show c1 --expanded | grep -Pz '  root:\n    path: /\n    pool: bla\n    type: disk\n'
 
   lxc storage volume create bla vol1
@@ -654,7 +699,7 @@ test_basic_usage() {
     false
   fi
 
-  lxc rm -f c1
+  lxc delete -f c1
   lxc storage volume delete bla vol1
   lxc storage volume delete bla vol2
   lxc storage delete bla
@@ -684,7 +729,7 @@ test_basic_usage() {
   # Test rebuilding an instance with an empty file system.
   lxc init testimage c1
   lxc rebuild c1 --empty
-  ! lxc config show c1 | grep -q 'image.*' || false
+  ! lxc config show c1 | grep -F 'image.' || false
   lxc delete c1 -f
 
   # Test assigning an empty profile (with no root disk device) to an instance.
@@ -715,21 +760,35 @@ EOF
   lxc profile delete foo
 
   # Multiple ephemeral instances delete
-  lxc launch testimage c1
-  lxc launch testimage c2
-  lxc launch testimage c3
+  lxc launch testimage c1 --ephemeral
+  lxc launch testimage c2 --ephemeral
+  lxc launch testimage c3 --ephemeral
 
+  lxc stop -f c1 c2 c3
+  [ "$(lxc list -f csv -c n)" = "" ]
+
+  # Cleanup
   fingerprint="$(lxc config trust ls --format csv | cut -d, -f4)"
   lxc config trust remove "${fingerprint}"
-  lxc delete -f c1 c2 c3
-  remaining_instances="$(lxc list --format csv)"
-  [ -z "${remaining_instances}" ]
+}
+
+test_basic_version() {
+  # XXX: add `fuidshift` to the list
+  for bin in lxc lxd lxd-agent lxd-benchmark lxd-migrate lxd-user; do
+    "${bin}" --version
+    "${bin}" --help
+  done
 }
 
 test_server_info() {
   # Ensure server always reports support for containers.
   lxc query /1.0 | jq -e '.environment.instance_types | contains(["container"])'
 
+  # Ensure server reports support for VMs if it should test them.
+  if [ "${LXD_VM_TESTS:-0}" = "1" ]; then
+    lxc query /1.0 | jq -e '.environment.instance_types | contains(["virtual-machine"])'
+  fi
+
   # Ensure the version number has the format (X.Y.Z for LTSes and X.Y otherwise)
   if lxc query /1.0 | jq -e '.environment.server_lts == true'; then
     lxc query /1.0 | jq -re '.environment.server_version' | grep -E '[0-9]+\.[0-9]+\.[0-9]+'
