@@ -10,14 +10,21 @@ test_container_devices_disk() {
   _container_devices_disk_cephfs
   _container_devices_disk_socket
   _container_devices_disk_char
+  _container_devices_disk_patch
 
-  lxc delete -f foo
+  lxc delete foo
 }
 
 _container_devices_disk_shift() {
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
 
+  # `tmpfs` does not support idmapped mounts on kernels older than 6.3
+  if [ "${LXD_TMPFS:-0}" = "1" ] && ! runsMinimumKernel 6.3; then
+    echo "==> SKIP: tmpfs (LXD_TMPFS=${LXD_TMPFS}) idmapped mount requires a kernel >= 6.3"
+    return
+  fi
+
   if [ -n "${LXD_IDMAPPED_MOUNTS_DISABLE:-}" ]; then
     return
   fi
@@ -50,8 +57,7 @@ _container_devices_disk_shift() {
   lxc config device add foo idmapped_mount disk source="${TEST_DIR}/shift-source" path=/mnt shift=true
   [ "$(lxc exec foo -- stat /mnt/a -c '%u:%g')" = "123:456" ]
 
-  lxc stop foo -f
-  lxc start foo
+  lxc restart foo -f
   [ "$(lxc exec foo -- stat /mnt/a -c '%u:%g')" = "123:456" ]
   lxc config device remove foo idmapped_mount
   lxc stop foo -f
@@ -179,3 +185,28 @@ _container_devices_disk_char() {
   lxc config device remove foo char
   lxc stop foo -f
 }
+
+_container_devices_disk_patch() {
+  lxc init c1 --empty
+
+  # Ensure no devices are present.
+  [ "$(lxc config device list c1 | awk 'NF' | wc -l)" -eq 0 ]
+
+  # Ensure a new device is added.
+  lxc query -X PATCH /1.0/instances/c1 -d '{\"devices\": {\"tmp\": {\"type\": \"disk\", \"source\": \"/etc/os-release\", \"path\": \"/tmp/release\"}}}'
+  [ "$(lxc config device list c1 | awk 'NF' | wc -l)" -eq 1 ]
+
+  # Ensure the device is updated.
+  lxc query -X PATCH /1.0/instances/c1 -d '{\"devices\": {\"tmp\": {\"type\": \"disk\", \"source\": \"/etc/os-release\", \"path\": \"/tmp/release-new\"}}}'
+  [ "$(lxc config device get c1 tmp path)" = "/tmp/release-new" ]
+
+  # Ensure the device is not removed when patching with an empty devices object.
+  lxc query -X PATCH /1.0/instances/c1 -d '{\"devices\": {}}'
+  [ "$(lxc config device list c1 | awk 'NF' | wc -l)" -eq 1 ]
+
+  # Ensure the device is removed when patching with a null device.
+  lxc query -X PATCH /1.0/instances/c1 -d '{\"devices\": {\"tmp\": null }}'
+  [ "$(lxc config device list c1 | awk 'NF' | wc -l)" -eq 0 ]
+
+  lxc delete --force c1
+}
