@@ -1,4 +1,9 @@
 test_container_devices_nic_bridged() {
+  if uname -r | grep -- -kvm$; then
+    echo "==> SKIP: the -kvm kernel flavor is missing CONFIG_NET_SCH_HTB which is required for 'tc qdisc htb'"
+    return
+  fi
+
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
@@ -31,16 +36,30 @@ test_container_devices_nic_bridged() {
   lxc network set "${brName}" ipv6.address 2001:db8::1/64
   lxc network set "${brName}" ipv4.routes 192.0.3.0/24
   lxc network set "${brName}" ipv6.routes 2001:db8::/64
-  [ "$(cat /sys/class/net/${brName}/address)" = "00:11:22:33:44:55" ]
+  [ "$(cat "/sys/class/net/${brName}/address")" = "00:11:22:33:44:55" ]
 
   # Record how many nics we started with.
   startNicCount=$(find /sys/class/net | wc -l)
 
-  # Test pre-launch profile config is applied at launch
+  # Test device renaming without "eth0" device applied by profile.
   lxc profile copy default "${ctName}"
+  lxc profile device remove "${ctName}" eth0
+  lxc init --empty "${ctName}" -d "${SMALL_ROOT_DISK}" -p "${ctName}"
+  lxc config device add "${ctName}" eth0 nic network="${brName}" name=eth0
+  [ "$(lxc config device get "${ctName}" eth0 name)" = "eth0" ]
+  [ "$(lxc config device get "${ctName}" eth0 network)" = "${brName}" ]
+  lxc config show -e "${ctName}" | sed '/^devices:/,/type:/ s/eth0/eth1/' | lxc config edit "${ctName}" # Rename added "eth0" to "eth1"
+  ! lxc config device get "${ctName}" eth0 name || false
+  [ "$(lxc config device get "${ctName}" eth1 name)" = "eth1" ]
+  [ "$(lxc config device get "${ctName}" eth1 network)" = "${brName}" ]
+  # Cleanup for remaining tests.
+  lxc delete "${ctName}" -f
+  lxc profile delete "${ctName}"
 
-  # Modify profile nictype and parent in atomic operation to ensure validation passes.
-  lxc profile show "${ctName}" | sed  "s/nictype: p2p/nictype: bridged\\n    parent: ${brName}/" | lxc profile edit "${ctName}"
+  echo "Test pre-launch profile config is applied at launch"
+
+  # Create profile for new containers by atomically modifying nictype and parent to ensure validation passes.
+  lxc profile show default | sed  "s/nictype: p2p/nictype: bridged\\n    parent: ${brName}/" | lxc profile create "${ctName}"
 
   lxc profile device set "${ctName}" eth0 ipv4.routes "192.0.2.1${ipRand}/32"
   lxc profile device set "${ctName}" eth0 ipv6.routes "2001:db8::1${ipRand}/128"
@@ -52,100 +71,100 @@ test_container_devices_nic_bridged() {
   lxc profile device set "${ctName}" eth0 queue.tx.length "1200"
   lxc profile device set "${ctName}" eth0 hwaddr "${ctMAC}"
 
-  lxc init testimage "${ctName}" -p "${ctName}"
+  lxc init testimage "${ctName}" -d "${SMALL_ROOT_DISK}" -p "${ctName}"
 
   # Check that adding another NIC to the same network fails because it triggers duplicate instance DNS name checks.
   # Because this would effectively cause 2 NICs with the same instance name to be connected to the same network.
-  ! lxc config device add "${ctName}" eth1 nic network=${brName} || false
+  ! lxc config device add "${ctName}" eth1 nic network="${brName}" || false
 
   # Test device name validation (use vlan=1 to avoid trigger instance DNS name conflict detection).
-  lxc config device add "${ctName}" 127.0.0.1 nic network=${brName} vlan=1
+  lxc config device add "${ctName}" 127.0.0.1 nic network="${brName}" vlan=1
   lxc config device remove "${ctName}" 127.0.0.1
-  lxc config device add "${ctName}" ::1 nic network=${brName} vlan=1
+  lxc config device add "${ctName}" ::1 nic network="${brName}" vlan=1
   lxc config device remove "${ctName}" ::1
-  lxc config device add "${ctName}" _valid-name nic network=${brName} vlan=1
+  lxc config device add "${ctName}" _valid-name nic network="${brName}" vlan=1
   lxc config device remove "${ctName}" _valid-name
-  lxc config device add "${ctName}" /foo nic network=${brName} vlan=1
+  lxc config device add "${ctName}" /foo nic network="${brName}" vlan=1
   lxc config device remove "${ctName}" /foo
-  ! lxc config device add "${ctName}" .invalid nic network=${brName} vlan=1 || false
-  ! lxc config device add "${ctName}" ./invalid nic network=${brName} vlan=1 || false
-  ! lxc config device add "${ctName}" ../invalid nic network=${brName} vlan=1 || false
+  ! lxc config device add "${ctName}" .invalid nic network="${brName}" vlan=1 || false
+  ! lxc config device add "${ctName}" ./invalid nic network="${brName}" vlan=1 || false
+  ! lxc config device add "${ctName}" ../invalid nic network="${brName}" vlan=1 || false
 
   # Start instance.
   lxc start "${ctName}"
 
   # Check profile routes are applied on boot.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.1${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.1${ipRand}" ; then
     echo "ipv4.routes invalid"
     false
   fi
-  if ! ip -6 r list dev "${brName}" | grep "2001:db8::1${ipRand}" ; then
+  if ! ip -6 r list dev "${brName}" | grep -F "2001:db8::1${ipRand}" ; then
     echo "ipv6.routes invalid"
     false
   fi
 
   # Check profile limits are applied on boot.
-  if ! tc class show dev "${vethHostName}" | grep "1Mbit" ; then
+  if ! tc class show dev "${vethHostName}" | grep -F "1Mbit" ; then
     echo "limits.ingress invalid"
     false
   fi
-  if ! tc filter show dev "${vethHostName}" egress | grep "2Mbit" ; then
+  if ! tc filter show dev "${vethHostName}" egress | grep -F "2Mbit" ; then
     echo "limits.egress invalid"
     false
   fi
 
   # Check that limits.priority was correctly configured in the firewall.
   if [ "$firewallDriver" = "xtables" ]; then
-    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
-    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0005"
+    [ "$(iptables -t mangle -S | grep -cF "${ctName} (${vethHostName}) netprio")" = "1" ]
+    iptables -t mangle -S | grep -F "${ctName} (${vethHostName}) netprio" | grep -F "0000:0005"
   else
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:5"
+    [ "$(nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -cF "meta priority set")" = "1" ]
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -F "meta priority set 0:5"
   fi
 
   # Check profile custom MTU is applied in container on boot.
-  if ! lxc exec "${ctName}" -- grep "1400" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1400" ]; then
     echo "container veth mtu invalid"
     false
   fi
 
   # Check profile custom MTU doesn't affect the host.
-  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
+  if [ "$(cat /sys/class/net/"${vethHostName}"/mtu)" != "1500" ]; then
     echo "host veth mtu invalid"
     false
   fi
 
   # Check profile custom txqueuelen is applied in container on boot.
-  if ! lxc exec "${ctName}" -- grep "1200" /sys/class/net/eth0/tx_queue_len ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/tx_queue_len)" != "1200" ]; then
     echo "container veth txqueuelen invalid"
     false
   fi
 
   # Check profile custom txqueuelen is applied on host side of veth.
-  if ! grep "1200" /sys/class/net/"${vethHostName}"/tx_queue_len ; then
+  if [ "$(cat /sys/class/net/"${vethHostName}"/tx_queue_len)" != "1200" ]; then
     echo "host veth txqueuelen invalid"
     false
   fi
 
   # Check profile custom MAC is applied in container on boot.
-  if ! lxc exec "${ctName}" -- grep -Fix "${ctMAC}" /sys/class/net/eth0/address ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/address)" != "${ctMAC}" ]; then
     echo "mac invalid"
     false
   fi
 
   # Add IP alias to container and check routes actually work.
+  lxc exec "${ctName}" -- ip -6 addr add "2001:db8::1${ipRand}/128" dev eth0
   lxc exec "${ctName}" -- ip -4 addr add "192.0.2.1${ipRand}/32" dev eth0
   lxc exec "${ctName}" -- ip -4 route add default dev eth0
-  ping -c2 -W5 "192.0.2.1${ipRand}"
-  lxc exec "${ctName}" -- ip -6 addr add "2001:db8::1${ipRand}/128" dev eth0
+  ping -nc2 -i0.1 -W1 "192.0.2.1${ipRand}"
   wait_for_dad "${ctName}" eth0
-  ping6 -c2 -W5 "2001:db8::1${ipRand}"
+  ping -6 -nc2 -i0.1 -W1 "2001:db8::1${ipRand}"
 
   # Test hot plugging a container nic with different settings to profile with the same name.
   lxc config device add "${ctName}" eth0 nic \
     nictype=bridged \
     name=eth0 \
-    parent=${brName} \
+    parent="${brName}" \
     ipv4.routes="192.0.2.2${ipRand}/32" \
     ipv6.routes="2001:db8::2${ipRand}/128" \
     limits.ingress=3Mbit \
@@ -155,59 +174,62 @@ test_container_devices_nic_bridged() {
     hwaddr="${ctMAC}" \
     mtu=1401
 
+  # Test hot plugging a container nic with a different name.
+  ! lxc config device add "${ctName}" eth1 nic nictype=bridged name=eth1 parent="${brName}" || false
+
   # Check profile routes are removed on hot-plug.
-  if ip -4 r list dev "${brName}" | grep "192.0.2.1${ipRand}" ; then
+  if ip -4 r list dev "${brName}" | grep -F "192.0.2.1${ipRand}" ; then
     echo "ipv4.routes remain"
     false
   fi
-  if ip -6 r list dev "${brName}" | grep "2001:db8::1${ipRand}" ; then
+  if ip -6 r list dev "${brName}" | grep -F "2001:db8::1${ipRand}" ; then
     echo "ipv6.routes remain"
     false
   fi
 
   # Check routes are applied on hot-plug.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.2${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.2${ipRand}" ; then
     echo "ipv4.routes invalid"
     false
   fi
-  if ! ip -6 r list dev "${brName}" | grep "2001:db8::2${ipRand}" ; then
+  if ! ip -6 r list dev "${brName}" | grep -F "2001:db8::2${ipRand}" ; then
     echo "ipv6.routes invalid"
     false
   fi
 
   # Check limits are applied on hot-plug.
-  if ! tc class show dev "${vethHostName}" | grep "3Mbit" ; then
+  if ! tc class show dev "${vethHostName}" | grep -F "3Mbit" ; then
     echo "limits.ingress invalid"
     false
   fi
-  if ! tc filter show dev "${vethHostName}" egress | grep "4Mbit" ; then
+  if ! tc filter show dev "${vethHostName}" egress | grep -F "4Mbit" ; then
     echo "limits.egress invalid"
     false
   fi
 
   # Check that limits.priority was correctly configured in the firewall.
   if [ "$firewallDriver" = "xtables" ]; then
-    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
-    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0006"
+    [ "$(iptables -t mangle -S | grep -cF "${ctName} (${vethHostName}) netprio")" = "1" ]
+    iptables -t mangle -S | grep -F "${ctName} (${vethHostName}) netprio" | grep -F "0000:0006"
   else
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:6"
+    [ "$(nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -cF "meta priority set")" = "1" ]
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -F "meta priority set 0:6"
   fi
 
   # Check custom MTU is applied on hot-plug.
-  if ! lxc exec "${ctName}" -- grep "1401" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1401" ]; then
     echo "container veth mtu invalid"
     false
   fi
 
   # Check custom MTU doesn't affect the host.
-  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
+  if [ "$(cat /sys/class/net/"${vethHostName}"/mtu)" != "1500" ]; then
     echo "host veth mtu invalid"
     false
   fi
 
   # Check custom MAC is applied on hot-plug.
-  if ! lxc exec "${ctName}" -- grep -Fix "${ctMAC}" /sys/class/net/eth0/address ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/address)" != "${ctMAC}" ]; then
     echo "mac invalid"
     false
   fi
@@ -216,52 +238,52 @@ test_container_devices_nic_bridged() {
   lxc config device remove "${ctName}" eth0
 
   # Check routes are removed on hot-plug.
-  if ip -4 r list dev "${brName}" | grep "192.0.2.2${ipRand}" ; then
+  if ip -4 r list dev "${brName}" | grep -F "192.0.2.2${ipRand}" ; then
     echo "ipv4.routes remain"
     false
   fi
-  if ip -6 r list dev "${brName}" | grep "2001:db8::2${ipRand}" ; then
+  if ip -6 r list dev "${brName}" | grep -F "2001:db8::2${ipRand}" ; then
     echo "ipv6.routes remain"
     false
   fi
 
   # Check profile routes are applied on hot-removal.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.1${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.1${ipRand}" ; then
     echo "ipv4.routes invalid"
     false
   fi
-  if ! ip -6 r list dev "${brName}" | grep "2001:db8::1${ipRand}" ; then
+  if ! ip -6 r list dev "${brName}" | grep -F "2001:db8::1${ipRand}" ; then
     echo "ipv6.routes invalid"
     false
   fi
 
   # Check profile limits are applie on hot-removal.
-  if ! tc class show dev "${vethHostName}" | grep "1Mbit" ; then
+  if ! tc class show dev "${vethHostName}" | grep -F "1Mbit" ; then
     echo "limits.ingress invalid"
     false
   fi
-  if ! tc filter show dev "${vethHostName}" egress | grep "2Mbit" ; then
+  if ! tc filter show dev "${vethHostName}" egress | grep -F "2Mbit" ; then
     echo "limits.egress invalid"
     false
   fi
 
   # Check that limits.priority was correctly configured in the firewall.
   if [ "$firewallDriver" = "xtables" ]; then
-    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
-    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0005"
+    [ "$(iptables -t mangle -S | grep -cF "${ctName} (${vethHostName}) netprio")" = "1" ]
+    iptables -t mangle -S | grep -F "${ctName} (${vethHostName}) netprio" | grep -F "0000:0005"
   else
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:5"
+    [ "$(nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -cF "meta priority set")" = "1" ]
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -F "meta priority set 0:5"
   fi
 
   # Check profile custom MTU is applied on hot-removal.
-  if ! lxc exec "${ctName}" -- grep "1400" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1400" ]; then
     echo "container veth mtu invalid"
     false
   fi
 
   # Check custom MTU doesn't affect the host.
-  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
+  if [ "$(cat /sys/class/net/"${vethHostName}"/mtu)" != "1500" ]; then
     echo "host veth mtu invalid"
     false
   fi
@@ -270,7 +292,7 @@ test_container_devices_nic_bridged() {
   lxc config device add "${ctName}" eth0 nic \
     nictype=bridged \
     name=eth0 \
-    parent=${brName} \
+    parent="${brName}" \
     host_name="${vethHostName}" \
     ipv4.routes="192.0.2.1${ipRand}/32" \
     ipv6.routes="2001:db8::1${ipRand}/128"
@@ -308,52 +330,52 @@ test_container_devices_nic_bridged() {
   lxc config device set "${ctName}" eth0 hwaddr "${ctMAC}"
 
   # Check original routes are removed on hot-plug.
-  if ip -4 r list dev "${brName}" | grep "192.0.2.1${ipRand}" ; then
+  if ip -4 r list dev "${brName}" | grep -F "192.0.2.1${ipRand}" ; then
     echo "ipv4.routes remain"
     false
   fi
-  if ip -6 r list dev "${brName}" | grep "2001:db8::1${ipRand}" ; then
+  if ip -6 r list dev "${brName}" | grep -F "2001:db8::1${ipRand}" ; then
     echo "ipv6.routes remain"
     false
   fi
 
   # Check routes are applied on update.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.2${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.2${ipRand}" ; then
     echo "ipv4.routes invalid"
     false
   fi
-  if ! ip -6 r list dev "${brName}" | grep "2001:db8::2${ipRand}" ; then
+  if ! ip -6 r list dev "${brName}" | grep -F "2001:db8::2${ipRand}" ; then
     echo "ipv6.routes invalid"
     false
   fi
 
   # Check limits are applied on update.
-  if ! tc class show dev "${vethHostName}" | grep "3Mbit" ; then
+  if ! tc class show dev "${vethHostName}" | grep -F "3Mbit" ; then
     echo "limits.ingress invalid"
     false
   fi
-  if ! tc filter show dev "${vethHostName}" egress | grep "4Mbit" ; then
+  if ! tc filter show dev "${vethHostName}" egress | grep -F "4Mbit" ; then
     echo "limits.egress invalid"
     false
   fi
 
   # Check that limits.priority was correctly configured in the firewall.
   if [ "$firewallDriver" = "xtables" ]; then
-    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
-    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0006"
+    [ "$(iptables -t mangle -S | grep -cF "${ctName} (${vethHostName}) netprio")" = "1" ]
+    iptables -t mangle -S | grep -F "${ctName} (${vethHostName}) netprio" | grep -F "0000:0006"
   else
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
-    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:6"
+    [ "$(nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -cF "meta priority set")" = "1" ]
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -F "meta priority set 0:6"
   fi
 
   # Check custom MTU is applied update.
-  if ! lxc exec "${ctName}" -- grep "1402" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1402" ]; then
     echo "mtu invalid"
     false
   fi
 
   # Check custom MAC is applied update.
-  if ! lxc exec "${ctName}" -- grep -Fix "${ctMAC}" /sys/class/net/eth0/address ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/address)" != "${ctMAC}" ]; then
     echo "mac invalid"
     false
   fi
@@ -363,7 +385,7 @@ test_container_devices_nic_bridged() {
   lxc config device unset "${ctName}" eth0 mtu
   lxc network set "${brName}" bridge.mtu "1405"
   lxc start "${ctName}"
-  if ! lxc exec "${ctName}" -- grep "1405" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1405" ]; then
     echo "mtu not inherited from parent"
     false
   fi
@@ -379,33 +401,33 @@ test_container_devices_nic_bridged() {
   lxc network set "${brName}" ipv4.nat false
 
   # Check external routes are applied on update.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2${ipRand}.0/24 via 192.0.2.1${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2${ipRand}.0/24 via 192.0.2.1${ipRand}" ; then
     echo "external ipv4 routes invalid after network update"
     false
   fi
 
   # Check container routes are applied on update.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.2${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.2${ipRand}" ; then
     echo "container ipv4 routes invalid after network update"
     false
   fi
 
   # Check network routes are applied on update.
-  if ! ip -4 r list dev "${brName}" | grep "192.0.3.0/24" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.3.0/24" ; then
     echo "network ipv4 routes invalid after network update"
     false
   fi
 
   # Check volatile cleanup on stop.
   lxc stop -f "${ctName}"
-  if lxc config show "${ctName}" | grep volatile.eth0 ; then
+  if lxc config show "${ctName}" | grep -F volatile.eth0 ; then
     echo "unexpected volatile key remains"
     false
   fi
 
   # Test DHCP lease clearance.
   lxc delete "${ctName}" -f
-  lxc launch testimage "${ctName}" -p "${ctName}"
+  lxc launch testimage "${ctName}" -d "${SMALL_ROOT_DISK}" -p "${ctName}"
 
   # Request DHCPv4 lease with custom name (to check managed name is allocated instead).
   lxc exec "${ctName}" -- udhcpc -f -i eth0 -n -q -t5 -F "${ctName}custom"
@@ -423,12 +445,7 @@ test_container_devices_nic_bridged() {
   fi
 
   # Request DHCPv6 lease (if udhcpc6 is in busybox image).
-  busyboxUdhcpc6=1
-  if ! lxc exec "${ctName}" -- busybox --list | grep udhcpc6 ; then
-    busyboxUdhcpc6=0
-  fi
-
-  if [ "$busyboxUdhcpc6" = "1" ]; then
+  if lxc exec "${ctName}" -- busybox --list | grep udhcpc6 ; then
         lxc exec "${ctName}" -- udhcpc6 -f -i eth0 -n -q -t5 2>&1 | grep 'IPv6 obtained'
   fi
 
@@ -453,16 +470,11 @@ test_container_devices_nic_bridged() {
   ip netns exec testdns ip link set veth_right name eth0
   ip netns exec testdns ip link set dev eth0 up
   ip netns exec testdns ip addr add 192.0.2.20/24 dev eth0
-  ip netns exec testdns ip addr add 2001:db8::20/64 dev eth0
+  ip netns exec testdns ip addr add 2001:db8::20/64 dev eth0 nodad
 
   ip addr
   ip netns exec testdns ip addr
 
-  # Give eth0 a chance to finish duplicate addr detection (ipv6)
-  while ip netns exec testdns ip a | grep "tentative"; do
-    sleep 0.5
-  done
-
   ip netns exec testdns dig -4 +retry=0 +notcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
   ip netns exec testdns dig -6 +retry=0 +notcp @2001:db8::1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
   ip netns exec testdns dig -4 +retry=0 +tcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
@@ -499,10 +511,10 @@ test_container_devices_nic_bridged() {
   ip addr
   ip netns exec testdns ip addr
 
-  ! ip netns exec testdns dig -4 +retry=0 +notcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
-  ! ip netns exec testdns dig -6 +retry=0 +notcp @2001:db8::1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
-  ! ip netns exec testdns dig -4 +retry=0 +tcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
-  ! ip netns exec testdns dig -6 +retry=0 +tcp @2001:db8::1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2."
+  ! ip netns exec testdns dig -4 +retry=0 +notcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2." || false
+  ! ip netns exec testdns dig -6 +retry=0 +notcp @2001:db8::1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2." || false
+  ! ip netns exec testdns dig -4 +retry=0 +tcp @192.0.2.1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2." || false
+  ! ip netns exec testdns dig -6 +retry=0 +tcp @2001:db8::1 A "${ctName}.${dnsDomain}" | grep "${ctName}.${dnsDomain}.\\+0.\\+IN.\\+A.\\+192.0.2." || false
 
   ip netns exec testdns ip link delete eth0
   ip netns delete testdns
@@ -533,29 +545,29 @@ test_container_devices_nic_bridged() {
   fi
 
   # Check dnsmasq host file is updated on new device.
-  lxc init testimage "${ctName}" -p "${ctName}"
+  lxc init testimage "${ctName}" -d "${SMALL_ROOT_DISK}" -p "${ctName}"
   lxc config device add "${ctName}" eth0 nic nictype=bridged parent="${brName}" name=eth0 ipv4.address=192.0.2.200 ipv6.address=2001:db8::200
 
   ls -lR "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/"
 
-  if ! grep "192.0.2.200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
+  if ! grep -F "192.0.2.200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
     echo "dnsmasq host config not updated with IPv4 address"
     false
   fi
 
-  if ! grep "2001:db8::200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
+  if ! grep -F "2001:db8::200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
     echo "dnsmasq host config not updated with IPv6 address"
     false
   fi
 
   lxc config device remove "${ctName}" eth0
 
-  if grep "192.0.2.200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
+  if grep -F "192.0.2.200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
     echo "dnsmasq host config still has old IPv4 address"
     false
   fi
 
-  if grep "2001:db8::200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
+  if grep -F "2001:db8::200" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctName}.eth0" ; then
     echo "dnsmasq host config still has old IPv6 address"
     false
   fi
@@ -568,7 +580,7 @@ test_container_devices_nic_bridged() {
   lxc network set "${brName}" ipv6.address none
 
   # Confirm IPv6 is disabled.
-  [ "$(cat /proc/sys/net/ipv6/conf/${brName}/disable_ipv6)" = "1" ]
+  [ "$(cat "/proc/sys/net/ipv6/conf/${brName}/disable_ipv6")" = "1" ]
 
   if [ -f "${LXD_DIR}/networks/${brName}/dnsmasq.leases" ] ; then
     echo "dnsmasq.leases file still present after disabling DHCP"
@@ -594,7 +606,7 @@ test_container_devices_nic_bridged() {
   lxc profile device set "${ctName}" eth0 ipv6.routes "2001:db8::1${ipRand}/128"
 
   # Confirm IPv6 is re-enabled.
-  [ "$(cat /proc/sys/net/ipv6/conf/${brName}/disable_ipv6)" = "0" ]
+  [ "$(cat "/proc/sys/net/ipv6/conf/${brName}/disable_ipv6")" = "0" ]
 
   # Check dnsmasq host file is created on add.
   lxc config device add "${ctName}" eth0 nic nictype=bridged parent="${brName}" name=eth0
@@ -615,39 +627,39 @@ test_container_devices_nic_bridged() {
 
   # Linked network tests.
   # Can't use network property when parent is set.
-  ! lxc profile device set "${ctName}" eth0 network="${brName}"
+  ! lxc profile device set "${ctName}" eth0 network="${brName}" || false
 
   # Remove mtu, nictype and parent settings and assign network in one command.
   lxc profile device set "${ctName}" eth0 mtu="" parent="" nictype="" network="${brName}"
 
   # Can't remove network if parent not specified.
-  ! lxc profile device unset "${ctName}" eth0 network
+  ! lxc profile device unset "${ctName}" eth0 network || false
 
   # Can't use some settings when network is set.
-  ! lxc profile device set "${ctName}" eth0 nictype="bridged"
-  ! lxc profile device set "${ctName}" eth0 mtu="1400"
-  ! lxc profile device set "${ctName}" eth0 maas.subnet.ipv4="test"
-  ! lxc profile device set "${ctName}" eth0 maas.subnet.ipv6="test"
+  ! lxc profile device set "${ctName}" eth0 nictype="bridged" || false
+  ! lxc profile device set "${ctName}" eth0 mtu="1400" || false
+  ! lxc profile device set "${ctName}" eth0 maas.subnet.ipv4="test" || false
+  ! lxc profile device set "${ctName}" eth0 maas.subnet.ipv6="test" || false
 
   # Can't set static IP that isn't part of network's subnet.
-  ! lxc profile device set "${ctName}" eth0 ipv4.address="192.0.4.2"
-  ! lxc profile device set "${ctName}" eth0 ipv6.address="2001:db8:2::2"
+  ! lxc profile device set "${ctName}" eth0 ipv4.address="192.0.4.2" || false
+  ! lxc profile device set "${ctName}" eth0 ipv6.address="2001:db8:2::2" || false
 
   # Test bridge MTU is inherited.
   lxc network set "${brName}" bridge.mtu 1400
   lxc config device remove "${ctName}" eth0
   lxc start "${ctName}"
-  if ! lxc exec "${ctName}" -- grep "1400" /sys/class/net/eth0/mtu ; then
+  if [ "$(lxc exec "${ctName}" -- cat /sys/class/net/eth0/mtu)" != "1400" ]; then
     echo "container mtu has not been inherited from network"
     false
   fi
 
   # Check profile routes are applied on boot (as these use derived nictype).
-  if ! ip -4 r list dev "${brName}" | grep "192.0.2.1${ipRand}" ; then
+  if ! ip -4 r list dev "${brName}" | grep -F "192.0.2.1${ipRand}" ; then
     echo "ipv4.routes invalid"
     false
   fi
-  if ! ip -6 r list dev "${brName}" | grep "2001:db8::1${ipRand}" ; then
+  if ! ip -6 r list dev "${brName}" | grep -F "2001:db8::1${ipRand}" ; then
     echo "ipv6.routes invalid"
     false
   fi
@@ -656,37 +668,37 @@ test_container_devices_nic_bridged() {
   lxc network unset "${brName}" bridge.mtu
 
   # Test stateful DHCP static IP checks.
-  ! lxc config device override "${ctName}" eth0 ipv4.address="192.0.4.2"
+  ! lxc config device override "${ctName}" eth0 ipv4.address="192.0.4.2" || false
 
   lxc network set "${brName}" ipv4.dhcp false
-  ! lxc config device override "${ctName}" eth0 ipv4.address="192.0.2.2"
+  ! lxc config device override "${ctName}" eth0 ipv4.address="192.0.2.2" || false
   lxc network unset "${brName}" ipv4.dhcp
   lxc config device override "${ctName}" eth0 ipv4.address="192.0.2.2"
 
-  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8:2::2"
+  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8:2::2" || false
 
   lxc network set "${brName}" ipv6.dhcp=false ipv6.dhcp.stateful=false
-  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2"
+  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2" || false
   lxc network set "${brName}" ipv6.dhcp=true ipv6.dhcp.stateful=false
-  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2"
+  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2" || false
   lxc network set "${brName}" ipv6.dhcp=false ipv6.dhcp.stateful=true
-  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2"
+  ! lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2" || false
 
   lxc network unset "${brName}" ipv6.dhcp
   lxc config device set "${ctName}" eth0 ipv6.address="2001:db8::2"
 
   # Test port isolation.
-  if bridge link set help 2>&1 | grep isolated ; then
+  if bridge link set help 2>&1 | grep -wF isolated ; then
     lxc config device set "${ctName}" eth0 security.port_isolation true
     lxc start "${ctName}"
-    bridge -d link show dev "${vethHostName}" | grep "isolated on"
+    bridge -d link show dev "${vethHostName}" | grep -F "isolated on"
     lxc stop -f "${ctName}"
   else
     echo "bridge command doesn't support port isolation, skipping port isolation checks"
   fi
 
   # Test interface naming scheme.
-  lxc init testimage test-naming
+  lxc init testimage -d "${SMALL_ROOT_DISK}" test-naming
   lxc start test-naming
   lxc query "/1.0/instances/test-naming/state" | jq -r .network.eth0.host_name | grep ^veth
   lxc stop -f test-naming
@@ -800,16 +812,21 @@ test_container_devices_nic_bridged() {
   lxc delete -f foo
 
   # Test container without extra network configuration can be restored from backup.
-  lxc init testimage foo -p "${ctName}"
+  lxc init testimage foo -d "${SMALL_ROOT_DISK}" -p "${ctName}"
   lxc export foo foo.tar.gz
   lxc import foo.tar.gz foo2
   rm foo.tar.gz
   lxc profile assign foo2 "${ctName}"
+  lxc snapshot foo snap0
 
   # Test container start will fail due to volatile MAC conflict.
   [ "$(lxc config get foo volatile.eth0.hwaddr)" = "$(lxc config get foo2 volatile.eth0.hwaddr)" ]
   ! lxc start foo2 || false
-  lxc delete -f foo foo2
+  lxc delete -f foo2
+
+  # Test snapshot can be copied to remote.
+  lxc copy foo/snap0 localhost:foo3
+  lxc delete -f foo foo3
 
   # Check we haven't left any NICS lying around.
   endNicCount=$(find /sys/class/net | wc -l)
