@@ -1,7 +1,4 @@
 test_container_devices_nic_bridged_filtering() {
-  ensure_import_testimage
-  ensure_has_localhost_remote "${LXD_ADDR}"
-
   firewallDriver=$(lxc info | awk -F ":" '/firewall:/{gsub(/ /, "", $0); print $2}')
 
   if [ "$firewallDriver" != "xtables" ] && [ "$firewallDriver" != "nftables" ]; then
@@ -10,12 +7,15 @@ test_container_devices_nic_bridged_filtering() {
   fi
 
   if [ "$firewallDriver" = "xtables" ]; then
-    if readlink -f "$(command -v ebtables)" | grep -q nft; then
+    if readlink -f "$(command -v ebtables)" | grep -wF nft >/dev/null; then
       echo "==> SKIP: ebtables must be legacy version (try update-alternatives --set ebtables /usr/sbin/ebtables-legacy)"
       return
     fi
   fi
 
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
   # Record how many nics we started with.
   startNicCount=$(find /sys/class/net | wc -l)
 
@@ -37,13 +37,10 @@ test_container_devices_nic_bridged_filtering() {
   lxc network set "${brName}" bridge.hwaddr 00:11:22:33:44:55
   lxc network set "${brName}" ipv4.address 192.0.2.1/24
   lxc network set "${brName}" ipv6.address 2001:db8:1::1/64
-  [ "$(cat /sys/class/net/${brName}/address)" = "00:11:22:33:44:55" ]
+  [ "$(cat "/sys/class/net/${brName}/address")" = "00:11:22:33:44:55" ]
 
-  # Create profile for new containers.
-  lxc profile copy default "${ctPrefix}"
-
-  # Modify profile nictype and parent in atomic operation to ensure validation passes.
-  lxc profile show "${ctPrefix}" | sed  "s/nictype: p2p/nictype: bridged\\n    parent: ${brName}/" | lxc profile edit "${ctPrefix}"
+  # Create profile for new containers by atomically modifying nictype and parent to ensure validation passes.
+  lxc profile show default | sed  "s/nictype: p2p/nictype: bridged\\n    parent: ${brName}/" | lxc profile create "${ctPrefix}"
 
   # Launch first container.
   lxc init testimage "${ctPrefix}A" -p "${ctPrefix}"
@@ -58,8 +55,8 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}B" -- ip a add 192.0.2.3/24 dev eth0
 
   # Check basic connectivity without any filtering.
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3
 
   # Enable MAC filtering on CT A and test.
   lxc config device set "${ctPrefix}A" eth0 security.mac_filtering true
@@ -68,7 +65,7 @@ test_container_devices_nic_bridged_filtering() {
   # Check MAC filter is present in firewall.
   ctAHost=$(lxc config get "${ctPrefix}A" volatile.eth0.host_name)
   if [ "$firewallDriver" = "xtables" ]; then
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
       echo "MAC filter not applied as part of mac_filtering in ebtables"
       false
     fi
@@ -82,11 +79,11 @@ test_container_devices_nic_bridged_filtering() {
     do
       rules=$(nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0")
 
-      if ! echo "${rules}" | grep -e "iifname \"${ctAHost}\" ether saddr != ${ctAMAC} drop"; then
+      if ! echo "${rules}" | grep -F "iifname \"${ctAHost}\" ether saddr != ${ctAMAC} drop"; then
         echo "MAC filter not applied as part of mac_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! echo "${rules}" | grep -e "iifname \"${ctAHost}\" arp saddr ether != ${ctAMAC} drop"; then
+      if ! echo "${rules}" | grep -F "iifname \"${ctAHost}\" arp saddr ether != ${ctAMAC} drop"; then
         echo "MAC ARP filter not applied as part of mac_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -101,13 +98,13 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}A" -- ip link set dev eth0 address 00:11:22:33:44:56 up
 
   # Check that ping is no longer working (i.e its filtered after fake MAC setup).
-  if lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1; then
+  if lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1; then
       echo "MAC filter not working to host"
       false
   fi
 
   # Check that ping is no longer working (i.e its filtered after fake MAC setup).
-  if lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3; then
+  if lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3; then
       echo "MAC filter not working to other container"
       false
   fi
@@ -116,20 +113,20 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}A" -- ip link set dev eth0 address "${ctAMAC}" up
 
   # Check basic connectivity with MAC filtering but real MAC configured.
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3
 
   # Stop CT A and check filters are cleaned up.
   lxc stop -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
         echo "MAC filter still applied as part of mac_filtering in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "MAC filter still applied as part of mac_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -140,8 +137,8 @@ test_container_devices_nic_bridged_filtering() {
   lxc start "${ctPrefix}A"
   lxc exec "${ctPrefix}A" -- ip link set dev eth0 address "${ctAMAC}" up
   lxc exec "${ctPrefix}A" -- ip a add 192.0.2.254/24 dev eth0
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3
 
   # Enable IPv4 filtering on CT A and test (disable security.mac_filtering to check its applied too).
   lxc config device set "${ctPrefix}A" eth0 ipv4.address 192.0.2.2
@@ -153,54 +150,54 @@ test_container_devices_nic_bridged_filtering() {
   # Check MAC and IPv4 filter is present in firewall.
   ctAHost=$(lxc config get "${ctPrefix}A" volatile.eth0.host_name)
   if [ "$firewallDriver" = "xtables" ]; then
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
       echo "MAC filter not applied as part of ipv4_filtering in ebtables"
       false
     fi
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "192.0.2.2" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "192.0.2.2" ; then
         echo "IPv4 filter not applied as part of ipv4_filtering in ebtables"
         false
     fi
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "198.51.100.0/24" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "198.51.100.0/24" ; then
         echo "IPv4 filter for ipv4.routes not applied as part of ipv4_filtering in ebtables"
         false
     fi
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "203.0.113.0/24" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "203.0.113.0/24" ; then
         echo "IPv4 filter for ipv4.routes.external not applied as part of ipv4_filtering in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether saddr != ${ctAMAC} drop"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether saddr != ${ctAMAC} drop"; then
         echo "MAC filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" arp saddr ether != ${ctAMAC} drop"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" arp saddr ether != ${ctAMAC} drop"; then
         echo "MAC ARP filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ip saddr 192.0.2.2 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ip saddr 192.0.2.2 accept"; then
         echo "IPv4 filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" arp saddr ip 192.0.2.2 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" arp saddr ip 192.0.2.2 accept"; then
         echo "IPv4 ARP filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ip saddr 198.51.100.0/24 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ip saddr 198.51.100.0/24 accept"; then
         echo "IPv4 filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" arp saddr ip 198.51.100.0/24 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" arp saddr ip 198.51.100.0/24 accept"; then
         echo "IPv4 ARP filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ip saddr 203.0.113.0/24 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ip saddr 203.0.113.0/24 accept"; then
         echo "IPv4 filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" arp saddr ip 203.0.113.0/24 accept"; then
+      if ! nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" arp saddr ip 203.0.113.0/24 accept"; then
         echo "IPv4 ARP filter not applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -214,21 +211,21 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}A" -- ip a add 192.0.2.2/24 dev eth0
 
   # Check basic connectivity with IPv4 filtering and real IPs configured.
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3
 
   # Add a fake IP
   lxc exec "${ctPrefix}A" -- ip a flush dev eth0
   lxc exec "${ctPrefix}A" -- ip a add 192.0.2.254/24 dev eth0
 
   # Check that ping is no longer working (i.e its filtered after fake IP setup).
-  if lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1; then
+  if lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1; then
       echo "IPv4 filter not working to host"
       false
   fi
 
   # Check that ping is no longer working (i.e its filtered after fake IP setup).
-  if lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3; then
+  if lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3; then
       echo "IPv4 filter not working to other container"
       false
   fi
@@ -240,13 +237,13 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}B" -- ip r add 198.51.100.0/24 dev eth0
 
   # Check that ping is still working (i.e the filter did not apply to the ipv4.routes subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1; then
+  if ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1; then
       echo "IPv4 filter is preventing traffic from within ipv4.routes"
       false
   fi
 
   # Check that ping is still working (i.e the filter did not apply to the ipv4.routes subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3; then
+  if ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3; then
       echo "IPv4 filter is preventing traffic from within ipv4.routes"
       false
   fi
@@ -258,13 +255,13 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}B" -- ip r add 203.0.113.0/24 dev eth0
 
   # Check that ping is still working (i.e the filter did not apply to the ipv4.routes.external subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1; then
+  if ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1; then
       echo "IPv4 filter is preventing traffic from within ipv4.routes.external"
       false
   fi
 
   # Check that ping is still working (i.e the filter did not apply to the ipv4.routes.external subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.3; then
+  if ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.3; then
       echo "IPv4 filter is preventing traffic from within ipv4.routes.external"
       false
   fi
@@ -272,14 +269,14 @@ test_container_devices_nic_bridged_filtering() {
   # Stop CT A and check filters are cleaned up in firewall.
   lxc stop -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "${ctAHost}" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F "${ctAHost}" ; then
         echo "IPv4 filter still applied as part of ipv4_filtering in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "IPv4 filter still applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -290,7 +287,7 @@ test_container_devices_nic_bridged_filtering() {
   rm "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0"
   lxc config device unset "${ctPrefix}A" eth0 ipv4.address
   lxc start "${ctPrefix}A"
-  if ! grep "192.0.2.2" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
+  if ! grep -F "192.0.2.2" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
     echo "dnsmasq host config doesnt contain previous lease as static IPv4 config"
     false
   fi
@@ -307,7 +304,7 @@ test_container_devices_nic_bridged_filtering() {
   lxc config device set "${ctPrefix}A" eth0 security.ipv4_filtering true
   lxc start "${ctPrefix}A"
 
-  if ! grep "192.0.2.3" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
+  if ! grep -F "192.0.2.3" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
     echo "dnsmasq host config doesnt contain sequentially allocated static IPv4 config"
     false
   fi
@@ -317,7 +314,7 @@ test_container_devices_nic_bridged_filtering() {
   lxc network set "${brName}" ipv4.dhcp.ranges "192.0.2.100-192.0.2.110"
   lxc start "${ctPrefix}A"
 
-  if ! grep "192.0.2.100" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
+  if ! grep -F "192.0.2.100" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
     echo "dnsmasq host config doesnt contain sequentially range allocated static IPv4 config"
     false
   fi
@@ -342,8 +339,8 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}A" -- ip link set dev eth0 address "${ctAMAC}" up
   lxc exec "${ctPrefix}A" -- ip -6 a add 2001:db8:1::254 dev eth0
   wait_for_dad "${ctPrefix}A" eth0
-  lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::1
-  lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::3
+  lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::1
+  lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::3
 
   # Enable IPv6 filtering on CT A and test (disable security.mac_filtering to check its applied too).
   lxc config device set "${ctPrefix}A" eth0 ipv6.address 2001:db8:1::2
@@ -360,43 +357,43 @@ test_container_devices_nic_bridged_filtering() {
   macHex=$(echo "${ctAMAC}" |sed "s/://g")
 
   if [ "$firewallDriver" = "xtables" ]; then
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
         echo "MAC filter not applied as part of ipv6_filtering in ebtables"
         false
     fi
 
     # Check NDP MAC filter is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "${macHex}" ; then
+    if ! ip6tables -S -w -t filter | grep -F "${macHex}" ; then
         echo "MAC NDP filter not applied as part of ipv6_filtering in ip6tables"
         false
     fi
 
     # Check NDP IPv6 filter is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "20010db8000100000000000000000002" ; then
+    if ! ip6tables -S -w -t filter | grep -F "20010db8000100000000000000000002" ; then
         echo "IPv6 NDP filter not applied as part of ipv6_filtering in ip6tables"
         false
     fi
 
     # Check NDP IPv6 filter for ipv6.routes is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "20010db800020000" ; then
+    if ! ip6tables -S -w -t filter | grep -F "20010db800020000" ; then
         echo "IPv6 NDP filter for ipv6.routes not applied as part of ipv6_filtering in ip6tables"
         false
     fi
 
     # Check NDP IPv6 filter for ipv6.routes.external is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "20010db800030000" ; then
+    if ! ip6tables -S -w -t filter | grep -F "20010db800030000" ; then
         echo "IPv6 NDP filter for ipv6.routes.external not applied as part of ipv6_filtering in ip6tables"
         false
     fi
 
     # Check IPv6 filter is present in ebtables.
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "2001:db8:1::2" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "2001:db8:1::2" ; then
         echo "IPv6 filter not applied as part of ipv6_filtering in ebtables"
         false
     fi
 
     # Check IPv6 RA filter is present in ebtables.
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-i ${ctAHost} --ip6-proto ipv6-icmp --ip6-icmp-type router-advertisement -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-i ${ctAHost} --ip6-proto ipv6-icmp --ip6-icmp-type router-advertisement -j DROP" ; then
         echo "IPv6 RA filter not applied as part of ipv6_filtering in ebtables"
         false
     fi
@@ -450,22 +447,17 @@ test_container_devices_nic_bridged_filtering() {
         echo "IPv6 filter not applied as part of ipv6_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
-      if ! echo "${rules}" | grep -e "iifname \"${ctAHost}\" icmpv6 type 134 drop"; then
+      if ! echo "${rules}" | grep -F "iifname \"${ctAHost}\" icmpv6 type 134 drop"; then
         echo "IPv6 RA filter not applied as part of ipv6_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
     done
   fi
 
-  # Check DHCPv6 allocation still works (if udhcpc6 is in busybox image).
   lxc exec "${ctPrefix}A" -- ip link set dev eth0 address "${ctAMAC}" up
 
-  busyboxUdhcpc6=1
-  if ! lxc exec "${ctPrefix}A" -- busybox --list | grep udhcpc6 ; then
-    busyboxUdhcpc6=0
-  fi
-
-  if [ "$busyboxUdhcpc6" = "1" ]; then
+  # Check DHCPv6 allocation still works (if udhcpc6 is in busybox image).
+  if lxc exec "${ctPrefix}A" -- busybox --list | grep udhcpc6 ; then
       lxc exec "${ctPrefix}A" -- udhcpc6 -f -i eth0 -n -q -t5 2>&1 | grep 'IPv6 obtained'
   fi
 
@@ -474,8 +466,8 @@ test_container_devices_nic_bridged_filtering() {
   wait_for_dad "${ctPrefix}A" eth0
 
   # Check basic connectivity with IPv6 filtering and real IPs configured.
-  lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::1
-  lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::3
+  lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::1
+  lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::3
 
   # Add a fake IP
   lxc exec "${ctPrefix}A" -- ip -6 a flush dev eth0
@@ -483,13 +475,13 @@ test_container_devices_nic_bridged_filtering() {
   wait_for_dad "${ctPrefix}A" eth0
 
   # Check that ping is no longer working (i.e its filtered after fake IP setup).
-  if lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::1; then
+  if lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::1; then
       echo "IPv6 filter not working to host"
       false
   fi
 
   # Check that ping is no longer working (i.e its filtered after fake IP setup).
-  if lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::3; then
+  if lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::3; then
       echo "IPv6 filter not working to other container"
       false
   fi
@@ -502,13 +494,13 @@ test_container_devices_nic_bridged_filtering() {
   wait_for_dad "${ctPrefix}A" eth0
 
   # Check that ping is still working (i.e the filter did not apply to the ipv6.routes subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::1; then
+  if ! lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::1; then
       echo "IPv6 filter is preventing traffic from from within ipv6.routes"
       false
   fi
 
   # Check that ping is still working (i.e the filter did not apply to the ipv6.routes subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::3; then
+  if ! lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::3; then
       echo "IPv6 filter is preventing traffic from within ipv6.routes"
       false
   fi
@@ -520,13 +512,13 @@ test_container_devices_nic_bridged_filtering() {
   wait_for_dad "${ctPrefix}A" eth0
 
   # Check that ping is still working (i.e the filter did not apply to the ipv6.routes.external subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::1; then
+  if ! lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::1; then
       echo "IPv6 filter is preventing traffic from within ipv6.routes.external"
       false
   fi
 
   # Check that ping is still working (i.e the filter did not apply to the ipv6.routes subnet).
-  if ! lxc exec "${ctPrefix}A" -- ping6 -c2 -W5 2001:db8:1::3; then
+  if ! lxc exec "${ctPrefix}A" -- ping -6 -nc2 -i0.1 -W1 2001:db8:1::3; then
       echo "IPv6 filter is preventing traffic from within ipv6.routes.external"
       false
   fi
@@ -534,14 +526,14 @@ test_container_devices_nic_bridged_filtering() {
   # Stop CT A and check filters are cleaned up.
   lxc stop -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "${ctAHost}" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F "${ctAHost}" ; then
         echo "IPv6 filter still applied as part of ipv6_filtering in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "IPv6 filter still applied as part of ipv4_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -549,7 +541,7 @@ test_container_devices_nic_bridged_filtering() {
   fi
 
   # Check volatile cleanup on stop.
-  if lxc config show "${ctPrefix}A" | grep volatile.eth0 | grep -v volatile.eth0.hwaddr ; then
+  if [ "$(lxc config show "${ctPrefix}A" | grep -F volatile.eth0 | grep -vF volatile.eth0.hwaddr)" != "" ]; then
     echo "unexpected volatile key remains"
     false
   fi
@@ -561,7 +553,7 @@ test_container_devices_nic_bridged_filtering() {
   rm "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0"
   lxc config device set "${ctPrefix}A" eth0 security.ipv6_filtering true
   lxc start "${ctPrefix}A"
-  if ! grep "\\[2001:db8:1:0:216:3eff:fe92:f3c1\\]" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
+  if ! grep -F "[2001:db8:1:0:216:3eff:fe92:f3c1]" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
     echo "dnsmasq host config doesnt contain dynamically allocated static IPv6 config"
     false
   fi
@@ -577,14 +569,11 @@ test_container_devices_nic_bridged_filtering() {
   respawn_lxd "${LXD_DIR}" true
   lxc config device set "${ctPrefix}A" eth0 security.ipv6_filtering true
   lxc start "${ctPrefix}A"
-  if ! grep "\\[2001:db8:1::2\\]" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
+  if ! grep -F "[2001:db8:1::2]" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/${ctPrefix}A.eth0" ; then
     echo "dnsmasq host config doesnt contain sequentially allocated static IPv6 config"
     false
   fi
 
-  lxc stop -f "${ctPrefix}A"
-  lxc stop -f "${ctPrefix}B"
-
   lxc delete -f "${ctPrefix}A"
   lxc delete -f "${ctPrefix}B"
 
@@ -603,18 +592,18 @@ test_container_devices_nic_bridged_filtering() {
 
   # When IPv{n} addresses are "none", every packet should be dropped.
   if [ "$firewallDriver" = "xtables" ]; then
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p ARP -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p ARP -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p IPv4 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p IPv4 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p IPv6 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p IPv6 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p ARP -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p ARP -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p IPv4 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p IPv4 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p IPv6 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p IPv6 -i ${ctAHost} -j DROP"
   else
     for table in "in" "fwd"
     do
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x0806 drop" # ARP
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x0800 drop" # IPv4
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x86dd drop" # IPv6
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x0806 drop" # ARP
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x0800 drop" # IPv4
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x86dd drop" # IPv6
     done
   fi
 
@@ -648,31 +637,31 @@ test_container_devices_nic_bridged_filtering() {
 
   if [ "$firewallDriver" = "xtables" ]; then
     # Check MAC filter is present in ebtables.
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
         echo "MAC filter not applied as part of ipv4_filtering in ebtables"
         false
     fi
 
     # Check MAC NDP filter is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "${macHex}" ; then
+    if ! ip6tables -S -w -t filter | grep -F "${macHex}" ; then
         echo "MAC NDP ip6tables filter not applied as part of ipv6_filtering in ip6tables"
         false
     fi
 
     # Check IPv4 filter is present in ebtables.
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "192.0.2.2" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "192.0.2.2" ; then
         echo "IPv4 filter not applied as part of ipv4_filtering in ebtables"
         false
     fi
 
     # Check IPv6 filter is present in ebtables.
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "2001:db8::2" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F "2001:db8::2" ; then
         echo "IPv6 filter not applied as part of ipv6_filtering in ebtables"
         false
     fi
 
     # Check IPv6 filter is present in ip6tables.
-    if ! ip6tables -S -w -t filter | grep -e "20010db8000000000000000000000002" ; then
+    if ! ip6tables -S -w -t filter | grep -F "20010db8000000000000000000000002" ; then
         echo "IPv6 filter not applied as part of ipv6_filtering in ip6tables"
         false
     fi
@@ -723,14 +712,14 @@ test_container_devices_nic_bridged_filtering() {
   # Delete container and check filters are cleaned up.
   lxc delete -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "${ctAHost}" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F "${ctAHost}" ; then
         echo "ebtables filter still applied after delete"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "nftables filter still applied after delete (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -757,7 +746,7 @@ test_container_devices_nic_bridged_filtering() {
   ctAMAC=$(lxc config get "${ctPrefix}A" volatile.eth0.hwaddr)
 
   if [ "$firewallDriver" = "xtables" ]; then
-    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -e "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
+    if ! ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-s ! ${ctAMAC} -i ${ctAHost} -j DROP" ; then
         echo "MAC ebtables filter not applied as part of mac_filtering in ebtables"
         false
     fi
@@ -790,14 +779,14 @@ test_container_devices_nic_bridged_filtering() {
   # Stop container and check filters are cleaned up.
   lxc stop -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "${ctAHost}" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F "${ctAHost}" ; then
         echo "MAC filter still applied as part of mac_filtering in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "MAC filter still applied as part of mac_filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
@@ -821,16 +810,16 @@ test_container_devices_nic_bridged_filtering() {
   wait_for_dad "${ctPrefix}A" eth0
 
   # Check basic connectivity without any filtering.
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1
-  lxc exec "${ctPrefix}A" -- ping -c2 -W5 2001:db8::1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1
+  lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 2001:db8::1
 
   # Check fraudulent IPs are blocked.
   lxc exec "${ctPrefix}A" -- ip a flush dev eth0
   lxc exec "${ctPrefix}A" -- ip a add 192.0.2.3/24 dev eth0
   lxc exec "${ctPrefix}A" -- ip a add 2001:db8::3/64 dev eth0
 
-  ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1 || false
-  ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 2001:db8::1 || false
+  ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1 || false
+  ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 2001:db8::1 || false
 
   # Check IP filtering can be enabled with IP assigned as none in LXD config.
   lxc config device set "${ctPrefix}A" eth0 ipv4.address=none security.ipv4_filtering=true
@@ -838,8 +827,8 @@ test_container_devices_nic_bridged_filtering() {
   lxc exec "${ctPrefix}A" -- ip a flush dev eth0
   lxc exec "${ctPrefix}A" -- ip a add 192.0.2.2/24 dev eth0
   lxc exec "${ctPrefix}A" -- ip a add 2001:db8::2/64 dev eth0
-  ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 192.0.2.1 || false
-  ! lxc exec "${ctPrefix}A" -- ping -c2 -W5 2001:db8::1 || false
+  ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 192.0.2.1 || false
+  ! lxc exec "${ctPrefix}A" -- ping -nc2 -i0.1 -W1 2001:db8::1 || false
 
   lxc delete -f "${ctPrefix}A"
   ip link delete "${brName}2"
@@ -863,32 +852,32 @@ test_container_devices_nic_bridged_filtering() {
   ctAHost=$(lxc config get "${ctPrefix}A" volatile.eth0.host_name)
 
   if [ "$firewallDriver" = "xtables" ]; then
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p ARP -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p ARP -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p IPv4 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p IPv4 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A INPUT -p IPv6 -i ${ctAHost} -j DROP"
-    ebtables --concurrent -L --Lmac2 --Lx | grep -e "-A FORWARD -p IPv6 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p ARP -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p ARP -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p IPv4 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p IPv4 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A INPUT -p IPv6 -i ${ctAHost} -j DROP"
+    ebtables --concurrent -L --Lmac2 --Lx | grep -F -- "-A FORWARD -p IPv6 -i ${ctAHost} -j DROP"
   else
     for table in "in" "fwd"
     do
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x0806 drop" # ARP
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x0800 drop" # IPv4
-      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "iifname \"${ctAHost}\" ether type 0x86dd drop" # IPv6
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x0806 drop" # ARP
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x0800 drop" # IPv4
+      nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "iifname \"${ctAHost}\" ether type 0x86dd drop" # IPv6
     done
   fi
 
   # Delete container and check filters are cleaned up.
   lxc delete -f "${ctPrefix}A"
   if [ "$firewallDriver" = "xtables" ]; then
-    if ebtables --concurrent -L --Lmac2 --Lx | grep -e "${ctAHost}" ; then
+    if ebtables --concurrent -L --Lmac2 --Lx | grep -F "${ctAHost}" ; then
         echo "Filters still applied as part of IP filter in ebtables"
         false
     fi
   else
     for table in "in" "fwd"
     do
-      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -e "${ctAHost}"; then
+      if nft -nn list chain bridge lxd "${table}.${ctPrefix}A.eth0" | grep -F "${ctAHost}"; then
         echo "Filters still applied as part of IP filtering in nftables (${table}.${ctPrefix}A.eth0)"
         false
       fi
