@@ -9,14 +9,13 @@ test_init_auto() {
   if [ "$(storage_backend "$LXD_DIR")" = "zfs" ]; then
     # lxd init --auto --storage-backend zfs --storage-pool <name>
     LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-    chmod +x "${LXD_INIT_DIR}"
     spawn_lxd "${LXD_INIT_DIR}" false
 
     configure_loop_device loop_file_1 loop_device_1
     # shellcheck disable=SC2154
     zpool create -m none -O compression=on "lxdtest-$(basename "${LXD_DIR}")-pool1-existing-pool" "${loop_device_1}"
     LXD_DIR=${LXD_INIT_DIR} lxd init --auto --storage-backend zfs --storage-pool "lxdtest-$(basename "${LXD_DIR}")-pool1-existing-pool"
-    LXD_DIR=${LXD_INIT_DIR} lxc profile show default | grep -q "pool: default"
+    [ "$(LXD_DIR=${LXD_INIT_DIR} lxc profile device get default root pool)" = "default" ]
 
     kill_lxd "${LXD_INIT_DIR}"
     sed -i "\\|^${loop_device_1}|d" "${TEST_DIR}/loops"
@@ -24,7 +23,6 @@ test_init_auto() {
 
     # lxd init --auto --storage-backend zfs --storage-pool <name>/<non-existing-dataset>
     LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-    chmod +x "${LXD_INIT_DIR}"
     spawn_lxd "${LXD_INIT_DIR}" false
 
     # shellcheck disable=SC2154
@@ -35,7 +33,6 @@ test_init_auto() {
 
     # lxd init --auto --storage-backend zfs --storage-pool <name>/<existing-dataset>
     LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-    chmod +x "${LXD_INIT_DIR}"
     spawn_lxd "${LXD_INIT_DIR}" false
 
     zfs create -p -o mountpoint=none "lxdtest-$(basename "${LXD_DIR}")-pool1-existing-pool/existing-dataset"
@@ -48,7 +45,6 @@ test_init_auto() {
 
     # lxd init --storage-backend zfs --storage-create-loop 1 --storage-pool <name> --auto
     LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-    chmod +x "${LXD_INIT_DIR}"
     spawn_lxd "${LXD_INIT_DIR}" false
 
     ZFS_POOL="lxdtest-$(basename "${LXD_DIR}")-init"
@@ -59,7 +55,6 @@ test_init_auto() {
 
   # lxd init --network-address 127.0.0.1 --network-port LOCAL --auto
   LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD_INIT_DIR}"
   spawn_lxd "${LXD_INIT_DIR}" false
 
   LXD_DIR=${LXD_INIT_DIR} lxd init --network-address 127.0.0.1 --network-port "$(local_tcp_port)" --auto
