@@ -5,7 +5,7 @@ test_metrics() {
   lxc config set core.https_address "${LXD_ADDR}"
 
   lxc launch testimage c1
-  lxc init testimage c2
+  lxc init --empty c2
 
   # create another container in the non default project
   lxc project create foo -c features.images=false -c features.profiles=false
@@ -13,126 +13,129 @@ test_metrics() {
 
   # create but dont start a container in separate non default project to check for stopped instance accounting.
   lxc project create foo2 -c features.images=false -c features.profiles=false
-  lxc init testimage c4 --project foo2
+  lxc init --empty c4 --project foo2
 
-  # c1 metrics should show as the container is running
+  echo "==> c1 metrics should show as the container is running"
   lxc query "/1.0/metrics" | grep "name=\"c1\""
   lxc query "/1.0/metrics?project=default" | grep "name=\"c1\""
 
-  # c2 metrics should not be shown as the container is stopped
+  echo "==> c2 metrics should not be shown as the container is stopped"
   ! lxc query "/1.0/metrics" | grep "name=\"c2\"" || false
   ! lxc query "/1.0/metrics?project=default" | grep "name=\"c2\"" || false
 
-  # Check that we can get the count of existing instances.
+  echo "==> Check that we can get the count of existing instances"
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="default",type="container"} 2'
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="foo",type="container"} 1'
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="foo2",type="container"} 1'
-  # Ensure lxd_instances reports VM count properly (0)
+  echo "==> Ensure lxd_instances reports VM count properly (0)"
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="default",type="virtual-machine"} 0'
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="foo",type="virtual-machine"} 0'
   lxc query /1.0/metrics | grep -xF 'lxd_instances{project="foo2",type="virtual-machine"} 0'
 
-  # c3 metrics from another project also show up for non metrics unrestricted certificate
+  echo "==> c3 metrics from another project also show up for non metrics unrestricted certificate"
   lxc query "/1.0/metrics" | grep "name=\"c3\""
   lxc query "/1.0/metrics?project=foo" | grep "name=\"c3\""
 
-  # create new certificate
+  echo "==> create new certificate"
   gen_cert_and_key "metrics"
 
-  # this should fail as the certificate is not trusted yet
+  echo "==> this should fail as the certificate is not trusted yet"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "\"error_code\":403"
 
-  # trust newly created certificate for metrics only
+  echo "==> trust newly created certificate for metrics only"
   lxc config trust add "${LXD_CONF}/metrics.crt" --type=metrics
-  lxc config trust show "$(openssl x509 -in "${LXD_CONF}/metrics.crt" -outform der | sha256sum | head -c12)" | grep -xF "restricted: false"
+  lxc config trust show "$(cert_fingerprint "${LXD_CONF}/metrics.crt")" | grep -xF "restricted: false"
 
-  # c1 metrics should show as the container is running
+  echo "==> c1 metrics should show as the container is running"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "name=\"c1\""
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=default" | grep "name=\"c1\""
 
-  # c2 metrics should not be shown as the container is stopped
+  echo "==> c2 metrics should not be shown as the container is stopped"
   ! CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "name=\"c2\"" || false
   ! CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=default" | grep "name=\"c2\"" || false
 
-  # c3 metrics from another project should be shown for unrestricted certificate
+  echo "==> c3 metrics from another project should be shown for unrestricted certificate"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "name=\"c3\""
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=foo" | grep "name=\"c3\""
 
-  # internal server metrics should be shown as the certificate is not restricted
+  echo "==> internal server metrics should be shown as the certificate is not restricted"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep -E "^lxd_warnings_total [0-9]+$"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=default" | grep -E "^lxd_warnings_total [0-9]+$"
 
-  # make sure nothing else can be done with this certificate
+  echo "==> make sure nothing else can be done with this certificate"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/instances" | grep "\"error_code\":403"
 
   metrics_addr="127.0.0.1:$(local_tcp_port)"
 
   lxc config set core.metrics_address "${metrics_addr}"
 
-  # c1 metrics should be shown as the container is running
+  echo "==> c1 metrics should be shown as the container is running"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics" | grep "name=\"c1\""
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep "name=\"c1\""
 
-  # c2 metrics should not be shown as the container is stopped
+  echo "==> c2 metrics should not be shown as the container is stopped"
   ! CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics" | grep "name=\"c2\"" || false
   ! CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep "name=\"c2\"" || false
 
-  # c3 metrics from another project should  be shown for unrestricted metrics certificate
+  echo "==> c3 metrics from another project should  be shown for unrestricted metrics certificate"
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "name=\"c3\""
   CERTNAME=metrics my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=foo" | grep "name=\"c3\""
 
-  # internal server metrics should be shown as the certificate is not restricted
+  echo "==> internal server metrics should be shown as the certificate is not restricted"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics" | grep -E "^lxd_warnings_total [0-9]+$"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep -E "^lxd_warnings_total [0-9]+$"
 
-  # make sure no other endpoint is available
+  echo "==> make sure no other endpoint is available"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/instances" | grep "\"error_code\":404"
 
-  # create new certificate
+  echo "==> create new (restricted) certificate"
   gen_cert_and_key "metrics-restricted"
 
-  # trust newly created certificate for metrics only and mark it as restricted for the foo project
+  echo "==> trust newly created certificate for metrics only and mark it as restricted for the foo project"
   lxc config trust add "${LXD_CONF}/metrics-restricted.crt" --type=metrics --restricted --projects foo
-  lxc config trust show "$(openssl x509 -in "${LXD_CONF}/metrics-restricted.crt" -outform der | sha256sum | head -c12)" | grep -xF "restricted: true"
+  lxc config trust show "$(cert_fingerprint "${LXD_CONF}/metrics-restricted.crt")" | grep -xF "restricted: true"
 
-  # c3 metrics should be showned
+  echo "==> c3 metrics should be showned"
   CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=foo" | grep "name=\"c3\""
 
-  # c3 metrics cannot be viewed via the generic metrics endpoint if the certificate is restricted
-  ! CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics"
+  echo "==> c3 metrics are shown even without specifying the project"
+  # If no project is specified, all metrics accessible by the restricted certificate are returned.
+  # This way, a certificate covering multiple projects can be used to scrape all the metrics in one scoop.
+  CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics" | grep "name=\"c3\""
 
-  # other projects metrics aren't visible as they aren't allowed for the restricted certificate
-  ! CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=default"
+  echo "==> other projects metrics aren't visible as they aren't allowed for the restricted certificate"
+  [ "$(CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=default" -w "%{http_code}" -o /dev/null)" = "403" ]
+  [ "$(CERTNAME=metrics-restricted my_curl -X GET "https://${LXD_ADDR}/1.0/metrics?project=foo2" -w "%{http_code}" -o /dev/null)" = "403" ]
 
-  # c1 and c2 metrics are not visible as they are in another project
-  ! CERTNAME=metrics-restricted my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=foo" | grep "name=\"c1\""
-  ! CERTNAME=metrics-restricted my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=foo" | grep "name=\"c2\""
+  echo "==> c1 and c2 metrics are not visible as they are in another project"
+  ! CERTNAME=metrics-restricted my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=foo" | grep "name=\"c1\"" || false
+  ! CERTNAME=metrics-restricted my_curl -X GET "https://${metrics_addr}/1.0/metrics?project=foo" | grep "name=\"c2\"" || false
 
-  # Check that we can get the count of existing containers. There should be two in the default project: c1 (RUNNING) and c2 (STOPPED).
+  echo "==> Check that we can get the count of existing containers. There should be two in the default project: c1 (RUNNING) and c2 (STOPPED)"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics" | grep -xF 'lxd_instances{project="default",type="container"} 2'
-  sleep 10
-  # Try again after the metric cache has expired. We should still see two containers.
+  sleep 9
+  echo "==> Try again after the metric cache has expired (TTL=8s). We should still see two containers"
   CERTNAME=metrics my_curl -X GET "https://${metrics_addr}/1.0/metrics" | grep -xF 'lxd_instances{project="default",type="container"} 2'
 
-  # test unauthenticated connections
+  echo "==> test unauthenticated connections"
   ! curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "name=\"c1\"" || false
   ! curl -k -s -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep "name=\"c1\"" || false
   lxc config set core.metrics_authentication=false
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "name=\"c1\""
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep "name=\"c1\""
 
-  # Filesystem metrics should contain instance type
+  echo "==> Filesystem metrics should contain instance type"
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "lxd_filesystem_avail_bytes" | grep "type=\"container\""
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics?project=default" | grep "lxd_filesystem_avail_bytes" | grep "type=\"container\""
 
-  # API requests metrics should be included
+  echo "==> API requests metrics should be included"
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "lxd_api_requests_completed_total" | grep "entity_type=\"server\""
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "lxd_api_requests_completed_total" | grep "result=\"succeeded\""
   curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep "lxd_api_requests_ongoing" | grep "entity_type=\"server\""
 
-  # Test lxd_api_requests_completed_total increment with different results.
+  echo "==> Test lxd_api_requests_completed_total increment with different results"
   previous="$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_completed_total{entity_type="instance",result="succeeded"}' | awk '{print $2}')"
-  lxc ls # Uses /1.0/instances
+  lxc list -c n # Uses /1.0/instances
   [ "$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_completed_total{entity_type="instance",result="succeeded"}' | awk '{print $2}')" -eq $((previous+1)) ]
 
   previous="$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_completed_total{entity_type="server",result="error_client"}' | awk '{print $2}')"
@@ -141,14 +144,14 @@ test_metrics() {
 
   previous="$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_completed_total{entity_type="instance",result="error_server"}' | awk '{print $2}')"
   lxc storage create broken dir
-  sudo rmdir "$(lxc storage get broken source)"/containers # Break the storage pool.
+  rmdir "$(lxc storage get broken source)"/containers # Break the storage pool.
   ! lxc init testimage failed-container -s broken || false  # Error when creating a container on broken.
   [ "$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_completed_total{entity_type="instance",result="error_server"}' | awk '{print $2}')" -eq $((previous+1)) ]
 
-  # Test lxd_api_requests_ongoing increment and decrement.
+  echo "==> Test lxd_api_requests_ongoing increment and decrement"
   previous="$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_ongoing{entity_type="instance"}' | awk '{print $2}')"
-  lxc exec c1 -- sleep 2 &
-  sleep 1
+  lxc exec c1 -- sleep 0.5 &
+  sleep 0.1
   [ "$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_ongoing{entity_type="instance"}' | awk '{print $2}')" -eq $((previous+1)) ]
   wait $!
   [ "$(curl -k -s -X GET "https://${metrics_addr}/1.0/metrics" | grep 'lxd_api_requests_ongoing{entity_type="instance"}' | awk '{print $2}')" -eq "$previous" ]
