@@ -4,13 +4,19 @@ test_migration() {
   # shellcheck disable=2153
   lxd_backend=$(storage_backend "$LXD_DIR")
 
+  if [ "${lxd_backend}" = "dir" ] && uname -r | grep -- -kvm$; then
+    echo "==> SKIP: the -kvm kernel flavor is does not work for this test on ${lxd_backend}"
+    return
+  fi
+
   LXD2_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD2_DIR}"
   spawn_lxd "${LXD2_DIR}" true
   LXD2_ADDR=$(cat "${LXD2_DIR}/lxd.addr")
 
-  # workaround for kernel/criu
-  umount /sys/kernel/debug >/dev/null 2>&1 || true
+  if command -v criu >/dev/null; then
+    # workaround for kernel/criu
+    umount /sys/kernel/debug >/dev/null 2>&1 || true
+  fi
 
   token="$(lxc config trust add --name foo -q)"
   # shellcheck disable=2153
@@ -50,15 +56,10 @@ test_migration() {
   if [ "${LXD_BACKEND}" = "zfs" ]; then
     # Test that block mode zfs backends work fine with migration.
     for fs in "ext4" "btrfs" "xfs"; do
-      if ! command -v "mkfs.${fs}" >/dev/null 2>&1; then
-        echo "==> SKIP: Skipping block mode test on ${fs} due to missing tools."
-        continue
-      fi
-
       local storage_pool1 storage_pool2
       # shellcheck disable=2153
-      storage_pool1="lxdtest-$(basename "${LXD_DIR}")-block-mode"
-      storage_pool2="lxdtest-$(basename "${LXD2_DIR}")-block-mode"
+      storage_pool1="lxdtest-$(basename "${LXD_DIR}")-block-mode-${fs}"
+      storage_pool2="lxdtest-$(basename "${LXD2_DIR}")-block-mode-${fs}"
       lxc_remote storage create l1:"$storage_pool1" zfs size=1GiB volume.zfs.block_mode=true volume.block.filesystem="${fs}"
       lxc_remote profile device set l1:default root pool "$storage_pool1"
 
@@ -75,6 +76,21 @@ test_migration() {
     done
   fi
 
+  # Test config overrides for migration of instance with snapshots
+  lxc_remote network create l1:foonet ipv4.address=10.100.10.1/24
+  lxc_remote network create l2:foonet2 ipv4.address=10.100.100.1/24
+  ensure_import_testimage
+  lxc_remote init testimage l1:u1
+  lxc_remote config device add l1:u1 eth1 nic name=eth1 network=foonet ipv4.address=10.100.10.10
+  lxc_remote snapshot l1:u1 snap
+
+  lxc_remote copy l1:u1 l2: -d eth1,ipv4.address=10.100.100.10 -d eth1,network=foonet2
+
+  lxc_remote delete l1:u1
+  lxc_remote delete l2:u1
+  lxc_remote network delete l1:foonet
+  lxc_remote network delete l2:foonet2
+
   lxc_remote remote remove l1
   lxc_remote remote remove l2
   kill_lxd "$LXD2_DIR"
@@ -184,18 +200,18 @@ migration() {
   lxc storage volume set "${pool}" container/cccp user.foo=postsnap1
 
   # Check storage volume creation times are set.
-  lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp | jq .created_at | grep -Fv '0001-01-01T00:00:00Z'
-  lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp/snapshots/snap0 | jq .created_at | grep -Fv '0001-01-01T00:00:00Z'
+  [ "$(lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp | jq -r .created_at)" != "0001-01-01T00:00:00Z" ]
+  [ "$(lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp/snapshots/snap0 | jq -r .created_at)" != "0001-01-01T00:00:00Z" ]
 
   # Local container only copy.
   lxc copy cccp udssr --instance-only
-  [ "$(lxc info udssr | grep -c snap)" -eq 0 ]
+  [ "$(lxc list -f csv -c S udssr)" = "0" ]
   [ "$(lxc file pull udssr/blah -)" = "after" ]
   lxc delete udssr
 
   # Local container with snapshots copy.
   lxc copy cccp udssr
-  [ "$(lxc info udssr | grep -c snap)" -eq 2 ]
+  [ "$(lxc list -f csv -c S udssr)" = "2" ]
   [ "$(lxc file pull udssr/blah -)" = "after" ]
   lxc storage volume show "${pool}" container/udssr
   [ "$(lxc storage volume get "${pool}" container/udssr user.foo)" = "postsnap1" ]
@@ -205,13 +221,13 @@ migration() {
 
   # Remote container only copy.
   lxc_remote copy l1:cccp l2:udssr --instance-only
-  [ "$(lxc_remote info l2:udssr | grep -c snap)" -eq 0 ]
+  [ "$(lxc_remote list -f csv -c S l2:udssr)" = "0" ]
   [ "$(lxc_remote file pull l2:udssr/blah -)" = "after" ]
   lxc_remote delete l2:udssr
 
   # Remote container with snapshots copy.
   lxc_remote copy l1:cccp l2:udssr
-  [ "$(lxc_remote info l2:udssr | grep -c snap)" -eq 2 ]
+  [ "$(lxc_remote list -f csv -c S l2:udssr)" = "2" ]
   [ "$(lxc_remote file pull l2:udssr/blah -)" = "after" ]
   lxc_remote storage volume show l2:"${remote_pool}" container/udssr
   [ "$(lxc_remote storage volume get l2:"${remote_pool}" container/udssr user.foo)" = "postsnap1" ]
@@ -222,7 +238,7 @@ migration() {
   # Remote container only move.
   lxc_remote move l1:cccp l2:udssr --instance-only --mode=relay
   ! lxc_remote info l1:cccp || false
-  [ "$(lxc_remote info l2:udssr | grep -c snap)" -eq 0 ]
+  [ "$(lxc_remote list -f csv -c S l2:udssr)" = "0" ]
   lxc_remote delete l2:udssr
 
   lxc_remote init testimage l1:cccp
@@ -232,7 +248,7 @@ migration() {
   # Remote container with snapshots move.
   lxc_remote move l1:cccp l2:udssr --mode=push
   ! lxc_remote info l1:cccp || false
-  [ "$(lxc_remote info l2:udssr | grep -c snap)" -eq 2 ]
+  [ "$(lxc_remote list -f csv -c S l2:udssr)" = "2" ]
   lxc_remote delete l2:udssr
 
   # Test container only copies
@@ -243,7 +259,7 @@ migration() {
   # Local container with snapshots move.
   lxc move cccp udssr --mode=pull
   ! lxc info cccp || false
-  [ "$(lxc info udssr | grep -c snap)" -eq 2 ]
+  [ "$(lxc list -f csv -c S udssr)" = "2" ]
   lxc delete udssr
 
   if [ "$lxd_backend" = "zfs" ]; then
@@ -255,12 +271,12 @@ migration() {
 
     # Test container only copies when zfs.clone_copy is set to false.
     lxc copy cccp udssr --instance-only
-    [ "$(lxc info udssr | grep -c snap)" -eq 0 ]
+    [ "$(lxc list -f csv -c S udssr)" = "0" ]
     lxc delete udssr
 
     # Test container with snapshots copy when zfs.clone_copy is set to false.
     lxc copy cccp udssr
-    [ "$(lxc info udssr | grep -c snap)" -eq 2 ]
+    [ "$(lxc list -f csv -c S udssr)" = "2" ]
     lxc delete cccp
     lxc delete udssr
 
@@ -353,10 +369,10 @@ migration() {
   lxc_remote copy l1:c1 l2:c2 --refresh
   lxc_remote ls l2:
   lxc_remote config show l2:c2/snap0
-  ! lxc_remote config show l2:c2/snap0 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
-  lxc_remote config device get l2:c2 testdev type | grep -q 'none'
+  ! lxc_remote config show l2:c2/snap0 | grep -F 'expires_at: 0001-01-01T00:00:00Z' || false
+  [ "$(lxc_remote config device get l2:c2 testdev type)" = "none" ]
   lxc_remote restore l2:c2 snap0
-  lxc_remote config device get l2:c2 testsnapdev type | grep -q 'none'
+  [ "$(lxc_remote config device get l2:c2 testsnapdev type)" = "none" ]
 
   # This will create snapshot c2/snap1
   lxc_remote snapshot l2:c2
@@ -371,7 +387,7 @@ migration() {
   remote_pool1="lxdtest-$(basename "${LXD_DIR}")"
   remote_pool2="lxdtest-$(basename "${lxd2_dir}")"
 
-  lxc_remote storage volume create l1:"$remote_pool1" vol1
+  lxc_remote storage volume create l1:"$remote_pool1" vol1 size=4MiB
   lxc_remote storage volume set l1:"$remote_pool1" vol1 user.foo=snap0vol1
   lxc_remote storage volume snapshot l1:"$remote_pool1" vol1
   lxc_remote storage volume set l1:"$remote_pool1" vol1 user.foo=snap1vol1
@@ -419,7 +435,7 @@ migration() {
   [ "$(lxc_remote storage volume get l2:"$remote_pool2" vol2/snapremove user.foo)" = "snapremovevol1" ]
 
   # check remote storage volume refresh from a different volume
-  lxc_remote storage volume create l1:"$remote_pool1" vol3
+  lxc_remote storage volume create l1:"$remote_pool1" vol3 size=4MiB
   lxc_remote storage volume set l1:"$remote_pool1" vol3 user.foo=snap0vol3
   lxc_remote storage volume snapshot l1:"$remote_pool1" vol3
   lxc_remote storage volume set l1:"$remote_pool1" vol3 user.foo=snap1vol3
@@ -440,7 +456,7 @@ migration() {
   lxc_remote storage volume delete l2:"$remote_pool2" vol2
 
   # check that a refresh doesn't change the volume's and snapshot's UUID.
-  lxc_remote storage volume create l1:"$remote_pool1" vol1
+  lxc_remote storage volume create l1:"$remote_pool1" vol1 size=4MiB
   lxc_remote storage volume snapshot l1:"$remote_pool1" vol1
   lxc_remote storage volume copy l1:"$remote_pool1"/vol1 l2:"$remote_pool2"/vol2
   old_uuid="$(lxc storage volume get l2:"$remote_pool2" vol2 volatile.uuid)"
@@ -452,8 +468,8 @@ migration() {
   lxc_remote storage volume delete l1:"$remote_pool1" vol1
 
   # remote storage volume migration in "push" mode
-  lxc_remote storage volume create l1:"$remote_pool1" vol1
-  lxc_remote storage volume create l1:"$remote_pool1" vol2
+  lxc_remote storage volume create l1:"$remote_pool1" vol1 size=4MiB
+  lxc_remote storage volume create l1:"$remote_pool1" vol2 size=4MiB
   lxc_remote storage volume snapshot l1:"$remote_pool1" vol2
 
   lxc_remote storage volume copy l1:"$remote_pool1/vol1" l2:"$remote_pool2/vol2" --mode=push
@@ -470,8 +486,8 @@ migration() {
   lxc_remote storage volume delete l2:"$remote_pool2" vol6
 
   # remote storage volume migration in "relay" mode
-  lxc_remote storage volume create l1:"$remote_pool1" vol1
-  lxc_remote storage volume create l1:"$remote_pool1" vol2
+  lxc_remote storage volume create l1:"$remote_pool1" vol1 size=4MiB
+  lxc_remote storage volume create l1:"$remote_pool1" vol2 size=4MiB
   lxc_remote storage volume snapshot l1:"$remote_pool1" vol2
 
   lxc_remote storage volume copy l1:"$remote_pool1/vol1" l2:"$remote_pool2/vol2" --mode=relay
@@ -489,12 +505,87 @@ migration() {
 
   # Test migration when rsync compression is disabled
   lxc_remote storage set l1:"$remote_pool1" rsync.compression false
-  lxc_remote storage volume create l1:"$remote_pool1" foo
+  lxc_remote storage volume create l1:"$remote_pool1" foo size=4MiB
   lxc_remote storage volume copy l1:"$remote_pool1"/foo l2:"$remote_pool2"/bar
   lxc_remote storage volume delete l1:"$remote_pool1" foo
   lxc_remote storage volume delete l2:"$remote_pool2" bar
   lxc_remote storage unset l1:"$remote_pool1" rsync.compression
 
+  echo "==> Test container migration with attached local volumes."
+
+  echo "==> Create a local storage with the same name."
+  lxc_remote storage create l1:dir dir
+  lxc_remote storage create l2:dir dir
+
+  echo "==> Create a volume to attach to container."
+  lxc_remote storage volume create l1:dir vol1 size=4MiB
+
+  echo "==> Create a container to test migration with attached local volume."
+  lxc_remote init --empty l1:c1
+  lxc_remote storage volume attach l1:dir vol1 c1 /files
+
+  echo "==> Check that copying a container with attached local volume fails, if the destination does not have a volume with the same name."
+  ! lxc_remote copy l1:c1 l2: || false
+
+  echo "==> Check that moving a container with attached local volume fails, if the destination does not have a volume with the same name."
+  ! lxc_remote move l1:c1 l2: || false
+
+  echo "==> Copy the volume."
+  lxc_remote storage volume copy l1:dir/vol1 l2:dir/vol1
+
+  echo "==> Check that copying a container with attached local volume succeeds, if the destination has a volume with the same name."
+  lxc_remote copy l1:c1 l2:
+
+  echo "==> Check that moving a container with attached local volume succeeds, if the destination has a volume with the same name."
+  lxc_remote move l2:c1 l1:c2
+
+  echo "==> Clean up the containers and local volumes."
+  lxc_remote delete -f l1:c1
+  lxc_remote delete -f l1:c2
+  lxc_remote storage volume delete l1:dir vol1
+  lxc_remote storage volume delete l2:dir vol1
+
+  # Test VM Migration.
+  if [ "${LXD_VM_TESTS:-0}" = "0" ]; then
+    echo "==> SKIP: VM tests are disabled"
+  elif [ "${LXD_TMPFS:-0}" = "1" ] && ! runsMinimumKernel 6.6; then
+    echo "==> SKIP: QEMU requires direct-io support which requires a kernel >= 6.6 for tmpfs support (LXD_TMPFS=${LXD_TMPFS})"
+  else
+    echo "==> Test VM migration with attached local volumes."
+
+    echo "==> Create a volume to attach to VM."
+    lxc_remote storage volume create l1:dir vol1 size=4MiB
+
+    echo "==> Create a VM to test migration with attached local volume."
+    lxc_remote init --vm --empty l1:v1 -c limits.memory=128MiB -d "${SMALL_ROOT_DISK}"
+    lxc_remote storage volume attach l1:dir vol1 v1 /files
+
+    echo "==> Check that copying a VM with attached local volume fails, if the destination does not have a volume with the same name."
+    ! lxc_remote copy l1:v1 l2: || false
+
+    echo "==> Check that moving a VM with attached local volume fails, if the destination does not have a volume with the same name."
+    ! lxc_remote move l1:v1 l2: || false
+
+    echo "==> Copy the volume."
+    lxc_remote storage volume copy l1:dir/vol1 l2:dir/vol1
+
+    echo "==> Check that copying a VM with attached local volume succeeds, if the destination has a volume with the same name."
+    lxc_remote copy l1:v1 l2:
+
+    echo "==> Check that moving a VM with attached local volume succeeds, if the destination has a volume with the same name."
+    lxc_remote move l2:v1 l1:v2
+
+    echo "==> Clean up the VMs and local volumes."
+    lxc_remote delete -f l1:v1
+    lxc_remote delete -f l1:v2
+    lxc_remote storage volume delete l1:dir vol1
+    lxc_remote storage volume delete l2:dir vol1
+  fi
+
+  echo "==> Clean up the storage pool."
+  lxc_remote storage delete l1:dir
+  lxc_remote storage delete l2:dir
+
   # Test some migration between projects
   lxc_remote project create l1:proj -c features.images=false -c features.profiles=false
   lxc_remote project switch l1:proj
@@ -530,18 +621,18 @@ migration() {
   # Check snapshot creation dates after migration.
   lxc_remote init testimage l1:c1
   lxc_remote snapshot l1:c1
-  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
-  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1/snap0 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1 | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1/snap0 | grep '^created_at: 0001-01-01T00:00:00Z' || false
   lxc_remote copy l1:c1 l2:c1
-  ! lxc_remote storage volume show "l2:${remote_pool2}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
-  [ "$(lxc_remote storage volume show "l1:${remote_pool1}" container/c1/snap0 | awk /created_at:/)" = "$(lxc_remote storage volume show "l2:${remote_pool2}" container/c1/snap0 | awk /created_at:/)" ]
+  ! lxc_remote storage volume show "l2:${remote_pool2}" container/c1 | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  [ "$(lxc_remote storage volume get --property "l1:${remote_pool1}" container/c1/snap0 created_at)" = "$(lxc_remote storage volume get --property "l2:${remote_pool2}" container/c1/snap0 created_at)" ]
   lxc_remote delete l1:c1 -f
   lxc_remote delete l2:c1 -f
 
   # Check migration with invalid snapshot config (disks attached with missing source pool and source path).
   lxc_remote init testimage l1:c1
   lxc_remote storage create l1:dir dir
-  lxc_remote storage volume create l1:dir vol1
+  lxc_remote storage volume create l1:dir vol1 size=4MiB
   lxc_remote storage volume attach l1:dir vol1 c1 /mnt
   mkdir "$LXD_DIR/testvol2"
   lxc_remote config device add l1:c1 vol2 disk source="$LXD_DIR/testvol2" path=/vol2
@@ -572,7 +663,7 @@ migration() {
   lxc_remote restore l2:c1 snap0
   lxc_remote start l2:c1
   lxc_remote file pull l2:c1/tmp/foo .
-  ! lxc_remote file pull l2:c1/tmp/bar . ||  false
+  ! lxc_remote file pull l2:c1/tmp/bar . || false
   lxc_remote stop l2:c1 -f
 
   rm foo bar
@@ -636,9 +727,9 @@ migration() {
   lxc storage volume import l1:"${pool}" ./foo.iso iso1
   lxc storage volume copy l1:"${pool}"/iso1 l2:"${remote_pool}"/iso1
 
-  lxc storage volume show l2:"${remote_pool}" iso1 | grep -q 'content_type: iso'
+  lxc storage volume show l2:"${remote_pool}" iso1 | grep -xF 'content_type: iso'
   lxc storage volume move l1:"${pool}"/iso1 l2:"${remote_pool}"/iso2
-  lxc storage volume show l2:"${remote_pool}" iso2 | grep -q 'content_type: iso'
+  lxc storage volume show l2:"${remote_pool}" iso2 | grep -xF 'content_type: iso'
   ! lxc storage volume show l1:"${pool}" iso1 || false
 
   lxc storage volume delete l2:"${remote_pool}" iso1
