@@ -16,6 +16,24 @@ test_network() {
   lxc delete -f def0
   lxc network delete lxdt$$
 
+  # Check that we return bridge informatin for ovs bridges
+  systemctl start openvswitch-switch
+  ip link add dummy0 type dummy
+  ovs-vsctl add-br ovs-br0
+  ovs-vsctl add-port ovs-br0 dummy0 tag=9
+  lxc network info ovs-br0 | grep -xF "Bridge:"
+
+  # Check that we are able to return linux bridge information if ovs service is disabled
+  systemctl stop openvswitch-switch
+  ip link add native-br0 type bridge
+  lxc network info native-br0 | grep -xF "Bridge:"
+
+  # Cleanup
+  systemctl start openvswitch-switch
+  ovs-vsctl del-br ovs-br0
+  ip link delete native-br0
+  ip link delete dummy0
+
   # Standard bridge with random subnet and a bunch of options
   lxc network create lxdt$$
   lxc network set lxdt$$ dns.mode dynamic
@@ -35,9 +53,9 @@ test_network() {
 
   # check ipv4.address and ipv6.address can be unset without triggering random subnet generation.
   lxc network unset lxdt$$ ipv4.address
-  ! lxc network show lxdt$$ | grep ipv4.address || false
+  ! lxc network show lxdt$$ | grep -F ipv4.address || false
   lxc network unset lxdt$$ ipv6.address
-  ! lxc network show lxdt$$ | grep ipv6.address || false
+  ! lxc network show lxdt$$ | grep -F ipv6.address || false
 
   # check ipv4.address and ipv6.address can be regenerated on update using "auto" value.
   lxc network set lxdt$$ ipv4.address auto
@@ -51,13 +69,13 @@ test_network() {
   # edit network description
   lxc network create lxdt$$
   lxc network show lxdt$$ | sed 's/^description:.*/description: foo/' | lxc network edit lxdt$$
-  lxc network show lxdt$$ | grep -q 'description: foo'
+  lxc network show lxdt$$ | grep -xF 'description: foo'
   lxc network delete lxdt$$
 
   # rename network
   lxc network create lxdt$$
   lxc network rename lxdt$$ newnet$$
-  lxc network list | grep -qv lxdt$$  # the old name is gone
+  ! lxc network list | grep -wF "lxdt$$" || false # the old name is gone
   lxc network delete newnet$$
 
   # Unconfigured bridge
@@ -70,13 +88,13 @@ test_network() {
 
   expected_state=$(lxc network info dummy0 | grep -F "State:")
   expected_type=$(lxc network info dummy0 | grep -F "Type:")
-  lxc network info lxdt$$ | grep -qF "${expected_state}"
-  lxc network info lxdt$$ | grep -qF "${expected_type}"
+  lxc network info lxdt$$ | grep -xF "${expected_state}"
+  lxc network info lxdt$$ | grep -xF "${expected_type}"
 
   # Delete physical network and check for expected response
   ip link delete dummy0
-  lxc network info lxdt$$ | grep -qF "State: unavailable"
-  lxc network info lxdt$$ | grep -qF "Type: unknown"
+  lxc network info lxdt$$ | grep -xF "State: unavailable"
+  lxc network info lxdt$$ | grep -xF "Type: unknown"
 
   lxc network delete lxdt$$
 
@@ -99,33 +117,28 @@ test_network() {
   lxc config device set outsider eth0 ipv4.address "${v4_addr_foo}" --project foo
   lxc config device set outsider eth0 ipv6.address "${v6_addr_foo}" --project foo
 
-  lxc network list-leases lxdt$$ | grep STATIC | grep -q "${v4_addr}"
-  lxc network list-leases lxdt$$ | grep STATIC | grep -q "${v6_addr}"
-  lxc network list-leases lxdt$$ --project foo | grep STATIC | grep -q "${v4_addr_foo}"
-  lxc network list-leases lxdt$$ --project foo | grep STATIC | grep -q "${v6_addr_foo}"
+  lxc network list-leases lxdt$$ | grep -wF STATIC | grep -F "${v4_addr}"
+  lxc network list-leases lxdt$$ | grep -wF STATIC | grep -F "${v6_addr}"
+  lxc network list-leases lxdt$$ --project foo | grep -wF STATIC | grep -F "${v4_addr_foo}"
+  lxc network list-leases lxdt$$ --project foo | grep -wF STATIC | grep -F "${v6_addr_foo}"
 
   # Request DHCPv6 lease (if udhcpc6 is in busybox image).
-  busyboxUdhcpc6=1
-  if ! lxc exec nettest -- busybox --list | grep udhcpc6 ; then
-    busyboxUdhcpc6=0
-  fi
-
-  if [ "$busyboxUdhcpc6" = "1" ]; then
-    lxc exec nettest -- udhcpc6 -f -i eth0 -n -q -t5 2>&1 | grep 'IPv6 obtained'
+  if lxc exec nettest -- busybox --list | grep -wF udhcpc6 ; then
+    lxc exec nettest -- udhcpc6 -f -i eth0 -n -q -t5 2>&1 | grep -F 'IPv6 obtained'
   fi
 
   # Check IPAM information
   net_ipv4="$(lxc network get lxdt$$ ipv4.address)"
   net_ipv6="$(lxc network get lxdt$$ ipv6.address)"
 
-  lxc network list-allocations | grep -e "${net_ipv4}" -e "${net_ipv6}"
-  lxc network list-allocations | grep -e "/1.0/networks/lxdt$$" -e "/1.0/instances/nettest"
-  lxc network list-allocations | grep -e "${v4_addr}" -e "${v6_addr}"
-  lxc network list-allocations localhost: | grep -e "${net_ipv4}" -e "${net_ipv6}"
-  lxc network list-allocations localhost: | grep -e "/1.0/networks/lxdt$$" -e "/1.0/instances/nettest"
-  lxc network list-allocations localhost: | grep -e "${v4_addr}" -e "${v6_addr}"
-  lxc network list-allocations --format csv | grep -q "/1.0/instances/outsider?project=foo,${v4_addr_foo}"
-  lxc network list-allocations --format csv | grep -q "/1.0/instances/outsider?project=foo,${v6_addr_foo}"
+  lxc network list-allocations | grep -F -e "${net_ipv4}" -e "${net_ipv6}"
+  lxc network list-allocations | grep -F -e "/1.0/networks/lxdt$$" -e "/1.0/instances/nettest"
+  lxc network list-allocations | grep -F -e "${v4_addr}" -e "${v6_addr}"
+  lxc network list-allocations localhost: | grep -F -e "${net_ipv4}" -e "${net_ipv6}"
+  lxc network list-allocations localhost: | grep -F -e "/1.0/networks/lxdt$$" -e "/1.0/instances/nettest"
+  lxc network list-allocations localhost: | grep -F -e "${v4_addr}" -e "${v6_addr}"
+  lxc network list-allocations --format csv | grep -F "/1.0/instances/outsider?project=foo,${v4_addr_foo}"
+  lxc network list-allocations --format csv | grep -F "/1.0/instances/outsider?project=foo,${v6_addr_foo}"
 
   lxc delete -f outsider --project foo
   lxc project delete foo
