@@ -10,39 +10,31 @@ test_pki() {
   (
     set -e
     cd "${TEST_DIR}/pki"
-    export EASYRSA_KEY_SIZE=4096
 
-    # shellcheck disable=SC1091
-    if [ -e pkitool ]; then
-        . ./vars
-        ./clean-all
-        ./pkitool --initca
-        ./pkitool lxd-client
-        ./pkitool lxd-client-revoked
-        # This will revoke the certificate but fail in the end as it tries to then verify the revoked certificate.
-        ./revoke-full lxd-client-revoked || true
-    else
-        ./easyrsa init-pki
-        echo "lxd" | ./easyrsa build-ca nopass
-        ./easyrsa gen-crl
-        ./easyrsa build-client-full restricted nopass
-        ./easyrsa build-client-full unrestricted nopass
-        ./easyrsa build-client-full fine-grained nopass
-        ./easyrsa build-client-full ca-trusted nopass
-        ./easyrsa build-client-full prior-revoked nopass
-        mkdir keys
-        cp pki/private/* keys/
-        cp pki/issued/* keys/
-        cp pki/ca.crt keys/
-        echo "yes" | ./easyrsa revoke prior-revoked
-        ./easyrsa gen-crl
-        cp pki/crl.pem keys/
-    fi
+    cat << EOF > vars
+set_var EASYRSA_ALGO ec
+set_var EASYRSA_BATCH yes
+EOF
+
+    ./easyrsa init-pki
+    echo "lxd" | ./easyrsa build-ca nopass
+    ./easyrsa gen-crl
+    ./easyrsa build-client-full restricted nopass
+    ./easyrsa build-client-full unrestricted nopass
+    ./easyrsa build-client-full fine-grained nopass
+    ./easyrsa build-client-full ca-trusted nopass
+    ./easyrsa build-client-full prior-revoked nopass
+    mkdir keys
+    cp pki/private/* keys/
+    cp pki/issued/* keys/
+    cp pki/ca.crt keys/
+    echo "yes" | ./easyrsa revoke prior-revoked
+    ./easyrsa gen-crl
+    cp pki/crl.pem keys/
   )
 
   # Setup the daemon.
   LXD5_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD5_DIR}"
   spawn_lxd "${LXD5_DIR}" true
   LXD5_ADDR=$(cat "${LXD5_DIR}/lxd.addr")
 
@@ -106,7 +98,7 @@ test_pki() {
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
     # Remove cert from truststore.
     fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
@@ -162,7 +154,7 @@ test_pki() {
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
     # Remove cert from truststore.
     fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
@@ -223,7 +215,7 @@ test_pki() {
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
     # Remove cert from truststore.
     lxc auth identity delete "tls/${fingerprint}"
@@ -273,7 +265,7 @@ test_pki() {
     # Check that we no longer have access.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
     # Re-enable `core.trust_ca_certificates`.
     lxc config set core.trust_ca_certificates true
@@ -289,7 +281,7 @@ test_pki() {
     # Check that we no longer have access (certificate was previously trusted, but is now revoked).
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
     # Remove remote.
     lxc remote remove pki-lxd
@@ -333,7 +325,7 @@ test_pki() {
     ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token=incorrect || false
 
     # Check we can't access anything with the revoked certificate.
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
   )
 
   # Confirm that we cannot add a remote using a certificate that is not signed by the CA.
@@ -349,7 +341,7 @@ test_pki() {
   lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
   ! lxc_remote ls pki-lxd: || false
   cat "${LXD_CONF}/client.crt" "${LXD_CONF}/client.key" > "${LXD_CONF}/client.pem"
-  [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
+  [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
 
 
   ### Show that mTLS still works for server certificates:
