@@ -1,14 +1,15 @@
 # Use the default project.
 test_projects_default() {
   # The default project is used by the default profile
-  lxc project show default | grep -q "/1.0/profiles/default$"
+  lxc project show default | grep -xF -- "- /1.0/profiles/default"
 
   # Containers and images are assigned to the default project
   ensure_import_testimage
   lxc init testimage c1
-  lxc project show default | grep -q "/1.0/profiles/default$"
-  lxc project show default | grep -q "/1.0/images/"
+  lxc project show default | grep -xF -- "- /1.0/profiles/default"
+  lxc project show default | grep -F -- "- /1.0/images/"
   lxc delete c1
+  lxc image delete testimage
 }
 
 # CRUD operations on project.
@@ -17,12 +18,13 @@ test_projects_crud() {
   lxc project create foo
 
   # All features are enabled by default
-  lxc project show foo | grep -q 'features.images: "true"'
-  lxc project get foo "features.profiles" | grep -q 'true'
+  lxc project show foo | grep -F 'features.images: "true"'
+  [ "$(lxc project get foo "features.profiles")" = "true" ]
 
   # Set a limit
   lxc project set foo limits.containers 10
-  lxc project show foo | grep -q 'limits.containers: "10"'
+  lxc project show foo | grep -F 'limits.containers: "10"'
+  [ "$(lxc project get foo limits.containers)" = "10" ]
 
   # Trying to create a project with the same name fails
   ! lxc project create foo || false
@@ -39,14 +41,14 @@ test_projects_crud() {
 
   # Edit the project
   lxc project show bar| sed 's/^description:.*/description: "Bar project"/' | lxc project edit bar
-  lxc project show bar | grep -q "description: Bar project"
+  lxc project show bar | grep -xF "description: Bar project"
 
   # Edit the project config via PATCH. Existing key/value pairs should remain or be updated.
   lxc query -X PATCH -d '{\"config\" : {\"limits.memory\":\"5GiB\",\"features.images\":\"false\"}}' /1.0/projects/bar
-  lxc project show bar | grep -q 'limits.memory: 5GiB'
-  lxc project show bar | grep -q 'features.images: "false"'
-  lxc project show bar | grep -q 'features.profiles: "true"'
-  lxc project show bar | grep -q 'limits.containers: "10"'
+  lxc project show bar | grep -F 'limits.memory: 5GiB'
+  lxc project show bar | grep -F 'features.images: "false"'
+  lxc project show bar | grep -F 'features.profiles: "true"'
+  lxc project show bar | grep -F 'limits.containers: "10"'
 
   # Create a second project
   lxc project create foo
@@ -59,12 +61,12 @@ test_projects_crud() {
   # Turning off the profiles feature makes the project see the default profile
   # from the default project.
   lxc project set foo features.profiles false
-  lxc profile show default | grep -E -q '^description: Default LXD profile$'
+  lxc profile show default | grep -xF 'description: Default LXD profile'
 
   # Turning on the profiles feature creates a project-specific default
   # profile.
   lxc project set foo features.profiles true
-  lxc profile show default | grep -E -q '^description: Default LXD profile for project foo$'
+  lxc profile show default | grep -xF 'description: Default LXD profile for project foo'
 
   # Invalid config values are rejected.
   ! lxc project set foo garbage xxx || false
@@ -76,7 +78,7 @@ test_projects_crud() {
   lxc project delete bar
 
   # We're back to the default project
-  lxc project list | grep -q "default (current)"
+  lxc project list | grep -F "default (current)"
 }
 
 # Use containers in a project.
@@ -96,30 +98,30 @@ test_projects_containers() {
   lxc init testimage c1
 
   # The container is listed when using this project
-  lxc list | grep -q c1
-  lxc info c1 | grep -q "Name: c1"
+  lxc list -c n | grep -wF c1
+  lxc info c1 | grep -xF "Name: c1"
 
   # The container's volume is listed too.
-  lxc storage volume list "${pool}" | grep container | grep -q c1
+  lxc storage volume list "${pool}" | grep -wF container | grep -wF c1
 
   # For backends with optimized storage, we can see the image volume inside the
   # project.
   driver="$(storage_backend "$LXD_DIR")"
   if [ "${driver}" != "dir" ]; then
-      lxc storage volume list "${pool}" | grep image | grep -q "${fingerprint}"
+      lxc storage volume list "${pool}" | grep -wF image | grep -F "${fingerprint}"
   fi
 
   # Start the container
   lxc start c1
-  lxc list | grep c1 | grep -q RUNNING
-  echo "abc" | lxc exec c1 cat | grep -q abc
+  [ "$(lxc list -f csv -c ns c1)" = "c1,RUNNING" ]
+  [ "$(echo "abc" | lxc exec c1 cat)" = "abc" ]
 
   # The container can't be managed when using the default project
   lxc project switch default
-  ! lxc list | grep -q c1 || false
+  ! lxc list -c n | grep -wF c1 || false
   ! lxc info c1 || false
   ! lxc delete c1 || false
-  ! lxc storage volume list "${pool}" | grep container | grep -q c1 || false
+  ! lxc storage volume list "${pool}" | grep -wF container | grep -wF c1 || false
 
   # Trying to delete a project which is in use fails
   ! lxc project delete foo || false
@@ -127,20 +129,19 @@ test_projects_containers() {
   # Trying to change features of a project which is in use fails
   ! lxc project show foo| sed 's/features.profiles:.*/features.profiles: "false"/' | lxc project edit foo || false
   ! lxc project set foo "features.profiles" "false" || false
-  lxc project show foo | grep -q 'features.profiles: "true"'
+  lxc project show foo | grep -F 'features.profiles: "true"'
 
   # Create a container with the same name in the default project
   ensure_import_testimage
   lxc init testimage c1
   lxc start c1
-  lxc list | grep c1 | grep -q RUNNING
+  [ "$(lxc list -f csv -c ns c1)" = "c1,RUNNING" ]
   lxc stop --force c1
 
   # Delete the container
   lxc project switch foo
 
-  lxc stop --force c1
-  lxc delete c1
+  lxc delete --force c1
   lxc image delete testimage
 
   # Delete the project
@@ -148,9 +149,8 @@ test_projects_containers() {
 
   # The container in the default project can still be used
   lxc start c1
-  lxc list | grep c1 | grep -q RUNNING
-  lxc stop --force c1
-  lxc delete c1
+  [ "$(lxc list -f csv -c ns c1)" = "c1,RUNNING" ]
+  lxc delete --force c1
 }
 
 # Copy/move between projects
@@ -214,8 +214,8 @@ test_projects_snapshots() {
 
   # Create, rename, restore and delete a snapshot
   lxc snapshot c1
-  lxc info c1 | grep -q snap0
-  lxc config show c1/snap0 | grep -q BusyBox
+  lxc info c1 | grep -wF snap0
+  lxc config show c1/snap0 | grep -wF BusyBox
   lxc rename c1/snap0 c1/foo
   lxc restore c1 foo
   lxc delete c1/foo
@@ -292,36 +292,36 @@ test_projects_profiles() {
   lxc project switch foo
 
   # List profiles
-  lxc profile list | grep -q 'default'
-  lxc profile show default | grep -q 'description: Default LXD profile for project foo'
+  lxc profile list | grep -wF 'default'
+  lxc profile show default | grep -xF 'description: Default LXD profile for project foo'
 
   # Create a profile in this project
   lxc profile create p1
-  lxc profile list | grep -q 'p1'
+  lxc profile list | grep -wF 'p1'
 
   # Set a config key on this profile
   lxc profile set p1 user.x y
-  lxc profile get p1 user.x | grep -q 'y'
+  [ "$(lxc profile get p1 user.x)" = "y" ]
 
   # The profile is not visible in the default project
   lxc project switch default
-  ! lxc profile list | grep -q 'p1' || false
+  ! lxc profile list | grep -wF 'p1' || false
 
   # The profile is visible in the default project when --all-projects is used
-  lxc profile list --all-projects | grep -q 'p1'
+  lxc profile list --all-projects | grep -wF 'p1'
 
   # A profile with the same name can be created in the default project
   lxc profile create p1
 
   # The same key can have a different value
   lxc profile set p1 user.x z
-  lxc profile get p1 user.x | grep -q 'z'
+  [ "$(lxc profile get p1 user.x)" = "z" ]
 
   # Switch back to the project
   lxc project switch foo
 
   # The profile has still the original config
-  lxc profile get p1 user.x | grep -q 'y'
+  [ "$(lxc profile get p1 user.x)" = "y" ]
 
   # Delete the profile from the project
   lxc profile delete p1
@@ -337,9 +337,9 @@ test_projects_profiles() {
   lxc profile set --project default default user.x z
   lxc profile copy --project default --target-project foo default bar
   # copy to an existing profile without --refresh should fail
-  ! lxc profile copy --project default --target-project foo default bar
+  ! lxc profile copy --project default --target-project foo default bar || false
   lxc profile copy --project default --target-project foo default bar --refresh
-  lxc profile get --project foo bar user.x | grep -q 'z'
+  [ "$(lxc profile get --project foo bar user.x)" = "z" ]
   lxc profile copy --project default --target-project foo default bar-non-existent --refresh
   lxc profile delete bar --project foo
   lxc profile delete bar-non-existent --project foo
@@ -410,7 +410,7 @@ test_projects_images() {
 
   # The imported image is not visible in the default project.
   lxc project switch default
-  ! lxc image list | grep -q "${fingerprint}" || false
+  ! lxc image list | grep -F "${fingerprint}" || false
 
   # Switch back to the project and clean it up.
   lxc project switch foo
@@ -420,11 +420,11 @@ test_projects_images() {
   deps/import-busybox --project foo --alias foo-image
 
   # The image alias shows up in the project
-  lxc image list | grep -q foo-image
+  lxc image list | grep -wF foo-image
 
   # However the image alias is not visible in the default project.
   lxc project switch default
-  ! lxc image list | grep -q foo-project || false
+  ! lxc image list | grep -wF foo-project || false
 
   # Let's import the same image in the default project
   ensure_import_testimage
@@ -433,7 +433,7 @@ test_projects_images() {
   lxc project switch foo
 
   # The image alias from the default project is not visible here
-  ! lxc image list | grep -q testimage || false
+  ! lxc image list | grep -wF testimage || false
 
   # Rename the image alias in the project using the same it has in the default
   # one.
@@ -450,7 +450,7 @@ test_projects_images() {
   lxc project delete foo
 
   # We automatically switched to the default project, which still has the alias
-  lxc image list | grep -q testimage
+  lxc image list | grep -wF testimage
 }
 
 # Use global images in a project.
@@ -459,33 +459,31 @@ test_projects_images_default() {
   ensure_import_testimage
 
   # Create a new project, without the features.images config.
-  lxc project create foo
+  lxc project create foo --config features.images=false
   lxc project switch foo
-  lxc project set foo "features.images" "false"
 
   # Create another project, without the features.images config.
-  lxc project create bar
-  lxc project set bar "features.images" "false"
+  lxc project create bar --config features.images=false
 
   # The project can see images from the default project
-  lxc image list | grep -q testimage
+  lxc image list | grep -wF testimage
 
   # The image from the default project has correct profile assigned
   fingerprint="$(lxc image list --format json | jq -r .[0].fingerprint)"
-  lxc query "/1.0/images/${fingerprint}?project=foo" | jq -r ".profiles[0]" | grep -xq default
+  [ "$(lxc query "/1.0/images/${fingerprint}?project=foo" | jq -r ".profiles[0]")" = "default" ]
 
   # The project can delete images in the default project
   lxc image delete testimage
 
   # Images imported into the project show up in the default project
   deps/import-busybox --project foo --alias foo-image
-  lxc image list | grep -q foo-image
+  lxc image list | grep -wF foo-image
   lxc project switch default
-  lxc image list | grep -q foo-image
+  lxc image list | grep -wF foo-image
 
   # Correct profile assigned to images from another project
   fingerprint="$(lxc image list --format json | jq -r '.[] | select(.aliases[0].name == "foo-image") | .fingerprint')"
-  lxc query "/1.0/images/${fingerprint}?project=bar" | jq -r ".profiles[0]" | grep -xq default
+  [ "$(lxc query "/1.0/images/${fingerprint}?project=bar" | jq -r ".profiles[0]")" = "default" ]
 
   lxc image delete foo-image
 
@@ -502,25 +500,25 @@ test_projects_storage() {
   lxc project create foo -c features.storage.volumes=false
   lxc project switch foo
 
-  lxc storage volume list "${pool}" | grep custom | grep -q vol
+  lxc storage volume list "${pool}" | grep -wF custom | grep -wF vol
 
   lxc storage volume delete "${pool}" vol
 
   lxc project switch default
 
-  ! lxc storage volume list "${pool}" | grep custom | grep -q vol || false
+  ! lxc storage volume list "${pool}" | grep -wF custom | grep -F vol || false
 
   lxc project set foo features.storage.volumes=true
   lxc storage volume create "${pool}" vol
   lxc project switch foo
-  ! lxc storage volume list "${pool}" | grep custom | grep -q vol
+  ! lxc storage volume list "${pool}" | grep -wF custom | grep -wF vol || false
 
   lxc storage volume create "${pool}" vol
   lxc storage volume delete "${pool}" vol
 
   lxc storage volume create "${pool}" vol2
   lxc project switch default
-  ! lxc storage volume list "${pool}" | grep custom | grep -q vol2
+  ! lxc storage volume list "${pool}" | grep -wF custom | grep -wF vol2 || false
 
   lxc project switch foo
   lxc storage volume delete "${pool}" vol2
@@ -548,7 +546,7 @@ test_projects_network() {
   # Create a container in the project
   lxc init -n "${network}" testimage c1
 
-  lxc network show "${network}" | grep -q "/1.0/instances/c1?project=foo"
+  lxc network show "${network}" | grep -xF -- "- /1.0/instances/c1?project=foo"
 
   # Delete the container
   lxc delete c1
@@ -585,8 +583,8 @@ test_projects_limits() {
   lxc project set p1 limits.disk.pool.limit1=0
   lxc project set p1 limits.disk.pool.limit2=0
 
-  ! lxc storage list | grep -q limit1 || false
-  ! lxc storage list | grep -q limit2 || false
+  ! lxc storage list | grep -F limit1 || false
+  ! lxc storage list | grep -F limit2 || false
 
   lxc storage volume create "${pool}" foo size=10MiB
   ! lxc storage volume create "${pool}" bar size=50MiB || false
@@ -807,7 +805,6 @@ test_projects_limits() {
     # Add a remote LXD to be used as image server.
     local LXD_REMOTE_DIR
     LXD_REMOTE_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-    chmod +x "${LXD_REMOTE_DIR}"
 
     # Switch to default project to spawn new LXD server, and then switch back to p1.
     lxc project switch default
@@ -881,62 +878,154 @@ test_projects_limits() {
   fi
 }
 
-# Set restrictions on projects.
 test_projects_restrictions() {
+  run_projects_restrictions local # Unix socket
+
+  ensure_has_localhost_remote "${LXD_ADDR}" # Unrestricted TLS client
+  run_projects_restrictions localhost
+
+  LXD_CONF1=$(mktemp -d -p "${TEST_DIR}" XXX)
+  LXD_CONF="${LXD_CONF1}" gen_cert_and_key "client"
+  trust_token="$(lxc config trust add --name test-user1 --quiet --restricted)"
+  LXD_CONF="${LXD_CONF1}" lxc remote add restricted "${trust_token}"
+  LXD_CONF="${LXD_CONF1}" run_projects_restrictions restricted # Restricted TLS client
+
+  LXD_CONF2=$(mktemp -d -p "${TEST_DIR}" XXX)
+  LXD_CONF="${LXD_CONF2}" gen_cert_and_key "client"
+  lxc auth group create test-group
+  pending_identity_token="$(lxc auth identity create tls/test-user2 --quiet --group test-group)"
+  LXD_CONF="${LXD_CONF2}" lxc remote add fine-grained "${pending_identity_token}"
+  LXD_CONF="${LXD_CONF2}" run_projects_restrictions fine-grained # Fine-grained TLS client
+
+  lxc config trust remove "$(cert_fingerprint "${LXD_CONF1}/client.crt")"
+  lxc auth identity delete tls/test-user2
+  rm -rf "${LXD_CONF1}" "${LXD_CONF2}"
+}
+
+# Set restrictions on projects.
+run_projects_restrictions() {
+  # Switch to the given remote. The remote name tells us what kind of access we have. We'll use the local: remote when
+  # we need to perform privileged actions to set up test assertions.
+  remote="${1}"
+  lxc remote switch "${remote}"
+
   # Add a managed network.
   netManaged="lxd$$"
-  lxc network create "${netManaged}"
+  lxc network create "local:${netManaged}"
 
   netUnmanaged="${netManaged}-unm"
   ip link add "${netUnmanaged}" type bridge
 
-  # Create a project and switch to it
-  lxc project create p1 -c features.storage.volumes=false
+  # Create a project
+  lxc project create local:p1 -c features.storage.volumes=false
+
+  # Grant access to the project.
+  if [ "${remote}" = "restricted" ]; then
+    # For the restricted client we need to grant access to the project first.
+    # shellcheck disable=SC2153
+    fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
+    lxc config trust show "${fingerprint}" | sed -e 's/projects: \[\]/projects: ["p1"]/' | lxc config trust edit "local:${fingerprint}"
+  elif [ "${remote}" = "fine-grained" ]; then
+    # For the fine grained client we'll grant (almost) equivalent access to the restricted certificate, with the exception
+    # that networks in the default project can never be modified by this user via the "punching through" that the feature flags allow
+    # with restricted certs.
+    lxc auth group permission add local:test-group project p1 operator
+    lxc auth group permission add local:test-group project default can_view_networks
+    lxc auth group permission add local:test-group server can_view_unmanaged_networks
+  fi
+
+  # Switch to the project. Also switch the local remote if not already set.
   lxc project switch p1
+  if [ "${remote}" != "local" ]; then
+    lxc remote switch local
+    lxc project switch p1
+    lxc remote switch "${remote}"
+  fi
 
   # Check with restricted unset and restricted.devices.nic unset that managed & unmanaged networks are accessible.
   lxc network list | grep -F "${netManaged}"
   lxc network list | grep -F "${netUnmanaged}"
   lxc network show "${netManaged}"
   lxc network show "${netUnmanaged}"
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    lxc network list --all-projects | grep -F "${netManaged}"
+    lxc network list --all-projects | grep -F "${netUnmanaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   # Check with restricted unset and restricted.devices.nic=block that managed & unmanaged networks are accessible.
-  lxc project set p1 restricted.devices.nic=block
+  lxc project set local:p1 restricted.devices.nic=block
   lxc network list | grep -F "${netManaged}"
   lxc network list | grep -F "${netUnmanaged}"
   lxc network show "${netManaged}"
   lxc network show "${netUnmanaged}"
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    lxc network list --all-projects | grep -F "${netManaged}"
+    lxc network list --all-projects | grep -F "${netUnmanaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   # Check with restricted=true and restricted.devices.nic=block that managed & unmanaged networks are inaccessible.
-  lxc project set p1 restricted=true
+  lxc project set local:p1 restricted=true
   ! lxc network list | grep -F "${netManaged}"|| false
   ! lxc network show "${netManaged}" || false
   ! lxc network list | grep -F "${netUnmanaged}"|| false
   ! lxc network show "${netUnmanaged}" || false
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    # Can view when performing an --all-projects request because the network is actually defined in the default project,
+    # and the default project is not restricted.
+    lxc network list --all-projects | grep -F "${netManaged}"
+    lxc network list --all-projects | grep -F "${netUnmanaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   # Check with restricted=true and restricted.devices.nic=managed that managed networks are accessible and that
   # unmanaged networks are inaccessible.
-  lxc project set p1 restricted.devices.nic=managed
+  lxc project set local:p1 restricted.devices.nic=managed
   lxc network list | grep -F "${netManaged}"
   lxc network show "${netManaged}"
   ! lxc network list | grep -F "${netUnmanaged}"|| false
-  ! lxc network show "${netUnmanaged}" || false
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    # Can view when performing an --all-projects request because the network is actually defined in the default project,
+    # and the default project is not restricted.
+    lxc network list --all-projects | grep -F "${netManaged}"
+    lxc network list --all-projects | grep -F "${netUnmanaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   # Check with restricted.devices.nic=allow and restricted.networks.access set to a network other than the existing
   # managed and unmanaged ones that they are inaccessible.
-  lxc project set p1 restricted.devices.nic=allow
-  lxc project set p1 restricted.networks.access=foo
+  lxc project set local:p1 restricted.devices.nic=allow
+  lxc project set local:p1 restricted.networks.access=foo
   ! lxc network list | grep -F "${netManaged}"|| false
   ! lxc network show "${netManaged}" || false
-  ! lxc network info "${netManaged}"|| false
+  ! lxc network info "${netManaged}" || false
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    # Can view when performing an --all-projects request because the network is actually defined in the default project,
+    # and the default project is not restricted.
+    lxc network list --all-projects | grep -F "${netManaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   ! lxc network list | grep -F "${netUnmanaged}"|| false
   ! lxc network show "${netUnmanaged}" || false
-  ! lxc network info "${netUnmanaged}"|| false
+  ! lxc network info "${netUnmanaged}" || false
+  if [ "${remote}" = "local" ] || [ "${remote}" = "localhost" ] || [ "${remote}" = "fine-grained" ]; then
+    # Can view when performing an --all-projects request because the network is actually defined in the default project,
+    # and the default project is not restricted.
+    lxc network list --all-projects | grep -F "${netUnmanaged}"
+  else
+    ! lxc network list --all-projects || false
+  fi
 
   ! lxc network set "${netManaged}" user.foo=bah || false
   ! lxc network get "${netManaged}" ipv4.address || false
-  ! lxc network info "${netManaged}"|| false
+  ! lxc network info "${netManaged}" || false
   ! lxc network delete "${netManaged}" || false
 
   ! lxc profile device add default eth0 nic nictype=bridge parent=netManaged || false
@@ -945,22 +1034,22 @@ test_projects_restrictions() {
   ip link delete "${netUnmanaged}"
 
   # Disable restrictions to allow devices to be added to profile.
-  lxc project unset p1 restricted.networks.access
-  lxc project set p1 restricted.devices.nic=managed
-  lxc project set p1 restricted=false
+  lxc project unset local:p1 restricted.networks.access
+  lxc project set local:p1 restricted.devices.nic=managed
+  lxc project set local:p1 restricted=false
 
   # Add a root device to the default profile of the project and import an image.
   pool="lxdtest-$(basename "${LXD_DIR}")"
-  lxc profile device add default root disk path="/" pool="${pool}"
+  lxc profile device add local:default root disk path="/" pool="${pool}"
 
   deps/import-busybox --project p1 --alias testimage
   fingerprint="$(lxc image list -c f --format json | jq -r .[0].fingerprint)"
 
   # Add a volume.
-  lxc storage volume create "${pool}" "v-proj$$"
+  lxc storage volume create "local:${pool}" "v-proj$$"
 
   # Enable all restrictions.
-  lxc project set p1 restricted=true
+  lxc project set local:p1 restricted=true
 
   # It's not possible to create nested containers.
   ! lxc profile set default security.nesting=true || false
@@ -1001,46 +1090,46 @@ test_projects_restrictions() {
 
   # It's not possible to set restricted.containers.nic to 'block' because
   # there's an instance using the managed network.
-  ! lxc project set p1 restricted.devices.nic=block || false
+  ! lxc project set local:p1 restricted.devices.nic=block || false
 
   # Relaxing restricted.containers.nic to 'allow' makes it possible to attach
   # raw network devices.
-  lxc project set p1 restricted.devices.nic=allow
+  lxc project set local:p1 restricted.devices.nic=allow
   lxc config device add c1 eth1 nic nictype=p2p
 
   # Relaxing restricted.containers.disk to 'allow' makes it possible to attach
   # non-managed disks.
-  lxc project set p1 restricted.devices.disk=allow
+  lxc project set local:p1 restricted.devices.disk=allow
   lxc config device add c1 testdir disk source="${TEST_DIR}" path=/foo
 
   # Relaxing restricted.containers.lowlevel to 'allow' makes it possible set
   # low-level keys.
-  lxc project set p1 restricted.containers.lowlevel=allow
+  lxc project set local:p1 restricted.containers.lowlevel=allow
   lxc config set c1 "raw.idmap=both 0 0"
 
   lxc delete c1
 
   # Setting restricted.containers.disk to 'block' allows only the root disk
   # device.
-  lxc project set p1 restricted.devices.disk=block
+  lxc project set local:p1 restricted.devices.disk=block
   ! lxc profile device add default data disk pool="${pool}" path=/mnt source="v-proj$$" || false
 
-  restrictedDir="/opt/projects_restricted"
+  restrictedDir="${TEST_DIR}/projects_restricted"
   mkdir "${restrictedDir}"
-  tmpDir=$(mktemp -d)
+  tmpDir=$(mktemp -d -p "${TEST_DIR}" XXX)
   optDir=$(mktemp -d --tmpdir="${restrictedDir}")
 
   # Block unmanaged disk devices
-  lxc project set p1 restricted.devices.disk=managed
+  lxc project set local:p1 restricted.devices.disk=managed
   ! lxc profile device add default data disk path=/mnt source="${tmpDir}" || false
 
   # Allow unmanaged disk devices
-  lxc project set p1 restricted.devices.disk=allow
+  lxc project set local:p1 restricted.devices.disk=allow
   lxc profile device add default data disk path=/mnt source="${tmpDir}"
   lxc profile device remove default data
 
   # Path restrictions
-  lxc project set p1 restricted.devices.disk.paths="${restrictedDir}"
+  lxc project set local:p1 restricted.devices.disk.paths="${restrictedDir}"
   ! lxc profile device add default data disk path=/mnt source="${tmpDir}" || false
   lxc profile device add default data disk path=/mnt source="${optDir}"
   lxc profile device remove default data
@@ -1049,54 +1138,58 @@ test_projects_restrictions() {
 
   # Setting restricted.containers.nesting to 'allow' makes it possible to create
   # nested containers.
-  lxc project set p1 restricted.containers.nesting=allow
+  lxc project set local:p1 restricted.containers.nesting=allow
   lxc init testimage c1 -c security.nesting=true
 
   # It's not possible to set restricted.containers.nesting back to 'block',
   # because there's an instance with security.nesting=true.
-  ! lxc project set p1 restricted.containers.nesting=block || false
+  ! lxc project set local:p1 restricted.containers.nesting=block || false
 
   lxc delete c1
 
   # Setting restricted.containers.lowlevel to 'allow' makes it possible to set
   # low-level options.
-  lxc project set p1 restricted.containers.lowlevel=allow
+  lxc project set local:p1 restricted.containers.lowlevel=allow
   lxc init testimage c1 -c "raw.idmap=both 0 0"
 
   # It's not possible to set restricted.containers.lowlevel back to 'block',
   # because there's an instance with raw.idmap set.
-  ! lxc project set p1 restricted.containers.lowlevel=block || false
+  ! lxc project set local:p1 restricted.containers.lowlevel=block || false
 
   lxc delete c1
 
   # Setting restricted.containers.privilege to 'allow' makes it possible to create
   # privileged containers.
-  lxc project set p1 restricted.containers.privilege=allow
+  lxc project set local:p1 restricted.containers.privilege=allow
   lxc init testimage c1 -c security.privileged=true
 
   # It's not possible to set restricted.containers.privilege back to
   # 'unprivileged', because there's an instance with security.privileged=true.
-  ! lxc project set p1 restricted.containers.privilege=unprivileged || false
+  ! lxc project set local:p1 restricted.containers.privilege=unprivileged || false
 
   # Test expected syscall interception behavior.
-  ! lxc config set c1 security.syscalls.intercept.mknod=true || false
-  lxc config set c1 security.syscalls.intercept.mknod=false
-  lxc project set p1 restricted.containers.interception=block
-  ! lxc config set c1 security.syscalls.intercept.mknod=true || false
-  lxc project set p1 restricted.containers.interception=allow
-  lxc config set c1 security.syscalls.intercept.mknod=true
-  lxc config set c1 security.syscalls.intercept.mount=true
-  ! lxc config set c1 security.syscalls.intercept.mount.allow=ext4 || false
+  ! lxc config set local:c1 security.syscalls.intercept.mknod=true || false
+  lxc config set local:c1 security.syscalls.intercept.mknod=false
+  lxc project set local:p1 restricted.containers.interception=block
+  ! lxc config set local:c1 security.syscalls.intercept.mknod=true || false
+  lxc project set local:p1 restricted.containers.interception=allow
+  lxc config set local:c1 security.syscalls.intercept.mknod=true
+  lxc config set local:c1 security.syscalls.intercept.mount=true
+  ! lxc config set local:c1 security.syscalls.intercept.mount.allow=ext4 || false
 
   lxc delete c1
 
   lxc image delete testimage
 
+  lxc profile device remove local:default root
+  lxc project switch default || true
+  lxc remote switch local
   lxc project switch default
   lxc project delete p1
 
   lxc network delete "${netManaged}"
   lxc storage volume delete "${pool}" "v-proj$$"
+  rm -rf "${restrictedDir}"
 }
 
 # Test project state api
@@ -1122,14 +1215,14 @@ test_projects_usage() {
   lxc init testimage c1 --project test-usage
   lxc project info test-usage
 
-  lxc project info test-usage --format csv | grep -q "CONTAINERS,UNLIMITED,1"
-  lxc project info test-usage --format csv | grep -q "CPU,5,1"
-  lxc project info test-usage --format csv | grep -q "DISK,10.00GiB,300.00MiB"
-  lxc project info test-usage --format csv | grep -q "INSTANCES,UNLIMITED,1"
-  lxc project info test-usage --format csv | grep -q "MEMORY,1.00GiB,512.00MiB"
-  lxc project info test-usage --format csv | grep -q "NETWORKS,3,0"
-  lxc project info test-usage --format csv | grep -q "PROCESSES,40,20"
-  lxc project info test-usage --format csv | grep -q "VIRTUAL-MACHINES,UNLIMITED,0"
+  lxc project info test-usage --format csv | grep -xF "CONTAINERS,UNLIMITED,1"
+  lxc project info test-usage --format csv | grep -xF "CPU,5,1"
+  lxc project info test-usage --format csv | grep -xF "DISK,10.00GiB,300.00MiB"
+  lxc project info test-usage --format csv | grep -xF "INSTANCES,UNLIMITED,1"
+  lxc project info test-usage --format csv | grep -xF "MEMORY,1.00GiB,512.00MiB"
+  lxc project info test-usage --format csv | grep -xF "NETWORKS,3,0"
+  lxc project info test-usage --format csv | grep -xF "PROCESSES,40,20"
+  lxc project info test-usage --format csv | grep -xF "VIRTUAL-MACHINES,UNLIMITED,0"
 
   lxc delete c1 --project test-usage
   lxc image delete testimage --project test-usage
@@ -1166,12 +1259,11 @@ EOF
 # Test project operations with an uninitialized LXD.
 test_projects_before_init() {
   LXD_INIT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD_INIT_DIR}"
   spawn_lxd "${LXD_INIT_DIR}" false
 
   # Check if projects can be created and modified without any pre-existing storage pools.
-  LXD_DIR=${LXD_INIT_DIR} lxc project create foo
-  LXD_DIR=${LXD_INIT_DIR} lxc project set foo user.foo test
+  LXD_DIR=${LXD_INIT_DIR} lxc project create foo --config user.foo=bar
+  [ "$(LXD_DIR=${LXD_INIT_DIR} lxc project get foo user.foo)" = "bar" ]
 
   shutdown_lxd "${LXD_INIT_DIR}"
 }
