@@ -20,7 +20,7 @@ test_remote_url() {
 
   # an invalid protocol returns an error
   ! lxc_remote remote add test "${url}" --protocol foo || false
-  [ "$(DEBUG="" lxc_remote remote add test "${url}" --protocol foo 2>&1)" = "Error: Invalid protocol: foo" ]
+  [ "$(CLIENT_DEBUG="" SHELL_TRACING="" lxc_remote remote add test "${url}" --protocol foo 2>&1)" = "Error: Invalid protocol: foo" ]
 
   for url in ${urls}; do
     lxc_remote remote add test "${url}"
@@ -92,7 +92,7 @@ test_remote_url_with_token() {
   [ "$(CERTNAME="token-client" my_curl "https://${LXD_ADDR}/1.0/instances" | jq '.error_code')" -eq 403 ]
 
   # Add valid token
-  CERTNAME="token-client" my_curl -X POST -d "{\"trust_token\": ${token}}" "https://${LXD_ADDR}/1.0/certificates"
+  CERTNAME="token-client" my_curl -X POST --fail-with-body -H 'Content-Type: application/json' -d "{\"trust_token\": ${token}}" "https://${LXD_ADDR}/1.0/certificates"
 
   # Check if we can see instances
   [ "$(CERTNAME="token-client" my_curl "https://${LXD_ADDR}/1.0/instances" | jq '.status_code')" -eq 200 ]
@@ -106,7 +106,7 @@ test_remote_url_with_token() {
   token="$(lxc config trust list-tokens -f json | jq '.[].Token')"
 
   # Add valid token but override projects
-  CERTNAME="token-client" my_curl -X POST -d "{\"trust_token\":${token},\"projects\":[\"default\",\"foo\"],\"restricted\":false}" "https://${LXD_ADDR}/1.0/certificates"
+  CERTNAME="token-client" my_curl -X POST --fail-with-body -H 'Content-Type: application/json' -d "{\"trust_token\":${token},\"projects\":[\"default\",\"foo\"],\"restricted\":false}" "https://${LXD_ADDR}/1.0/certificates"
 
   # Check if we can see instances in the foo project
   [ "$(CERTNAME="token-client" my_curl "https://${LXD_ADDR}/1.0/instances?project=foo" | jq '.status_code')" -eq 200 ]
@@ -156,7 +156,11 @@ test_remote_url_with_token() {
 }
 
 test_remote_admin() {
-  lxc_remote remote add badtoken "${LXD_ADDR}" --token badtoken 2>&1 | grep -F "Error: Failed to decode trust token:"
+  echo "Verify error due to bad token and inspect error message"
+  OUTPUT="$(! lxc_remote remote add badtoken "${LXD_ADDR}" --token badtoken 2>&1 || false)"
+  echo "${OUTPUT}" | grep -F "Error: Failed to decode trust token:"
+
+  echo "Verify that a bad token does not succeed in adding remote"
   ! lxc_remote remote add badtoken "${LXD_ADDR}" --token badtoken || false
   ! lxc_remote remote list | grep -wF badtoken || false
 
@@ -166,14 +170,14 @@ test_remote_admin() {
   ! lxc_remote remote add foo "${LXD_ADDR}" --accept-certificate --token "${token}" || false
 
   lxc_remote remote add foo "${LXD_ADDR}" --token "${token}"
-  lxc_remote remote list | grep 'foo'
+  lxc_remote remote list | grep -wF 'foo'
 
   lxc_remote remote set-default foo
   [ "$(lxc_remote remote get-default)" = "foo" ]
 
   lxc_remote remote rename foo bar
   lxc_remote remote list | grep 'bar'
-  lxc_remote remote list | grep -v 'foo'
+  ! lxc_remote remote list | grep -F 'foo' || false
   [ "$(lxc_remote remote get-default)" = "bar" ]
 
   ! lxc_remote remote remove bar || false
@@ -205,7 +209,6 @@ test_remote_admin() {
 test_remote_usage() {
   local LXD2_DIR LXD2_ADDR
   LXD2_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
-  chmod +x "${LXD2_DIR}"
   spawn_lxd "${LXD2_DIR}" true
   LXD2_ADDR=$(cat "${LXD2_DIR}/lxd.addr")
 
@@ -225,45 +228,47 @@ test_remote_usage() {
 
   lxc_remote image delete "lxd2:${sum}" || true
 
-  lxc_remote image copy localhost:testimage lxd2: --copy-aliases --public
+  lxc_remote image copy --quiet localhost:testimage lxd2: --copy-aliases --public
   lxc_remote image delete "localhost:${sum}"
-  lxc_remote image copy "lxd2:${sum}" local: --copy-aliases --public
+  lxc_remote image copy --quiet "lxd2:${sum}" local: --copy-aliases --public
   lxc_remote image info localhost:testimage
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2:
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2:
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:$(echo "${sum}" | colrm 3)" lxd2:
+  lxc_remote image copy --quiet "localhost:${sum:0:12}" lxd2:
   lxc_remote image delete "lxd2:${sum}"
 
   # test a private image
-  lxc_remote image copy "localhost:${sum}" lxd2:
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2:
   lxc_remote image delete "localhost:${sum}"
-  lxc_remote init "lxd2:${sum}" localhost:c1
+  lxc_remote init --quiet "lxd2:${sum}" localhost:c1
   lxc_remote delete localhost:c1
 
   lxc_remote image alias create localhost:testimage "${sum}"
 
   # test remote publish
-  lxc_remote init testimage pub
-  lxc_remote publish pub lxd2: --alias bar --public a=b
-  lxc_remote image show lxd2:bar | grep -q "a: b"
-  lxc_remote image show lxd2:bar | grep -q "public: true"
+  lxc_remote init --quiet testimage pub
+  lxc_remote publish --quiet pub lxd2: --alias bar --public a=b
+  lxc_remote image show lxd2:bar | grep -F "a: b"
+  lxc_remote image show lxd2:bar | grep -xF "public: true"
+  fingerprint="$(lxc image list -f csv -c F lxd2:bar)"
   ! lxc_remote image show bar || false
   lxc_remote delete pub
 
   # test spawn from public server
   lxc_remote remote add lxd2-public "${LXD2_ADDR}" --public --accept-certificate
-  lxc_remote init lxd2-public:bar pub
+  lxc_remote init --quiet lxd2-public:bar pub
   lxc_remote image delete lxd2:bar
   lxc_remote delete pub
+  lxc_remote image delete "${fingerprint}"
 
   # Double launch to test if the image downloads only once.
-  lxc_remote init localhost:testimage lxd2:c1 &
+  lxc_remote init --quiet localhost:testimage lxd2:c1 &
   C1PID=$!
 
-  lxc_remote init localhost:testimage lxd2:c2
+  lxc_remote init --quiet localhost:testimage lxd2:c2
   lxc_remote delete lxd2:c2
 
   wait "${C1PID}"
@@ -283,7 +288,7 @@ test_remote_usage() {
   mv "${LXD_CONF}/client.key" "${LXD_CONF}/client.key.bak"
 
   # testimage should still exist on the local server.
-  lxc_remote image list local: | grep -q testimage
+  lxc_remote image list local: | grep -wF testimage
 
   # Skip the truly remote servers in offline mode.
   # There should always be Ubuntu images in the results from cloud-images.ubuntu.com remote.
@@ -300,37 +305,37 @@ test_remote_usage() {
 
   lxc_remote image alias create localhost:foo "${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=push
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=push
   lxc_remote image show lxd2:"${sum}"
-  lxc_remote image show lxd2:"${sum}" | grep -q 'public: false'
+  lxc_remote image show lxd2:"${sum}" | grep -xF 'public: false'
   ! lxc_remote image show lxd2:foo || false
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=push --copy-aliases --public
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=push --copy-aliases --public
   lxc_remote image show lxd2:"${sum}"
-  lxc_remote image show lxd2:"${sum}" | grep -q 'public: true'
+  lxc_remote image show lxd2:"${sum}" | grep -xF 'public: true'
   lxc_remote image show lxd2:foo
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=push --copy-aliases --alias=bar
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=push --copy-aliases --alias=bar
   lxc_remote image show lxd2:"${sum}"
   lxc_remote image show lxd2:foo
   lxc_remote image show lxd2:bar
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=relay
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=relay
   lxc_remote image show lxd2:"${sum}"
-  lxc_remote image show lxd2:"${sum}" | grep -q 'public: false'
+  lxc_remote image show lxd2:"${sum}" | grep -xF 'public: false'
   ! lxc_remote image show lxd2:foo || false
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=relay --copy-aliases --public
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=relay --copy-aliases --public
   lxc_remote image show lxd2:"${sum}"
-  lxc_remote image show lxd2:"${sum}" | grep -q 'public: true'
+  lxc_remote image show lxd2:"${sum}" | grep -xF 'public: true'
   lxc_remote image show lxd2:foo
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --mode=relay --copy-aliases --alias=bar
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --mode=relay --copy-aliases --alias=bar
   lxc_remote image show lxd2:"${sum}"
   lxc_remote image show lxd2:foo
   lxc_remote image show lxd2:bar
@@ -338,36 +343,36 @@ test_remote_usage() {
 
   # Test image copy between projects
   lxc_remote project create lxd2:foo
-  lxc_remote image copy "localhost:${sum}" lxd2: --target-project foo
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --target-project foo
   lxc_remote image show lxd2:"${sum}" --project foo
   lxc_remote image delete "lxd2:${sum}" --project foo
-  lxc_remote image copy "localhost:${sum}" lxd2: --target-project foo --mode=push
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --target-project foo --mode=push
   lxc_remote image show lxd2:"${sum}" --project foo
   lxc_remote image delete "lxd2:${sum}" --project foo
-  lxc_remote image copy "localhost:${sum}" lxd2: --target-project foo --mode=relay
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --target-project foo --mode=relay
   lxc_remote image show lxd2:"${sum}" --project foo
   lxc_remote image delete "lxd2:${sum}" --project foo
   lxc_remote project delete lxd2:foo
 
   # Test image copy with --profile option
   lxc_remote profile create lxd2:foo
-  lxc_remote image copy "localhost:${sum}" lxd2: --profile foo
-  lxc_remote image show lxd2:"${sum}" | grep -q '\- foo'
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --profile foo
+  lxc_remote image show lxd2:"${sum}" | grep -xF -- '- foo'
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --profile foo --mode=push
-  lxc_remote image show lxd2:"${sum}" | grep -q '\- foo'
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --profile foo --mode=push
+  lxc_remote image show lxd2:"${sum}" | grep -xF -- '- foo'
   lxc_remote image delete "lxd2:${sum}"
 
-  lxc_remote image copy "localhost:${sum}" lxd2: --profile foo --mode=relay
-  lxc_remote image show lxd2:"${sum}" | grep -q '\- foo'
+  lxc_remote image copy --quiet "localhost:${sum}" lxd2: --profile foo --mode=relay
+  lxc_remote image show lxd2:"${sum}" | grep -xF -- '- foo'
   lxc_remote image delete "lxd2:${sum}"
   lxc_remote profile delete lxd2:foo
 
-  lxc_remote image copy localhost:testimage lxd2: --alias bar
+  lxc_remote image copy --quiet localhost:testimage lxd2: --alias bar
   # Get the `cached` and `aliases` fields for the image `bar` in lxd2
-  cached=$(lxc_remote image info lxd2:bar | awk '/Cached/ { print $2 }')
-  alias=$(lxc_remote image info lxd2:bar | grep -A 1 "Aliases:" | tail -n1 | awk '{print $2}')
+  cached=$(lxc_remote image info lxd2:bar | awk '/^Cached/ { print $2 }')
+  alias=$(lxc_remote image info lxd2:bar | grep -xF -A 1 "Aliases:" | tail -n1 | awk '{print $2}')
 
   # Check that image is not cached
   [ "${cached}" = "no" ]
@@ -376,19 +381,19 @@ test_remote_usage() {
 
   # Now, lets delete the image and observe that when its downloaded implicitly as part of an instance create,
   # the image becomes `cached` and has no alias.
-  fingerprint=$(lxc_remote image info lxd2:bar | awk '/Fingerprint/ { print $2 }')
+  fingerprint=$(lxc_remote image info lxd2:bar | awk '/^Fingerprint/ { print $2 }')
   lxc_remote image delete lxd2:bar
-  lxc_remote init localhost:testimage lxd2:c1
-  cached=$(lxc_remote image info "lxd2:${fingerprint}" | awk '/Cached/ { print $2 }')
+  lxc_remote init --quiet localhost:testimage lxd2:c1
+  cached=$(lxc_remote image info "lxd2:${fingerprint}" | awk '/^Cached/ { print $2 }')
   # The `cached` field should be set to `yes` since the image was implicitly downloaded by the instance create operation
   [ "${cached}" = "yes" ]
   # There should be no alias for the image
-  ! lxc_remote image info "lxd2:${fingerprint}" | grep -q "Aliases:"
+  [ "$(lxc_remote image list "lxd2:${fingerprint}" -f csv -c l)" = "" ]
 
   # Finally, lets copy the remote image explicitly to the local server with an alias like we did before
-  lxc_remote image copy localhost:testimage lxd2: --alias bar
-  cached=$(lxc_remote image info lxd2:bar | awk '/Cached/ { print $2 }')
-  alias=$(lxc_remote image info lxd2:bar | grep -A 1 "Aliases:" | tail -n1 | awk '{print $2}')
+  lxc_remote image copy --quiet localhost:testimage lxd2: --alias bar
+  cached=$(lxc_remote image info lxd2:bar | awk '/^Cached/ { print $2 }')
+  alias=$(lxc_remote image info lxd2:bar | grep -xF -A 1 "Aliases:" | tail -n1 | awk '{print $2}')
   # The `cached` field should be set to `no` since the image was explicitly copied.
   [ "${cached}" = "no" ]
   # The alias should be set to `bar`.
@@ -396,6 +401,7 @@ test_remote_usage() {
 
   lxc_remote image alias delete localhost:foo
 
+  lxc_remote image delete localhost:testimage
   lxc_remote remote remove lxd2
   lxc_remote remote remove lxd2-public
 
