@@ -60,6 +60,35 @@ test_storage_buckets() {
   ! lxc storage bucket create "${poolName}" fooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo || false
   ! lxc storage bucket create "${poolName}" "foo bar" || false
 
+  # Ensure storage bucket size can be configured.
+  if [ "$lxd_backend" != "dir" ]; then
+    lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-size" size=128MiB
+    lxc storage bucket delete "${poolName}" "${bucketPrefix}.foo-size"
+  else
+    # dir backend doesn't support size.
+    ! lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-size" size=128MiB || false
+  fi
+
+  # Test some storage backend specific options.
+  if [ "$lxd_backend" = "zfs" ]; then
+    # Ensure block.* options are not allowed on non-block-backed volumes.
+    ! lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-fs-invalid" block.filesystem=btrfs || false
+    ! lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-fs-invalid" block.filesystem=btrfs zfs.block_mode=false || false
+
+    # Ensure block.* options are allowed on block-backed volumes.
+    lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-fs" block.filesystem=btrfs zfs.block_mode=true
+    [ "$(lxc storage bucket get "${poolName}" "${bucketPrefix}.foo-fs" block.filesystem)" = "btrfs" ]
+    [ "$(lxc storage bucket get "${poolName}" "${bucketPrefix}.foo-fs" block.mount_options)" = "discard" ]
+    [ "$(lxc storage bucket get "${poolName}" "${bucketPrefix}.foo-fs" zfs.block_mode)" = "true" ]
+    lxc storage bucket delete "${poolName}" "${bucketPrefix}.foo-fs"
+  elif [ "$lxd_backend" = "lvm" ]; then
+    # Ensure filesystem can be configured on LVM pools.
+    lxc storage bucket create "${poolName}" "${bucketPrefix}.foo-fs" block.filesystem=btrfs
+    [ "$(lxc storage bucket get "${poolName}" "${bucketPrefix}.foo-fs" block.filesystem)" = "btrfs" ]
+    [ "$(lxc storage bucket get "${poolName}" "${bucketPrefix}.foo-fs" block.mount_options)" = "discard" ]
+    lxc storage bucket delete "${poolName}" "${bucketPrefix}.foo-fs"
+  fi
+
   # Create bucket.
   initCreds=$(lxc storage bucket create "${poolName}" "${bucketPrefix}.foo" user.foo=comment)
   initAccessKey=$(echo "${initCreds}" | awk '{ if ($2 == "access" && $3 == "key:") {print $4}}')
@@ -125,7 +154,8 @@ EOF
   [ "${ORIG_MD5SUM}" = "${INFO_MD5SUM}" ]
   [ "${ORIG_MD5SUM}" = "$(md5sum < "${lxdTestFile}.get")" ]
   rm "${lxdTestFile}.get"
-  INFO_MD5SUM="$(s3cmdrun "${lxd_backend}" "${roAccessKey}" "${roSecretKey}" info "s3://${bucketPrefix}.foo/${lxdTestFile}" | awk '{ if ($1 == "MD5") {print $3}}')  -"
+  # roAccessKey cannot get the `info`
+  ! s3cmdrun "${lxd_backend}" "${roAccessKey}" "${roSecretKey}" info "s3://${bucketPrefix}.foo/${lxdTestFile}" || false
   s3cmdrun "${lxd_backend}" "${roAccessKey}" "${roSecretKey}" get "s3://${bucketPrefix}.foo/${lxdTestFile}" "${lxdTestFile}.get"
   [ "${ORIG_MD5SUM}" = "${INFO_MD5SUM}" ]
   [ "${ORIG_MD5SUM}" = "$(md5sum < "${lxdTestFile}.get")" ]
