@@ -1,13 +1,15 @@
 test_storage_driver_btrfs() {
-  local LXD_STORAGE_DIR lxd_backend
+  local lxd_backend
 
-  lxd_backend=$(storage_backend "$LXD_DIR")
-  if [ "$lxd_backend" != "btrfs" ]; then
+  lxd_backend=$(storage_backend "${LXD_DIR}")
+  if [ "${lxd_backend}" != "btrfs" ]; then
+    echo "==> SKIP: test_storage_driver_btrfs only supports 'btrfs', not ${lxd_backend}"
     return
   fi
 
+  local LXD_STORAGE_DIR
+
   LXD_STORAGE_DIR=$(mktemp -d -p "${TEST_DIR}" XXXXXXXXX)
-  chmod +x "${LXD_STORAGE_DIR}"
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   (
