@@ -1,13 +1,15 @@
 test_storage_driver_ceph() {
-  local LXD_STORAGE_DIR lxd_backend
+  local lxd_backend
 
-  lxd_backend=$(storage_backend "$LXD_DIR")
-  if [ "$lxd_backend" != "ceph" ]; then
+  lxd_backend=$(storage_backend "${LXD_DIR}")
+  if [ "${lxd_backend}" != "ceph" ]; then
+    echo "==> SKIP: test_storage_driver_ceph only supports 'ceph', not ${lxd_backend}"
     return
   fi
 
+  local LXD_STORAGE_DIR
+
   LXD_STORAGE_DIR=$(mktemp -d -p "${TEST_DIR}" XXXXXXXXX)
-  chmod +x "${LXD_STORAGE_DIR}"
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   (
@@ -53,9 +55,7 @@ test_storage_driver_ceph() {
     lxc list -c b c4pool2 | grep "lxdtest-$(basename "${LXD_DIR}")-pool2"
 
     lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool1" volume.block.filesystem xfs
-    # xfs is unhappy with block devices < 48 MiB. It seems to calculate the
-    # ag{count,size} parameters wrong and/or sets the data area too big.
-    lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool1" volume.size 48MiB
+    lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool1" volume.size 300MiB # modern xfs requires 300MiB or more
     lxc init testimage c5pool1 -s "lxdtest-$(basename "${LXD_DIR}")-pool1"
 
     # Test whether dependency tracking is working correctly. We should be able
@@ -129,7 +129,7 @@ test_storage_driver_ceph() {
     lxc profile device remove default root
     lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-pool1"
     lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-pool2"
-    ! ceph --cluster "${LXD_CEPH_CLUSTER}" osd pool ls | grep -q "lxdtest-$(basename "${LXD_DIR}")-existing-osd-pool" || false
+    ! ceph --cluster "${LXD_CEPH_CLUSTER}" osd pool ls | grep -F "lxdtest-$(basename "${LXD_DIR}")-existing-osd-pool" || false
 
 
     # Test that pre-existing OSD pools are not affected by the config option in LXD. Only the associated pool should be affected.
