@@ -6,7 +6,6 @@ test_storage_volume_snapshots() {
 
   lxd_backend=$(storage_backend "$LXD_DIR")
   LXD_STORAGE_DIR=$(mktemp -d -p "${TEST_DIR}" XXXXXXXXX)
-  chmod +x "${LXD_STORAGE_DIR}"
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   local storage_pool storage_volume
@@ -20,7 +19,7 @@ test_storage_volume_snapshots() {
   else
     lxc storage create "$storage_pool" "$lxd_backend"
   fi
-  lxc storage volume create "${storage_pool}" "${storage_volume}"
+  lxc storage volume create "${storage_pool}" "${storage_volume}" size=1MiB
   lxc launch testimage c1 -s "${storage_pool}"
   lxc storage volume attach "${storage_pool}" "${storage_volume}" c1 /mnt
   # Create file on volume
@@ -81,23 +80,33 @@ EOF
 
   # edit volume snapshot description
   lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | sed 's/^description:.*/description: foo/' | lxc storage volume edit "${storage_pool}" "${storage_volume}/snap0"
-  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep -q 'description: foo'
+  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep -xF 'description: foo'
 
   # edit volume snapshot expiry date
   lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | sed 's/^expires_at:.*/expires_at: 2100-01-02T15:04:05Z/' | lxc storage volume edit "${storage_pool}" "${storage_volume}/snap0"
   # Depending on the timezone of the runner, some values will be different.
   # Both the year (2100) and the month (01) will be constant though.
-  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep -q '^expires_at: 2100-01'
+  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep '^expires_at: 2100-01'
   # Reset/remove expiry date
   lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | sed '/^expires_at:/d' | lxc storage volume edit "${storage_pool}" "${storage_volume}/snap0"
-  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep -q '^expires_at: 0001-01-01T00:00:00Z'
+  lxc storage volume show "${storage_pool}" "${storage_volume}/snap0" | grep '^expires_at: 0001-01-01T00:00:00Z'
+
+  # Check the API returns the zero time representation when listing all snapshots in recursive mode.
+  [ "$(lxc query "/1.0/storage-pools/${storage_pool}/volumes/custom/${storage_volume}/snapshots?recursion=2" | jq -r '.[] | select(.name == "'"${storage_volume}/snap0"'") | .expires_at')" = "0001-01-01T00:00:00Z" ]
 
   lxc storage volume set "${storage_pool}" "${storage_volume}" snapshots.expiry '1d'
   lxc storage volume snapshot "${storage_pool}" "${storage_volume}"
-  ! lxc storage volume show "${storage_pool}" "${storage_volume}/snap1" | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
+
+  # Get snapshot created_at and expires_at properties.
+  # Remove the " +0000 UTC" from the end of the timestamp so we can add one day using `date`.
+  created_at="$(lxc storage volume get "${storage_pool}" "${storage_volume}/snap1" --property created_at | awk -F' +' '{print $1}')"
+  expires_at="$(lxc storage volume get "${storage_pool}" "${storage_volume}/snap1" --property expires_at | awk -F' +' '{print $1}')"
+
+  # Check if the expires_at propery is exactly 1d ahead.
+  [ "$(date -d "${created_at} today + 1days")" = "$(date -d "${expires_at}")" ]
 
   lxc storage volume snapshot "${storage_pool}" "${storage_volume}" --no-expiry
-  lxc storage volume show "${storage_pool}" "${storage_volume}/snap2" | grep -q 'expires_at: 0001-01-01T00:00:00Z'
+  lxc storage volume show "${storage_pool}" "${storage_volume}/snap2" | grep 'expires_at: 0001-01-01T00:00:00Z'
 
   lxc storage volume rm "${storage_pool}" "${storage_volume}/snap2"
   lxc storage volume rm "${storage_pool}" "${storage_volume}/snap1"
@@ -142,8 +151,8 @@ EOF
   lxc storage volume delete "${storage_pool}" "${storage_volume}"
 
   # Check snapshots naming conflicts.
-  lxc storage volume create "${storage_pool}" "vol1"
-  lxc storage volume create "${storage_pool}" "vol1-snap0"
+  lxc storage volume create "${storage_pool}" "vol1" size=1MiB
+  lxc storage volume create "${storage_pool}" "vol1-snap0" size=1MiB
   lxc storage volume snapshot "${storage_pool}" "vol1" "snap0"
   lxc storage volume delete "${storage_pool}" "vol1"
   lxc storage volume delete "${storage_pool}" "vol1-snap0"
@@ -159,19 +168,19 @@ EOF
   ! lxc storage volume create "${storage_pool}" "vol1" --type block block.mount_options=xyz || false
 
   # Check snapshot creation dates.
-  lxc storage volume create "${storage_pool}" "vol1"
+  lxc storage volume create "${storage_pool}" "vol1" size=1MiB
   lxc storage volume snapshot "${storage_pool}" "vol1" "snap0"
-  ! lxc storage volume show "${storage_pool}" "vol1" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
-  ! lxc storage volume show "${storage_pool}" "vol1/snap0" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "vol1" | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "vol1/snap0" | grep '^created_at: 0001-01-01T00:00:00Z' || false
   lxc storage volume copy "${storage_pool}/vol1" "${storage_pool}/vol2"
-  ! lxc storage volume show "${storage_pool}" "vol2" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "vol2" | grep '^created_at: 0001-01-01T00:00:00Z' || false
   [ "$(lxc storage volume show "${storage_pool}" "vol1/snap0" | awk /created_at:/)" = "$(lxc storage volume show "${storage_pool}" "vol2/snap0" | awk /created_at:/)" ]
   lxc storage volume delete "${storage_pool}" "vol1"
   lxc storage volume delete "${storage_pool}" "vol2"
 
   # Check snapshot copy (mode pull).
   lxc launch testimage "c1"
-  lxc storage volume create "${storage_pool}" "vol1"
+  lxc storage volume create "${storage_pool}" "vol1" size=1MiB
   lxc storage volume attach "${storage_pool}" "vol1" "c1" /mnt
   lxc exec "c1" -- touch /mnt/foo
   lxc delete -f "c1"
@@ -284,12 +293,12 @@ EOF
   lxc storage volume delete "${storage_pool}" "vol1"
 
   # Check snapshot creation dates (remote).
-  lxc storage volume create "${storage_pool}" "vol1"
+  lxc storage volume create "${storage_pool}" "vol1" size=1MiB
   lxc storage volume snapshot "${storage_pool}" "vol1" "snap0"
-  ! lxc storage volume show "${storage_pool}" "vol1" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
-  ! lxc storage volume show "${storage_pool}" "vol1/snap0" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "vol1" | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "vol1/snap0" | grep '^created_at: 0001-01-01T00:00:00Z' || false
   lxc storage volume copy "${storage_pool}/vol1" "localhost:${storage_pool}/vol1-copy"
-  ! lxc storage volume show "${storage_pool}" "localhost:${storage_pool}" "vol1-copy" | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${storage_pool}" "localhost:${storage_pool}" "vol1-copy" | grep '^created_at: 0001-01-01T00:00:00Z' || false
   [ "$(lxc storage volume show "${storage_pool}" "vol1/snap0" | awk /created_at:/)" = "$(lxc storage volume show "localhost:${storage_pool}" "vol1-copy/snap0" | awk /created_at:/)" ]
   lxc storage volume delete "${storage_pool}" "vol1"
   lxc storage volume delete "${storage_pool}" "vol1-copy"
