@@ -55,7 +55,7 @@ test_warnings() {
 
     # Delete warning
     lxc warning rm "${uuid}"
-    ! lxc warning list | grep -q "${uuid}" || false
+    ! lxc warning list | grep -F "${uuid}" || false
     ! lxc warning show "${uuid}" || false
 
     # Delete all warnings
